function [PredEventData,PredictionEvent,ShockMean,ShockCov,ParameterCov] = LoadBVARPredictionUncertainty(OutputDirectory,CondStr,NameOfModel,PriorType,OmegaPriorType,h,NumDraws,NumPaths,AnnualizeData)
% LoadBVARPredictionUncertainty: Loads the results for the prediction uncertainty decomposition for a Bayesian VAR model
%                                from disk. It also loads any data it finds about prediction events.
%
% USAGE:
%
%       [PredEventData,PredictionEvent,ShockMean,ShockCov,ParameterCov] = LoadBVARPredictionUncertainty(OutputDirectory,CondStr,...
%                                             NameOfModel,PriorType,OmegaPriorType,h,NumDraws,NumPaths,AnnualizeData)
%
% REQUIRED INPUT:  OutputDirectory (string) with the full path to the base output directory.
%
%                  CondStr (string) that takes the values "" or "Cond".
%
%                  NameOfModel (string) with the name of the model.
%
%                  PriorType (integer) determining the type of prior used for the parameters on lagged endogenous
%                            variables of the Bayesian VAR. The value "1" is the Minnesota-style prior, "2" is the
%                            normal conditional on the covariance matrix prior, and "3" the diffuse prior.
%
%                  OmegaPriorType (integer) determining the type of prior used for the covariance matrix parameters.
%                            The value "1" is a diffuse prior, and "2" and inverse Wishart prior.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPaths (integer) with the total number of prediction paths per parameter draw.
%
%                  AnnualizeData (boolean) that take the value 1 if annualized results should be checked and 0 otherwise.
%
% REQUIRED OUTPUT: PredEventData (matrix) of dim px8 where the first column holds the number of times
%                            the PredPaths data satisfies the event, the second holds the number
%                            of times the PredPaths data is greater than the event, the third
%                            the number of time the data is above the upper bound, while the
%                            fourth column holds the number of times the event has been tested.
%                            The fifth and sixth column measure the average deviation from the
%                            lower bound when below the lower bound and the square of this, while
%                            the seventh and eighth columns hold similar data for the upper bound
%                            of the event when the variables are above the upper bound.
%
%                  PredictionEvent (matrix) of dim px3 with the prediction event information. The first
%                            column holds the upper bound, the second the lower bound for the event,
%                            while the third holds the event length in periods.
%
%                  ShockMean (matrix) of dimension pxh (or empty) with the population mean of the
%                            BVAR residuals at the population mean prediction of the observed
%                            variables.
%
%                  ShockCov (matrix) of dimension pxpxh with the estimated shock uncertainty.
%
%                  ParameterCov (matrix) of dimension pxpxh with the estimated parameter uncertainty.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 10, 2007.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 11-05-2007: Updated the documentation.
%
% * 06-06-2007: The PredEventData matrix now has three columns.
%
% * 13-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 18-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 19-11-2010: Took into account that "PredEventData" should have 8 columns.
%
% * 05-01-2011: Updated the documentation.
%
% * 13-01-2011: The function now supports unix and mac paths.
%
% * 08-03-2011: Added the "CondStr" input variable, and the "ShockMean" output variable.
%
% * 02-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 09-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
PredEventData = [];
PredictionEvent = [];
ShockMean = [];
ShockCov = [];
ParameterCov = [];
if AnnualizeData==0;
   file = FixFilePath([OutputDirectory '\predictions\BVAR-' CondStr 'Prediction-Original-' NameOfModel '-' num2str(PriorType,'%0.0f') '.' num2str(OmegaPriorType,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(NumDraws,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
else;
   file = FixFilePath([OutputDirectory '\predictions\BVAR-' CondStr 'Prediction-Annual-' NameOfModel '-' num2str(PriorType,'%0.0f') '.' num2str(OmegaPriorType,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(NumDraws,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
end;
if FileExist(file)==1;
   PredData = load(file);
   if isfield(PredData,'ShockCov')==1;
      ShockCov = PredData.ShockCov;
      ParameterCov = PredData.ParameterCov;
   end;
   if isfield(PredData,'PredEventData')==1;
      if size(PredData.PredEventData,2)==8;
         PredEventData = PredData.PredEventData;
      end;
   end;
   if isfield(PredData,'PredictionEvent')==1;
      PredictionEvent = PredData.PredictionEvent;
   end;
   if isfield(PredData,'ShockMean')==1;
      ShockMean = PredData.ShockMean;
   end;
end;

%
% end of LoadBVARPredictionUncertainty.m
%
