function UpdatePartial = GetUpdatePartial(UpdateValue,theta,UpdateNames,ModelParameters,thetaPositions,UpdateParameterFile)
% GetUpdatePartial: Computes numerical partial derivatives of the parameters that are a function of the estimated
%                   parameters.
%
% USAGE:
%
%       UpdatePartial = GetUpdatePartial(UpdateValue,theta,UpdateNames,ModelParameters,thetaPositions,UpdateParameterFile)
%
% REQUIRED INPUT:  UpdateValue (vector) of dimension u with the number of parameters that are updated.
%
%                  theta (vector) of dimension p with the estimated parameters.
%
%                  UpdateNames (string matrix) with the names of the parameters that are updated.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  UpdateParameterFile (string) with the name of the function that computes the update
%                                 parameters.
%
% REQUIRED OUTPUT: UpdatePartial (matrix) of dimension uxp with the numerical partial derivatives.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 3, 2010.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed the dos copy call to a "CopyFile" call.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 11-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 21-02-2022: Changed an eval call to feval.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
u = length(UpdateValue);
p = length(theta);
UpdatePartial = zeros(u,p);
delta = 1e-6;
scale = delta*eye(p);
%
% copy the update parameter file to the tmp folder
%
[stat,msg] = CopyFile(UpdateParameterFile,[pwd '\tmp']);
%
% change the original parameters one-by-one
%
for i=1:p;
   thetaNew = theta+scale(:,i);
   %
   % update the model parameter structure
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,thetaNew,thetaPositions);
   %
   % Update any parameteters that need updating
   %
   ModelParameters = feval(GetName(UpdateParameterFile),ModelParameters);
   %
   % determine the new values of the update parameters
   %
   UpdateNew = zeros(u,1);
   for j=1:u;
      UpdateNew(j) = getfield(ModelParameters,StringTrim(UpdateNames(j,:)));
   end;
   UpdatePartial(:,i) = (1/delta)*(UpdateNew-UpdateValue);
end;

%
% end of GetUpdatePartial.m
%
