function [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = GetUpdateSystemPriorDensity(DSGEModel,CurrINI,controls,CurrentChain,NumChains,NumBurnin,NumIter)
% GetUpdateSystemPriorDensity: Tries to estimate prior densities for the parameters that are a function of
%                              the estimated parameters by drawing from the prior distribution of the latter
%                              parameters.
%
% USAGE:
%
%       [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = ...
%                  GetUpdateSystemPriorDensity(DSGEModel,CurrINI,controls,CurrentChain,NumChains,NumBurnin,NumIter)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the model,
%                            such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  CurrentChain (integer) with the number of the currently used MCMC draws.
%
%                  NumChains (integer) with the total number of MCMC chains that we can choose from.
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  NumIter (integer) with the total number of posterior draws that would have
%                            been draws based on the user settings.
%
% REQUIRED OUTPUT: UpdatePriorX (matrix) of dimension NumAdditional x NumValues with the
%                            values on the X-axis for the prior densities of the
%                            NumAdditional additional parameters.
%
%                  UpdatePriorY (matrix) of dimension NumAdditional x NumValues with the
%                            density values for the Y-axis of the prior densities of
%                            the NumAdditional additional parameters.
%
%                  UpdatePriorBounds (matrix) of dimension NumAdditional x 2 with suitable
%                            lower and upper bounds for the additional parameters.
%
%                  UpdateParameters (vector) with the entries of the updated parameters that actually
%                            are a function of the estimated parameters.
%
% OPTIONAL OUTPUT: UpdateInitialValue (vector) with the initial values of the additional
%                            parameters.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 8, 2017.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-06-2017: Updated the documentation.
%
% * 11-08-2017: The function now supports the SMC with likelihood tempering posterior sampler.
%
% * 11-01-2018: Updated the documentation.
%
% * 14-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 16-10-2018: The function now supports the SMC with data tempering posterior sampler.
%
% * 19-10-2018: Updated the documentation.
%
% * 25-10-2018: Made sure that the name of the prior draws file reflects using a system prior.
%
% * 30-10-2018: Removed a check for LogMargLikeSystemLaplace when opening the prior draws file.
%               With the renaming of the system prior draws file, this check is no longer necessary.
%
% * 02-11-2018: The function now checks if update draws have already been stored to the prior
%               draws file and makes use of them when this is the case. The code now also makes
%               sure not to remove any update parameters due to small differences or very high
%               densities heights.
%
% * 03-12-2018: The function should now support IS based on MitISEM.
%
% * 04-12-2018: Made sure that NumBurnin is removed from AddFile for IS.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed eval calls to feval.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
UpdatePriorX = [];
UpdatePriorY = [];
UpdatePriorBounds = [];
UpdateParameters = [];
if nargout==5;
   UpdateInitialValue = [];
end;
%
% make sure that the prior draws are already available
%
ErrorStr = '';
PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
if FileExist(PriorDrawsFile)==1;
   %
   % check if the file contains the correct information
   %
   PriorDrawsData = load(PriorDrawsFile);
else;
   ErrorStr = 'The prior draws file doesn''t exist. You must run prior sampling before YADA can graph the prior densities when these include a system prior.';
end;
%
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','Prior Densities - Update Parameters',500,600,CurrINI);
   drawnow;
   return;
end;
%
% number of tempering stages
%
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
% first we check if the prior densities have already been calculated and saved to file
%
if NumChains>0;
   if DSGEModel.PosteriorSampler==1;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==2;
      stat = MakeDir(DSGEModel.OutputDirectory,'slice');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Slice-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==3;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==4;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==5;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==6;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==7;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==8;
      stat = MakeDir(DSGEModel.OutputDirectory,'smc');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-Additional-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==9;
      stat = MakeDir(DSGEModel.OutputDirectory,'smc');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==10;
      stat = MakeDir(DSGEModel.OutputDirectory,'is');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\is\IS-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumIter,'%0.0f') '.mat']);
   end;
else;
   stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-Additional-' DSGEModel.NameOfModel '.mat']);
end;
if FileExist(AddFile)==1;
   %
   % check if we can load the data
   % 
   AddData = load(AddFile);
   if isfield(AddData,'UpdatePriorX')==1;
      %
      % ask if we should load prior draws from disk
      %
      nup = size(AddData.UpdatePriorX,1);
      if nup>1;
         txt = ['The prior distributions for the ' int2str(nup) ' update parameters have already been calculated. Would you like to reuse these distributions?'];
      else;
         txt = ['The prior distribution for the single update parameter has already been calculated. Would you like to reuse this distribution?'];
      end;
      answer = Query(txt,'question',200,'Prior Densities - Update Parameters',500,'yes',CurrINI);
      if strcmp(lower(answer),'yes')==1;
         UpdatePriorX = AddData.UpdatePriorX;
         UpdatePriorY = AddData.UpdatePriorY;
         UpdatePriorBounds = AddData.UpdatePriorBounds;
         if isfield(AddData,'UpdateParameters')==1;
            UpdateParameters = AddData.UpdateParameters;
         else;
            UpdateParameters = (1:size(UpdatePriorX,1))
         end;
         if (nargout==5)&(isfield(AddData,'UpdateInitialValue')==1);
            UpdateInitialValue = AddData.UpdateInitialValue;
         end;
         return;
      end;
   end;
end;
%
% check if we can load the posterior mode data
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
if NumChains>0;
   if FileExist(ModeFile)==1;
      ModeData = load(ModeFile);
      theta = ModeData.theta;
      thetaDist = ModeData.thetaDist;
      LowerBound = ModeData.LowerBound;
      PriorDist = ModeData.PriorDist;
      ParameterNames = ModeData.ParameterNames;
      thetaPositions = ModeData.thetaPositions;
      ModelParameters = ModeData.ModelParameters;
   else;
      [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI)
      if InitStatus==0;
         return;
      end;
      %
      % check if we have a file with parameters to initialize
      %
      if FileExist(DSGEModel.InitializeParameterFile)==1;
         [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
         if DSGEModel.RunInitializeFirst==1;
            try;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            catch;
               return;
            end;
         end;
      end;
   end;
else;
   %
   % collect input from CurrINI
   %
   theta = CurrINI.theta;
   thetaDist = CurrINI.thetaDist;
   LowerBound = CurrINI.LowerBound;
   PriorDist = CurrINI.PriorDist;
   ParameterNames = CurrINI.ParameterNames;
   thetaPositions = CurrINI.thetaPositions;
   ModelParameters = CurrINI.ModelParameters;
   if FileExist(DSGEModel.InitializeParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
      if DSGEModel.RunInitializeFirst==1;
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            catch ME;
               ErrorStr = ['There was en error run the file with parameters to initialize. YADA received the following message: ' YADALastError(ME)];
            end;
         else;
            try;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            catch;
               ErrorStr = ['There was en error run the file with parameters to initialize. YADA received the following message: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==0;
            About(ErrorStr,'error','YADA - Error',500,600,CurrINI);
            return;
         end;
      end;
   end;
end;
%
% copy update parameter file to tmp folder
%
[stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
%
% make sure the ModelParameters structure is update with the initial values of the
% estimated parameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% run the update parameter function
%
ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
%
% check if we should run the initialize parameter function
%
if (FileExist(DSGEModel.InitializeParameterFile)==1)&((FileExist(ModeFile)==0)|(NumChains==0))&(DSGEModel.RunInitializeFirst==0);
   try;
      ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
   catch;
      return;
   end;
end;
%
% the ModelParameter structure now only has values that depend on the initial parameter values.
% determine the values for the additional parameters
%
NumParam = length(theta);
NumAdditional = size(ParameterNames.additional,1);
UpdateInitial = zeros(NumAdditional,1);
for j=1:NumAdditional;
   UpdateInitial(j,1) = getfield(ModelParameters,StringTrim(ParameterNames.additional(j,:)));
end;
%
% get the draws of the estimated parameters from the system prior
%
thetaDraws = PriorDrawsData.thetaDraws;
NumPriorDraws = size(thetaDraws,2);
%
% check if the update prior draws already exist
%
if (isfield(PriorDrawsData,'UpdatePriorSample')==1)&(size(PriorDrawsData.UpdatePriorSample,1)==NumAdditional);
   ComputeUpdateDraws = 0;
   UpdateDraws = PriorDrawsData.UpdatePriorSample;
else;
   ComputeUpdateDraws = 1;
end;
%
if ComputeUpdateDraws==1;
   %
   % we now turn to computing the values of the additional parameters under the draws from the prior
   % of the estimated parameters
   %
   if NumPriorDraws>=1000;
      %
      % display a wait dialog
      %
      txt = ['Please wait while YADA computes the prior draws of the ' int2str(NumAdditional) ' update parameters using ' int2str(NumPriorDraws) ' prior draws of the estimated parameters. Computations started at: ' StringTrim(datestr(now,14))  '.'];
      WaitHandle = WaitDLG(txt,'information','Prior Densities - Update Parameters',500,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
      drawnow;
      pause(0.02);
   end;
   %
   UpdateDraws = zeros(NumAdditional,NumPriorDraws);
   %
   for i=1:NumPriorDraws;
      %
      % update ModelParameters
      %
      ModelParameters = ThetaToModelParameters(ModelParameters,thetaDraws(:,i),thetaPositions);
      %
      % run the update parameters function
      %
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
      %
      % collect the update parameter values
      %
      for j=1:NumAdditional;
         UpdateDraws(j,i) = getfield(ModelParameters,StringTrim(ParameterNames.additional(j,:)));
      end;
      drawnow;
   end;
   %
   if NumPriorDraws>=1000;
      %
      % close the wait dialog
      %
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
% determine upper and lower bounds for the update parameters
%
LowerB = floor(min(UpdateDraws'));
UpperB = ceil(max(UpdateDraws'));
if size(LowerB,2)>1
   LowerB = LowerB';
end;
if size(UpperB,2)>1;
   UpperB = UpperB';
end;
UpdatePriorBounds = [LowerB UpperB];
%
% we may now turn to the Kernel density estimation
%
ErrorMsg = '';
for i=1:NumAdditional;
   if MatlabNumber>=8.0;
      try;
         [XValues,YValues] = KernelDensityPlot(UpdateDraws(i,:),DSGEModel.KernelDensityEstimator);
      catch ME;
         XValues = NaN*ones(1,512);
         YValues = NaN*ones(1,512);
         ErrorMsg = ['YADA was unable to estimate the prior density of the parameter "' StringTrim(CurrINI.UpdateNames(i,:)) '". The following error message was caught: ' YADALastError(ME)];
         About(ErrorMsg,'information','Prior Densities - Update Parameters',500,500,CurrINI);
      end;
   else;
      try;
         [XValues,YValues] = KernelDensityPlot(UpdateDraws(i,:),DSGEModel.KernelDensityEstimator);
      catch;
         XValues = NaN*ones(1,512);
         YValues = NaN*ones(1,512);
         ErrorMsg = ['YADA was unable to estimate the prior density of the parameter "' StringTrim(CurrINI.UpdateNames(i,:)) '". The following error message was caught: ' YADALastError];
         About(ErrorMsg,'information','Prior Densities - Update Parameters',500,500,CurrINI);
      end;
   end;
   if i==1;
      UpdatePriorX = zeros(NumAdditional,length(XValues));
      UpdatePriorY = UpdatePriorX;
   end;
   UpdatePriorX(i,:) = XValues;
   UpdatePriorY(i,:) = YValues;
   drawnow;
end;
%
% save the results to file so that they can be quickly retrieved
%
if nargout==5;
   UpdateInitialValue = UpdateInitial;
end;
if isempty(ErrorMsg)==1;
   if NumChains>0;
      save(AddFile,'UpdatePriorX','UpdatePriorY','UpdatePriorBounds','UpdateParameters','-APPEND');
   else;
      save(AddFile,'UpdatePriorX','UpdatePriorY','UpdatePriorBounds','UpdateParameters','UpdateInitialValue');
   end;
end;

%
% end of GetUpdateSystemPriorDensity.m
%
