function [lnS,rhoTerm] = logMultiStudentTPDF(z,mu,LogDetSigma,InvSigma,df)
% logMultiStudentTPDF: Calculates the height of the log of the multivariate Student-t density.
%
% USAGE:
%
%       [lnS,rhoTerm] = logMultiStudentTPDF(z,mu,LogDetSigma,InvSigma,df)
%
% REQUIRED INPUT:  z (vector) with values of the random variables.
%
%                  mu (vector) with means.
%
%                  LogDetSigma (scalar) with the log of the determinant of the
%                              covariance matrix.
%
%                  InvSigma (matrix) with the inverse of the covariance matrix.
%
%                  df (scalar) with the number of degrees of freedom.
%
% REQUIRED OUTPUT: lnS (scalar) with the log height of the density.
%
%                  rhoTerm (scalar) with the value of the squared deviation term, i.e.
%                          (z-mu)'*InvSigma*(z-mu).
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 5, 2018.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-12-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

n = length(mu);
rhoTerm = (z-mu)'*InvSigma*(z-mu);
lnS = gammaln((df+n)/2)-gammaln(df/2)-((n/2)*log(pi*df))-((1/2)*LogDetSigma)-(((df+n)/2)*log(1+(rhoTerm/df)));

%
% end of logMultiStudentTPDF.m
%
