function [SOmega,Omega] = DSGEVARCoherenceParam(Phi,SigmaEpsilon,lambda,EstStr,DSGEModel,CurrINI)
% DSGEVARCoherenceParam: Computes coherence between observed variables of a the DSGE-VAR model for a fixed
%                        parameter value.
%
% USAGE:
%
%       [SOmega,Omega] = DSGEVARCoherenceParam(Phi,SigmaEpsilon,lambda,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nxnp with parameters on lagged endogenous variables.
%
%                  SigmaEpsilon (matrix) of dimension nxn with residual covariances.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  EstStr (string) that takes the values "Initial Values", "Marginal Posterior mode", or
%                         "Joint Posterior Mode".
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                         input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: SOmega (matrix) of dimension n(n-1)/2 x f with the n(n-1)/2 unique coherence statistics for
%                                 f frequencies from 0 to pi of the m variables.
%
%                  Omega (vector) of dimension f with the frequencies from 0 to pi.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 21, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-06-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initilize
%
SOmega = [];
%
% determine dimensions
%
[n,np] = size(Phi);
p = np/n;
Omega = (0:pi/299:pi);
f = length(Omega);
%
% compute a matrix with the transposes of the submatrices of Phi, i.e. with
% Phi = [Phi(1) ... Phi(p)], we're looking for PhiT = [Phi(1)' ... Phi(p)'];
%
PhiT = zeros(n,np);
for k=1:p;
   PhiT(:,1+(n*(k-1)):n*k) = Phi(:,1+(n*(k-1)):n*k)';
end;
In = eye(n);
%
% compute pxf matrices with exp(-i*k*omega) and exp(i*k*omega), respectively, where k=1,...,p.
%
NegOmega = exp(-i*(1:p)'*Omega);
PosOmega = exp(i*(1:p)'*Omega);
%
% display a wait dialog
%
if n>=7;
   txt = ['Please wait while YADA computes coherence for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' at ' lower(EstStr) '.'];
   WaitHandle = WaitDLG(txt,'information',['Coherence - ' EstStr],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
else;
   WaitHandle = [];
end;
%
% compute the population spectrum
%
SyOmega = cell(1,f);
for freq=1:f;
   InvPhiPos = (In-(Phi*kron(NegOmega(:,freq),In)))\In;
   InvPhiTNeg = (In-(PhiT*kron(PosOmega(:,freq),In)))\In;
   SyOmega{freq} = (1/(2*pi))*InvPhiPos*SigmaEpsilon*InvPhiTNeg;
   drawnow;
end;
SOmega = zeros(n*(n-1)/2,f);
for freq=1:f;
   num = 0;
   SpecMat = SyOmega{freq};
   for k=1:n-1;
      for l=k+1:n;
         num = num+1;
         if (real(SpecMat(k,k))>1.0e-010)&(real(SpecMat(l,l))>1.0e-010);
            SOmega(num,freq) = (abs(SpecMat(k,l))^2)/(real(SpecMat(k,k))*real(SpecMat(l,l)));
         end;
         drawnow;
      end;
   end;
end;
%
% close the wait dialog
%
if n>=7;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;

%
% end of DSGEVARCoherenceParam.m
%
