function DoneCalc = DSGEVARCorrelationEstimation(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,CurrChain,IsPosterior,PME,lambda,DSGEModel,CurrINI,controls)
% DSGEVARCorrelationEstimation: Computes the population correlations based on a DSGE-VAR model with
%                               parameters drawn from the prior or the posterior distribution.
%
% USAGE:
%
%       DoneCalc = DSGEVARCorrelationEstimation(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,CurrChain,IsPosterior,...
%                                               PME,lambda,DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                           VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  SigmaDraws (matrix) of dimension UsedDraws x n(n+1)/2 with the unique draws
%                           of the SigmaEpsilon parameters. Each row holds vech(Sigma)'.
%
%                  PhiMode (matrix) of dimension nx(np+k) with fixed values for the Phi parameters
%                           that can be used as replacements for PhiDraws.
%
%                  SigmaModeEpsilon (matrix) of dimension nxn with fixed values of the SigmaEpsilon
%                           parameters that can be used as replacements for SigmaDraws.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                           are used and 0 if draws from the prior are supplied.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                           are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                           used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                           when generating the posterior draws.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                           model, such as input files, variable and sample data. Actual observations
%                           are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                           main dialog.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 17, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-06-2010: Updated the documentation.
%
% * 26-10-2010: Added info about the lag order and lambda on the wait dialog.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 12-01-2011: Added additional support for unix and mac paths.
%
% * 25-11-2011: The function now supports skewness.
%
% * 02-12-2011: And it now also supports kurtosis.
%
% * 03-01-2012: Updated the documentation.
%
% * 19-01-2012: Added try-catch code around the call to "DoublingAlgorithmLyapunov".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Made sure that AutoCovHorizon is nonnegative.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
UsedDraws = size(PhiDraws,1);
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'state');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\state". The computation of the observed variable correlations have therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/state". The computation of the observed variable correlations have therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% get the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
%
% compute population covariances and autocorrelations
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
%
AutoCovHorizon = max(min([SampleFreq*3 T-20 18]),0);
if k>0;
   MeanY = zeros(n,k);
   MeanX = (1/T)*sum(x,2);
   MeanX2 = (1/T)*sum(x.*x,2);
   MeanX3 = (1/T)*sum(x.*x.*x,2);
   MeanX4 = (1/T)*sum(x.*x.*x.*x,2);
   Xbar = zeros(k,k,AutoCovHorizon+1);
   Xbar(:,:,1) = (1/T)*x*x';
   for i=1:AutoCovHorizon;
      Xbar(:,:,1+i) = (1/(T-i))*x(:,1+i:T)*x(:,1:T-i)';
   end;
end;
%
% backup parameters
%
if k>0;
   Phi0star = PhiMode(:,1:k);
else;
   Phi0star = zeros(n,1);
end;
Phistar = PhiMode(:,k+1:(n*p)+k);
J = eye(n);
if p>1;
   Phistar = [Phistar;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
Dn = DuplicationMatrix(n);
%
% check if we should setup a wait handle
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' - Prior';
end;
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(UsedDraws,'%0.0f') ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = ['Correlation Distribution' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the population based observed variable correlation distribution for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Correlation Distribution' PriorHeader],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
SaveAfterDraws = min(UsedDraws,1000);
NumFiles = ceil(UsedDraws/SaveAfterDraws);
%
% begin the loop
%
abort = '';
LastDraw = 0;
MeanEstimationTime = 0;
ThisSave = 0;
SVE = [];
MatFiles = '';
for ParamIndex=1:UsedDraws;
   LastDraw = LastDraw+1;
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if strcmp(abort,'cancel')==1;
         break;
      else;
         ProgressDLG([ParamIndex/UsedDraws MeanEstimationTime]);
      end;
   end;
   %
   % determine the new parameters
   %
   tic;
   SigmaEpsilon = reshape(Dn*SigmaDraws(ParamIndex,:)',n,n);
   if IsPositiveDefinite(SigmaEpsilon)==1;
      Phi = reshape(PhiDraws(ParamIndex,:)',n,(n*p)+k);
      if k>0;
         Phi0 = Phi(:,1:k);
         Phi = Phi(:,k+1:(n*p)+k);
      else;
         Phi0 = [];
      end;
      if p>1;
         Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
      end;
   else;
      SigmaEpsilon = SigmaModeEpsilon;
      Phi = Phistar;
      Phi0 = Phi0star;
   end;
   %
   % compute the mean
   %
   SVE(LastDraw).MeanY = J'*((eye(n*p)-Phi)\J)*Phi0;
   MeanY = MeanY+((1/UsedDraws)*SVE(LastDraw).MeanY);
   if (k==0)|((k==1)&(max(x)==min(x)));
      SVE(LastDraw).meanerror = [];
   else;
      %
      % compute the largest approximation error
      %
      SVE(LastDraw).meanerror = zeros(n,1);
      mu = SVE(LastDraw).MeanY;
      muY = zeros(n,T);
      myYlag = zeros(n*p,1);
      for i=1:p;
         myYlag(1+(n*(i-1)):n*i,1) = mu*X(:,p-i+1);
      end;
      for t=1:T;
         muY(:,t) = (Phi0*x(:,i))+(Phi(1:n,:)*myYlag);
         if p>1;
            myYlag = [mu*x(:,i);myYlag(1:n*(p-1),1)];
         else;
            myYlag = mu*x(:,i);
         end;
      end;
      MeanError = sort((mu*x)-muY,2);
      for i=1:n;
         if abs(MeanError(i,1))>abs(MeanError(i,T))
            SVE(LastDraw).meanerror(i) = MeanError(i,1);
         else;
            SVE(LastDraw).meanerror(i) = MeanError(i,T);
         end;
      end;
   end;
   %
   % compute the second moments
   %
   try;
      [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*SigmaEpsilon*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   catch;
      vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(SigmaEpsilon);
      SigmaZ = reshape(vSigmaZ,n*p,n*p);
   end;
   Cov = SigmaZ(1:n,1:n);
   StdY = diag(Cov);
   SVE(LastDraw).StdY = StdY;
   SVE(LastDraw).SkewY = zeros(n,1);
   SVE(LastDraw).KurtY = 3*(StdY.*StdY);
   %
   SVE(LastDraw).CorrY = zeros(n,n,AutoCovHorizon+1);
   SVE(LastDraw).CorrY(:,:,1) = Cov;
   SigmaYStack = SigmaZ(:,(n*(p-1))+1:n*p);
   for c=1:AutoCovHorizon;
      if c<=p-1;
         CovMat = SigmaZ(1:n,1+(n*c):n*(c+1));
      else;
         CovMat = Phi(1:n,:)*SigmaYStack;
         if p>1;
            SigmaYStack = [CovMat;SigmaYStack(1:n*(p-1),:)];
         else;
            SigmaYStack = CovMat;
         end;
      end;
      SVE(LastDraw).CorrY(:,:,1+c) = CovMat;
   end;
   MeanEstimationTime = (((ParamIndex-1)/ParamIndex)*MeanEstimationTime)+((1/ParamIndex)*toc);
   %
   % check if we should save to disk
   %
   if (LastDraw==SaveAfterDraws)|(ParamIndex==UsedDraws);
      ThisSave = ThisSave+1;
      LastDraw = 0;
      %
      % set up the file name and then save
      %
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
      end;
      save(file,'SVE');
      MatFiles = strvcat(MatFiles,file);
      %
      % restore the data structure
      %
      SVE = [];
   end;
   drawnow;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% correct the standard deviation, autocorrelations, and skewness for mean effect
%
if k>0;
   txt = ['Please wait while YADA corrects the population based observed variable correlation distribution for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Correlation Distribution' PriorHeader],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   NFiles = size(MatFiles,1);
   for i=1:NFiles;
      file = StringTrim(MatFiles(i,:));
      FData = load(file);
      SVE = FData.SVE;
      for j=1:length(SVE);
         DevMeanY = SVE(j).MeanY-MeanY;
         %
         % compute kurtosis
         %
         SVE(j).KurtY = SVE(j).KurtY+((DevMeanY.*DevMeanY.*DevMeanY.*DevMeanY)*MeanX4)+(6*(SVE(j).StdY.*((DevMeanY.*DevMeanY)*MeanX2)));
         %
         % compute skewness
         %
         SVE(j).SkewY = ((DevMeanY.*DevMeanY.*DevMeanY)*MeanX3)+(3*(SVE(j).StdY.*(DevMeanY*MeanX)));
         %
         % correct standard deviation
         %
         SVE(j).StdY = sqrt(SVE(j).StdY+diag(DevMeanY*Xbar(:,:,1)*DevMeanY'));
         StdY = SVE(j).StdY;
         if min(StdY)<1.0e-10;
            pos = (StdY<1.0e-10).*(1:n)';
            pos = pos(pos>0);
            StdY(pos) = 1;
         end;
         %
         % correct skewness and kurtosis for the standard deviation
         %
         SVE(j).SkewY = SVE(j).SkewY./(StdY.*StdY.*StdY);
         SVE(j).KurtY = SVE(j).KurtY./(StdY.*StdY.*StdY.*StdY);
         %
         % correct autocovariances
         %
         for k=1:AutoCovHorizon+1;
            SVE(j).CorrY(:,:,k) = (SVE(j).CorrY(:,:,k)+(DevMeanY*Xbar(:,:,k)*DevMeanY'))./(StdY*StdY');
         end;
      end;
      save(file,'SVE','-APPEND');
      %
      drawnow;
   end;
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we have computed all output
%
if strcmp(abort,'cancel')==0;
   DoneCalc = 1;
end;

%
% end of DSGEVARCorrelationEstimation.m
%
