function SVEs = DSGEVARCorrelationParam(Phi,SigmaEpsilon,DSGEModel)
% DSGEVARCorrelationParam: Computes the population correlations based on a DSGE-VAR model with
%                          fixed parameter values.
%
% USAGE:
%
%       SVEs = DSGEVARCorrelationParam(Phi,SigmaEpsilon,DSGEModel)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nx(np+k) with parameters on exogenous variables and
%                      on lagged endogenous variables.
%
%                  SigmaEpsilon (matrix) of dimension nxn with residual covariances.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                      input files, variable and sample data.
%
% REQUIRED OUTPUT: SVEs (structure) with population means, standard deviations, autocorrelations, and
%                      skewness.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 16, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-06-2010: Updated the documentation.
%
% * 21-06-2010: Removed the "lambda" input variable as it was not used.
%
% * 05-01-2011: Updated the documentation.
%
% * 25-11-2011: The function now also computes skewness.
%
% * 02-12-2011: And furthermore it handles kurtosis.
%
% * 03-01-2012: Updated the documentation.
%
% * 19-01-2012: Added try-catch code around the call to "DoublingAlgorithmLyapunov".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Made sure that AutoCovHorizon is nonnegative.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SVEs = [];
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
% remove parameters on the exogenous variables
%
if k>0;
   Phi0 = Phi(:,1:k);
else;
   Phi0 = zeros(n,1);
end;
Phi = Phi(:,k+1:(n*p)+k);
PhiOld = Phi;
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% we approximate the mean by J'*inv(I-Phi)*J*Phi0
%
SVEs.Mean = J'*((eye(n*p)-Phi)\J)*Phi0;
%
% get the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
%
if (k==0)|((k==1)&(max(x)==min(x)));
   SVEs.meanerror = [];
else;
   %
   % compute the largest approximation error
   %
   SVEs.meanerror = zeros(n,1);
   mu = SVEs.Mean;
   muY = zeros(n,T);
   myYlag = zeros(n*p,1);
   for i=1:p;
      myYlag(1+(n*(i-1)):n*i,1) = mu*X(:,p-i+1);
   end;
   for t=1:T;
      muY(:,t) = (Phi0*x(:,i))+(PhiOld*myYlag);
      if p>1;
         myYlag = [mu*x(:,i);myYlag(1:n*(p-1),1)];
      else;
         myYlag = mu*x(:,i);
      end;
   end;
   MeanError = sort((mu*x)-muY,2);
   for i=1:n;
      if abs(MeanError(i,1))>abs(MeanError(i,T))
         SVEs.meanerror(i) = MeanError(i,1);
      else;
         SVEs.meanerror(i) = MeanError(i,T);
      end;
   end;
end;
%
% compute population covariances and autocorrelations
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
%
AutoCovHorizon = max(min([SampleFreq*3 T-20 18]),0);
SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
%
% compute the autocovariances for the stacked Z vector
%
try;
   [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*SigmaEpsilon*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
catch;
   vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(SigmaEpsilon);
   SigmaZ = reshape(vSigmaZ,n*p,n*p);
end;
SVEs.Cov = SigmaZ(1:n,1:n);
StdVec = sqrt(diag(SVEs.Cov));
SVEs.Corr = zeros((n*(n+1))/2,(2*AutoCovHorizon)+1);
SVEs.Corr(:,AutoCovHorizon+1) = vech(SVEs.Cov./(StdVec*StdVec'));
SVEs.Std = StdVec;
SVEs.Skew = zeros(n,1);
SVEs.Kurt = 3*ones(n,1);
%
% initialize matrix SigmaYStack
%
SigmaYStack = SigmaZ(:,(n*(p-1))+1:n*p);
for c=1:AutoCovHorizon;
   if c<=p-1;
      CovMat = SigmaZ(1:n,1+(n*c):n*(c+1));
   else;
      CovMat = PhiOld*SigmaYStack;
      if p>1;
         SigmaYStack = [CovMat;SigmaYStack(1:n*(p-1),:)];
      else;
         SigmaYStack = CovMat;
      end;
   end;
   CorrMat = CovMat./(StdVec*StdVec');
   SVEs.Corr(:,AutoCovHorizon+1+c) = vech(CorrMat');
   SVEs.Corr(:,AutoCovHorizon+1-c) = vech(CorrMat);
end;
%
% compute the data measures
%
if k>0;
   SVEs.Data.Mean = (y*x')/(x*x');
   y = y-(SVEs.Data.Mean*x);
else;
   SVEs.Data.Mean = (1/T)*sum(y,2);
   y = y-(SVEs.Data.Mean*ones(1,T));
end;
AutoCov = (1/T)*y*y';
StdVec = sqrt(diag(AutoCov));
SVEs.Data.Cov = AutoCov;
SVEs.Data.Std = StdVec;
SVEs.Data.Skew = ((1/T)*sum(y.*y.*y,2))./(StdVec.*StdVec.*StdVec);
SVEs.Data.Kurt = ((1/T)*sum(y.*y.*y.*y,2))./(StdVec.*StdVec.*StdVec.*StdVec);
SVEs.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
SVEs.Data.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(StdVec*StdVec'));
for c=1:AutoCovHorizon;
   ys = y(:,c+1:T);
   ylag = y(:,1:T-c);
   AutoCov = (1/T)*ys*ylag';
   CorrMat = AutoCov./(StdVec*StdVec');
   SVEs.Data.Corr(:,AutoCovHorizon+1+c) = vech(CorrMat');
   SVEs.Data.Corr(:,AutoCovHorizon+1-c) = vech(CorrMat);
end;

%
% end of DSGEVARCorrelationParam.m
%
