function IRFs = DSGEVARIRFsDistribution(DSGEModel,CurrINI,controls,ShockType,UsedDraws,h,IRType,CurrChain,IsPosterior,Weights,PME,lambda)
% DSGEVARIRFsDistribution: Computes the mean value of impulse responses as well as quantiles by loading data from disk.
%
% USAGE:
%
%       IRFs = DSGEVARIRFsDistribution(DSGEModel,CurrINI,controls,ShockType,UsedDraws,h,IRType,CurrChain,...
%                                      IsPosterior,Weights,PME,lambda)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the
%                            model, such as input files, variable and sample data. Actual observations
%                            are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                            main dialog.
%
%                  ShockType (string vector) taking the values "individual" or "group".
%
%                  UsedDraws (integer) with the number of draws of the DSGE-VAR parameters.
%
%                  h (integer) with the response horizon.
%
%                  IRType (integer) that takes the value 1 if the responses concern the
%                            original data, 2 if annualized, and 3 if levels.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                            are used and 0 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the SMC based weights. Is empty for
%                            all MCMC samplers.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  lambda (scalar)  with the value of the lambda hyperparameter for the DSGE-VAR.
%
% REQUIRED OUTPUT: IRFs (structure) with fields "Y" and "Quantiles". The "Y" field holds a 3D matrix of dimension
%                            n x q x h+1, where n is the number of observed variables, q the number of shocks or
%                            shock groups, and h the impulse response horizon. The values are the p(oste)rior mean
%                            impulse responses. The field "Quantiles" is a vector structure with fields "Y" and
%                            "percent", where "Y" holds a 3D matrix of dimension n x q x h+1 with values from
%                            the percent quantile of the marginal distribution of the impulse responses and
%                            "percent" holds the percentile value in percent.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 15, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-04-2010: Updated the documentation.
%
% * 27-04-2010: Updated the documentation.
%
% * 21-05-2010: Fixed an initialization bug for "IRFs.Quantiles(i).Y" where the second
%               dimension was set to n instead of q.
%
% * 22-10-2010: The wait dialog now informs the user about the number of lags.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 24-08-2012: Added support for highest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 04-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
% * 16-01-2018: Updated the documentation.
%
% * 10-04-2018: Memory issue code for large models.
%
% * 29-08-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
IRFs = [];
if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
if IRType==1;
   DataType = 'Original';
elseif IRType==2;
   DataType = 'Annualized';
elseif IRType==3;
   DataType = 'Levels';
end;
%
% set up the shocks to use
%
p = DSGEModel.BVARLags;
n = DSGEModel.n;
ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
if strcmp(ShockType,'individual')==1;
   IMatrix = [];
   q = n;
   ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
   ShockStr = 'shock';
else;
   IMatrix = SubIdentityMatrix(ShockGroups)';
   q = size(IMatrix,1);
   ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
   ShockStr = 'shock group';
   for j=1:size(ShockNames,1);
      ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
   end;
end;
%
% initialize index for saving
%
SaveAfterDraws = min(UsedDraws,1000);
NumFiles = ceil(UsedDraws/SaveAfterDraws);
NumQuants = length(DSGEModel.Percentiles);
%
if n>20;
   YADApack;
end;
%
% initialize output
%
IRFs.Y = zeros(n,q,h+1);
for i=1:NumQuants;
   IRFs.Quantiles(i).Y = zeros(n,q,h+1);
   IRFs.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% set up a wait dialog
%
if IsPosterior==1;
   txt = ['Please wait while YADA computes the posterior mean impulse responses as well as percentile values of the observed variables for the DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ')'];
else;
   txt = ['Please wait while YADA computes the prior mean impulse responses as well as percentile values of the observed variables for the DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ')'];
end;
WaitHandle = WaitDLG(txt,'information',[DataType ' Impulse Responses' PriorHeader],500,300,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% now we can start loading the data
%
for i=1:n;
   for j=1:q;
      %
      % update the wait dialog
      %
      if IsPosterior==1;
         txt = ['Please wait while YADA computes the posterior mean impulse responses as well as percentile values for the observed variables for the DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ')'];
      else;
         txt = ['Please wait while YADA computes the prior mean impulse responses as well as percentile values for the observed variables for the DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ')'];
      end;
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempVar = zeros(UsedDraws,h+1);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-VAR-IR-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-VAR-IR-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
            end;
            IRData = load(file);
            IR = IRData.IR;
            clear('IRData');
            NumSaves = length(IR);
            %
            % load data from CurrSaves to NumSaves+1-CurrSaves
            %
            if isempty(IMatrix)==1;
               for s=1:NumSaves;
                  TempVar(CurrSaves+s,:) = IR(s).IRF(i,j,:);
               end;
            else;
               for s=1:NumSaves;
                  TempVar(CurrSaves+s,:) = IMatrix(j,:)*reshape(IR(s).IRF(i,:,:),size(IR(s).IRF,2),h+1);
               end;
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % now we sort and compute the mean the data
         %
         for k=1:h+1;
            if max(abs(TempVar(:,k)))>1.0e-10;
               IRFs.Y(i,j,k) = mean(TempVar(:,k).*Weights);
            end;
            TempVar(:,k) = sort(TempVar(:,k));
            %
            % now we put the quantile information in its correct place
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws);
                  end;
                  if abs(TempVar(QuantVal,k))>1.0e-10;
                     IRFs.Quantiles(l).Y(i,j,k) = TempVar(QuantVal,k);
                  end;
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar(:,k),ConfDraws);
                  if abs(Bounds(1))>1.0e-10;
                     IRFs.Quantiles(l).Y(i,j,k) = Bounds(1);
                  end;
                  if abs(Bounds(2))>1.0e-10;
                     IRFs.Quantiles(NumQuants-l+1).Y(i,j,k) = Bounds(2);
                  end;
               end;
            end;
            drawnow;
         end;
      catch
         for k=1:h+1;
            TempVar = zeros(UsedDraws,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-VAR-IR-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-VAR-IR-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
               end;
               IRData = load(file);
               IR = IRData.IR;
               clear('IRData');
               NumSaves = length(IR);
               %
               % load data from CurrSaves to NumSaves+1-CurrSaves
               %
               if isempty(IMatrix)==1;
                  for s=1:NumSaves;
                     TempVar(CurrSaves+s,1) = IR(s).IRF(i,j,k);
                  end;
               else;
                  for s=1:NumSaves;
                     MT = IMatrix(j,:)*reshape(IR(s).IRF(i,:,:),size(IR(s).IRF,2),h+1);
                     TempVar(CurrSaves+s,1) = MT(k);
                  end;
               end;
               CurrSaves = CurrSaves+NumSaves;
               drawnow;
            end;
            %
            % now we sort and compute the mean the data
            %
            if max(abs(TempVar))>1.0e-10;
               IRFs.Y(i,j,k) = mean(TempVar.*Weights);
            end;
            TempVar = sort(TempVar);
            %
            % now we put the quantile information in its correct place
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws);
                  end;
                  if abs(TempVar(QuantVal,1))>1.0e-10;
                     IRFs.Quantiles(l).Y(i,j,k) = TempVar(QuantVal,1);
                  end;
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
                  if abs(Bounds(1))>1.0e-10;
                     IRFs.Quantiles(l).Y(i,j,k) = Bounds(1);
                  end;
                  if abs(Bounds(2))>1.0e-10;
                     IRFs.Quantiles(NumQuants-l+1).Y(i,j,k) = Bounds(2);
                  end;
               end;
            end;
            drawnow;
         end;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEVARIRFsDistribution.m
%
