function IRF = DSGEVARImpulseResponses(Phi,A0,p,h,IRType,levels,annual,annualscale)
% DSGEVARImpulseResponses: Computes impulse responses for the original, the annualized,
%                          and the levels data for the DSGE-VAR model.
%
% USAGE:
%
%       IRF = DSGEVARImpulseResponses(Phi,A0,p,h,IRType,levels,annual,annualscale)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nxnp with the coefficient on lagged endogenous
%                      variables in the VAR.
%
%                  A0 (matrix) of dimension nxn with the contemporaneous responses in the
%                      endogenous variables from unit shocks.
%
%                  p (integer) with the lag order of the VAR.
%
%                  h (integer) with the response horizon.
%
%                  IRType (integer) that takes the value 1 if the responses concern the
%                      original data, 2 if annualized, and 3 if levels.
%
%                  levels (vector) of dimension n which indicates if a variable appears
%                      in levels (1) or first differences (0).
%
%                  annual (nx1) vector with entries 1 (annual), 4 (quarterly), or 12 (monthly).
%
%                  annualscale (nx1) vector with coefficients that the annualized rows will be
%                      multiplied with.
%
% REQUIRED OUTPUT: IRF (matrix) of dimension nxnx(h+1) with variables in rows, shocks in columns,
%                      and the response horizon in the 3rd dimension.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 23, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 24-02-2010: Updated the documentation.
%
% * 20-05-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 14-01-2015: The function now swaps the horizon and shock dimensions using "permute" when
%               annualized responses are computed.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

n = size(Phi,1);
IRF = zeros(n,n,h+1);
%
% set small responses to zero
%
Apos = (A0>1.0e-12);
Aneg = (A0<-1.0e-12);
A0 = (A0 .* Apos)+(A0 .* Aneg);
IRF(:,:,1) = A0;
%
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% check which type of impulse responses we should compute
%
if IRType<=2;
   Phih = Phi;
   for i=1:h;
      IR = J'*Phih*J*A0;
      IRF(:,:,1+i) = (IR .* (IR>1.0e-12))+(IR .* (IR<-1.0e-12));
      Phih = Phih*Phi;
   end;
   %
   % check if we should annualize
   %
   if IRType==2;
      %
      % swap horizon and shock dimensions
      %
      IRFY = permute(IRF,[1 3 2]);
%      IRFY = zeros(n,h+1,n);
%      for i=1:h+1;
%         for j=1:n;
%            IRFY(:,i,j) = IRF(:,j,i);
%         end;
%      end;
      %
      % annualize the responses
      %
      for j=1:n;
         IRFY(:,:,j) = Annualize([zeros(n,max(annual)-1) IRFY(:,:,j)],annual,annualscale);
      end;
      %
      % restore the shock and horizon dimension
      %
      IRF = permute(IRFY,[1 3 2]);
%      for i=1:h+1;
%         for j=1:n;
%            IRF(:,j,i) = IRFY(:,i,j);
%         end;
%      end;
   end;
else;
   AccMat = diag(1-levels);
   Phih = Phi;
   for i=1:h;
      IR = (J'*Phih*J*A0)+(AccMat*IRF(:,:,i));
      IRF(:,:,1+i) = (IR .* (IR>1.0e-12))+(IR .* (IR<-1.0e-12));
      Phih = Phih*Phi;
   end;
end;

%
% end of DSGEVARImpulseResponses.m
%
