function [JointInverseHessian,InvHessphi,Status] = DSGEVARJointPosteriorInvHessian(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,StepLength,OrderQZ)
% DSGEVARJointPosteriorInvHessian: Computes the inverse Hessian of the joint log posterior of the
%                                  DSGE-VAR when the transformed parameters are used.
%
% USAGE:
%
%       [JointInverseHessian,InvHessphi,Status] = DSGEVARJointPosteriorInvHessian(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions, ...
%                                           thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy, ...
%                                           GammaHatyY,GammaHatYY,DetProductMoments,HSample,StepLength,OrderQZ)
%
% REQUIRED INPUT:  phi (vector) with the transformed parameters of the DSGE model.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                             are 0 1 except for the uniform and the beta distribution, where they represent
%                             the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  T (interger) with the sample size of the endogenous variables
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  npk (integer) with the number of regressors in the VAR, equal to the number of endogenous
%                             variables times the number of lags plus the number of exogenous variables.
%
%                  GammaHatyy (matrix) of dimension nxn with the sample product moments of the endogenous
%                             variables.
%
%                  GammaHatyY (matrix) of dimension nxnpk with the sample product moments of the endogenous
%                             variables and the regressors.
%
%                  GammaHatYY (matrix) of dimension npkxnpk with the sample product moments of the
%                             regressors.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                             are located.
%
%                  StepLength (scalar) with the step length for the finite difference Hessian matrix.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
% REQUIRED OUTPUT: JointInverseHessian (matrix) of dimension (n(npk + (n+1)/2) + m)x(n(npk + (n+1)/2) + m) with
%                             the inverse Hessian of the joint log posterior evaluated at the mode. The parameters
%                             are ordered as vec(Phi), vech(SigmaEpsilon), and phi.
%
%                  InvHessphi (matrix of dimension mxm with the inverse Hessian of phi at the joint posterior mode.
%
%                  Status (boolean) which takes the value 1 if the calculations were successfully performed with
%                             a positive definite inverse Hessian, and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 2, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-02-2010: Finished the function.
%
% * 05-02-2010: Added the output variable "InvHessphi".
%
% * 08-02-2010: Deleted an unnecessary command.
%
% * 17-12-2010: Added the input variable "OrderQZ".
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 26-04-2012: Fixed the sum(isnan(theta)) test.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
JointInverseHessian = [];
InvHessphi = [];
Status = 0;
%
% transform to theta
%
theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
if (sum(isinf(theta))>=1)|(sum(isnan(theta))>=1);
   return;
end;
%
% update ModelParameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameters that need to be updated
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
end;
%
% See if we can solve the model
%
[A,H,R,F,B0] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,OrderQZ);
%
% determine the DSGE-VAR parameters
%
[PhiHat,SigmaHatEpsilon,Gammayy,GammayY,GammaYY] = DSGEVARParameters(A,H,R,F,B0,lambda,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
SigmaBarEpsilon = (((1+lambda)*T)/(((1+lambda)*T)+n+1))*SigmaHatEpsilon;
%
% compute the partions of the Hessian term-by-term. We start with common terms
%
% NOTE: All terms in the YADA manual are divided by -T*lambda to improve numerical
%       precision. This means that the inverse Hessian needs to be divided by T*lambda
%       to compensate.
%
JGamma = GammaYY+((1/lambda)*GammaHatYY);
InvSigmaBarEpsilon = SigmaBarEpsilon\eye(n);
KronGammaSigma = kron(JGamma,InvSigmaBarEpsilon);Dn = DuplicationMatrix(n);
Dnplus = (Dn'*Dn)\Dn';
Dnpk = DuplicationMatrix(npk);
%
% compute the partial of Gammayy wrt phi
%
[GammayyPartialMatrix,PPStatus] = YADANumericalPartialsMatrix('DSGEVARPhiHatSigmaHat',phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,p,DetProductMoments,HSample,1,OrderQZ);
if PPStatus==0;
   return;
end;
[GammayYPartialMatrix,PPStatus] = YADANumericalPartialsMatrix('DSGEVARPhiHatSigmaHat',phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,p,DetProductMoments,HSample,2,OrderQZ);
if PPStatus==0;
   return;
end;
[GammaYYPartialMatrix,PPStatus] = YADANumericalPartialsMatrix('DSGEVARPhiHatSigmaHat',phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,p,DetProductMoments,HSample,3,OrderQZ);
if PPStatus==0;
   return;
end;
%
% compute the partial of PhiHat wrt phi
%
PhiHatPartialMatrix = (kron(JGamma\eye(npk),eye(n))*GammayYPartialMatrix)-(kron(inv(JGamma),PhiHat)*Dnpk*GammaYYPartialMatrix);
SigmaHatPartialMatrix = (lambda/(1+lambda))*(GammayyPartialMatrix + (Dnplus*kron(PhiHat,PhiHat)*Dnpk*GammaYYPartialMatrix) - (2*Dnplus*kron(PhiHat,eye(n))*GammayYPartialMatrix) );
%
% compute the terms that rely on the partial derivative matrices
%
HessPhiphi = -KronGammaSigma*PhiHatPartialMatrix;
HessSigmaphi = -((1+lambda)/(2*lambda))*Dn'*kron(InvSigmaBarEpsilon,InvSigmaBarEpsilon)*Dn*SigmaHatPartialMatrix;
%
% compute the first part of Hessphiphi
%
Hessphiphi = PhiHatPartialMatrix'*KronGammaSigma*PhiHatPartialMatrix;
%
% compute the second partial derivatives of GammaYY wrt phi
%
GammaYYHessian = YADAHessianMatrix('DSGEVARJointSecondPartials',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,GammaHatyy,GammaHatyY,GammaHatYY,SigmaBarEpsilon,p,DetProductMoments,HSample,1,OrderQZ);
GammaYYHessian = reshape(GammaYYHessian,length(phi),length(phi));
%
SigmaEpsilonHessian = YADAHessianMatrix('DSGEVARJointSecondPartials',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,GammaHatyy,GammaHatyY,GammaHatYY,SigmaBarEpsilon,p,DetProductMoments,HSample,2,OrderQZ);
SigmaEpsilonHessian = reshape(SigmaEpsilonHessian,length(phi),length(phi));
%
LogPriorHessian = YADAHessianMatrix('DSGEVARJointSecondPartials',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,GammaHatyy,GammaHatyY,GammaHatYY,SigmaBarEpsilon,p,DetProductMoments,HSample,3,OrderQZ);
LogPriorHessian = reshape(LogPriorHessian,length(phi),length(phi));
%
TraceSigmaEpsilonHessian = YADAHessianMatrix('DSGEVARJointSecondPartials',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,GammaHatyy,GammaHatyY,GammaHatYY,SigmaBarEpsilon,p,DetProductMoments,HSample,4,OrderQZ);
TraceSigmaEpsilonHessian = reshape(TraceSigmaEpsilonHessian,length(phi),length(phi));
%
% now compute the Hessian wrt phi only
%
Hessphiphi = Hessphiphi+(((1+lambda)/(2*lambda))*TraceSigmaEpsilonHessian)-((n/(2*lambda*T))*GammaYYHessian) ...
             -((((lambda*T)-npk)/(2*lambda*T))*SigmaEpsilonHessian)-((1/(lambda*T))*LogPriorHessian);
%
% test matrix
%
invHPhi = kron(JGamma\eye(npk),SigmaBarEpsilon);
invHSigma = ((2*lambda*T)/(((1+lambda)*T)+n+1))*Dnplus*kron(SigmaBarEpsilon,SigmaBarEpsilon)*Dnplus';
Q = Hessphiphi-(HessSigmaphi'*invHSigma*HessSigmaphi)-(HessPhiphi'*invHPhi*HessPhiphi);
%
if IsPositiveDefinite(Q)==0;
   return;
end;
%
% compute the inverse using results for partioned matrices
%
Hphiphi = Q\eye(length(phi));
%
HSigmaSigma = invHSigma+(invHSigma*HessSigmaphi*Hphiphi*HessSigmaphi'*invHSigma);
HPhiPhi = invHPhi+(invHPhi*HessPhiphi*Hphiphi*HessPhiphi'*invHPhi);
HSigmaphi = -invHSigma*HessSigmaphi*Hphiphi;
HPhiphi = -invHPhi*HessPhiphi*Hphiphi;
HPhiSigma = invHPhi*HessPhiphi*Hphiphi*HessSigmaphi'*invHSigma;
%
% we can now put everything together
%
JointInverseHessian = (1/(lambda*T))*[HPhiPhi HPhiSigma HPhiphi;HPhiSigma' HSigmaSigma HSigmaphi;HPhiphi' HSigmaphi' Hphiphi];
InvHessphi = (1/(lambda*T))*Hphiphi;
Status = 1;

%
% end of DSGEVARJointPosteriorInvHessian.m
%
