function logCPC = DSGEVARLogConcLikelihoodConstant(lambda,T,n,npk)
% DSGEVARLogConcLikelihoodConstant: Computes the constant of the log of the concentrated
%                                   posterior distribution of the DSGE model parameters, where
%                                   the joint log posterior has been concentrated by setting
%                                   the VAR parameters to their mode function values.
%
% USAGE:
%
%       logCPC = DSGEVARLogConcLikelihoodConstant(lambda,T,n,npk)
%
% REQUIRED INPUT:  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  T (interger) with the sample size of the endogenous variables
%
%                  n (integer) with the number of endogenous variables.
%
%                  npk (integer) equal to the number of lagged endogenous and exogenous variables per
%                      equation of the VAR.
%
% REQUIRED OUTPUT: logCPC (scalar) with the value of the constant term for the log of the concentrated
%                      posterior density.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 28, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-01-2010: Corrected the scalar that is multiplied to log(lambda*T).
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

logCPC = - ((n*(((1+lambda)*T)+n+1))/2) + (((n*(((1+lambda)*T)+n+1))/2)*log(((1+lambda)*T)+n+1)) ...
         - (((n*(T+n+1))/2)*log(lambda*T)) - (((n*(T+npk))/2)*log(pi)) - (((n*(1+lambda)*T)/2)*log(2)) ...
         - (((n*(n-1))/4)*log(pi)) - LogGammaProduct(n,(lambda*T)-npk);

%
% end of DSGEVARLogConcLikelihoodConstant.m
%
