function OVDPs = DSGEVAROVPsDistribution(DSGEModel,CurrINI,controls,ShockType,UsedDraws,DecType,CurrChain,IsPosterior,Weights,PME,lambda,T)
% DSGEVAROVPsDistribution: Computes the mean value of the observed variable decomposition paths as well as
%                          quantiles by loading data from disk.
%
% USAGE:
%
%       OVDPs = DSGEVAROVPsDistribution(DSGEModel,CurrINI,controls,ShockType,UsedDraws,DecType,CurrChain,...
%                                       IsPosterior,Weights,PME,lambda,T)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the
%                            model, such as input files, variable and sample data. Actual observations
%                            are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                            main dialog.
%
%                  ShockType (string vector) taking the values "individual" or "group".
%
%                  UsedDraws (integer) with the number of draws of the DSGE-VAR parameters.
%
%                  DecType (integer) that takes the value 1 when using the original data and
%                            2 when using the levels.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                            are used and 0 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the SMC based weights. Is empty for
%                            all MCMC samplers.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  T (integer) with the number of time periods for the decomposition.
%
% REQUIRED OUTPUT: OVDPs (structure) with fields "yMean", "yInitial", "yShocks" and "Quantiles".
%                            The last field is a vector with sub-fields "percent", and "DecompY".
%                            The "DecompY" field is an nxTxq matrix where for each i=1,...,q it
%                            holds the term "yMean+yInitial+yShocks(:,:,i)".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 27, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-05-2010: Updated the documentation.
%
% * 22-10-2010: The wait dialog now display the lambda value and log order of the DSGE-VAR.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 27-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 04-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
OVDPs = [];
if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
WeightsT = Weights*ones(1,T);
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
if DecType==1;
   DataType = 'Original';
elseif DecType==2;
   DataType = 'Annualized';
end;
%
% set up the shocks to use
%
p = DSGEModel.BVARLags;
n = DSGEModel.n;
ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
if strcmp(ShockType,'individual')==1;
   IMatrix = [];
   q = n;
   ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
   ShockStr = 'shock';
else;
   IMatrix = SubIdentityMatrix(ShockGroups);
   q = size(IMatrix,2);
   ShockNames = DSGEModel.ShockGroupNames(unique(ShockGroups),:);
   ShockStr = 'shock group';
   for j=1:size(ShockNames,1);
      ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
   end;
end;
%
% initialize index for saving
%
SaveAfterDraws = min(UsedDraws,1000);
NumFiles = ceil(UsedDraws/SaveAfterDraws);
NumQuants = length(DSGEModel.Percentiles);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check to make sure that T matches the number of data points in the files
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\ovp\DSGE-VAR-OVP-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-1-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\ovp\DSGE-VAR-OVP-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-1-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
end;
OVPData = load(file);
if T~=size(OVPData.OVP(1).yMean,2);
   txt = ['The number of observations for the decomposition (' int2str(size(OVPData.OVP(1).yMean,2)) ') doesn''t match the number of observations that have currently set for the DSGE-VAR model (' int2str(T) '). You need to rerun the computation of the historical observed variable decomposition paths.'];
   About(txt,'information',[DataType ' Decomposition Paths' PriorHeader],500,500,CurrINI);
   return;
end;
%
% initialize output
%
OVDPs.yMean = zeros(n,T);
OVDPs.yInitial = zeros(n,T);
OVDPs.yShocks = zeros(n,T,q);
for i=1:NumQuants;
   OVDPs.Quantiles(i).DecompY = zeros(n,T,q);
   OVDPs.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
% set up a wait dialog
%
if IsPosterior==1;
   txt = ['Please wait while YADA computes the posterior mean as well as percentile values for the historical observed variable decomposition paths for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ')'];
else;
   txt = ['Please wait while YADA computes the prior mean as well as percentile values for the historical observed variable decomposition paths for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ')'];
end;
WaitHandle = WaitDLG(txt,'information',[DataType ' Decomposition Paths' PriorHeader],500,300,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% now we can start loading the data
%
for i=1:n;
   for j=1:q;
      if IsPosterior==1;
         txt = ['Please wait while YADA computes the posterior mean as well as percentile values for the historical observed variable decomposition paths for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ')'];
      else;
         txt = ['Please wait while YADA computes the prior mean as well as percentile values for the historical observed variable decomposition paths for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ')'];
      end;
      set(WaitControls.text,'String',txt);
      drawnow;
      try;
         if j==1;
            %
            % make sure we only do this once per variable
            %
            TmpyMean = zeros(UsedDraws,T);
            TmpyInitial = zeros(UsedDraws,T);
         end;
         TmpyShocks = zeros(UsedDraws,T);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\ovp\DSGE-VAR-OVP-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\ovp\DSGE-VAR-OVP-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
            end;
            OVPData = load(file);
            OVP = OVPData.OVP;
            NumSaves = length(OVP);
            %
            % load the draws
            %
            if j==1;
               for s=1:NumSaves;
                  TmpyMean(CurrSaves+s,:) = OVP(s).yMean(i,:);
                  TmpyInitial(CurrSaves+s,:) = OVP(s).yInitial(i,:);
               end;
            end;
            %
            % and now the shocks
            %
            if strcmp(ShockType,'individual')==1;
               %
               % individual shocks
               %
               for s=1:NumSaves;
                  TmpyShocks(CurrSaves+s,:) = OVP(s).yShocks(i,:,j);
               end;
            else;
               %
               % shock groups
               %
               for s=1:NumSaves;
                  TmpyShocks(CurrSaves+s,:) = shiftdim(OVP(s).yShocks(i,:,:))*IMatrix(:,j);
               end;
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % compute the mean
         %
         if j==1;
            OVDPs.yMean(i,:) = (1/UsedDraws)*sum(TmpyMean.*WeightsT,1);
            OVDPs.yInitial(i,:) = (1/UsedDraws)*sum(TmpyInitial.*WeightsT,1);
         end;
         OVDPs.yShocks(i,:,j) = (1/UsedDraws)*sum(TmpyShocks.*WeightsT,1);
         %
         % now deal with the quantiles
         %
         TmpDecompY = sort(TmpyMean+TmpyInitial+TmpyShocks);
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws);
               end;
               OVDPs.Quantiles(l).DecompY(i,:,j) = TmpDecompY(QuantVal,:);
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws/100);
               for t=1:T;
                  Bounds = HighestProbabilityDensityBand(TmpDecompY(:,t),ConfDraws);
                  OVDPs.Quantiles(l).DecompY(i,t,j) = Bounds(1);
                  OVDPs.Quantiles(NumQuants-l+1).DecompY(i,t,j) = Bounds(2);
               end;
            end;
         end;
      catch;
         for t=1:T;
            if j==1;
               %
               % make sure we only do this once per variable
               %
               TmpyMean = zeros(UsedDraws,1);
               TmpyInitial = zeros(UsedDraws,1);
            end;
            TmpyShocks = zeros(UsedDraws,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\ovp\DSGE-VAR-OVP-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\ovp\DSGE-VAR-OVP-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
               end;
               OVPData = load(file);
               OVP = OVPData.OVP;
               NumSaves = length(OVP);
               %
               % load the draws
               %
               if j==1;
                  for s=1:NumSaves;
                     TmpyMean(CurrSaves+s,1) = OVP(s).yMean(i,t);
                     TmpyInitial(CurrSaves+s,1) = OVP(s).yInitial(i,t);
                  end;
               end;
               %
               % and now the shocks
               %
               if strcmp(ShockType,'individual')==1;
                  %
                  % individual shocks
                  %
                  for s=1:NumSaves;
                     TmpyShocks(CurrSaves+s,1) = OVP(s).yShocks(i,t,j);
                  end;
               else;
                  %
                  % shock groups
                  %
                  for s=1:NumSaves;
                     MT = shiftdim(OVP(s).yShocks(i,:,:))*IMatrix(:,j);
                     TmpyShocks(CurrSaves+s,1) = MT(t);
                  end;
               end;
               CurrSaves = CurrSaves+NumSaves;
               drawnow;
            end;
            %
            % compute the mean
            %
            if j==1;
               OVDPs.yMean(i,t) = (1/UsedDraws)*sum(TmpyMean.*Weights,1);
               OVDPs.yInitial(i,t) = (1/UsedDraws)*sum(TmpyInitial.*Weights,1);
            end;
            OVDPs.yShocks(i,t,j) = (1/UsedDraws)*sum(TmpyShocks.*Weights,1);
            %
            % now deal with the quantiles
            %
            TmpDecompY = sort(TmpyMean+TmpyInitial+TmpyShocks);
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws);
                  end;
                  OVDPs.Quantiles(l).DecompY(i,t,j) = TmpDecompY(QuantVal,1);
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws/100);
                  Bounds = HighestProbabilityDensityBand(TmpDecompY,ConfDraws);
                  OVDPs.Quantiles(l).DecompY(i,t,j) = Bounds(1);
                  OVDPs.Quantiles(NumQuants-l+1).DecompY(i,t,j) = Bounds(2);
               end;
            end;
         end;
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEVAROVPsDistribution.m
%
