function DSGEVARObsVarDecPathsDLG(selector,y,yMean,yInitial,yShocks,Quantiles,DateVector,ShockNames,EstStr,DecType,lambda,SampleStrMat,CurrINI,DSGEModel)
% DSGEVARObsVarDecPathsDLG: Displays historical observed variable decomposition paths based on a DSGE-VAR model.
%
% USAGE:
%
%       DSGEVARObsVarDecPathsDLG(selector,y,yMean,yInitial,yShocks,Quantiles,DateVector,ShockNames,EstStr,...
%                                DecType,lambda,SampleStrMat,CurrINI,DSGEModel)
%
% REQUIRED INPUT: selector (string) that takes the values 'init', 'samplebegin', 'sampleend',
%                          'showgraph', and 'done'.
%
% OPTIONAL INPUT: y (matrix) with the data on the observed variables.
%
%                 yMean (matrix) of dimension nxT with the term due to the exogenous variables in the
%                          decomposition of the endogenous variables.
%
%                 yInitial (matrix) of dimension nxT with the term due to the initial values in the
%                          decomposition of the endogenous variables.
%
%                 yShocks (matrix) of dimension nxTxq with the terms due to each structural shock or group in
%                          the decomposition of the endogenous variables. The 3rd dimension is the shock (group)
%                          dimension.
%
%                 Quantiles (structure) with quantile information about the decomposition paths.
%
%                 DateVector (vector) with numerical dates for the sample.
%
%                 ShockNames (string matrix) with the names of the structural shocks or their groups.
%
%                 EstStr (string) that takes the values 'Initial Values', 'Marginal Posterior Mode',
%                          'Joint Posterior Mode', 'Prior Distribution', or 'Posterior Distribution'.
%
%                 DecType (integer) that takes the value 1 if the decomposition concerns the original
%                          data and 2 if the annualized data.
%
%                 lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                 SampleStrMat (string matrix) with the sample dates.
%
%                 CurrINI (structure) with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information about the
%                          model, such as input files, variable and sample data. Actual observations
%                          are not stored in this file.
%
% NOTE: All optional variables are required when "selector" is equal to 'init'.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 26, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-05-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 23-03-2018: Fixed an issue to make legend compatible with matlab 2017a and later.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   CurrINI.y = y;
   CurrINI.yMean = yMean;
   CurrINI.yInitial = yInitial;
   CurrINI.yShocks = yShocks;
   CurrINI.Quantiles = Quantiles;
   CurrINI.DateVector = DateVector;
   CurrINI.ShockNames = ShockNames;
   CurrINI.EstStr = EstStr;
   CurrINI.DecType = DecType;
   CurrINI.lambda = lambda;
   if size(ShockNames,1)<DSGEModel.n;
      ShockNames = char(zeros(size(ShockNames)));
      for j=1:size(ShockNames,1);
         ShockNames(j,:) = strrep(CurrINI.ShockNames(j,:),'_',' ');
      end;
      CurrINI.ShockNames = ShockNames;
   end;
else;
   ObsVarDecPathsGUI = findobj('Type','figure','Tag','DSGEVARObsVarDecPathsDLG');
   controls = get(ObsVarDecPathsGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
switch selector;
   case 'init';
      ObsVarDecPathsGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','DSGEVARObsVarDecPathsDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-600)/2 (CurrINI.scrsz(4)-135)/2 600 135], ...
         'Tag','DSGEVARObsVarDecPathsDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Historical Observed Variable Decomposition Paths - ' EstStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','DSGEVARObsVarDecPathsDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(ObsVarDecPathsGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 598 125],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Select the sample
      %
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 80 125 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','DSGEVARObsVarDecPathsDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[340 80 125 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','DSGEVARObsVarDecPathsDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SampleStrMat,1));
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 50 255 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVarPopup',...
         'Value',1);
      %
      %
      % select shock or shock group
      %
      controls.shockstext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 15 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select shock:', ...
         'HorizontalAlignment','left', ...
         'Tag','ShockText');
      %
      controls.shocks = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 20 255 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ShockNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ShockPopup',...
         'Value',1);
      % 
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARObsVarDecPathsDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARObsVarDecPathsDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(ObsVarDecPathsGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
   case 'showgraph'
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.VariableNames(VariableValue,:));
      ShockValue = get(controls.shocks,'Value');
      CurrShock = StringTrim(CurrINI.ShockNames(ShockValue,:));
      if ShockValue<size(CurrINI.ShockNames,1);
         set(controls.shocks,'Value',ShockValue+1);
      else;
         set(controls.shocks,'Value',1);
         if VariableValue<size(DSGEModel.VariableNames,1);
            set(controls.variables,'Value',VariableValue+1);
         else;
            set(controls.variables,'Value',1);
         end;
      end;
      %
      % get the sample values
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      %
      % compute the decomposition term
      %
      YDec = CurrINI.yInitial(VariableValue,:)+CurrINI.yMean(VariableValue,:)+CurrINI.yShocks(VariableValue,:,ShockValue);
      Y = CurrINI.y(VariableValue,:);
      %
      % try to compute the correlation
      %
      YDiff = Y-CurrINI.yInitial(VariableValue,:)+CurrINI.yMean(VariableValue,:);
      YDiff = YDiff-((1/length(YDiff))*sum(YDiff));
      %
      % make sure that only the shock term is included in the correlation
      %
      YShock = CurrINI.yShocks(VariableValue,:,ShockValue);
      YShockDiff = YShock-(sum(YShock)/length(YShock));
      StdY = sqrt(sum(YDiff.*YDiff)/length(Y));
      StdYShock = sqrt(sum(YShockDiff.*YShockDiff)/length(YShock));
      if (StdYShock>1.0e-06)&(StdY>1.0e-06);
         CorrYYShock = (sum(YDiff.*YShockDiff)/length(Y))/(StdY*StdYShock);
         CorrStr = [' (corr=' num2str(CorrYYShock,'%0.2f') ')'];
      else;
         CorrStr = ' (corr=0)';
      end;
      %
      % open a figure
      %
      FigHandle = figure('Visible','off', ...
                     'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                     'Tag',['OVP-' CurrVariable '-' strrep(CurrShock,' ','-')], ...
                     'Name',['Observed Variable Decomposition Path - ' CurrVariable ' and ' CurrShock]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % check if we have any quantile data
      %
      Handles = [];
      LegendStr = '';
      if isempty(CurrINI.Quantiles)==0;
         NumBounds = floor(length(CurrINI.Quantiles)/2);
         for i=1:NumBounds;
            Lower = CurrINI.Quantiles(i).DecompY(VariableValue,samplebeg:sampleend,ShockValue);
            Upper = CurrINI.Quantiles(length(CurrINI.Quantiles)-i+1).DecompY(VariableValue,samplebeg:sampleend,ShockValue);
            PatchColor = ((1+(2*(i-1)))/length(CurrINI.Quantiles))*DSGEModel.ConfidenceBandBaseColor;
            ConfBound = CurrINI.Quantiles(length(CurrINI.Quantiles)-i+1).percent-CurrINI.Quantiles(i).percent;
            ConfPatch = patch([CurrINI.DateVector(samplebeg:sampleend) fliplr(CurrINI.DateVector(samplebeg:sampleend))],[Upper fliplr(Lower)],PatchColor);
            set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',PatchColor);
            hold('on');
            Handles = [Handles ConfPatch];
            LegendStr = strvcat(LegendStr,[num2str(ConfBound,'%0.0f') ' confidence']);
         end;
         set(gca,'Layer','Top');
      end;
      %
      % plot the observed variable
      %
      ObsHandle = plot(CurrINI.DateVector(samplebeg:sampleend),Y(samplebeg:sampleend),'.-','Color',[0.75 0 0],'LineWidth',2);
      hold('on');
      %
      % plot the decomposition
      %
      DecHandle = plot(CurrINI.DateVector(samplebeg:sampleend),YDec(samplebeg:sampleend),'--','Color',[0 0 0.76],'LineWidth',2);
      hold('on');
      %
      % setup the figure title
      %
      if CurrINI.DecType==1;
         if (strcmp(lower(CurrINI.EstStr),'prior distribution')==0)&(strcmp(lower(CurrINI.EstStr),'posterior distribution')==0);
            SetAxesFonts(CurrINI,'Title',[CurrVariable ' (original data) and ' CurrShock ' term at ' lower(CurrINI.EstStr) ' for lambda=' num2str(CurrINI.lambda,'%0.4f')]);
         else;
            SetAxesFonts(CurrINI,'Title',[CurrVariable ' (original data) and ' CurrShock ' term for lambda=' num2str(CurrINI.lambda,'%0.4f')]);
         end;
      else;
         if (strcmp(lower(CurrINI.EstStr),'prior distribution')==0)&(strcmp(lower(CurrINI.EstStr),'posterior distribution')==0);
            SetAxesFonts(CurrINI,'Title',[CurrVariable ' (annual data) and ' CurrShock ' term at ' lower(CurrINI.EstStr) ' for lambda=' num2str(CurrINI.lambda,'%0.4f')]);
         else;
            SetAxesFonts(CurrINI,'Title',[CurrVariable ' (annual data) and ' CurrShock ' term for lambda=' num2str(CurrINI.lambda,'%0.4f')]);
         end;
      end;
      %
      % see if we should add a zero line
      %
      ylim = get(gca,'YLim');
      AddOn = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-AddOn ylim(2)+AddOn];
      set(gca,'YLim',ylim,'XLim',[CurrINI.DateVector(samplebeg) CurrINI.DateVector(sampleend)]);
      if (ylim(1)<0)&(ylim(2)>0);
         hold('on');
         ZeroHandle = plot([CurrINI.DateVector(samplebeg) CurrINI.DateVector(sampleend)],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         set(gca,'Children',[ObsHandle DecHandle ZeroHandle fliplr(Handles)]);
      end;
      if MatlabNumber>=7;
         legHandle = legend([ObsHandle DecHandle Handles],strvcat(CurrVariable,[CurrShock ' term' CorrStr],LegendStr),'Location','NorthWest');
      else;
         legHandle = legend([ObsHandle DecHandle Handles],strvcat(CurrVariable,[CurrShock ' term' CorrStr],LegendStr),2);
      end;
      SetAxesFonts(CurrINI,legHandle);
      hold('off');
      %
      SetXTickIntegers(gca);
      %
      % display the graph
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(controls.done,'UserData','done');
      delete(ObsVarDecPathsGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(ObsVarDecPathsGUI,'Visible','on');
   drawnow;
   SetYADAIcon(ObsVarDecPathsGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of DSGEVARObsVarDecPathsDLG.m
%
