function [yMean,yInitial,yShocks] = DSGEVARObsVarDecomp(Phi,A0,p,k,y,Y,IsConstant)
% DSGEVARObsVarDecomp: Computes the observed variable decomposition for a VAR model.
%
% USAGE:
%
%       [yMean,yInitial,yShocks] = DSGEVARObsVarDecomp(Phi,A0,p,k,y,Y,ConstantMean)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nx(np+k) with coefficients on exogenous and lagged
%                      endogenous variables.
%
%                  A0 (matrix) of dimension nxn with coefficients on the structural shocks of the VAR.
%
%                  p (integer) with the number of lags of the VAR.
%
%                  k (integer) with the number of exogenous variables
%
%                  y (matrix) of dimension nxT with the observations on the endogenous variables.
%
%                  Y (matrix) of dimension (np+k)xT with the observations on the exogenous and lagged
%                      endogenous variables.
%
%                  IsConstant (boolean) that takes the value 1 if the exogenous variable is a constant
%                      and 0 otherwise.
%
% REQUIRED OUTPUT: yMean (matrix) of dimension nxT with the term due to the exogenous variables in the
%                      decomposition of the endogenous variables.
%
%                  yInitial (matrix) of dimension nxT with the term due to the initial values in the
%                      decomposition of the endogenous variables.
%
%                  yShocks (matrix) of dimension nxTxn with the terms due to each structural shock in
%                      the decomposition of the endogenous variables. The 3rd dimension is the shock
%                      dimension.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 25, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-05-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 25-05-2011: At the 1-year anniversary a number of empty lines were deleted.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[n,T] = size(y);
yMean = zeros(n,T);
yInitial = zeros(n,T);
yShocks = zeros(n,T,n);
%
% set small responses to zero
%
Apos = (A0>1.0e-12);
Aneg = (A0<-1.0e-12);
A0 = (A0 .* Apos)+(A0 .* Aneg);
%
% estimate the structural shocks
%
u = (A0\eye(n))*(y-(Phi*Y));
%
% determine some relevant matrices
%
if k>=1;
   Phi0 = Phi(:,1:k);
   Phi = Phi(:,k+1:(n*p)+k);
   X = Y(1:k,:);
   Y0 = Y(k+1:(n*p)+k,1);
else;
   Phi0 = [];
   X = [];
   Y0 = Y(:,1);
end;
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% we can now compute the decomposition
%
if IsConstant==1;
   %
   % compute the mean
   %
   muY = J'*((eye(n*p)-Phi)\J)*Phi0;
   Y0 = Y0-(kron(ones(p,1),eye(n))*muY);
   Phii = eye(n*p);
   for ti=0:T-1;
      %
      % select a shock
      %
      for j=1:n;
         for t=ti+1:T;
            yShocks(:,t,j) = yShocks(:,t,j)+(J'*Phii*J*A0(:,j)*u(j,t-ti));
         end;
      end;
      %
      % update Fi. This is the speediest algorithm since F may have high dimension making
      % F^i a costly computation
      %
      Phii = Phii*Phi;
      yInitial(:,ti+1) = J'*Phii*Y0;
      drawnow;
   end;
   yMean = muY*ones(1,T);
else;
   if k>0;
      Phii = eye(n*p);
      for ti=0:T-1;
         %
         % select a shock
         %
         for t=ti+1:T;
            for j=1:n;
               yShocks(:,t,j) = yShocks(:,t,j)+(J'*Phii*J*A0(:,j)*u(j,t-ti));
            end;
            yMean(:,t) = yMean(:,t)+(J'*Phii*J*Phi0*X(:,t-ti));
         end;
         %
         % update Fi. This is the speediest algorithm since F may have high dimension making
         % F^i a costly computation
         %
         Phii = Phii*Phi;
         yInitial(:,ti+1) = J'*Phii*Y0;
         drawnow;
      end;
   else;
      Phii = eye(n*p);
      for ti=0:T-1;
         %
         % select a shock
         %
         for j=1:n;
            for t=ti+1:T;
               yShocks(:,t,j) = yShocks(:,t,j)+(J'*Phii*J*A0(:,j)*u(j,t-ti));
            end;
         end;
         %
         % update Fi. This is the speediest algorithm since F may have high dimension making
         % F^i a costly computation
         %
         Phii = Phii*Phi;
         yInitial(:,ti+1) = J'*Phii*Y0;
         drawnow;
      end;
   end;
end;

%
% end of DSGEVARObsVarDecomp.m
%
