function DSGEVARPredictiveDistributionDLG(selector,DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,KernelX,KernelY,lambda,EstStr,DataType,ForecastTypeStr,UData)
% DSGEVARPredictiveDistributionDLG: Displays predictive distributions of the observed variables along with the mean
%                                   predictions and when possible the observed values from a DSGE-VAR.
%
% USAGE:
%
%       DSGEVARPredictiveDistributionDLG(selector,DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,...
%                                        KernelX,KernelY,lambda,EstStr,DataType,ForecastTypeStr,UData)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 Y (matrix) of dimension nxT with data on the endogenous variables.
%
%                 DateVector (vector) of dimension T with numeric dates.
%
%                 SampleStrMat (string matrix) with dates for the observed variables prior
%                           to the forecast sample.
%
%                 MeanPrediction (matrix) of dimension nxh with the mean predictions.
%
%                 Quantiles (structure) of dimension NumQuants where each has a field called "Quantile" which
%                           is an nxh matrix with the quantiles of the prediction paths. The second field
%                           is "percent" that gives the percentile value as a string vector.
%
%                 KernelX (matrix) of dimension (n x NumGridPoints x h) with the x-axis values of the
%                           kernel density estimates of the marginal predictive densities.
%
%                 KernelY (matrix) of dimension (n x NumGridPoints x h) with the y-axis values of the
%                           kernel density estimates of the marginal predictive densities.
%
%                 lambda (scalar) with the lambda hyperparameter of the DSGE-VAR.
%
%                 EstStr (string) that is equal to "Initial Values", "Prior Distribution", "Marginal
%                           Posterior Mode", "Joint Posterior Mode" or "Posterior Distribution".
%
%                 DataType (string) that is equal to "Original", "Annualized" or "Transformed".
%
%                 ForecastTypeStr (string) which is equal to "Conditional" or "Unconditional".
%
%                 UData (structure) with data on uncertainty measures for the variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 4, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 07-06-2010: Updated the documentation.
%
% * 08-06-2010: Fixed a bugf where the wrong argument was used for the tag of the graphs of
%               prediction uncertainty decompositions.
%
% * 23-07-2010: The function can now plot a normal approximation of the marginal predictive
%               densities.
%
% * 05-01-2011: Updated the documentation.
%
% * 24-02-2011: Added the input variable "ForecastTypeStr".
%
% * 03-01-2012: Updated the documentation.
%
% * 09-03-2012: Made sure that missing observations don't affect the adjusted of the predictive mean
%               to the observed data.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 21-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 19-10-2016: Added the option of selecting different actual data when plotting.
%
% * 08-11-2016: It is now possible to add a small number to the upper and substract the same
%               number from the lower bound when plotting. This is only useful when using HG2
%               graphics.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 20-03-2018: The code now checks if the number of paths is unity and skips the patch plots
%               for confidence regions for this case.
%
% * 21-03-2018: The confidence region patches now stretch back to the last historical value.
%
% * 23-03-2018: Fixed an issue to make legend compatible with matlab 2017a and later.
%
% * 29-08-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 16-04-2021: The function can now display bounds for the prior and posterior distributions also
%               when the number of paths per parameter value is equal to 1.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.Y = Y;
   DSGEModel.DateVector = DateVector;
   DSGEModel.MeanPrediction = MeanPrediction;
   DSGEModel.Quantiles = Quantiles;
   DSGEModel.lambda = lambda;
   DSGEModel.EstStr = EstStr;
   DSGEModel.DataType = DataType;
   DSGEModel.ForecastTypeStr = ForecastTypeStr;
   DSGEModel.UData = UData;
   [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,DSGEModel.PredDateVector,DSGEModel.PredSampleStrMat] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
   if isempty(setdiff(DSGEModel.PredDateVector,DateVector))==1;
      DSGEModel.ShowAdjustOption = 1;
      AddHeight = 35;
   else;
      DSGEModel.ShowAdjustOption = 0;
      AddHeight = 0;
   end;
   if isempty(KernelX)==0;
      DSGEModel.ShowKernel = 1;
      DSGEModel.KernelX = KernelX;
      DSGEModel.KernelY = KernelY;
      AddKHeight = 25+(10*(1-DSGEModel.ShowAdjustOption));
   else;
      DSGEModel.ShowKernel = 0;
      AddKHeight = 0;
   end;
   if isempty(DSGEModel.Actuals)==0;
      AddActHeight = 35;
      ActualsStr = 'Actuals from original sample';
      for i=1:length(DSGEModel.Actuals);
         ActualsStr = strvcat(ActualsStr,DSGEModel.Actuals(i).title);
      end;
   else;
      AddActHeight = 0;
      ActualsStr = '';
   end;
   if strcmp(lower(ForecastTypeStr),'conditional')==1;
      AddCondHeight = 35;
      AddCondWidth = 40;
   else;
      AddCondHeight = 0;
      AddCondWidth = 0;
   end;
else;
   DSGEVARPredictiveDistributionGUI = findobj('Type','figure','Tag','DSGEVARPredictiveDistributionDLG');
   PredictiveControls = get(DSGEVARPredictiveDistributionGUI,'UserData');
   CurrINI = get(PredictiveControls.varnames,'UserData');
   DSGEModel = get(PredictiveControls.variables,'UserData');
   maingui = findobj('Type','figure','Tag','YADA');
   controls = get(maingui,'UserData');
end;
%
% create the dialog
%
switch selector;
   case 'init';
      DSGEVARPredictiveDistributionGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','DSGEVARPredictiveDistributionDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-600)/2 (CurrINI.scrsz(4)-140-AddCondHeight-AddHeight-AddKHeight-AddActHeight)/2 600 140+AddCondHeight+AddHeight+AddKHeight+AddActHeight], ...
         'Tag','DSGEVARPredictiveDistributionDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['DSGE-VAR(' num2str(lambda,'%0.4f') ') Predictive Distributions - ' EstStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','DSGEVARPredictiveDistributionDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(DSGEVARPredictiveDistributionGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 598 130+AddCondHeight+AddHeight+AddKHeight+AddActHeight],['Multistep ' ForecastTypeStr ' Predictions - ' DataType ' Data'],45,[0.5 1],'on',CurrINI);
      %
      PredictiveControls.sample = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85+AddCondHeight+AddHeight+AddKHeight+AddActHeight 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String',['Prediction Sample: ' PredStartYear ':' PredStartPeriod ' - ' PredEndYear ':' PredEndPeriod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % data sample
      %
      PredictiveControls.startdatatext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50+AddCondHeight+AddHeight+AddKHeight+AddActHeight 180+AddCondWidth 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Start period for variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','StartDataText');
      %
      PredictiveControls.startdata = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210+AddCondWidth 55+AddCondHeight+AddHeight+AddKHeight+AddActHeight 255-AddCondWidth 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Tag','StartDataPopup',...
         'Value',1);
      %
      % Select variable
      %
      PredictiveControls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 15+AddCondHeight+AddHeight+AddKHeight+AddActHeight 180+AddCondWidth 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableNamesText');
      %
      PredictiveControls.varnames = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210+AddCondWidth 20+AddCondHeight+AddHeight+AddKHeight+AddActHeight 255-AddCondWidth 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','VariableNamesPopup',...
         'Value',1);
      %
      if AddActHeight>0;
         %
         % select actuals
         %
         PredictiveControls.actualstext = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 15+AddCondHeight+AddHeight+AddKHeight 180+AddCondWidth 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','text', ...
            'String','Select the actuals dataset', ...
            'HorizontalAlignment','left', ...
            'Tag','ActualsText');
         %
         PredictiveControls.actuals = uicontrol('Units','pixels', ...
            'BackgroundColor',[1 1 1], ...
            'Position',[210+AddCondWidth 20+AddCondHeight+AddHeight+AddKHeight 255-AddCondWidth 20], ...
            'Style','popupmenu', ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'UserData',CurrINI, ...
            'String',ActualsStr, ...
            'HorizontalAlignment','center', ...
            'Tag','ActualsPopup',...
            'Value',1);
      else;
         PredictiveControls.actualstext = [];
         PredictiveControls.actuals = [];
      end;
      %
      % display control for fixing the upper and lower bounds for conditional forecasts
      %
      if AddCondHeight>0;
         %
         PredictiveControls.upperlowerfixtext = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 15+AddHeight+AddKHeight 220 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','text', ...
            'String','Upper/lower bound correction (HG2 issue):', ...
            'HorizontalAlignment','left', ...
            'Tag','UpperLowerFixText');
         %
         FixValue = 1;
         if MatlabNumber>8.3;
            FixValue = 9;
         end;
         %
         PredictiveControls.upperlowerfix = uicontrol('Units','pixels', ...
            'BackgroundColor',[1 1 1], ...
            'Position',[250 20+AddHeight+AddKHeight 215 20], ...
            'Style','popupmenu', ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'UserData',CurrINI, ...
            'String','0|1.0e-01|1.0e-02|1.0e-03|1.0e-04|1.0e-05|1.0e-06|1.0e-07|1.0e-08|1.0e-09|1.0e-10|1.0e-11|1.0e-12|1.0e-13|1.0e-14', ...
            'HorizontalAlignment','center', ...
            'Tag','UpperLowerFixPopup',...
            'Value',FixValue);
      else;
         PredictiveControls.upperlowerfix = [];
      end;
      %
      if DSGEModel.ShowKernel==1;
         PredictiveControls.ShowKernelDensity = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 15+min(AddHeight,25) 441 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Value',0, ...
            'String','Show kernel density estimates of marginal predictive densities', ...
            'HorizontalAlignment','left', ...
            'Tag','ShowKernelDensityCheckbox');
      else;
         PredictiveControls.ShowKernelDensity = [];
      end;
      %
      if DSGEModel.ShowAdjustOption==1;
         PredictiveControls.AdjustMean = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 15 441 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Value',0, ...
            'String','Set mean of predictive dist. equal to realized values', ...
            'HorizontalAlignment','left', ...
            'Tag','AdjustMeanCheckbox');
      else;
         PredictiveControls.AdjustMean = [];
      end;
      %
      % Create graph button
      %
      PredictiveControls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARPredictiveDistributionDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      PredictiveControls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARPredictiveDistributionDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(DSGEVARPredictiveDistributionGUI,'UserData',PredictiveControls,...
         'HandleVisibility','callback');
   case 'showgraph'
      VariableValue = get(PredictiveControls.varnames,'Value');
      VariableStr = get(PredictiveControls.varnames,'String');
      CurrVariable = StringTrim(VariableStr(VariableValue,:));
      if VariableValue<size(VariableStr,1);
         set(PredictiveControls.varnames,'Value',VariableValue+1);
      else;
         set(PredictiveControls.varnames,'Value',1);
      end;
      %
      NumPredPathsStr = get(controls.forecast.predpaths,'String');
      NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
      %
      % create a vector with dates
      %
      DateVector = DSGEModel.DateVector;
      h = length(DSGEModel.PredDateVector);
      y = DSGEModel.Y(VariableValue,:);
      if (isempty(PredictiveControls.AdjustMean)==0)&(get(PredictiveControls.AdjustMean,'Value')==1);
         AddOnVector = y(length(y)-h+1:length(y))-DSGEModel.MeanPrediction(VariableValue,:);
         AddOnVector(isnan(AddOnVector)) = 0;
      else;
         AddOnVector = zeros(1,length(DSGEModel.PredDateVector));
      end;
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                     'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                     'Tag',[CurrVariable '-' DSGEModel.EstStr], ...
                     'Name',['DSGE-VAR(' num2str(DSGEModel.lambda,'%0.4f') ') ' DSGEModel.ForecastTypeStr ' Multistep Predictions - ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      %
      % check which actuals data to use
      %
      if isempty(PredictiveControls.actuals)==1;
         YActuals = DSGEModel.Y;
      else;
         SelData = get(PredictiveControls.actuals,'Value');
         if SelData==1;
            YActuals = DSGEModel.Y;
         else;
            YActuals = DSGEModel.Actuals(SelData-1).data;
         end;
      end;
      DSGEModel.Y = YActuals;
      %
      % first we plot the predictive bounds
      %
      if isempty(PredictiveControls.upperlowerfix)==0;
         SelFixValue = get(PredictiveControls.upperlowerfix,'Value');
         if SelFixValue==1;
            FixValue = 0;
         else;
            FixValue = 10^(1-SelFixValue);
         end;
      else;
         FixValue = 0;
      end;
      NumBounds = floor(length(DSGEModel.Quantiles)/2);
      Handles = [];
      LegendStr = '';
      if (NumPredPaths>1)|(strcmp(lower(DSGEModel.EstStr),'prior distribution')==1)|(strcmp(lower(DSGEModel.EstStr),'posterior distribution')==1);
         for i=1:NumBounds;
            Lower = DSGEModel.Quantiles(i).Quantile(VariableValue,:)+AddOnVector;
            Upper = DSGEModel.Quantiles(length(DSGEModel.Quantiles)-i+1).Quantile(VariableValue,:)+AddOnVector;
            PatchColor = ((1+(2*(i-1)))/length(DSGEModel.Quantiles))*DSGEModel.ConfidenceBandBaseColor;
            ConfBound = DSGEModel.Quantiles(length(DSGEModel.Quantiles)-i+1).percent-DSGEModel.Quantiles(i).percent;
            ConfPatch = patch([DSGEModel.PredDateVector fliplr(DSGEModel.PredDateVector)],[Upper+FixValue fliplr(Lower)-FixValue],PatchColor);
            set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',PatchColor);
            hold('on');
            Handles = [Handles ConfPatch];
            LegendStr = strvcat(LegendStr,[num2str(ConfBound,'%0.0f') ' confidence']);
         end;
         set(gca,'Layer','Top');
      end;
      %
      % plot the observed variable
      %
      YHandle = plot(DateVector,y,'+-','Color',[0.75 0 0],'LineWidth',2);
      hold('on');
      %
      % set up a DateVector which does not go beyond the largest value in PredDateVector
      %
      if isempty(Handles)==0;
         PDMax = max(DSGEModel.PredDateVector);
         XData = get(YHandle,'XData');
         PDMaxPos = max((XData==PDMax).*(1:length(XData)));
         if PDMaxPos>0;
            DV = XData(1:PDMaxPos);
         else;
            DV = XData;
         end;
         [DV,Ind] = setdiff(DV,DSGEModel.PredDateVector);
         DateVec = DV(length(DV));
         Ind = Ind(length(Ind));
         YData = get(YHandle,'YData');
         YData = YData(Ind);
         %
         % we should add the position x,y = DateVec,YData to the patches
         %
         for i=1:length(Handles);
            XD = get(Handles(i),'XData')';
            YD = get(Handles(i),'YData')';
            PatchColor = get(Handles(i),'FaceColor');
            delete(Handles(i));
            Handles(i) = patch([DateVec XD DateVec],[YData YD YData],PatchColor);
            set(Handles(i),'FaceColor',PatchColor,'EdgeColor',PatchColor);
         end;
      end;
      %
      % plot the mean prediction
      %
      if (isempty(PredictiveControls.AdjustMean)==0)&(get(PredictiveControls.AdjustMean,'Value')==1);
         PredHandle = [];
         PredStr = '';
      else;
         PredHandle = plot(DSGEModel.PredDateVector,DSGEModel.MeanPrediction(VariableValue,:),'--','Color',[0 0 0.75],'LineWidth',2);
         PredStr = 'Mean predictions';
      end;
      hold('on');
      Handles = [YHandle PredHandle Handles];
      LegendStr = strvcat(CurrVariable,PredStr,LegendStr);
      %
      ylim = get(gca,'YLim');
      add_on = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-add_on ylim(2)+add_on];
      xlim = [DateVector(get(PredictiveControls.startdata,'Value')) DSGEModel.PredDateVector(h)];
      set(gca,'XLim',xlim,'YLim',ylim);
      %
      % add a vertical line from the forecast point
      %
      fp = plot((((2*DSGEModel.PredDateVector(1))-(DateVector(2)-DateVector(1)))/2)*[1 1],ylim,':','Color',[0 0 0]);
      hold('on');
      %
      if (ylim(1)<0)&(ylim(2)>0);
         %
         % add a zero line
         %
         zl = plot(get(gca,'XLim'),[0 0],'-','Color',[0 0 0]);
      else;
         zl = [];
      end;
      %
      if isempty(PredHandle)==0;
         set(gca,'Children',[PredHandle;YHandle;zl;fp;fliplr(Handles(3:length(Handles)))']);
      else;
         set(gca,'Children',[YHandle;zl;fp;fliplr(Handles(2:length(Handles)))']);
      end;
      %
      SetAxesFonts(CurrINI,'Title',[CurrVariable ' (' lower(DSGEModel.DataType) ' data) and out-of-sample predictions - ' lower(DSGEModel.EstStr)]);
      hold('off');
      %
      % add the legend
      %
      if MatlabNumber>=7;
         legHandle = legend(Handles,LegendStr,'Location','NorthWest');
      else;
         legHandle = legend(Handles,LegendStr,2);
      end;
      SetAxesFonts(CurrINI,legHandle);
      %
      SetXTickIntegers(gca);
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      TopFigHandle = FigHandle;
      %
      % check if we should make another figure
      %
      if isempty(DSGEModel.UData)==0;
         %
         % create the figure
         %
         FigHandle = figure('Visible','off', ...
                        'Position',[((CurrINI.scrsz(3)-750)/2)+22 ((CurrINI.scrsz(4)-500)/2)-22 750 500], ...
                        'Tag',[CurrVariable '-' DSGEModel.EstStr], ...
                        'Name',['DSGE-VAR(' num2str(DSGEModel.lambda,'%0.4f') ') Prediction Uncertainty Decompositions - ' CurrVariable]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         %
         Ydata = [DSGEModel.UData.ShockVar(VariableValue,:);DSGEModel.UData.ParameterVar(VariableValue,:)];
         ColorMat = [0.5 0.75 1;0.75 0 0];
         BarVector = (1:size(Ydata,2));
         if MatlabNumber>8.3;
            BarHandles = bar(BarVector',Ydata','stacked');
         else;
            if MatlabNumber>=7.0;
               %
               % make sure the bars are written as patches
               %
               BarHandles = bar('v6',BarVector',Ydata','stacked');
            else;
               BarHandles = bar(BarVector',Ydata','stacked');
            end;
         end;
         %
         for i=1:length(BarHandles);
            set(BarHandles(i),'FaceColor',ColorMat(i,:),'EdgeColor',ColorMat(i,:));
         end;
         %
         set(gca,'YLim',[0 100],'Layer','Top');
         %
         SetAxesFonts(CurrINI,'Title',['Decomposition of Prediction Uncertainty for ' CurrVariable],'XLabel','Prediction Horizon','YLabel','Percent');
         %
         % add the legend
         %
         if MatlabNumber>=7;
            legHandle = legend(BarHandles,strvcat('Shocks','Parameter'),'Location','BestOutside');
         else;
            legHandle = legend(BarHandles,strvcat('Shocks','Parameter'),-1);
         end;
         SetAxesFonts(CurrINI,legHandle);
         %
         SetToolbarImages(FigHandle,CurrINI.images);
         PredFigHandle = FigHandle;
      else;
         PredFigHandle = [];
      end;
      %
      if (IsHandle(PredictiveControls.ShowKernelDensity)==1)&(get(PredictiveControls.ShowKernelDensity,'Value')==1);
         %
         % the prediction sample date strings are located in: DSGEModel.PredSampleStrMat
         %
         [n,NumGridPoints,h] = size(DSGEModel.KernelX);
         %
         % get the observed data and check if we can use it
         %
         y = get(YHandle,'YData');
         DateVector = get(YHandle,'XData');
         %
         % find the position in DateVector that corresponds to the first obs in PredDateVector
         %
         PredPos = (DateVector==DSGEModel.PredDateVector(1)).*(1:length(DateVector));
         PredPos = PredPos(PredPos>0);
         if isempty(PredPos)==0;
            y = y(PredPos:length(y));
            if length(y)<h;
               y = [y NaN*ones(1,h-length(y))];
            end;
         else;
            y = [];
         end;
         %
         % check how many plots we should make
         %
         NumPlots = 0;
         for i=1:h;
            if isequal(DSGEModel.KernelX(VariableValue,:,i),zeros(1,NumGridPoints))==0;
               NumPlots = NumPlots+1;
            end;
         end;
         NumFigs = ceil(NumPlots/16);
         if NumFigs==1;
            [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumPlots);
         elseif NumFigs>1;
            NumRows = 4;
            NumCols = 4;
         end;
         CurrPlot = 0;
         MadePlots = 0;
         pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
         %
         for i=1:NumFigs;
            if i>1;
               pos(1) = pos(1)+22;
               pos(2) = pos(2)-22;
               if pos(1)+1000>CurrINI.scrsz(3);
                  pos(1) = 0;
               end;
               if pos(2)<=32;
                  pos(2) = CurrINI.scrsz(4)-700;
               end;
            end;
            FigHandles{i} = figure('Visible','off','Position',pos);
            if NumFigs==1;
               set(FigHandles{i},'Name',['DSGE-VAR(' num2str(DSGEModel.lambda,'%0.4f') ') Marginal Predictive Densities - ' CurrVariable]);
            else;
               set(FigHandles{i},'Name',['DSGE-VAR(' num2str(DSGEModel.lambda,'%0.4f') ') Marginal Predictive Densities - Figure ' int2str(i) ' - ' CurrVariable]);
            end;
            SetFigureProperties(FigHandles{i},CurrINI);
            %
            k = 0;
            DataHandle = [];
            DataStr = '';
            for r=1:NumRows;
               for c=1:NumCols;
                  if MadePlots<NumPlots;
                     CurrPlot = CurrPlot+1;
                     while isequal(DSGEModel.KernelX(VariableValue,:,CurrPlot),zeros(1,NumGridPoints))==1;
                        CurrPlot = CurrPlot+1;
                     end;
                     k = k+1;
                     MadePlots = MadePlots+1;
                     %
                     % ensure that the handle to the current figure is correct (HG2 issue)
                     %
                     set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
                     subplot(NumRows,NumCols,k);
                     %
                     % check if we have a normal approximation available for the predictive density
                     %
                     if (isempty(DSGEModel.UData)==0)&(isfield(DSGEModel.UData,'KernelX')==1);
                        NormalApproxHandle = plot(DSGEModel.UData.KernelX(VariableValue,:,CurrPlot)+AddOnVector(CurrPlot),DSGEModel.UData.KernelY(VariableValue,:,CurrPlot),':','Color',[0 0.5 1],'LineWidth',2);
                        hold('on');
                        NormalApproxStr = 'Normal approx.';
                     else;
                        NormalApproxHandle = [];
                        NormalApproxStr = '';
                     end;
                     %
                     % graph the predictive density
                     %
                     PredDensityHandle = plot(DSGEModel.KernelX(VariableValue,:,CurrPlot)+AddOnVector(CurrPlot),DSGEModel.KernelY(VariableValue,:,CurrPlot),'-','Color',[0 0.25 0],'LineWidth',2);
                     hold('on');
                     %
                     SetAxesFonts(CurrINI,'Title',[CurrVariable ' - ' DSGEModel.PredSampleStrMat(CurrPlot,:)]);
                     %
                     % deal with axis limits
                     %
                     axis('tight');
                     xlim = [min(DSGEModel.KernelX(VariableValue,:,CurrPlot)) max(DSGEModel.KernelX(VariableValue,:,CurrPlot))];
                     set(gca,'XLim',xlim);
                     %
                     ylim = [0 1.2*max(DSGEModel.KernelY(VariableValue,:,CurrPlot))];
                     %
                     % display the mean prediction
                     %
                     if (isempty(PredictiveControls.AdjustMean)==0)&(get(PredictiveControls.AdjustMean,'Value')==1);
                        MeanPredHandle = plot((DSGEModel.MeanPrediction(VariableValue,CurrPlot)+AddOnVector(CurrPlot))*ones(1,2),ylim,'-.','Color',[0.75 0 0],'LineWidth',2);
                        MeanStr = 'Observation';
                     else;
                        %
                        %
                        if (isempty(y)==0)&(isnan(y(CurrPlot))==0);
                           DataHandle = plot(y(CurrPlot)*ones(1,2),ylim,'-.','Color',[0.75 0 0],'LineWidth',2);
                           hold('on');
                           DataStr = 'Observation';
                        end;
                        MeanPredHandle = plot(DSGEModel.MeanPrediction(VariableValue,CurrPlot)*ones(1,2),ylim,'--','Color',[0 0 0.75],'LineWidth',2);
                        MeanStr = 'Mean';
                     end;
                     hold('on');
                     %
                     x_lim = get(gca,'XLim');
                     add_x = 0.1*(xlim(2)-xlim(1));
                     x_lim = [xlim(1)-add_x xlim(2)+add_x];
                     %
                     set(gca,'XLim',x_lim,'YLim',ylim);
                     hold('off');
                  end;
               end;
            end;
            legHandle = legend([PredDensityHandle NormalApproxHandle MeanPredHandle DataHandle],strvcat('Density',NormalApproxStr,MeanStr,DataStr));
            %
            SetAxesFonts(CurrINI,legHandle);
            %
            % transform units temporarily into pixels
            %
            OldAxesUnits = get(gca,'Units');
            OldLegUnits = get(legHandle,'Units');
            set(gca,'Units','pixels');
            set(legHandle,'Units','pixels');
            %
            % check if we can move legend
            %
            LegPos = get(legHandle,'Position');
            AxPos = get(gca,'Position');
            if LegPos(1)<AxPos(1)+AxPos(3);
               if AxPos(1)+AxPos(3)+10+LegPos(3)<1000;
                  LegPos(1) = AxPos(1)+AxPos(3)+10;
                  set(legHandle,'Position',LegPos);
               end;
            end;
            set(gca,'Units',OldAxesUnits);
            set(legHandle,'Units',OldLegUnits);
            %
            % Fix the toolbar cdata
            %
            SetToolbarImages(FigHandles{i},CurrINI.images);
            %
            % Display the current figure
            %
            set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
            drawnow;
            pause(0.02);
         end;
      end;
      %
      if isempty(PredFigHandle)==0;
         set(PredFigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      end;
      %
      set(TopFigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(PredictiveControls.done,'UserData','done');
      delete(DSGEVARPredictiveDistributionGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(DSGEVARPredictiveDistributionGUI,'Visible','on');
   drawnow;
   SetYADAIcon(DSGEVARPredictiveDistributionGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(PredictiveControls.done);
   end;
   waitfor(PredictiveControls.done,'UserData','done');
end;

%
% end of DSGEVARPredictiveDistributionDLG.m
%
