function [SyOmega,SyOmegaAnnual,OriginalVariance,AnnualVariance,Omega] = DSGEVARSpectralDecomposition(Phi,A0,lambda,EstStr,DSGEModel,CurrINI)
% DSGEVARSpectralDecomposition: Computes a spectral decomposition of the contemporaneous covariance matrix
%                               for a DSGE-VAR model. The decomposition is made in terms of all structural
%                               shocks.
%
% USAGE:
%
%       [SyOmega,SyOmegaAnnual,OriginalVariance,AnnualVariance,Omega] = 
%                               DSGEVARSpectralDecomposition(Phi,A0,lambda,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nxnp with the coefficient on lagged endogenous
%                      variables in the VAR.
%
%                  A0 (matrix) of dimension nxn with the contemporaneous responses in the
%                      endogenous variables from unit shocks.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  EstStr (string) that takes the value 'Initial Values', 'Marginal Posterior Mode',
%                      or 'Joint Posterior Mode'.
%
%                  DSGEModel (structure) whose fields contain all required information
%                                 about the model, such as input files, variable and sample
%                                 data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: SyOmega (cell array) of dimension nxf with nxn matrices with the conditional
%                      spectral density for the f frequencies and the n structural shocks.
%
%                  SyOmegaAnnual (cell array) of dimension nxf with nxn matrices with the annualized
%                      conditional spectral density for the f frequencies and the n structural shocks.
%
%                  OriginalVariance (vector) with the DSGE-VAR model based population variances for
%                      the endogenous variables.
%
%                  AnnualVariance (vector) with the DSGE-VAR model based population variances for
%                      the annualized endogenous variables.
%
%                  Omega (vector) of dimension f with the frequencies from 0 to pi.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 27, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-05-2010: Updated the documentation.
%
% * 31-05-2010: Updated the documentation.
%
% * 26-10-2010: Added info about the lag order on the wait dialog.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 19-01-2012: Added try-catch code around the call to "DoublingAlgorithmLyapunov".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% determine dimensions
%
[n,np] = size(Phi);
p = np/n;
Omega = (0:pi/299:pi);
f = length(Omega);
%
% initialize
%
SyOmega = cell(n,f);
SyOmegaAnnual = [];
OriginalVariance = [];
AnnualVariance = [];
%
% set small responses to zero
%
Apos = (A0>1.0e-12);
Aneg = (A0<-1.0e-12);
A0 = (A0 .* Apos)+(A0 .* Aneg);
%
% compute the population covariance matrix using the model
%
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
try;
   [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*A0*A0'*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
catch
   vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(A0*A0');
   SigmaZ = reshape(vSigmaZ,n*p,n*p);
end;
SigmaY = SigmaZ(1:n,1:n);
OriginalVariance = diag(SigmaY);
%
%
if max(DSGEModel.annual)>1;
   SyOmegaAnnual = cell(n,f);
   AnnualVariance = OriginalVariance;
   ComputeAnnual = 1;
   %
   % compute the annualized variances of the endogenous variables
   %
   if max(DSGEModel.annual)==4;
      Cov1 = J'*Phi*SigmaZ*J;
      Cov2 = J'*(Phi^2)*SigmaZ*J;
      Cov3 = J'*(Phi^3)*SigmaZ*J;
      AnnCov = (4*SigmaY)+(3*(Cov1+Cov1'))+(2*(Cov2+Cov2'))+(Cov3+Cov3');
      AnnualVariance = ((DSGEModel.annual==1)' .* diag(SigmaY))+((DSGEModel.annual==4)' .* DSGEModel.annualscale' .* DSGEModel.annualscale' .* diag(AnnCov));
   elseif max(DSGEModel.annual)==12;
      Cov1 = J'*Phi*SigmaZ*J;
      Cov2 = J'*(Phi^2)*SigmaZ*J;
      Cov3 = J'*(Phi^3)*SigmaZ*J;
      Cov4 = J'*(Phi^4)*SigmaZ*J;
      Cov5 = J'*(Phi^5)*SigmaZ*J;
      Cov6 = J'*(Phi^6)*SigmaZ*J;
      Cov7 = J'*(Phi^7)*SigmaZ*J;
      Cov8 = J'*(Phi^8)*SigmaZ*J;
      Cov9 = J'*(Phi^9)*SigmaZ*J;
      Cov10 = J'*(Phi^10)*SigmaZ*J;
      Cov11 = J'*(Phi^11)*SigmaZ*J;
      AnnCov = (12*SigmaY)+(11*(Cov1+Cov1'))+(10*(Cov2+Cov2'))+(9*(Cov3+Cov3'))+(8*(Cov4+Cov4'))+(7*(Cov5+Cov5'))+ ...
               (6*(Cov6+Cov6'))+(5*(Cov7+Cov7'))+(4*(Cov8+Cov8'))+(3*(Cov9+Cov9'))+(2*(Cov10+Cov10'))+(Cov11+Cov11');
      AnnualVariance = ((DSGEModel.annual==1)' .* diag(SigmaY))+((DSGEModel.annual==12)' .* DSGEModel.annualscale' .* DSGEModel.annualscale' .* diag(AnnCov));
   end;
else;
   ComputeAnnual = 0;
end;
%
% compute pxf matrices with exp(-i*k*omega) and exp(i*k*omega), respectively, where k=1,...,p.
%
NegOmega = exp(-i*(1:p)'*Omega);
PosOmega = exp(i*(1:p)'*Omega);
%
Phi = J'*Phi;
%
% compute a matrix with the transposes of the submatrices of Phi, i.e. with
% Phi = [Phi(1) ... Phi(p)], we're looking for PhiT = [Phi(1)' ... Phi(p)'];
%
PhiT = zeros(n,np);
for k=1:p;
   PhiT(:,1+(n*(k-1)):n*k) = Phi(:,1+(n*(k-1)):n*k)';
end;
In = eye(n);
if n>=7;
   %
   % display a wait dialog
   %
   txt = ['Please wait while YADA computes spectral decompositions for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and p=' int2str(p) ' at ' lower(EstStr) '.'];
   WaitHandle = WaitDLG(txt,'information',['Spectral Decompositions - ' EstStr],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% compute the spectral decompositions
%
for freq=1:f;
   if ComputeAnnual==1;
      [AnnMatrix,ConjAnnMatrix] = AnnualFilter(DSGEModel.annual,DSGEModel.annualscale,Omega(freq));
   end;
   InvPhiPos = (In-(Phi*kron(NegOmega(:,freq),In)))\In;
   InvPhiTNeg = (In-(PhiT*kron(PosOmega(:,freq),In)))\In;
   for l=1:n;
      SyOmega{l,freq} = (1/(2*pi))*InvPhiPos*A0(:,l)*A0(:,l)'*InvPhiTNeg;
      if ComputeAnnual==1;
         SyOmegaAnnual{l,freq} = AnnMatrix*SyOmega{l,freq}*ConjAnnMatrix;
      end;
   end;
   drawnow;
end;
%
% close the wait dialog
%
if n>=7;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;

%
% end of DSGEVARSpectralDecomposition.m
%
