function LogMargs = PlotMargLikeChibJeliazkovDSGEVAR(PostSample,LogPost,NumBurnin,CurrChain,lambda,PME,DSGEModel,CurrINI)
% PlotMargLikeChibJeliazkovDSGEVAR: Calculates sequentially the Chib and Jeliazkov marginal likelihood density
%                                   based estimator of the log marginal likelihood for DSGE-VAR models.
%
% USAGE:
%
%       LogMargs = PlotMargLikeChibJeliazkovDSGEVAR(PostSample,LogPost,NumBurnin,CurrChain,lambda,PME,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  PostSample (matrix) of dimension NumIterxNumParam with the posterior sample draws
%                             of the phi parameters.
%
%                  LogPost (vector) of dimension (NumIterx1) with all the log posterior values for the
%                             non-discarded posterior draws. These posterior values are calculated
%                             using the "logPosteriorPhiDSGE" function.
%
%                  NumBurnin (integer), with the number of burn-in draws.
%
%                  CurrChain (integer) with the current MCMC chain number.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                             are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                             used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                             when generating the posterior draws.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% OPTIONAL OUTPUT: LogMargs (matrix) with the sequential estimates of the log marginal likelihood
%                             in the second column and the numerical standard errors in the
%                             third column. The first column holds the number of parameter draws
%                             that were used for the estimate of the marginal likelihood.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 27, 2012.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-02-2012: Finished the function.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 30-08-2017: Took into account that "logPosteriorPhiDSGEVAR" and "MargLikeChibJeliazkovDSGEVAR" require
%               the input variable "NumWorkers".
%
% * 16-01-2018: Updated the documentation.
%
% * 22-03-2018: Made sure that the "SystemPriorFile" is copied to the tmp directory.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 15-04-2021: Corrected the mode files under PME 2 and 3.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 05-07-2023: The function now tries to the the AIMData file directly.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% check if we should initialize the output.
%
if nargout==1;
   LogMargs = [];
end;
%
% locate the controls structure
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
%
% Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% Scale factor for the random walk Metropolis algorithm, i.e., c in "YADA Manual - Computational Details".
%
ScaleStr = get(controls.posterior.scalefactor,'String');
c = str2num(StringTrim(ScaleStr(DSGEModel.MHScaleFactor,:)));
%
% sequential marginal likelihood start value and step length
%
SeqStartStr = get(controls.dsge.sequentialstart,'String');
DSGEModel.SequentialStartIterationValue = str2num(strrep(StringTrim(SeqStartStr(DSGEModel.SequentialStartIteration,:)),',',''));
SeqStepStr = get(controls.dsge.sequentialstep,'String');
DSGEModel.SequentialStepLengthValue = str2num(strrep(StringTrim(SeqStepStr(DSGEModel.SequentialStepLength,:)),',',''));
%
% determine the posterior mode results file to open
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
if PME==1;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
elseif PME==2;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
elseif PME==3;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
end;
%
% check if we can load the posterior mode data
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'error','DSGE-VAR Seq. Marginal Likelihood - Chib & Jeliazkov',500,600,CurrINI);
   return;
end;
%
% initialize dimensions
%
p = DSGEModel.BVARLags;
k = DSGEModel.k;
n = DSGEModel.n;
npk = (n*p)+k;
%
% create sample dates
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Check if we should adjust this based on p, the lag order
%
IVAdjusted = 0;
if FirstPeriod+KalmanFirstObservation-1<p+1;
   KalmanFirstObservation = p+2-FirstPeriod;
   IVAdjusted = 1;
end;
%
% actual sample that we will use
%
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
DSGEModel.KalmanFirstObservation = KalmanFirstObservation;
%
% get the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
if k>0;
   DetProductMoments = CalcDeterministicProductMoments(x,X);
else;
   DetProductMoments = [];
end;
%
% compute sample product moments
%
GammaHatyy = (1/T)*y*y';
GammaHatyY = (1/T)*y*Y';
GammaHatYY = (1/T)*Y*Y';
%
if isinf(lambda)==0;
   logGPR = LogGammaProductRatio(n,((1+lambda)*T)-npk,(lambda*T)-npk);
else;
   logGPR = [];
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% try to load the AIMData, other wise use ModeData
%
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      AIMData = ModeData.AIMData;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      AIMData = ModeData.AIMData;
   end;
end;
%
% now try to 
%
phiMode = ModeData.phiMode;
if MatlabNumber>=8.0;
   try;
      logPostMode = -logPosteriorPhiDSGEVAR(phiMode,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
   catch ME;
      txt = ['Unable to compute the log posterior at the posterior mode values. Message caught is: ' YADALastError(ME)];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
      return;
   end;
else;
   try;
      logPostMode = -logPosteriorPhiDSGEVAR(phiMode,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
   catch;
      txt = ['Unable to compute the log posterior at the posterior mode values. Message caught is: ' YADALastError];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
      return;
   end;
end;
%
% determine the covariance matrix used for the posterior simulator
%
NumParam = length(phiMode);
if DSGEModel.InverseHessianEstimator==1;
   SigmaMode = ModeData.InverseHessian;
elseif DSGEModel.InverseHessianEstimator==2;
   if sum(isnan(ModeData.phiFittedStd))==0;
      SigmaStd = sqrt(abs(diag(ModeData.InverseHessian)));
      SigmaCorr = ModeData.InverseHessian ./ (SigmaStd * SigmaStd');
      phiFittedStd = ModeData.phiFittedStd;
      oldphiFittedStd = phiFittedStd;
      if DSGEModel.ModifiedHessian==1;
         FullSet = (1:length(phiFittedStd));
         for i=1:length(phiFittedStd);
            iExcluded = setdiff(FullSet,i);
%            phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*inv(SigmaCorr(iExcluded,iExcluded))*SigmaCorr(iExcluded,i)));
            phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*(SigmaCorr(iExcluded,iExcluded)\SigmaCorr(iExcluded,i))));
         end;
      end;
      SigmaMode = SigmaCorr .* (phiFittedStd * phiFittedStd');
   else;
      txt = 'YADA has not fitted a quadratic to the evaluated log posterior. Please run the "Check posterior mode" function first. Alternatively, you can use the estimator of the inverse Hessian from the optimization routine.';
      About(txt,'information','YADA - Modified Inverse Hessian',200,500,CurrINI);
      drawnow;
      return;
   end;
elseif DSGEModel.InverseHessianEstimator==3;
   if sum(sum(isnan(ModeData.phiInverseHessianFinDiff)))==0;
      SigmaMode = ModeData.phiInverseHessianFinDiff;
   else;
      txt = 'YADA has not estimated the inverse Hessian using finite difference during the posterior mode estimation. YADA is therefore unable to continue.';
      About(txt,'question','YADA - Finite Difference Inverse Hessian',200,500,CurrINI);
      drawnow;
      return;
   end;
elseif DSGEModel.InverseHessianEstimator==4;
   %
   % try to load the file
   %
   CovMatData = load(DSGEModel.ParameterCovMatrix);
   SigmaMode = CovMatData.ParameterCovarianceMatrix;
   if size(SigmaMode,1)~=NumParam;
      txt = ['The dimension of the parameter covariance matrix you have selected from the file "' GetName(DSGEModel.ParameterCovMatrix) '" is ' int2str(size(SigmaMode,1)) ', while the number of parameters to estimate is ' int2str(NumParam) '. YADA is therefore unable to continue.'];
      About(txt,'information','YADA - Own Parameter Covariance Matrix',200,500,CurrINI);
      return;
   end;
end;
%
% check if we should manipulate the correlations for the proposal density
%
if DSGEModel.MaxCorrelationValue>1;
   MaxCorrStr = get(controls.posterior.maxcorrelation,'String');
   MaxCorr = str2num(StringTrim(MaxCorrStr(DSGEModel.MaxCorrelationValue,:)));
   %
   % create the correlation matrix
   %
   StdVec = sqrt(diag(SigmaMode));
   CorrMat = SigmaMode ./ (StdVec*StdVec');
   for i=1:length(StdVec);
      CorrMat(i,i) = 0;
   end;
   ScaleFactor = max(max(abs(CorrMat)));
   ScaleMatrix = min(1,MaxCorr/ScaleFactor)*ones(length(StdVec),length(StdVec));
   for i=1:length(StdVec);
      ScaleMatrix(i,i) = 1;
   end;
   SigmaMode = SigmaMode .* ScaleMatrix;
else;
   MaxCorr = 1;
end;
%
% test if SigmaMode is positive definite
%
if IsPositiveDefinite(SigmaMode)==0;
   EigValues = eig(SigmaMode);
   EigValues = EigValues(EigValues<=0);
   txt = ['The inverse Hessian is not positive definite. YADA has found that ' int2str(length(EigValues)) ' eigenvalues of the estimated inverse Hessian are less than or equal to zero. This means that the inverse Hessian estimator can''t be used for when computing the log marginal likelihood. Do you want YADA to try to create a positive definite matrix using your matrix as input?'];
   answer = Query(txt,'question',300,'YADA - Inverse Hessian Estimation Problem',500,'no',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      [NewSigmaMode,pdstatus] = CreatePositiveDefinite(SigmaMode);
      if pdstatus==1;
         SigmaMode = NewSigmaMode;
      else;
         txt = 'YADA wasn''t able to "make" the inverse Hessian positive definite. The calculation of the log marginal likelihood has therefore been aborted.';
         About(txt,'information','YADA - Inverse Hessian Estimation Problem',200,500,CurrINI);
         return;
      end;
   else;
      return;
   end;
end;
%
% initialize the random number generators
%
if DSGEModel.RandomNumberValue==1;
   rand('state',0);
   randn('state',0);
else;
   rand('state',sum(100*clock));
   randn('state',sum(100*clock));
end;
%
% Open up a wait dialog.
%
WaitTitle = 'Sequential Marginal Likelihood - Chib & Jeliazkov';
ShowCancel = 1;
if (nargout==1)&(DSGEModel.SequentialML==0);
   WaitTitle = 'Marginal Likelihood - Chib & Jeliazkov';
   ShowCancel = 0;
end;
txt = ['Please wait while YADA computes Chib and Jeliazkov''s marginal likelihood identity based estimator of the log marginal likelihood for a DSGE-VAR(' num2str(lambda,'%0.4f') ') with ' int2str(p) ' lags. Estimation started at: ' StringTrim(datestr(now,14)) '. This will take some time!'];
WaitHandle = WaitDLG(txt,'information',WaitTitle,550,300,CurrINI,ShowCancel);
WaitControls = get(WaitHandle,'UserData');
if isfield(WaitControls,'cancel')==0;
   WaitControls.cancel = [];
end;
%
% run the marginal likelihood estimation
%
if nargout==0;
   LogMargsCJ = MargLikeChibJeliazkovDSGEVAR(PostSample,LogPost,phiMode,SigmaMode,c,logPostMode,NumBurnin,1,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers,WaitControls.cancel);
else;
   LogMargsCJ = MargLikeChibJeliazkovDSGEVAR(PostSample,LogPost,phiMode,SigmaMode,c,logPostMode,NumBurnin,DSGEModel.SequentialML,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers,WaitControls.cancel);
end;
%
% shut down the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
% clear draws of transformed parameters from memory
%
clear('PostSample','LogPost');
%
if isempty(LogMargsCJ)==0;
   %
   % ask if we write to file
   %
   txt = 'Would you like to save the sequential estimation data to file?';
   answer = Query(txt,'question',200,'Save - Chib & Jeliazkov',500,'yes',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      Xvalues = LogMargsCJ(:,1)+NumBurnin;
      LogMargLikeCJ = LogMargsCJ(:,2:3);
      %
      file = FixFilePath([DSGEModel.OutputDirectory '\SeqChibJeliazkov-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.mat']);
      save(file,'Xvalues','LogMargLikeCJ');
      %
      mfile = FixFilePath([DSGEModel.OutputDirectory '\SeqCJDSGEVAR' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(p) strrep(DSGEModel.NameOfModel,'-','') int2str(NumPostDraws) int2str(CurrChain) '.m']);
      fid = fopen(mfile,'wt');
      fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
      fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
      fprintf(fid,'%%\n%% Number of posterior draws used for sequential estimation (N x 1) vector\n%%\n');
      fprintf(fid,'Xvalues = DataStructure.Xvalues;\n');
      fprintf(fid,'%%\n%% N x 2 matrix with log marginal likelihood values and numerical standard\n%% error using the Chib and Jeliazkov estimator\n%%\n');
      fprintf(fid,'LogMargLikeCJ = DataStructure.LogMargLikeCJ;\n');
      fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
      %
      % end the script file
      %
      fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
      fclose(fid);
      %
      txt = ['The data on sequential estimation of the log marginal likelihood has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: Xvalues (Nx1 vector with the number of posteror draws used for the sequential estimation), and LogMargLikeCJ (Nx2 matrix with estimates of the log marginal likelihood and the numerical standard errors with the Chib and Jeliazkov approach). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
      About(txt,'information',WaitTitle,200,500,CurrINI);
   end;
   %
   if nargout==1;
      LogMargs = LogMargsCJ;
      return;
   end;
   %
   GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
   %
   FigHandle =figure('Visible','off', ...
          'Position',[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500], ...
          'Tag','MargLikeCJ', ...
          'Name','Sequential Marginal Likelihood - Chib and Jeliazkov');
   %
   SetFigureProperties(FigHandle,CurrINI);
   %
   Xvalues = LogMargsCJ(:,1)+NumBurnin;
   Yvalues = LogMargsCJ(:,2);
   %
   plot(Xvalues,Yvalues,'LineStyle','-','Color',[0.75 0 0],'LineWidth',2,'Marker','.');
   hold('off');
   SetAxesFonts(CurrINI,'Title','Chib and Jeliazkov Estimates','XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
   axis('tight');
   %
   % check the YLim
   %
   ylim = get(gca,'YLim');
   add_on = 0.1*(ylim(2)-ylim(1));
   ylim = [ylim(1)-add_on ylim(2)+add_on];
   set(gca,'YLim',ylim);
   %
   % save the grasph
   %
   if DirExist(GraphDirectory);
      file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ChibJeliazkov-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.fig']);
      saveas(FigHandle,file);
   end;
   %
   % display the graph
   %
   SetToolbarImages(FigHandle,CurrINI.images);
   set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of PlotMargLikeChibJeliazkovDSGEVAR.m
%
