function PrintDSGEVARAutoCorrelations(ResFile,lambda,DSGEModel,SVEs,CurrINI,EstStr,NumPaths,UsedDraws)
% PrintDSGEVARAutoCorrelations: Writes the results from calculating autocorrelations based on a DSGE-VAR
%                               model and based on the data to disk.
%
% USAGE:
%
%       PrintDSGEVARAutoCorrelations(ResFile,lambda,DSGEModel,SVEs,CurrINI,EstStr,NumPaths,UsedDraws)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the text-file where the results
%                         are written.
%
%                 lambda (scalar) with the lambda hyperparameter of the DSGE-VAR.
%
%                 DSGEModel (structure) whose fields contain all required information
%                         about the model, such as input files, variable and sample data.
%
%                 SVEs (structure) with means standard deviations and possibly also quantiles.
%
%                 CurrINI (structure) with initialization information.
%
%                 EstStr (string) that takes the values "Initial Values", "Marginal Posterior mode", or
%                         "Joint Posterior Mode".
%
%                 NumPaths (integer) with the number of paths to simulate. 0 means that the moments
%                         are population based.
%
%                 UsedDraws (integer) with the number of parameter draws.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 17, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-06-2010: Ensured that the file can be used also for prior and posterior
%               distribution based moments.
%
% * 25-11-2011: The function now also writes skewness results to file.
%
% * 02-12-2011: The function now also provides results on kurtosis.
%
% * 03-01-2012: Updated the documentation.
%
% * 27-08-2012: Changed "DSGE Model" label to "DSGE-VAR Model".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
if NumPaths==0;
   fprintf(fid,'*                       P O P U L A T I O N - B A S E D                        *\n');
else;
   fprintf(fid,'*                           S A M P L E - B A S E D                            *\n');
end;
fprintf(fid,'*            M E A N S   &   S T A N D A R D   D E V I A T I O N S             *\n');
fprintf(fid,'*                  S K E W N E S S   A N D   K U R T O S I S                   *\n');
fprintf(fid,'*                         F O R   A   D S G E - V A R                          *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% initializing the formatting of output numbers. we usually apply the 12.6f format.
%
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
%
if (NumPaths==0)&(UsedDraws==1);
   fprintf(fid,'NOTE: Using the %s values.\n\n',lower(EstStr));
   [n,vn] = size(DSGEModel.VariableNames);
   if vn<15;
      AddNameStr = SpaceStr(15-vn);
   else;
      AddNameStr = '';
   end;
   %
   % check first if there are exogenous variables
   %
   PrtStr = '';
   for i=1:size(SVEs.Mean,2);
      PrtStr = [PrtStr '   ' PrtVal];
   end;
   PrtStr = ['%s%s    ' PrtStr '   |' PrtStr '\n'];
   TopStr = MultiCharStr('=',23+(30*size(SVEs.Mean,2)));
   DivStr = MultiCharStr('-',23+(30*size(SVEs.Mean,2)));
   %
   fprintf(fid,'%s\n',TopStr);
   fprintf(fid,'%sAverages\n',SpaceStr(19+(15*size(SVEs.Mean,2))));
   fprintf(fid,'%sDSGE-VAR Model   |   Data\n',SpaceStr(5+(15*size(SVEs.Mean,2))));
   fprintf(fid,'%s\n',DivStr);
   for i=1:n;
      fprintf(fid,PrtStr,AddNameStr,DSGEModel.VariableNames(i,:),[SVEs.Mean(i,:) SVEs.Data.Mean(i,:)]');
   end;
   fprintf(fid,'%s\n\n',TopStr);
   if DSGEModel.k>0;
      XVarNames = StrWrap(vecstr(DSGEModel.XVariableNames,' '),40);
   else;
      XVarNames = 'None';
   end;
   fprintf(fid,'Names of exogenous variables:     %s\n',XVarNames(1,:));
   for i=2:size(XVarNames,1);
      if strcmp(StringTrim(XVarNames(i,:)),'')==0;
         fprintf(fid,'                                  %s\n',XVarNames(i,:));
      end;
   end;
   fprintf(fid,'Number of lags:                   %0.0f\n',DSGEModel.BVARLags);
   fprintf(fid,'Lambda hyperparameter:            %0.4f\n\n',lambda);
   if isempty(SVEs.meanerror)==0;
      MeanErrorStr = ['%s%s     ' PrtVal ' \n'];
      fprintf(fid,'====================================\n');
      fprintf(fid,'       Variable  Maximum Mean Error\n');
      fprintf(fid,'------------------------------------\n');
      for i=1:n;
         fprintf(fid,MeanErrorStr,AddNameStr,DSGEModel.VariableNames(i,:),SVEs.meanerror(i));
      end;
      fprintf(fid,'====================================\n\n');
      fprintf(fid,'NOTE: The maximum mean error concerns the approximation of the mean with\n       mu(t) = c*x(t) rather than mu(t) = sum(j=1,t) c(j)*x(j).\n\n');
   end;
   %
   % Now we deal with the standard deviations
   %
   PrtStr = ['%s%s       ' PrtVal '   |   ' PrtVal '\n'];
   TopStr = MultiCharStr('=',53);
   DivStr = MultiCharStr('-',53);
   fprintf(fid,'%s\n',TopStr);
   fprintf(fid,'%sStandard Deviations\n',SpaceStr(29));                         
   fprintf(fid,'%sDSGE-VAR Model   |   Data\n',SpaceStr(20));
   fprintf(fid,'%s\n',DivStr);
   for i=1:n;
      fprintf(fid,PrtStr,AddNameStr,DSGEModel.VariableNames(i,:),SVEs.Std(i),SVEs.Data.Std(i));
   end;
   fprintf(fid,'%s\n\n',TopStr);
   fprintf(fid,'NOTE: The data standard deviation above is based on first correcting for\n      the data mean.\n\n');
   %
   % skewness
   %
   fprintf(fid,'%s\n',TopStr);
   fprintf(fid,'%sSkewness\n',SpaceStr(33));                         
   fprintf(fid,'%sDSGE-VAR Model   |   Data\n',SpaceStr(20));
   fprintf(fid,'%s\n',DivStr);
   for i=1:n;
      fprintf(fid,PrtStr,AddNameStr,DSGEModel.VariableNames(i,:),SVEs.Skew(i),SVEs.Data.Skew(i));
   end;
   fprintf(fid,'%s\n\n',TopStr);
   %
   % kurtosis
   %
   fprintf(fid,'%s\n',TopStr);
   fprintf(fid,'%sKurtosis\n',SpaceStr(33));                         
   fprintf(fid,'%sDSGE-VAR Model   |   Data\n',SpaceStr(20));
   fprintf(fid,'%s\n',DivStr);
   for i=1:n;
      fprintf(fid,PrtStr,AddNameStr,DSGEModel.VariableNames(i,:),SVEs.Kurt(i),SVEs.Data.Kurt(i));
   end;
   fprintf(fid,'%s\n\n',TopStr);
else;
   %
   if NumPaths>0;
      fprintf(fid,'Number of paths for the observed variables:       %0.0f\n',NumPaths);
   end;
   if UsedDraws>1;
      fprintf(fid,'Number of parameter draws:                        %0.0f\n',UsedDraws);
      if NumPaths>0;
         fprintf(fid,'Total number of paths for the observed variables: %0.0f\n',NumPaths*UsedDraws);
      end;
   end;
   fprintf(fid,'\nNOTE: Using the %s parameter values.\n\n',lower(EstStr));
   fprintf(fid,'Number of lags:                                   %0.0f\n',DSGEModel.BVARLags);
   fprintf(fid,'Lambda hyperparameter:                            %0.4f\n\n',lambda);
   %
   [n,vn] = size(DSGEModel.VariableNames);
   if vn<20;
      AddNameStr = SpaceStr(20-vn);
   else;
      AddNameStr = '';
   end;
   %
   % deal with the means
   %
   NumBounds = floor(length(SVEs.Quantiles)/2);
   QuantVec = zeros(NumBounds,1);
   QuantStd = zeros(NumBounds,2,n);
   QuantSkew = zeros(NumBounds,2,n);
   QuantKurt = zeros(NumBounds,2,n);
   k = size(SVEs.Mean,2);
   if (isempty(DSGEModel.XVariableNames)==1)&(k>0);
      DSGEModel.XVariableNames = 'const';
   end;
   for j=1:k;
      Quant(j).Mean = zeros(NumBounds,2,n);
   end;
   for nb=1:NumBounds;
      QuantVec(NumBounds-nb+1,1) = SVEs.Quantiles(length(DSGEModel.Percentiles)-nb+1).percent-SVEs.Quantiles(nb).percent;
      for i=1:n;
         QuantStd(NumBounds-nb+1,1,i) = SVEs.Quantiles(nb).Std(i,1);
         QuantStd(NumBounds-nb+1,2,i) = SVEs.Quantiles(length(DSGEModel.Percentiles)-nb+1).Std(i,1);
         QuantSkew(NumBounds-nb+1,1,i) = SVEs.Quantiles(nb).Skew(i,1);
         QuantSkew(NumBounds-nb+1,2,i) = SVEs.Quantiles(length(DSGEModel.Percentiles)-nb+1).Skew(i,1);
         QuantKurt(NumBounds-nb+1,1,i) = SVEs.Quantiles(nb).Kurt(i,1);
         QuantKurt(NumBounds-nb+1,2,i) = SVEs.Quantiles(length(DSGEModel.Percentiles)-nb+1).Kurt(i,1);
         for j=1:k;
            Quant(j).Mean(NumBounds-nb+1,1,i) = SVEs.Quantiles(nb).Mean(i,j);
            Quant(j).Mean(NumBounds-nb+1,2,i) = SVEs.Quantiles(length(DSGEModel.Percentiles)-nb+1).Mean(i,j);
         end;
      end;
   end;
   %
   % define some print string
   %
   PrtStdStr = ['%s%s           ' PrtVal '          |   ' PrtVal ' \n'];
   PrtQuantStr = ['%3.0f percent            [' PrtVal ',' PrtVal ']   |\n'];
   if k>0;
      kn = size(DSGEModel.XVariableNames,2);
      if kn<20;
         AddXNameStr = SpaceStr(20-kn);
      else;
         AddXNameStr = '';
      end;
      %
      % print the mean results
      %
      for j=1:k;
         fprintf(fid,'======================================================================\n');
         fprintf(fid,'                                                  Averages \n');
         fprintf(fid,'                                        Variable: %s%s \n',DSGEModel.XVariableNames(j,:),AddXNameStr);
         fprintf(fid,'Variable                         DSGE-VAR Model      |       Data \n');
         fprintf(fid,'======================================================================\n');
         for i=1:n;
            fprintf(fid,PrtStdStr,AddNameStr,DSGEModel.VariableNames(i,:),SVEs.Mean(i,j),SVEs.Data.Mean(i,j));
            for nb=1:NumBounds;
               fprintf(fid,PrtQuantStr,QuantVec(nb,1),Quant(j).Mean(nb,:,i)');
            end;
            if i<n;
               fprintf(fid,'----------------------------------------------------------------------\n');
            else;
               fprintf(fid,'======================================================================\n\n');
            end;
         end;
      end;
   end;
   %
   % the standard deviations
   %
   fprintf(fid,'======================================================================\n');
   fprintf(fid,'                                             Standard Deviations \n');
   fprintf(fid,'Variable                         DSGE-VAR Model      |       Data \n');
   fprintf(fid,'======================================================================\n');
   for i=1:n;
      fprintf(fid,PrtStdStr,AddNameStr,DSGEModel.VariableNames(i,:),SVEs.Std(i,1),SVEs.Data.Std(i,1));
      for j=1:NumBounds;
         fprintf(fid,PrtQuantStr,QuantVec(j,1),QuantStd(j,:,i)');
      end;
      if i<n;
         fprintf(fid,'----------------------------------------------------------------------\n');
      else;
         fprintf(fid,'======================================================================\n\n');
      end;
   end;
   %
   % skewness
   %
   fprintf(fid,'======================================================================\n');
   fprintf(fid,'                                                 Skewness \n');
   fprintf(fid,'Variable                         DSGE-VAR Model      |       Data \n');
   fprintf(fid,'======================================================================\n');
   for i=1:n;
      fprintf(fid,PrtStdStr,AddNameStr,DSGEModel.VariableNames(i,:),SVEs.Skew(i,1),SVEs.Data.Skew(i,1));
      for j=1:NumBounds;
         fprintf(fid,PrtQuantStr,QuantVec(j,1),QuantSkew(j,:,i)');
      end;
      if i<n;
         fprintf(fid,'----------------------------------------------------------------------\n');
      else;
         fprintf(fid,'======================================================================\n\n');
      end;
   end;
   %
   % kurtosis
   %
   fprintf(fid,'======================================================================\n');
   fprintf(fid,'                                                 Kurtosis \n');
   fprintf(fid,'Variable                         DSGE-VAR Model      |       Data \n');
   fprintf(fid,'======================================================================\n');
   for i=1:n;
      fprintf(fid,PrtStdStr,AddNameStr,DSGEModel.VariableNames(i,:),SVEs.Kurt(i,1),SVEs.Data.Kurt(i,1));
      for j=1:NumBounds;
         fprintf(fid,PrtQuantStr,QuantVec(j,1),QuantKurt(j,:,i)');
      end;
      if i<n;
         fprintf(fid,'----------------------------------------------------------------------\n');
      else;
         fprintf(fid,'======================================================================\n\n');
      end;
   end;
end;
%
% bottom file data
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintDSGEVARAutoCorrelations.m
%
