function PrintDSGEVARShockCorrelations(ResFile,lambda,DSGEModel,ShockRes,CurrINI,EstStr,TotalDraws,UsedDraws)
% PrintDSGEVARShockCorrelations: Writes the results from calculating the sample mean, standard deviation and 
%                                correlations of the estimated structural shocks based on a DSGE-VAR model
%                                and based on the data to disk.
%
% USAGE:
%
%       PrintDSGEVARShockCorrelations(ResFile,lambda,DSGEModel,ShockRes,CurrINI,EstStr,UsedDraws)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the text-file where the results
%                         are written.
%
%                 lambda (scalar) with the lambda hyperparameter of the DSGE-VAR.
%
%                 DSGEModel (structure) whose fields contain all required information
%                         about the model, such as input files, variable and sample data.
%
%                 ShockRes (structure) with shock names, means, standard deviations, correlations,
%                         and quantiles.
%
%                 CurrINI (structure) with initialization information.
%
%                 EstStr (string) that takes the values "Prior" or "Posterior".
%
%                 TotalDraws (integer) with the number of draws from the posterior distribution of the parameters
%                         that could have been used.
%
%                 UsedDraws (integer) with the number of parameter draws.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 19, 2014.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-03-2014: Finished the function.
%
% * 25-03-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(ResFile,'wt');
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*            M E A N S   &   S T A N D A R D   D E V I A T I O N S             *\n');
fprintf(fid,'*                        A N D   C O R R E L A T I O N                         *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
%
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
if isempty(TotalDraws)==0
   fprintf(fid,'Number of available parameter draws: %0.0f\n',TotalDraws);
end;
fprintf(fid,'Number of parameter draws used:      %0.0f\n',UsedDraws);
fprintf(fid,'Parameter distribution:              %s\n',EstStr);
fprintf(fid,'Number of lags:                      %0.0f\n',DSGEModel.BVARLags);
fprintf(fid,'Lambda hyperparameter:               %0.4f\n\n',lambda);
%
if size(ShockRes.ShockNames)<=15;
   ShockNames = ShockRes.ShockNames;
else;
   ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
end;
[n,vn] = size(ShockNames);
if vn<15;
   AddNameStr = SpaceStr(15-vn);
else;
   AddNameStr = '';
end;
PrtMeanStr = ['%s%s           ' PrtVal '                  ' PrtVal ' \n'];
PrtQuantStr = ['%3.0f percent       [' PrtVal ',' PrtVal ']   [' PrtVal ',' PrtVal '] \n'];
%
% determine all the quantile data
%
NumBounds = floor(length(ShockRes.Quantiles)/2);
QuantVec = zeros(NumBounds,1);
QuantMean = zeros(NumBounds,2,n);
QuantStd = zeros(NumBounds,2,n);
for nb=1:NumBounds;
   QuantVec(NumBounds-nb+1,1) = ShockRes.Quantiles(length(DSGEModel.Percentiles)-nb+1).percent-ShockRes.Quantiles(nb).percent;
   for i=1:n;
      QuantMean(NumBounds-nb+1,1,i) = ShockRes.Quantiles(nb).SampleMean(i);
      QuantMean(NumBounds-nb+1,2,i) = ShockRes.Quantiles(length(DSGEModel.Percentiles)-nb+1).SampleMean(i);
      QuantStd(NumBounds-nb+1,1,i) = ShockRes.Quantiles(nb).StdShocks(i);
      QuantStd(NumBounds-nb+1,2,i) = ShockRes.Quantiles(length(DSGEModel.Percentiles)-nb+1).StdShocks(i);
   end;
end;
%
% header for the mean and the standard deviation
%
fprintf(fid,'============================================================================\n');
fprintf(fid,'                                                          Sample\n');
fprintf(fid,'          Shock               Mean                  Standard Deviation\n');
fprintf(fid,'============================================================================\n');
for i=1:n;
   fprintf(fid,PrtMeanStr,AddNameStr,ShockNames(i,:),ShockRes.SampleMean(i),ShockRes.StdShocks(i));
   for j=1:NumBounds;
      fprintf(fid,PrtQuantStr,QuantVec(j),QuantMean(j,:,i)',QuantStd(j,:,i)');
   end;
   if i<n;
      fprintf(fid,'----------------------------------------------------------------------------\n');
   else;
      fprintf(fid,'============================================================================\n\n');
   end;
end;
fprintf(fid,'NOTE: The population mean and standard deviation of the structural shocks is 0 and 1.\n\n');
%
% and now the correlations
%
if n>1;
   ShockStrMat = '';
   CorrQuant = zeros(NumBounds,2,n*(n-1)/2);
   for j=1:n-1;
      for i=j+1:n;
         ShockStrMat = strvcat(ShockStrMat,[ShockNames(j,:) ' ' ShockNames(i,:)]);
      end;
   end;
   for nb=1:NumBounds;
      CorrQuant(NumBounds-nb+1,1,:) = vechl(ShockRes.Quantiles(nb).CorrShocks);
      CorrQuant(NumBounds-nb+1,2,:) = vechl(ShockRes.Quantiles(length(DSGEModel.Percentiles)-nb+1).CorrShocks);
   end;
   CorrShocks = vechl(ShockRes.CorrShocks);
   PrtCorrStr = ['%s%s%s           ' PrtVal ' \n'];
   PrtQuantQtr = ['            %3.0f percent           [' PrtVal ',' PrtVal '] \n'];
   %
   fprintf(fid,'==============================================================\n');
   fprintf(fid,'                                              Sample \n');
   fprintf(fid,'        Shock 1/Shock 2                    Correlation \n');
   fprintf(fid,'==============================================================\n');
   for i=1:n*(n-1)/2;
      fprintf(fid,PrtCorrStr,AddNameStr,ShockStrMat(i,:),AddNameStr,CorrShocks(i));
      for j=1:NumBounds;
         fprintf(fid,PrtQuantQtr,QuantVec(j,1),CorrQuant(j,:,i)');
      end;
      if i<n*(n-1)/2;
         fprintf(fid,'--------------------------------------------------------------\n');
      else;
         fprintf(fid,'==============================================================\n\n');
      end;
   end;
   fprintf(fid,'NOTE: The population correlation of the structural shocks is 0.\n\n');
end;
%
% bottom file data
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintDSGEVARShockCorrelations.m
%
