function [status,PostModeEstimator] = VerifyVARDSGESamplingTxtFiles(DSGEModel,CurrentChain,controls)
% VerifyVARDSGESamplingTxtFiles: checks which VAR posterior sampling summary files exist on disk.
%
% USAGE:
%
%       [status,PostModeEstimator] = VerifyVARDSGESamplingTxtFiles(DSGEModel,CurrentChain,controls)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrentChain (integer) the value of the parallel chain to load data from.
%
% OPTIONAL INPUT:  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% REQUIRED OUTPUT: status (boolean), which is 1 if a file exists for one lambda and 0 otherwise.
%
% OPTIONAL OUTPUT: PostModeEstimator (vector structure) with 3 elements. Each element has the
%                            field Lambda (vector) with the lambda hyperparameter values for
%                            which the results file exists on disk. The first element of the
%                            PostModeEstimator structure is the DSGE model posterior mode estimator,
%                            the second is the marginal DSGE-VAR posterior mode estimator, and
%                            the third is the joint DSGE-VAR posterior mode estimator.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 9, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 02-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 14-08-2017: The function now supports SMC with likelihood tempering.
%
% * 11-09-2017: Took into account that SMC with likelihood tempering sets NumBurnin to zero and
%               UsedDraws to NumPostDraws.
%
% * 15-01-2018: Updated the documentation.
%
% * 16-10-2018: The function now supports SMC with data tempering.
%
% * 04-12-2018: The function should now support IS based on MitISEM.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
if nargout==2;
   PostModeEstimator(1).Lambda = [];
   PostModeEstimator(2).Lambda = [];
   PostModeEstimator(3).Lambda = [];
end;
%
% locate the controls structure
%
if nargin<3;
   maingui = findobj('Type','figure','Tag','YADA');
   controls = get(maingui,'UserData');
end;
%
% get the candidate values of lambda
%
lambda = GetLambda(DSGEModel);
p = DSGEModel.BVARLags;
%
%
% Get the base values number of draws, etc.
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% remove burnin draws
%
if DSGEModel.PosteriorSampler<=7
   BurninStr = get(controls.posterior.burnin,'String');
   NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
else;
   NumBurnin = 0;
end;
NumPostDraws = NumPostDraws-NumBurnin;
%
% and the share used
%
if DSGEModel.PosteriorSampler<=7;
   DrawsPercentStr = get(controls.posterior.usepostdraws,'String');
   DrawsPercent = str2num(strrep(StringTrim(DrawsPercentStr(DSGEModel.PostDrawsPercentValue,:)),'%',''));
   UsedDraws = ceil(DrawsPercent*NumPostDraws/100);
else;
   UsedDraws = NumPostDraws;
end;
%
% 3. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% number of tempering stages
%
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
PostSampleStr = [int2str(DSGEModel.InverseHessianEstimator) '.' int2str(DSGEModel.PosteriorSampler)];
if DSGEModel.PosteriorSampler==3;
   PostSampleStr = [PostSampleStr '-Student-' int2str(DSGEModel.StudenttDegFree)];
elseif DSGEModel.PosteriorSampler==4;
   PostSampleStr = [PostSampleStr '-Fixed-' int2str(DSGEModel.FixedNumParamBlocks)];
elseif DSGEModel.PosteriorSampler==5;
   PostSampleStr = [PostSampleStr '-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-Student-' int2str(DSGEModel.StudenttDegFree)];
elseif DSGEModel.PosteriorSampler==6;
   PostSampleStr = [PostSampleStr '-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2))];
elseif DSGEModel.PosteriorSampler==7;
   PostSampleStr = [PostSampleStr '-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-Student-' int2str(DSGEModel.StudenttDegFree)];
elseif DSGEModel.PosteriorSampler==8;
   PostSampleStr = [int2str(DSGEModel.PosteriorSampler) '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm)];
elseif DSGEModel.PosteriorSampler==9;
   PostSampleStr = [int2str(DSGEModel.PosteriorSampler) '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm)];
elseif DSGEModel.PosteriorSampler==10;
   PostSampleStr = [PostSampleStr '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF)];
end;
%
for PME=1:3
   for i=1:length(lambda);
      lambdai =lambda(i);
      TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(UsedDraws) '.' int2str(NumPostDraws) '.txt']);
      if FileExist(TextFile)==1;
         status = 1;
         if nargout==2;
            PostModeEstimator(PME).Lambda = [PostModeEstimator(PME).Lambda lambdai];
         end;
      end;
   end;
   if (status==1)&(nargout==1);
      break;
   end;
end;

%
% end of VerifyVARDSGESamplingTxtFiles.m
%
