function [logPredLike,PredictedVars] = logPredLikeDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,Dn,OrderQZ,J,Y,YData,X,k,h,AddlnL)
% logPredLikeDSGEVAR: Computes the height of the joint predictive likelihood for DSGE-VAR models.
%
% USAGE:
%
%       [logPredLike,PredictedVars] = logPredLikeDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions, ...
%                  thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY, ...
%                  GammaHatYY,DetProductMoments,HSample,Dn,OrderQZ,J,Y,YData,X,k,h,AddlnL)
%
% REQUIRED INPUT:  phi (vector) with all parameters to be estimated.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                             are 0 1 except for the uniform and the beta distribution, where they represent
%                             the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  T (interger) with the sample size of the endogenous variables
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  npk (integer) with the number of regressors in the VAR, equal to the number of endogenous
%                             variables times the number of lags plus the number of exogenous variables.
%
%                  GammaHatyy (matrix) of dimension nxn with the sample product moments of the endogenous
%                             variables.
%
%                  GammaHatyY (matrix) of dimension nxnpk with the sample product moments of the endogenous
%                             variables and the regressors.
%
%                  GammaHatYY (matrix) of dimension npkxnpk with the sample product moments of the
%                             regressors.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                             are located.
%
%                  Dn (matrix) of dimension n^2xn(n+1)/2 with the duplication matrix.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
%                  J (matrix) of dimension npxn which selects the y(t) vector from the stacked Y(t) vector,
%                             i.e., y(t) = J'*Y(t), where J has an nxn identity matrix on top and zeros below.
%
%                  Y (vector) of dimension np with the historical data on the endogenous variables
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                             sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  k (integer) with the number of exoegnous variables.
%
%                  h (integer) with the prediction horizon.
%
%                  AddlnL (boolean) that takes the value 1 if the historical sample log-likelihood should be added
%                             to the prediction sample log-likelihood, and 0 otherwise.
%
% REQUIRED OUTPUT: logPredLike (vector) of dimension h with the value of the log marginal predictive likelihood at phi
%                             for h-step-ahead forecasts.
%
% OPTIONAL OUTPUT: PredictedVars (cell array) with the positions of the predicted variables in the vector of observed
%                             variables for each prediction period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 5, 2012.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-05-2012: Updated the documentation.
%
% * 26-04-2012: Fixed the sum(isnan(theta)) test.
%
% * 14-05-2012: Updated the documentation.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 30-08-2012: Made sure that the log posterior is computed also using the VAR parameters and not from
%               the marginalized or the concentrated likelihood function.
%
% * 25-10-2012: Removed the "ModeValue" input variable.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 22-06-2017: Took into account that logPosteriorKernelDSGEVAR now requires the input variable
%               OrderQZ.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

logPredLike = [];
PredVars = [];
if nargout==2;
   PredictedVars = [];
end;
drawnow;
%
if isinf(lambda)==0;
   Phi = reshape(phi(1:n*npk),n,npk);
   SigmaEpsilon = reshape(Dn*phi((n*npk)+1:(n*npk)+(n*(n+1)/2)),n,n);
   phi = phi((n*npk)+(n*(n+1)/2)+1:length(phi));
end;
%
% initialize
%
theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
if (sum(isinf(theta))>=1)|(sum(isnan(theta))>=1);
   logPredLike = NaN*ones(h,1);
   if nargout==2;
      PredictedVars = [];
   end;
   return;
end;
%
% determine the prior population matrices from which the prior VAR parameters
% can be computed
%
[A,H,R,F,B0] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,OrderQZ);
[Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
if isinf(lambda)==1;
   %
   % compute the VAR parameters
   %
   Phi = GammayY/GammaYY;
   SigmaEpsilon = Gammayy-((GammayY/GammaYY)*GammayY');
end;
%
if AddlnL==1;
   LogPostKernel = logPosteriorKernelDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,OrderQZ,lambda,Phi,SigmaEpsilon,GammaHatyy,GammaHatyY,GammaHatYY,Gammayy,GammaYY,GammayY,T,n,npk);
end;
%
% determine the constant term and the Psi matrix
%
if k>0;
   Phi0 = Phi(:,1:k);
   Xbar = Phi0*X;
   Phi = Phi(:,k+1:npk);
else;
   Xbar = zeros(n,h);
end;
Psi = Phi;
if p>1;
   Psi = [Psi;eye(n*(p-1)) zeros(n*(p-1),n)];
end;
%
% compute the likelihood over the prediction sample
%
SigmaEpsilonJ = J*SigmaEpsilon*J';
JXbar = J*Xbar;
Ytt1 = JXbar(:,1)+(Psi*Y);
Omegatt1 = SigmaEpsilonJ;
KInit = eye(n);
KInit = KInit(:,DSGEModel.PredictionVariables);
logLikeValue = zeros(h,1);
YData = YData(DSGEModel.PredictionVariables,:);
log2pi = log(2*pi);
for i=1:h;
   Yhatjoint = J'*Ytt1;
   SigmaYjoint = J'*Omegatt1*J;
   %
   % check for missing obs
   %
   pos = ~isnan(YData(:,i));
   nstar = sum(pos);
   if nstar>0;
      K = KInit(:,pos);
      %
      % select the subset
      %
      Yhatjoint = Yhatjoint(DSGEModel.PredictionVariables);
      SigmaYjoint = SigmaYjoint(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      %
      % and now the likelihood
      %
      Ystarerror = YData(pos,i)-Yhatjoint(pos);
      SigmaYstar = SigmaYjoint(pos,pos);
      logLikeValue(i) = -((nstar*log2pi)+LogDet(SigmaYstar)+((Ystarerror'/SigmaYstar)*Ystarerror))/2;
      PredVars{i} = DSGEModel.PredictionVariables(pos);
      %
      % now we compute the Kalman filter updates
      %
      if i<h;
         KalmanGain = Psi*Omegatt1*J*(K/SigmaYstar);
         Ytt1 = JXbar(:,i+1)+(Psi*Ytt1)+(KalmanGain*Ystarerror);
         PsiDev = Psi-(KalmanGain*K'*J');
         Omegatt1 = (PsiDev*Omegatt1*PsiDev')+SigmaEpsilonJ;
      end;
   else;
      PredVars{i} = [];
      %
      % now we compute the Kalman filter updates
      %
      if i<h;
         Ytt1 = JXbar(:,i+1)+(Psi*Ytt1);
         Omegatt1 = (Psi*Omegatt1*Psi')+SigmaEpsilonJ;
      end;
   end;
end;
logLikeValue = cumsum(logLikeValue);
%
% and we add things up
%
logLikeValue(logLikeValue==0) = NaN;
if AddlnL==0;
   logPredLike = -logLikeValue;
else;
   if LogPostKernel~=1000000;
      logPredLike = -logLikeValue+LogPostKernel;
   else;
      logPredLike = NaN*ones(h,1);
   end;
end;
%
if nargout>1;
   PredictedVars = PredVars;
end;

%
% end of logPredLikeDSGEVAR.m
%
