function [logPredLike,PredVars] = logPredLikePriorDSGEVARAnnual(YData,YDataAnnual,X,h,IsMarginal,DSGEModel,n,p,k,npk,J,Y,Phi,SigmaEpsilon)
% logPredLikePriorDSGEVARAnnual: Computes the log-likelihood, the log predictive likelihood, and the log prior at given parameters
%                                values for either the h-step-ahead joint or the marginal predictive likelihood for a DSGE-VAR,
%                                taking annualization into account.
%
% USAGE:
%
%       [logPredLike,PredVars] = logPredLikePriorDSGEVAR(YData,YDataAnnual,X,h,IsMarginal,DSGEModel,n,p,k,npk,J,Y, ...
%                                                        Phi,SigmaEpsilon)
%
% REQUIRED INPUT:  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                             sample.
%
%                  YDataAnnual (matrix) of dimension nxh with realizations for the annualized observed variables over
%                             the prediction sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  h (integer) with the prediction horizon.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive likelihood should be
%                             computed and 0 if the joint predictive likelihood is the object of interest.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  k (integer) with the number of exogenous variables.
%
%                  npk (integer) with the number of regressors in the VAR, equal to the number of endogenous
%                             variables times the number of lags plus the number of exogenous variables.
%
%                  J (matrix) of dimension npxn which selects the y(t) vector from the stacked Y(t) vector,
%                             i.e., y(t) = J'*Y(t), where J has an nxn identity matrix on top and zeros below.
%
%                  Y (vector) of dimension np with the historical data on the endogenous variables.
%
%                  Phi (matrix) of dimension nxnpk with the parameters on the exogenous variables and the
%                             lagged endogenous of the VAR.
%
%                  SigmaEpsilon (matrix) of dimension nxn with the residual covariances of the VAR.
%
% REQUIRED OUTPUT: logPredLike (vector) of dimension h with the value of the log marginal predictive likelihood at phi
%                             for h-step-ahead forecasts.
%
%                  PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                             variables for each prediction period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 19, 2012.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-10-2012: Finished the function.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
drawnow;
%
if IsMarginal==1;
   [logPredLike,PredVars] = logMargCondLikeDSGEVARAnnual(DSGEModel,n,p,npk,J,Y,YDataAnnual,X,k,h,Phi,SigmaEpsilon);
else;
   [logPredLike,PredVars] = logCondLikeDSGEVAR(DSGEModel,n,p,npk,J,Y,YData,X,k,h,Phi,SigmaEpsilon);
end;

%
% end of logPredLikePriorDSGEVARAnnual.m
%
