function [status,DynareErrorStr] = DynareParser(DynareModelFile,NameOfModel,OutputDirectory,DynarePath,CurrINI)
% DynareParser: Runs the dynare parser and creates the AiM files for solving a DSGE model.
%
% USAGE:
%
%       status = DynareParser(DynareModelFile,NameOfModel,OutputDirectory,DynarePath,CurrINI)
%
% REQUIRED INPUT:  DynareModelFile (string) with full path and name of the model file to parse with dynare.
%
%                  NameOfModel (string) with a name for the model.
%
%                  OutputDirectory (string) with the full path to the output directory.
%
%                  DynarePath (string) with the path to the main dynare directory.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: status (integer) indicates the status when parsing the dynare model file where 1
%                         means success and 0 means errors from dynare; -1 means no dynare
%                         installation; -2 means no stoch_simul call in model file.
%
%                  DynareErrorStr (string) with error messages from trying to run dynare with
%                         the model file.
%
%
% NOTE: This function is based on Paolo Zagaglia's dynare2yada program. It has been updated to handle
%       more recent dynare versions (from about 4.3 and later), and has been adapted to the YADA
%       environment.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: November 23, 2022.
%                        This version: January 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 24-11-2022: Continued work on the function.
%
% * 25-11-2022: Continued work on the function from replacing temporary terms to the h matrix.
%
% * 28-11-2022: Finished the function.
%
% * 29-11-2022: Made sure that estimation calls are ignored in the model file being parsed.
%
% * 01-12-2022: The code is now backward compatible with, e.g., dynare 4.3.3 and probably even older versions.
%
% * 02-12-2022: Updated the documentation.
%
% * 05-12-2022: Dealt with parameters.
%
% * 09-12-2022: Added warning off after running dynare since it has turned it on and older dynare version don't
%               support the nowarn option.
%
% * 04-01-2023: Took into account that older dynare versions leave an m-file with the same filename as the
%               mod-file.
%
% * 15-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
DynareErrorStr = '';
%
% deal with directories: we need to add dynare (temporarily) to the path if it is not already there
%                        and cd to model file directory (temporarily)
%
IsDynare = (exist('dynare')==2);
OrigPath = path;
YADADirectory = pwd;
ModelDirectory = GetPath(DynareModelFile);
if IsDynare==0;
   path(DynarePath,path);
   if exist('dynare')~=2;
      status = -1;
      path(OrigPath);
      return;
   end;
end;
%
% check the model file to ensure that it uses the aim solver
%
DynModelTxt = ReadText(DynareModelFile);
%
% the output is a cell array of strings
%
CheckStochSim = 0;
CheckAiM = 0;
for i=1:length(DynModelTxt);
   %
   % search for line with stoch_sim command
   %
   TxtStr = DynModelTxt{i};
   if CheckStochSim==0;
      ssind = strfind(TxtStr,'stoch_simul(');
      if isempty(ssind)==0;
         %
         % we've located the call
         %
         aimind = strfind(TxtStr,'aim_solver');
         if isempty(aimind)==1;
            %
            % include a call to the aim solver
            %
            RepStr = TxtStr;
            FirstStr = TxtStr(1:ssind+12);
            SecondStr = TxtStr(ssind+13:length(TxtStr));
            DynModelTxt{i} = [FirstStr 'aim_solver,' SecondStr];
         else;
            CheckAiM = 1;
         end;
         CheckStochSim = 1;
      end;
   end;
end;
if CheckStochSim==0;
   path(OrigPath);
   status = -2;
   return;
end;
%
% make sure that any estimation calls are ignored
%
EstInd = FindStringRow(char(DynModelTxt),'estimation(');
for i=1:length(EstInd);
   DynModelTxt{EstInd(i)} = '';
end;
%
% see if the model file commands have been edited
%
if (CheckAiM==0)|(length(EstInd)>0);
   %
   % copy the old model file temporarily
   %
   TmpFile = [GetPath(DynareModelFile) NameOfModel '_tmp' GetExt(DynareModelFile)];
   [stat,msg] = CopyFile(DynareModelFile,TmpFile);
   %
   % now write to the original model file
   %
   stat = TextToFile(DynareModelFile,DynModelTxt);
end;
%
% dynare is now fully setup to run. Change to the model directory
%
cd(ModelDirectory);
%
% run dynare with try-catch code
%
Message = '';
if MatlabNumber>=8.0;
   try;
      %EvalStr = ['dynare(''' NameOfModel ''',''onlyclearglobals'',''nolog'',''console'',''nowarn'',''nointeractive'');'];
      EvalStr = ['dynare(''' NameOfModel ''',''noclearall'',''nolog'',''console'');'];
      Message = evalc(EvalStr);
   catch ME;
      DynareErrorStr = ['There was an error trying to run dynare using the model file "' NameOfModel '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      %EvalStr = ['dynare(''' NameOfModel ''',''onlyclearglobals'',''nolog'',''console'',''nowarn'',''nointeractive'');'];
      EvalStr = ['dynare(''' NameOfModel ''',''noclearall'',''nolog'',''console'');'];
      Message = evalc(EvalStr);
   catch;
      DynareErrorStr = ['There was an error trying to run dynare using the model file "' NameOfModel '". Message caught is: ' YADALastError];
   end;
end;
%
% restore the old model file if necessary
%
if (CheckAiM==0)|(length(EstInd)>0);
   [stat,msg] = CopyFile(TmpFile,DynareModelFile);
   delete(TmpFile);
end;
%
% change to YADA's base directory
%
cd(YADADirectory);
%
% make sure warnings are off after running dynare
%
warning('off');
%
% the dynare based aim solution files are located in the directory
%
OutDirPlus = FixFilePath([ModelDirectory '+' NameOfModel '\']);
DynareOutputMatFile = FixFilePath([OutputDirectory '\Output\' NameOfModel '_results.mat']);
%
% check if there are any errors running dynare
%
if isempty(DynareErrorStr)==0;
   %
   % delete files created by dynare and keep only those linked to YADA
   %
   CheckSumFile = FixFilePath([OutputDirectory '\checksum']);
   if FileExist(CheckSumFile)==1;
      delete(CheckSumFile);
   end;
   if FileExist(DynareOutputMatFile)==1;
      delete(DynareOutputMatFile);
      stat = rmdir(GetPath(DynareOutputMatFile));
   end;
   %
   % delete all files in + directory
   %
   if FileExist(FixFilePath([OutDirPlus 'dynamic_g1.m']))==1;
      delete([OutDirPlus '*.m']);
      stat = rmdir(OutDirPlus);
   end;
   path(OrigPath);
   return;
end;
%
% we can now check if the required files are located in the appropriate folders
%
IsNewerDynare = 1;
mfile = '';
if FileExist(DynareOutputMatFile)==0;
   %
   % check if an older version of dynare has been used.
   %
   AltDynareOutputMatFile = [ModelDirectory NameOfModel '_results.mat'];
   if FileExist(AltDynareOutputMatFile)==1;
      DynareOutputMatFile = AltDynareOutputMatFile;
      mfile = [ModelDirectory NameOfModel '.m'];
      IsNewerDynare = 0;
   else;
      DynareErrorStr = ['The dynare output mat file "' GetFilename(DynareOutputMatFile) '" does not exist in the directory "' GetPath(DynareOutputMatFile) '". Something must have gone wrong. Please check the log-file "' NameOfModel '.log" in the directory "' GetPath(DynareOutputMatFile) '".'];
      stat = TextToFile([ModelDirectory NameOfModel '.log'],cellstr(Message));
      SetLogText(strvcat('Missing dynare output mat file','Parsing failed',' '));
      path(OrigPath);
      return;
   end;
end;
%
% check if the dynamic_g1.m file exists
%
if IsNewerDynare==1;
   DynamicFile = FixFilePath([OutDirPlus 'dynamic_g1.m']);
   if FileExist(DynamicFile)==0;
      DynareErrorStr = ['The "dynamic_g1.m" file was not created in the directory "' OutDirPlus '". Please check if your dynare model file calls the stoch_simul function or has other errors. '];
      SetLogText(strvcat('Missing "dynamic_g1.m" file','Parsing failed',' '));
      path(OrigPath);
      return;
   end;
else;
   DynamicFile = FixFilePath([ModelDirectory NameOfModel '_dynamic.m']);
   if FileExist(DynamicFile)==0;
      DynareErrorStr = ['The "' NameOfModel '_dynamic.m" file was not created in the directory "' ModelDirectory '". Please check if your dynare model file calls the stoch_simul function or has other errors. '];
      SetLogText(strvcat(['Missing "' NameOfModel '_dynamic.m" file'],'Parsing failed',' '));
      path(OrigPath);
      return;
   end;
end;
%
% and if dynamic_resid_tt exists
%
if IsNewerDynare==1;
   DynamicTTFile = FixFilePath([OutDirPlus 'dynamic_resid_tt.m']);
   if FileExist(DynamicFile)==0;
      DynareErrorStr = ['The "dynamic_resid_tt.m" file was not created in the directory "' OutDirPlus '". Please check if your dynare model file calls the stoch_simul function or has other errors. '];
      SetLogText(strvcat('Missing "dynamic_resid_tt.m" file','Parsing failed',' '));
      path(OrigPath);
      return;
   end;
end;
%
% we should now have the necessary information to create the aim related files
%
% 1. create an empty aim file
%
AiMFile = FixFilePath([ModelDirectory NameOfModel '.aim']);
SetLogText(['Writing to an empty aim model file: ' GetFilename(AiMFile)]);
AiMText = strvcat(['MODEL > ' NameOfModel],' ','END');
stat = TextToFile(AiMFile,cellstr(AiMText));
%
% next we create the compute_aim_data.m file
%
% load required data
%
DynMD = load(DynareOutputMatFile);
DM = DynMD.M_;
DO = DynMD.oo_;
%
%CAD = cell(13,1)
CAD{1} = 'function [param_,np,modname,neq,nlag,nlead,eqname_,eqtype_,endog_,delay_,vtype_] = compute_aim_data()';
CAD{2} = '';
%
n_eq     = size(DM.endo_names,1)+size(DM.exo_names,1)+1;
n_lag    = DM.maximum_lag;
n_lead   = DM.maximum_lead;
%n_params = size(DM.params,1)+size(DM.endo_names,1);
n_params = size(DM.params,1);
if (n_lag>1)|(n_lead>1);
   DynareErrorStr = ['The number of leads (' int2str(n_lead) ') or lags (' int2str(n_lag) ') is too large. YADA required that they are no greater than 1. Please consider rewriting your model.'];
   SetLogText('The number of leads or lags is too large.');
   path(OrigPath);
   return;
end;
CAD{3} =  ['neq=' int2str(n_eq) ';'];
CAD{4} =  ['nlag=' int2str(n_lag) ';'];
CAD{5} =  ['nlead=' int2str(n_lead) ';'];
CAD{6} =  ['np=' int2str(n_params) ';'];
CAD{7} =  ['modname=''' NameOfModel '.aim' ''';'];
CAD{8} =  ['eqname = cell('  int2str(n_eq) ', 1);'];
CAD{9} =  ['param  = cell('  int2str(n_params) ', 1);'];
CAD{10} = ['endog  = cell('  int2str(n_eq) ', 1);'];
CAD{11} = ['delay  = zeros(' int2str(n_eq) ', 1);'];
CAD{12} = ['vtype  = zeros(' int2str(n_eq) ', 1);'];
CAD{13} = ['eqtype = zeros(' int2str(n_eq) ', 1);'];
CAD = CAD';
%
% setup the cell arrays for equations, parameters, etc
%
eqname = cell(n_eq,1);
eqtype = cell(n_eq,1);
delay = cell(n_eq,1);
endog = cell(n_eq,1);
vtype = cell(n_eq,1);
param = cell(n_params,1);
%
% prepare all the variable names
%
VariableNames = [cellstr(DM.endo_names);cellstr(DM.exo_names);cellstr('one')];
%
for i=1:n_eq;
   eqname{i} = ['eqname(' int2str(i) ')=cellstr(''EQ' int2str(i) ''');'];
   eqtype{i} = ['eqtype(' int2str(i) ')=' int2str(i) ';'];
   endog{i} = ['endog(' int2str(i) ')=cellstr(''' VariableNames{i} ''');'];
   delay{i} = ['delay(' int2str(i) ')=0;'];
   vtype{i} = ['vtype(' int2str(i) ')=1;'];
end;
%
% and now the parameters
%
%cellParameterNames = [cellstr(DM.param_names);cellstr(DM.endo_names)];
cellParameterNames = cellstr(DM.param_names);
for i=1:n_params;
   param{i} = ['param(' int2str(i) ')=cellstr(''' StringTrim(cellParameterNames{i}) ''');'];
end;
%
% last commands
%
LC = cell(11,1);
LC{1} = ['eqname_ = char(eqname);'];
LC{2} = ['eqtype_ = eqtype;'];
LC{3} = ['param_ = char(param);'];
LC{4} = ['endog_ = char(endog);'];
LC{5} = ['delay_ = delay;'];
LC{6} = ['vtype_ = vtype;'];
LC{8} = '%';
LC{9} = '% end of compute_aim_data.m';
LC{10} = '%';
%
% put everything together
%
CADText = [CAD;eqname;eqtype;param;endog;delay;vtype;LC];
%
% write to file
%
SetLogText(strvcat('Writing "compute_aim_data.m" in the directory:',['   ' OutputDirectory]));
AiMDataFile = FixFilePath([OutputDirectory '\compute_aim_data.m']);
stat = TextToFile(AiMDataFile,CADText);
%
% saving the data to a mat file
%
[stat,msg] = CopyFile(AiMDataFile,[pwd '\tmp']);
pause(0.5);
clear('param','eqname','eqtype','endog','delay','vtype');
[param,np,modname,neq,nlag,nlead,eqname,eqtype,endog,delay,vtype] = compute_aim_data;
%
% saving the data to a mat file
%
DataFile = FixFilePath([OutputDirectory '\' NameOfModel '.mat']);
save(DataFile,'param','eqname','eqtype','endog','neq','nlag','nlead','np','modname','delay','vtype');
SetLogText(strvcat(['Saving "' NameOfModel '.mat" to the directory:'],['   ' OutputDirectory]));
%
% now we turn to the compute_aim_matrices.m file
%
if IsNewerDynare==1;
   DynamicTxt = char(ReadText(DynamicFile));
   DynamicTTTxt = char(ReadText(DynamicTTFile));
   %
   % get the parameter names as a cell array of strings
   %
   ParameterNames = DM.param_names;
   Pos1 = FindStringRow(DynamicTxt,'%');
   Pos2 = FindStringRow(DynamicTxt,'function');
   Pos3 = FindStringRow(DynamicTxt,'if');
   Pos4 = FindStringRow(DynamicTxt,'end');
   %
   % remove the rows
   %
   DynamicTxt([Pos1 Pos2 Pos3 Pos4],:) = [];
   %
   % need to replace any temporary terms from dynamic_resid_tt instead of function call
   %
   Tstr = [];
   for i=1:size(DynamicTxt,1);
      col = strfind(DynamicTxt(i,:),'T');
      if isempty(col)==0;
         Tstr = [Tstr i];
      end;
   end;
   Tstri = 0;
   for i=1:length(Tstr);
      %
      % find the line with the call:
      %
      %        T = 'NameOfMode'.dynamic_g1_tt(T, y, x, params, steady_state, it_);
      %
      TestStr = findstr(DynamicTxt(Tstr(i),:),NameOfModel);
      if TestStr>0;
         Tstri = Tstr(i);
         break;
      end;
   end;
   if Tstri>0;
      %
      % remove the function call line
      %
      DynamicTxt(Tstri,:) = [];
      %
      % find the lines to replace T = ...
      %
      Pos1 = FindStringRow(DynamicTTTxt,'%');
      Pos2 = FindStringRow(DynamicTTTxt,'function');
      Pos3 = FindStringRow(DynamicTTTxt,'assert');
      Pos4 = FindStringRow(DynamicTTTxt,'end');
      %
      % remove these lines
      %
      DynamicTTTxt([Pos1 Pos2 Pos3 Pos4],:) = [];
      %
      % prepend to 
      %
      DynamicTxt = strvcat(DynamicTTTxt,DynamicTxt);
   end;
else;
   DynamicTxt = char(ReadText(DynamicFile));
   ParameterNames = DM.param_names;
   Pos1 = FindStringRow(DynamicTxt,'%');
   Pos2 = FindStringRow(DynamicTxt,'function');
   Pos3 = FindStringRow(DynamicTxt,'if');
   Pos4 = FindStringRow(DynamicTxt,'end');
   Pos5 = FindStringRow(DynamicTxt,'lhs');
   Pos6 = FindStringRow(DynamicTxt,'rhs');
   Pos7 = FindStringRow(DynamicTxt,'residual');
   Pos8 = FindStringRow(DynamicTxt,'g2');
   Pos9 = FindStringRow(DynamicTxt,'g3');
   %
   % remove the rows
   %
   DynamicTxt([Pos1 Pos2 Pos3 Pos4 Pos5 Pos6 Pos7 Pos8 Pos9],:) = [];
end;
%
% replace instances of params(i) with the corresponding name
% of the parameter
%
% convert to cellstr
%
cellDynamicTxt = cellstr(DynamicTxt);
cellStateVariableNames = cellstr(DM.endo_names);
%
% first we use ModelParameters.
%
cellDynamicTxt = regexprep(cellDynamicTxt,'params(','(ModelParameters.');
%
% and now the parameter names
%
for i=1:n_params;
   LookForStr = ['ModelParameters.' int2str(i) ')'];
   ReplaceWithStr = ['ModelParameters.', cellParameterNames{i}, ')'];
   cellDynamicTxt = regexprep(cellDynamicTxt,LookForStr,ReplaceWithStr);
end;
%
% replace all instances of 'g1(' and 'g1 =' with 'g(' and 'g ='
%
cellDynamicTxt = regexprep(cellDynamicTxt,'g1(','g(');
cellDynamicTxt = regexprep(cellDynamicTxt,'g1 =','g =');
cellDynamicText = cellstr(char(zeros(size(cellDynamicTxt,2)+8,size(cellDynamicTxt,2))));
cellDynamicText(9:size(cellDynamicTxt,1)+8) = cellDynamicTxt;
cellDynamicText(1:7) = cellstr(strvcat('function [g,h] = compute_aim_matrices(ModelParameters)',' ', ...
                                      '% function [g,h] = compute_aim_matrices(ModelParameters)', ...
                                      '%     This function computes the g and h matrices from the parameter values in the fields of "ModelParameters".. ','%', ...
                                      ['% This file was created by YADA on: ' datestr(now,0)],'%'));
%cellDynamicText(size(cellDynamicTxt,1)+9:size(cellDynamicTxt,1)+11) = cellstr(strvcat('%','% end of compute_aim_matrices.m','%'));
%
% number of leads, contemp and lags (should be 3)
%
LLnum = n_lag+n_lead+1;
%
% obtain the lead-lag incidence where first column is t-1, second is t and third is t+1
%
LLIncidence  = DM.lead_lag_incidence';
LLIncidence  = vec(LLIncidence);
%
% determine the non-zero elements
%
LLIncNonZero = find(LLIncidence) ;
%
% expand the state/model variables cellstring
% using the information in the non-zero elements
% lead-lag incidence
%
StateVarsLLInc = repmat(cellStateVariableNames,1,LLnum);
StateVarsLLInc = StateVarsLLInc(LLIncNonZero);
%
LLIncInd = (length(LLIncNonZero):-1:1)';
%
% replace all y(i)'s with ModelParameters.'StateVarName'
%
for i=1:length(LLIncNonZero);
   old_param = ['y(' int2str(LLIncInd(i))];
   new_param = ['(ModelParameters.' StateVarsLLInc{LLIncInd(i),:}];
   cellDynamicText = regexprep(cellDynamicText,old_param,new_param);
end;
DynamicText = char(cellDynamicText);
%
% set the correct model dimensions
%
RowsFullDim = n_eq;
ColsFullDim = n_eq*LLnum;
%
NewLine = ['g = zeros(', int2str(RowsFullDim) '.gekko, ' int2str(ColsFullDim) '.gekko);']; 
LineNum = FindStringRow(DynamicText,'g = zeros(');
cellDynamicText(LineNum,:) = cellstr(['g = zeros(', int2str(RowsFullDim) '.gekko,' int2str(ColsFullDim) '.gekko);']);
%
% setup the lead-lag incidence for the model with correct dimensions
%
ExpandedLLIncidence = [DM.lead_lag_incidence zeros(size(DM.lead_lag_incidence,1),size(DM.exo_names,1)) zeros(size(DM.lead_lag_incidence,1),1)];
%
% pick out the lag part, the contemporaneous and the lead parts
%
LagsLLInc    = ExpandedLLIncidence(1:n_lag,:);
ConLLinc = ExpandedLLIncidence((n_lag+1),:);
LeadsLLInc   = ExpandedLLIncidence((n_lag+1+1):(n_lag+1+n_lead),:);
%
% create row vectors of these
%
LagsLLInc = reshape(LagsLLInc',1,n_lag*RowsFullDim);
ConLLinc = reshape(ConLLinc',1,RowsFullDim);
LeadsLLInc = reshape(LeadsLLInc',1,n_lead*RowsFullDim);
%
% arrange the full row vector format
%
LagConLeadMat = [LagsLLInc ConLLinc LeadsLLInc];
%
% adjust the indices of the g matrix
%
% find the nonzero elements in LagConLeadMat
%
colsp = find(LagConLeadMat~=0);
for j=1:numel(colsp);
   OldStr = [',' int2str(j) ')'];
   NewStr = [',' int2str(colsp(j)) '.gekko)'];
   cellDynamicText = regexprep(cellDynamicText,OldStr,NewStr);
end;
%
% pick up the column index of the shocks in the full model form
%
RepInd = (RowsFullDim*(1+n_lag)-size(DM.exo_names,1)):(RowsFullDim*(1+n_lag)-1);
num = 0;
for i=(nnz(ExpandedLLIncidence )+1):(nnz(ExpandedLLIncidence)+size(DM.exo_names,1));
   num = num+1;
   OldStr = [',' int2str(i) ')'];
   NewStr = [',' int2str(RepInd(num)) '.gekko)'];  
   cellDynamicText = regexprep(cellDynamicText,OldStr,NewStr);
end;
%
% pick up the column indices corresponding to the shocks and 'one'
%
ColInd = (RowsFullDim*(1+n_lag)-size(DM.exo_names,1)):(RowsFullDim*(1+n_lag));
RowInd = (RowsFullDim-size(DM.exo_names,1)):(RowsFullDim);
%
% fill in the shocks + one part
%
ShocksPart = cellstr(char(zeros(size(DM.exo_names,1)+1,100)));
for i=1:(size(DM.exo_names,1)+1);
   ShocksPart(i,:) = cellstr(['g(' int2str(RowInd(i) ) ',' int2str(ColInd(i)) ')' '=1;']);
end;
%
% fix the cellDynamicText
%
cellDynamicText = regexprep(cellDynamicText,'.gekko',' ');
%
% add append to the cell string with the compute_aim_matrices code
%
cellDynamicText = [cellDynamicText;ShocksPart];
DynamicText = char(cellDynamicText);
%
% create the coefficient matrix h for the forward looking variables
%
nrows_h  = numel((RowsFullDim*(n_lag+1)+1):ColsFullDim);
newmat_h = cellstr(char(zeros(nrows_h+1,100)));
newmat_h(1,:) = cellstr(['h = zeros(' int2str(RowsFullDim) ',' int2str(ColsFullDim) ');']);
%
num = 1;
for i=(RowsFullDim*(n_lag+1)+1):ColsFullDim;
   OldString = [',' int2str(i) ' )'];
   RowIndex = FindStringRow(DynamicText,OldString);
   UseIndex = [];
   for j=1:numel(RowIndex);
      num = num+1;
      CheckStr = findstr(DynamicText(RowIndex(j),:),'g = zeros(');
      if isempty(CheckStr)==1;
         newmat_h(num,:)= cellstr(DynamicText(RowIndex(j),:));
         UseIndex = [UseIndex RowIndex(j)];
      end;
   end;
   %
   % clean up
   %
   DynamicText(UseIndex,:) = [];
end;
%
% make sure we replace g with h in
%
newmat_h = regexprep(newmat_h,'g(','h(');
%
% now we fix the dimension of g such that it only covers lagged and contemporaneous
% coefficients
%
ColsBackDim = RowsFullDim*(n_lag+1);
NewLine = ['g = zeros(' int2str(RowsFullDim) ' ,', int2str(ColsBackDim) ' );'];
LineNum = FindStringRow(DynamicText,'g = zeros(');
cellDynamicText = cellstr(DynamicText);
cellDynamicText(LineNum,:) = cellstr(NewLine);
%
% now prepend newmat_h and bottom of file information
%
cellDynamicText = [cellDynamicText;newmat_h;cellstr(strvcat('%','% end of compute_aim_matrices.m','%'))];
%
% now we can write to file
%
SetLogText(strvcat('Writing "compute_aim_matrices.m" in the directory:',['   ' OutputDirectory]));
AiMMatricesFile = FixFilePath([OutputDirectory '\compute_aim_matrices.m']);
stat = TextToFile(AiMMatricesFile,cellDynamicText);
%
% delete files created by dynare and keep only those linked to YADA
%
if IsNewerDynare==1;
   CheckSumFile = FixFilePath([OutputDirectory '\checksum']);
   if FileExist(CheckSumFile)==1;
      delete(CheckSumFile);
   end;
   if FileExist(DynareOutputMatFile)==1;
      delete(DynareOutputMatFile);
      stat = rmdir(GetPath(DynareOutputMatFile));
   end;
   %
   % delete all files in + directory
   %
   if FileExist(DynamicFile)==1;
      delete([OutDirPlus '*.m']);
      stat = rmdir(OutDirPlus);
   end;
else;
   %
   % clean up the older stuff
   %
   if FileExist(DynareOutputMatFile)==1;
      delete(DynareOutputMatFile);
   end;
   if DirExist(FixFilePath([OutputDirectory '\Output\']))==1;
      stat = rmdir(FixFilePath([OutputDirectory '\Output\']));
   end;
   if FileExist(DynamicFile)==1;
      delete([ModelDirectory NameOfModel '_*.m']);
   end;
   if FileExist(mfile)==1;
      delete(mfile);
   end;
end;
%
% reset the path
%
path(OrigPath);
if (FileExist(AiMFile)==1)&(FileExist(AiMDataFile)==1)&(FileExist(AiMMatricesFile)==1);
   status = 1;
end;

%
% end of DynareParser.m
%
