function StructureForData = DataConstFile
% DataConstFile: Shows how the data construction output should be provided for YADA. This example is based
%                on the An-Schorfheide model, but uses actual US data as described in Herbst and Schorfheide
%
% USAGE:
%
%       StructureForData = DataConstFile
%
% where StructureForData is a proper structure in matlab. The file is run in matlab via the eval
% function.
%
% INPUTS:          None!
%
% REQUIRED OUTPUT: StructureForData (structure). Any name for this structure will do since YADA renames it internally.
%                  REQUIRED FIELDS:  Y.data              (nxT) data-matrix of the n observable variables that should be evaluated by the
%                                                        Kalman filter
%                                    X.data              (kxT) data-matrix with exogenous data, may be an empty matrix, but must be stated
%                                    Y.names             (n dimensional cellstring) with variable names
%                                    X.names             (k dimensional cellstring) with names for exogenous variables (may be empty)
%                                    sample              (1x4 matrix) containing [start-year start-period end-yer end-period]. Must match
%                                                        the number of observations T.
%                                    frequency           (string) data frequency. Valid entries are: a(nnual), q(uarterly), m(onthly)
%                                                        if invalid frequency is reported, it is set to annual with sample being set
%                                                        to [1 1 T 1]
%                  OPTIONAL FIELDS:  Lambda              (vector) with lambda values to use for the DSGE-VAR models.
%                                    Y.levels            (1xn) vector, where 0 indicates a first differenced variable and 1 a levels
%                                                        variable
%                                    Y.annual            (1xn) vector, where 1 indicates that the variable should be annualized based
%                                                        on the data frequency, while 0 means that it should not be annualized (i.e.,
%                                                        can already be regarded as annualized).
%                                    Y.annualscale       (1xn) vector with scaling coefficient to multiply the annualized variables
%                                                        with.
%                                    Y.BVAR              (1xn1) vector where n1<=n that gives the indices for the variables in Y that
%                                                        should be included in a Bayesian VAR.
%                                    X.BVAR              (1xk1) vector where k1<=k that gives the indices for the variables in X that
%                                                        should be included in a Bayesian VAR.
%                                    X.extradata         (kxT1) data matrix with out-of-sample exogenous data.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 18, 2015.
%                        This version: January 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-07-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 15-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 15-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%warning('off');
%Data = xlsread([pwd '\data\SmallScaleNewKeynesianDSGEModel-USData-FRED.xls']);
%
% load the US data from a spreadsheet
%
Data = xlsread(FixFilePath([pwd '\example\HerbstSchorfheide\data\SmallScaleNewKeynesianDSGEModel-USData-FRED.xls']));
if size(Data,2)~=4;
   Data = Data(:,2:5);
end;
%
% construct the time series
%
% GDP per capita is variable 1 divided by variable 2
%
GDPPC = Data(:,1)./Data(:,2);
%
% consumer price index is variable 3
%
CPI = Data(:,3);
%
% federal funds rate is variable 4
%
FFR = Data(:,4);
%
% transform the variables
%
T = length(FFR);
Y = zeros(3,T-1);
Y(3,:) = FFR(2:T);
Y(2,:) = 400*diff(log(CPI));
Y(1,:) = 100*diff(log(GDPPC));
%
StructureForData.Y.data = Y;
StructureForData.Y.names = {'DY' 'PI' 'FFR'};
%
StructureForData.Y.levels = [0 0 1];
StructureForData.Y.annual = [1 1 0];
StructureForData.Y.annualscale = [1 0.25 1];
StructureForData.X.data = ones(1,T-1);
StructureForData.X.extradata = ones(1,120);
StructureForData.X.names = {'const'};
StructureForData.Y.BVAR = [1 2 3];
StructureForData.Y.ZLBdata = 0;
StructureForData.Y.policyrate = 3;
StructureForData.X.BVAR = [1];
StructureForData.sample = [1970 2 2014 4];
StructureForData.frequency = 'q';
StructureForData.percentiles = [2.5 5 15 25 75 85 95 97.5];
StructureForData.Lambda = [0.25 0.5 0.75 1 1.25 1.5 1.75 2 2.5 3 3.5 4 4.5 5 6 7 8 9 10 25 Inf];

%
% end of DataConstFile.m
%
