function [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = KalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c)
% KalmanFilterMOHt: Calculates the value of the log-likelihood function (based on Gaussian errors and initial entry
%                   for the state vector) for the observed data using the state-space representation. The function
%                   allows for missing observations and a time-varying matrix on the state variables in the
%                   measurement equation.
%
% USAGE:
%
%       [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = KalmanFilterMOHt(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,...
%                                                                  StartPeriod,AllowUnitRoot,c)
%
% REQUIRED INPUT:  Y (matrix) of dimension nxT with observed data.
%
%                  X (matrix) of dimesnion with exogenous of predetermined data.
%
%                  A (matrix) of dimension kxn with parameter values on the k X variables in the measurement equation.
%
%                  H (matrix) of dimension rxnxT with parameter values on the r state variables (ksi) in the measurement equation.
%
%                  F (matrix) of dimension rxr with parameter values on lagged state variables in the state equation.
%
%                  Q (matrix) of dimension rxr with covariances for the innovations in the state equation.
%
%                  R (matrix) of dimension nxn with covariances for the innovations in the measurement equation.
%
%                  KsiLast (vector) of dimension rx1 with initial values for the state vector.
%
%                  initP (integer) indicating the method for determining initial value for P(1|0), the MSE matrix for
%                        ksi(1|0). If initP=1 (default), the P(1|0) is given by the unconditional covariance matrix of ksi,
%                        calculated through the vec operator. Since this operation can be excessively slow for large systems
%                        (when r^2 is a large number) the Doubling Algorthm can be chosen to approximate the unconditional
%                        variance. This algorithm is called if initP=2. Finally, if initP=3, then P(1|0) = c*eye(r*r).
%                  NOTE: If F has eigenvalues outside the unit circle, then initP is always set to 3!
%
%                  MaxIter (integer), the maximum number of iterations to use with the doubling algorithm
%                        divided by 100. Must be multiplied by 100 when calling DoublingAlgorithmLyapunov.
%
%                  Tolerance (integer), determines the tolerance value for the doubling algorithm.
%
%                  StartPeriod, positive integer indicating first observation to use when calculating the log-likelihood value. The
%                        default value is 1.
%
%                  AllowUnitRoot (boolean) which is 1 if unit roots should be allowed for 0 otherwise.
%
%                  c (scalar) indicating the variance of P(1|0) when initP=3. The default value is 1000.
%
% REQUIRED OUTPUT: lnL (scalar) value of the log-likelihood function.
%
%                  status (boolean) indicating the covariance stationarity of the state variables. If all the
%                        eigenvalues of F are inside the unit circle, then status=0 and the log-likelihood function
%                        is calculated. If not, then status=1 and initP is set to 3. In addition, status can be
%                        -1 if ths forecast covariance matrix is not positive definite for some time period.
%
% OPTIONAL OUTPUT: lnLt (vector) of dimension 1x(T-StartPeriod+1) with the time t values of the log-densities that sum to the value lnL.
%
%                  Yhat (matrix) of dimension nx(T-StartPeriod+1) with the predictions [y(StartPeriod|StartPeriod-1) ... y(T|T-1).
%
%                  MSEY (matrix) of dimension nxnx(T-StartPeriod+1) with MSEY(:,:,t-StartPeriod+1) = H'*P(t|t-1)*H + R, the mean
%                        squared error matrix for y(t|t-1).
%
%                  Ksitt1 (matrix) of dimension rx(T-StartPeriod+1) with the predictions [ksi(StartPeriod|StartPeriod-1) ... ksi(T|T-1)].
%
%                  Ptt1 (matrix) of dimension rxrx(T-StartPeriod+1) with Ptt1(:,:,t-StartPeriod+1) = P(t|t-1)
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 14, 2011.
%                        This version: January 16, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-03-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 03-01-2013: Updated the documentation.
%
% * 25-06-2013: The predicted values of the observables now also cover variables that are missing.
%
% * 27-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 13-01-2015: Updated the documentation.
%
% * 02-09-2015: The recursive estimation breaks if the log-likelihood is a NaN.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% initializing
%
lnL = 0;
status = 0;
%
% check the covariance stationarity of the state vector.
%
if max(abs(eig(F)))>=1;
   if AllowUnitRoot==0;
      status = 1;
   elseif (AllowUnitRoot==1)&(max(abs(eig(F)))>1)
      status = 1;
   end;
   initP = 3;
end;
%
% Determine dimensions. Checking if dimensions match is not undertaken!
%
[n,T] = size(Y);
if isempty(X)==0;
   Ax = A'*X;
else;
   Ax = zeros(n,T);
end;
r = size(H,1);
In = eye(n);
%
% Compute initial values
%
if initP==1;
   PLast = reshape((eye(r*r)-kron(F,F))\vec(Q),r,r);
elseif initP==2;
   %
   % The maximum number of iterations is set to MaxIter*100, while the convergence value is 1.0e-8
   %
   [PLast,stat] = DoublingAlgorithmLyapunov(F,Q,MaxIter*100,10^(-(Tolerance+1)));
else;
   if c<=0;
      c = 1000;
   end;
   PLast = c*eye(r);
end;
%
% Initialize temporary matrices
%
if nargout>2;
   YhatTmp = NaN*ones(n,T);
   MSEYTmp = NaN*ones(n,n,T);
   Ksitt1Tmp = zeros(r,T);
   Ksitt1Tmp(:,1) = KsiLast;
   Ptt1Tmp = zeros(r,r,T);
end;
lnLtTmp = zeros(1,T);
%
% Now we can compute the Kalman filtered series
%
ln2pi = -log(2*pi);
if rank(Q)+rank(R)>=n;
   if nargout==2;
      for t=1:T;
         pos = (~isnan(Y(:,t)));
         n1 = sum(pos);
         if n1>0;
            HT = H(:,pos,t)';
            PLastH = PLast*H(:,pos,t);
            InvMSEY = In(pos,pos)/((HT*PLastH)+R(pos,pos));
            yPred = Ax(pos,t)+(HT*KsiLast);
            yError = Y(pos,t)-yPred;
            %
            % calculate the Kalman gain matrix
            %
            KalmanGain = F*PLastH*InvMSEY;
            lnLtTmp(1,t) = (n1*ln2pi)+LogDet(InvMSEY)-(yError'*InvMSEY*yError);
            if isnan(lnLtTmp(1,t))==1;
               break;
            end;
            %
            % now we update KsiLast and PLast
            %
            KsiLast = (F*KsiLast)+(KalmanGain*yError);
            FDev = F-(KalmanGain*HT);
            PLast = (FDev*PLast*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+Q;
         else;
            KsiLast = F*KsiLast;
            PLast = (F*PLast*F')+Q;
         end;
      end;
   else;
      for t=1:T;
         pos = (~isnan(Y(:,t)));
         n1 = sum(pos);
         if n1>0;
            HT = H(:,pos,t)';
            PLastH = PLast*H(:,pos,t);
            InvMSEY = In(pos,pos)/((HT*PLastH)+R(pos,pos));
            yPred = Ax(pos,t)+(HT*KsiLast);
            yError = Y(pos,t)-yPred;
            %
            % calculate the Kalman gain matrix
            %
            KalmanGain = F*PLastH*InvMSEY;
            lnLtTmp(1,t) = (n1*ln2pi)+LogDet(InvMSEY)-(yError'*InvMSEY*yError);
            if isnan(lnLtTmp(1,t))==1;
               break;
            end;
            %
            % store values if asked to do so
            %
            Ht = H(:,:,t);
            YhatTmp(:,t) = Ax(:,t)+(Ht'*KsiLast);
            MSEYTmp(:,:,t) = (Ht'*PLast*Ht)+R;
            Ksitt1Tmp(:,t) = KsiLast;
            Ptt1Tmp(:,:,t) = PLast;
            %
            % now we update KsiLast and PLast
            %
            KsiLast = (F*KsiLast)+(KalmanGain*yError);
            FDev = F-(KalmanGain*HT);
            PLast = (FDev*PLast*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+Q;
         else;
            Ksitt1Tmp(:,t) = KsiLast;
            Ptt1Tmp(:,:,t) = PLast;
            Ht = H(:,:,t);
            YhatTmp(:,t) = Ax(:,t)+(Ht'*KsiLast);
            MSEYTmp(:,:,t) = (Ht'*PLast*Ht)+R;
            KsiLast = F*KsiLast;
            PLast = (F*PLast*F')+Q;
         end;
      end;
   end;
else;
   if nargout==2;
      for t=1:T;
         pos = (~isnan(Y(:,t)));
         n1 = sum(pos);
         if n1>0;
            HT = H(:,pos,t)';
            PLastH = PLast*H(:,pos,t);
            InvMSEY = pinv((HT*PLastH)+R(pos,pos));
            yPred = Ax(pos,t)+(HT*KsiLast);
            yError = Y(pos,t)-yPred;
            %
            % calculate the Kalman gain matrix
            %
            KalmanGain = F*PLastH*InvMSEY;
            lnLtTmp(1,t) = (n1*ln2pi)+LogDet(InvMSEY)-(yError'*InvMSEY*yError);
            if isnan(lnLtTmp(1,t))==1;
               break;
            end;
            %
            % now we update KsiLast and PLast
            %
            KsiLast = (F*KsiLast)+(KalmanGain*yError);
            FDev = F-(KalmanGain*HT);
            PLast = (FDev*PLast*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+Q;
         else;
            KsiLast = F*KsiLast;
            PLast = (F*PLast*F')+Q;
         end;
      end;
   else;
      for t=1:T;
         pos = (~isnan(Y(:,t)));
         n1 = sum(pos);
         if n1>0;
            HT = H(:,pos,t)';
            PLastH = PLast*H(:,pos,t);
            InvMSEY = pinv((HT*PLastH)+R(pos,pos));
            yPred = Ax(pos,t)+(HT*KsiLast);
            yError = Y(pos,t)-yPred;
            %
            % calculate the Kalman gain matrix
            %
            KalmanGain = F*PLastH*InvMSEY;
            lnLtTmp(1,t) = (n1*ln2pi)+LogDet(InvMSEY)-(yError'*InvMSEY*yError);
            if isnan(lnLtTmp(1,t))==1;
               break;
            end;
            %
            % store values if asked to do so
            %
            Ht = H(:,:,t);
            YhatTmp(:,t) = Ax(:,t)+(Ht'*KsiLast);
            MSEYTmp(:,:,t) = (Ht'*PLast*Ht)+R;
            Ksitt1Tmp(:,t) = KsiLast;
            Ptt1Tmp(:,:,t) = PLast;
            %
            % now we update KsiLast and PLast
            %
            KsiLast = (F*KsiLast)+(KalmanGain*yError);
            FDev = F-(KalmanGain*HT);
            PLast = (FDev*PLast*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+Q;
         else;
            Ksitt1Tmp(:,t) = KsiLast;
            Ptt1Tmp(:,:,t) = PLast;
            Ht = H(:,:,t);
            YhatTmp(:,t) = Ax(:,t)+(Ht'*KsiLast);
            MSEYTmp(:,:,t) = (Ht'*PLast*Ht)+R;
            KsiLast = F*KsiLast;
            PLast = (F*PLast*F')+Q;
         end;
      end;
   end;
end;
lnL = (1/2)*sum(lnLtTmp(1,StartPeriod:T));
%
% test if the log-likelihood is real
%
if (isreal(lnL)==0)|(isnan(lnL)==1);
   status = -1;
end;
%
% Fix output to match StartPeriod
%
if nargout==3;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
elseif nargout==4;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
elseif nargout==5;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
elseif nargout==6;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
elseif nargout==7;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
end;

%
% end of KalmanFilterMOHt.m
%
