function [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,NumLead,StateVariablePositions,StateEquationPositions,StateShockPositions,AIMTolerance,OrderQZ)
% SimsSolver: Solves a linear rational expectations model with Chris Sims' gensys algorithm.
%
% USAGE:
%
%       [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,NumLead,StateVariablePositions,...
%                                             StateEquationPositions,StateShockPositions,AIMTolerance,OrderQZ)
%
% REQUIRED INPUT:  ModelParameters (structure) contains numeric values for the model parameters.
%
%                  NumLead (integer) with the number of leads.
%
%                  StateVariablePositions (vector) with N<=n containing the positions of the actual state
%                                         variables. AiM will typically stack the state variables and the
%                                         state shocks given that the shocks have been formulated as
%                                         equations!
%
%                  StateEquationPositions (vector) of dimension N with the positions of the state equations.
%
%                  StateShockPositions (vector) with M<=n containing the positions of the state shocks.
%
%                  AIMTolerance (positive integer) that determines the tolerance for the DSGE model solver.
%
%                  OrderQZ (boolean) which takes the value 1 if "ordqz" is a built-in matlab function
%                                         and 0 otherwise.
%
% REQUIRED OUTPUT: F (matrix) of dimension kxk with the parameters on the lags of the state-equation in
%                                         the state-space representation.
%
%                  B0 (matrix) of dimension kxq with the parameters on the shocks in the state equation.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not. mcode is
%                                         1 when unique, 0 if not computed, and various other values when
%                                         there are problems with the solution. Specifically:
%
%                                              mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                              numbers.
%                                              mcode = 7:      The function "compute_aim_matrices" returns infinite
%                                                              or NaN values.
%                                              mcode = -1:     No stable solution.
%                                              mcode = -2:     Too many large generalized eigenvalues.
%                                              mcode = -3:     Too few large generalized eigenvalues.
%
% OPTIONAL OUTPUT: MaxAbsError (non-negative real). The maximum absolute error when computing the solution.
%                                         If this variable is NaN, it has not been calculated.
%
%                  ModelEigenvalues (structure) with fields "Roots", "LargeRoots", "DimCompanion", "Amplitude",
%                                         and "Period".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 15, 2010.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-12-2010: Updated the documentation.
%
% * 17-12-2010: Fixed a bug when extracting data from "gev".
%
% * 24-12-2010: Fixed issues with the generalized eigenvalues.
%
% * 03-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 05-06-2012: Added the "StateEquationPositions" input vector.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-02-2014: Fixed a small bug where ModelEigenvalues wasn't calculated for mcode=-1.
%
% * 13-01-2015: Updated the documentation.
%
% * 19-06-2015: Made sure that the function doesn't try to takes elements from f after running
%               YADAgensys if its first dimension is smaller than the first dimension of F (k).
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 12-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 19-11-2020: Added the mcode value of 7 for Inf or NaN values.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
mcode = 0;
F = [];
B0 = [];
condn = 10^(-AIMTolerance-2);
%
% Construct structural coefficient matrices.
%
[GMatrix,HMatrix] = compute_aim_matrices(ModelParameters);
if isreal(GMatrix)==0;
   mcode = 8;
   if nargout==5;
      MaxAbsError = NaN;
      ModelEigenvalues = [];
   end;
   return;
end;
if isreal(HMatrix)==0;
   mcode = 8;
   if nargout==5;
      MaxAbsError = NaN;
      ModelEigenvalues = [];
   end;
   return;
end;
if (max(max(isnan(GMatrix)))==1)|(max(max(isinf(GMatrix))));
   mcode = 7;
   if nargout==5;
      MaxAbsError = NaN;
      ModelEigenvalues = [];
   end;
   return;
end;
if (max(max(isnan(HMatrix)))==1)|(max(max(isinf(HMatrix))));
   mcode = 7;
   if nargout==5;
      MaxAbsError = NaN;
      ModelEigenvalues = [];
   end;
   return;
end;
%
[rh,ch] = size(HMatrix);
[rg,cg] = size(GMatrix);
H = zeros(rh,ch);
H(1:rg,1:cg) = GMatrix;
H = H+HMatrix;
Hlag = H(:,1:rh);
Hexp = H(:,(2*rh)+1:(2+NumLead)*rh);
H = H(:,rh+1:2*rh);
StateVarPos = StateVariablePositions;
if NumLead>1;
   for j=2:NumLead;
      StateVarPos = [StateVarPos StateVariablePositions+(rh*(j-1))];
   end;
end;
Hlag = Hlag(StateEquationPositions,StateVariablePositions);
Hexp = Hexp(StateEquationPositions,StateVarPos);
D = -H(StateEquationPositions,StateShockPositions);
H = H(StateEquationPositions,StateVariablePositions);
%disp(Hexp([2 3 4 10 13 16 17 18],:))
[k,l] = size(Hexp);
q = size(D,2);
G0 = [H Hexp;eye(l) zeros(l,k)];
G1 = [-Hlag zeros(k,l);zeros(l,k) eye(l)];
Psi = [D;zeros(l,q)];
Pi = [zeros(k,l);eye(l)];
%
% run the YADA version of gensys
%
[f,b0,gev,eu] = YADAgensys(G0,G1,Psi,Pi,OrderQZ,1+condn);
if (eu(1)==1)&(eu(2)==1);
   mcode = 1;
elseif (eu(1)==1)&(eu(2)~=1);
   %
   % we have existence but not uniqueness
   %
   if abs(gev(k,2))>abs(gev(k,1))+condn;
      %
      % too many large eigenvalues
      %
      mcode = -2;
   elseif abs(gev(k+1,2))<abs(gev(k+1,1));
      %
      % too few large eigenvalue
      %
      mcode = -3;
   end;
   if size(f,1)<k;
      return;
   end;
else;
   mcode = -1;
   if nargout==5;
      MaxAbsError = NaN;
      ModelEigenvalues = [];
      if size(f,1)<k;
         return;
      end;
   else;
      return;
   end;
end;
F = f(1:k,1:k);
B0 = b0(1:k,1:q);
%
% check if we should compute additional data about the solution
%
if nargout==5;
   S0 = H;
   Hcheck = Hlag+(H*F);
   Fmat = F;
   for j=1:NumLead;
      ind = (1:k)+(k*(j-1));
      S0 = S0+(Hexp(:,ind)*Fmat);
      Fmat = Fmat*F;
      Hcheck = Hcheck+(Hexp(:,ind)*Fmat);
   end;
   Bcheck = B0-(S0\D);
   MaxAbsError = max(max(abs([Hcheck Bcheck])));
   ModelEigenvalues = [];
   s = gev(:,1);
   t = gev(:,2);
   %
   % remove infinite generalized eigenvalues
   %
   pos = (abs(s)>condn).*(1:k+l)';
   pos = pos(pos>0);
   s = s(pos);
   t = t(pos);
   GEValue = t./s;
   [MVals,Ind] = sort(abs(GEValue));
   ModelEigenvalues.Roots = GEValue(flipud(Ind));
   ModelEigenvalues.LargeRoots = sum(abs(ModelEigenvalues.Roots)>1+condn);
   ModelEigenvalues.DimCompanion = l+k;
   %
   % calculate the amplitude and the periodicity
   %
   lambda = ModelEigenvalues.Roots;
   amp = abs(lambda);
   keep = find(lambda);
   lambda = lambda(keep);
   amp = amp(keep);
   %
   n = length(keep);
   angel = zeros(n,1);
   period = zeros(n,1)+NaN;
   %
   i = find(abs(lambda)>condn);
   %
   for j=i
      angel(j) = abs(angle(lambda(j)));
   end
   %
   k = find(angel>condn);
   if ~isempty(k);
      period(k) = (2*pi)./angel(k);
   end;
   ModelEigenvalues.Amplitude = amp;
   ModelEigenvalues.Period = period;
end;

%
% end of SimsSolver.m
%
