function [F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,AIMTolerance,OrderQZ)
% SolveExtendedDSGEModel: Solves the DSGE model extended for the ZLB case with separate equations for
%                         the restricted and unrestricted policy rate
%
% USAGE:
%
%       [F,B0,BA] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,AIMTolerance,OrderQZ)
%
% REQUIRED INPUT:  Hlag (matrix) of dimension (r+1) x (r+1) with the coefficient values on the lagged state variables.
%
%                  H0 (matrix) of dimension (r+1) x r+1 with the coefficient values on the contemporaneous state variables.
%
%                  Hexp (matrix) of dimension (r+1) x (r+1) with the coefficient values on the expectated future values
%                                  of the state variables.
%
%                  D (matrix) of dimension (r+1) x q with the coefficient values on the structural shocks.
%
%                  DA (matrix) of dimension (r+1) x T_ZLB with the coefficients on the anticipated structural shocks.
%
%                  AIMTolerance (positive integer) that determines the tolerance for the DSGE model solver.
%
%                  OrderQZ (boolean) which takes the value 1 if "ordqz" is a built-in matlab function
%                          and 0 otherwise.
%
% REQUIRED OUTPUT: F (matrix) of dimension (r+1) x (r+1) with the coefficients on the lagged model variables
%
%                  B0 (matrix) of diemsnion (r+1) x q with the coefficients on the structural shocks.
%
%                  BA (matrix) of diemsnion (r+1) x T_ZLB with coefficients on the anticipated shocks.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not. mcode is
%                                         1 when unique, 0 if not computed, and various other values when
%                                         there are problems with the solution. Specifically:
%
%                                              mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                              numbers.
%                                              mcode = 7:      The function "compute_aim_matrices" returns infinite
%                                                              or NaN values.
%                                              mcode = -1:     No stable solution.
%                                              mcode = -2:     Too many large generalized eigenvalues.
%                                              mcode = -3:     Too few large generalized eigenvalues.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 28, 2024.
%                        This version: March 4, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-03-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

mcode = 0;
F = [];
B0 = [];
BA = [];
condn = 10^(-AIMTolerance-2);
%
[k,l] = size(Hexp);
%
% transform the system to Klein's A and B matrices
%
A = [zeros(k,k) -Hexp;eye(l,l) zeros(l,k)];
if l==k;
   B = [Hlag H0;zeros(k,k) eye(k)];
else;
   B = [Hlag H0 zeros(k,l-k);zeros(l,k) eye(l,l)];
end;
%
% compute the QZ decomposition
%
[S,T,Q,Z] = qz(A,B);
if OrderQZ==1;
   [S,T,Q,Z] = ordqz(S,T,Q,Z,'udo');
else;
   [S,T,Q,Z] = YADAqzdiv(1+condn,S,T,Q,Z);
end;
Z11 = Z(1:k,1:k);
if rank(Z11)<k;
   mcode = -1;
   return;
end;
%
% check for other errors
%
mcode = 1;
if abs(T(k,k))>abs(S(k,k))+condn;
   %
   % too many large eigenvalues
   %
   mcode = -2;
elseif abs(T(k+1,k+1))<abs(S(k+1,k+1));
   %
   % too few large eigenvalue
   %
   mcode = -3;
end;
%
% compute the transition matrix
%
S11 = S(1:k,1:k);
T11 = T(1:k,1:k);
F = real((Z11*(S11\T11))/Z11);
%
% solve for B0
%
Z22 = Z(k+1:k+l,k+1:k+l);
Z12 = Z(1:k,k+1:k+l);
Z21 = Z(k+1:k+l,1:k);
T22 = T(k+1:k+l,k+1:k+l);
Q2 = Q(k+1:k+l,:);
B = real((Z22-(Z21*(Z11\Z12)))*(T22\Q2)*[D;zeros(l,size(D,2))]);
B0 = B(1:k,:);
%
% solve for BA
%
B = real((Z22-(Z21*(Z11\Z12)))*(T22\Q2)*[DA;zeros(l,size(DA,2))]);
BA = B(1:k,:);

%
% end of SolveExtendedDSGEModel.m
%
