function CoherenceDistributionDLG(selector,SOmega,Omega,Quantiles,VariableNames,VarStr,EstStr,ModelStr,DSGEModel,CurrINI)
% CoherenceDistributionDLG: Displays coherence results for pairs of observed or state variables for various
%                           frequencies.
%
% USAGE:
%
%       CoherenceDistributionDLG(selector,SOmega,Omega,Quantiles,VariableNames,VarStr,EstStr,ModelStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: selector (string) taking on the values "init", "checkvars", "lowerperiod", "upperperiod",
%                          "showgraph", and "done".
%
% OPTIONAL INPUT: SOmega (matrix) of dimension m(m-1)/2 x f with the m(m-1)/2 unique coherence statistics for
%                          f frequencies from 0 to pi of the m variables.
%
%                 Omega (vector) of dimension f with the frequencies from 0 to pi.
%
%                 Quantiles (vector structure) with fields "SOmega" and "percent". Can be empty.
%
%                 VariableNames (string matrix) with the names of the variables.
%
%                 VarStr (string) that takes the values "Observed" or "State".
%
%                 EstStr (string) that takes the values "Initial Values", "Prior Distribution", "Posterior Mode",
%                          or "Posterior Distribution".
%
%                 ModelStr (string) that takes the values "" or "DSGE-VAR(lambda) ".
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure) with initialization information.
%
% NOTE: All optional variables are required when selector is "init".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 12, 2010.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-05-2010: Updated the documentation.
%
% * 15-05-2010: Changed the line ordering so that the dashed vertical lines appear
%               above the quantile data.
%
% * 16-05-2010: Updated the documentation.
%
% * 21-06-2010: Added the "ModelStr" input variable.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(lower(selector),'init')==1;
   DSGEModel.SOmega = SOmega;
   DSGEModel.Omega = Omega;
   DSGEModel.Quantiles = Quantiles;
   DSGEModel.VarNames = VariableNames;
   DSGEModel.EstStr = EstStr;
   DSGEModel.ModelStr = ModelStr;
   %
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      PeriodStrMat = '1 quarter';
      for j=2:80;
         PeriodStrMat = strvcat(PeriodStrMat,[int2str(j) ' quarters']);
      end;
      LowerValue = 4;
      UpperValue = 32;
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      PeriodStrMat = '1 month';
      for j=2:240;
         PeriodStrMat = strvcat(PeriodStrMat,[int2str(j) ' months']);
      end;
      LowerValue = 12;
      UpperValue = 96;
   else;
      PeriodStrMat = '1 year';
      for j=2:20;
         PeriodStrMat = strvcat(PeriodStrMat,[int2str(j) ' years']);
      end;
      LowerValue = 1;
      UpperValue = 8;
   end;
   n = size(VariableNames,1);
   DSGEModel.PosMatrix = zeros(n,n);
   num = 0;
   for i=1:n-1;
      for j=i+1:n;
         num = num+1;
         DSGEModel.PosMatrix(i,j) = num;
         DSGEModel.PosMatrix(j,i) = num;
      end;
   end;
else;
   CoherenceDistributionGUI = findobj('Type','figure','Tag','CoherenceDistributionDLG');
   controls = get(CoherenceDistributionGUI,'UserData');
   CurrINI = get(controls.firstvariable,'UserData');
   DSGEModel = get(controls.firstvariabletext,'UserData');
end;
%
switch selector;
   case 'init';
      CoherenceDistributionGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','CoherenceDistributionDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-525)/2 (CurrINI.scrsz(4)-130)/2 525 130], ...
         'Tag','CoherenceDistributionDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',[ModelStr 'Coherence - ' EstStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','CoherenceDistributionDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(CoherenceDistributionGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 523 120],'Coherence Results',45,[0.5 1],'on',CurrINI);
      %
      % select business cycle frequencies
      %
      controls.frequencytxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select cyclical period:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');
      %
      controls.lowerperiod = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 80 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',PeriodStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','CoherenceDistributionDLG lowerperiod', ...
         'Tag','LowerPeriodPopup',...
         'Value',LowerValue);
      %
      controls.upperperiod = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[303 80 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',PeriodStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','CoherenceDistributionDLG upperperiod', ...
         'Tag','UpperPeriodPopup',...
         'Value',UpperValue);
      %
      % Select variable 1
      %
      controls.firstvariabletext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select first variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVar1Text');
      %
      controls.firstvariable = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 50 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',VariableNames, ...
         'Callback','CoherenceDistributionDLG checkvars', ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVar1Popup',...
         'Value',1);
      %
      % select variable 2
      %
      controls.secondvariabletext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select second variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVar2Text');
      %
      controls.secondvariable = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 20 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',VariableNames, ...
         'Callback','CoherenceDistributionDLG checkvars', ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVar2Popup',...
         'Value',2);
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','CoherenceDistributionDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','CoherenceDistributionDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(CoherenceDistributionGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'showgraph'
      FirstValue = get(controls.firstvariable,'Value');
      SecondValue = get(controls.secondvariable,'Value');
      VarStrMat = get(controls.firstvariable,'String');
      n = size(VarStrMat,1);
      FirstVariable = StringTrim(VarStrMat(FirstValue,:));
      SecondVariable = StringTrim(VarStrMat(SecondValue,:));
      %
      % update the controls
      %
      if FirstValue<SecondValue;
         if SecondValue<n
            set(controls.secondvariable,'Value',SecondValue+1);
         else;
            if FirstValue<n-1;
               set(controls.firstvariable,'Value',FirstValue+1);
               set(controls.secondvariable,'Value',FirstValue+2);
            else;
               set(controls.firstvariable,'Value',1);
               set(controls.secondvariable,'Value',2);
            end;
         end;
      else;
         if FirstValue<n;
            set(controls.firstvariable,'Value',FirstValue+1);
         else;
            if SecondValue<n-1;
               set(controls.secondvariable,'Value',SecondValue+1);
               set(controls.firstvariable,'Value',SecondValue+2);
            else;
               set(controls.firstvariable,'Value',1);
               set(controls.secondvariable,'Value',2);
            end;
         end;
      end;
      %
      % determine which entry we should take
      %
      pos = DSGEModel.PosMatrix(FirstValue,SecondValue);
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                     'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                     'Tag',[FirstVariable '-' SecondVariable], ...
                     'Name',['Coherence - ' FirstVariable ' and ' SecondVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % mark up the cyclical area
      %
      LowerFrequency = 2*pi/get(controls.upperperiod,'Value');
      UpperFrequency = 2*pi/get(controls.lowerperiod,'Value');
      Omega = DSGEModel.Omega;
      SOmega = DSGEModel.SOmega;
      f = length(Omega);
      BgPatchColor = [0.9 0.9 0.9];
      BgPatchHandle = patch([LowerFrequency UpperFrequency UpperFrequency LowerFrequency],[1.1 1.1 0 0],BgPatchColor);
      set(BgPatchHandle,'FaceColor',BgPatchColor,'EdgeColor',BgPatchColor);
      hold('on');
      set(gca,'Layer','Top');
      %
      % check if we have quantiles
      %
      Handles = [];
      LegendStr = '';
      if isempty(DSGEModel.Quantiles)==0;
         NumBounds = floor(length(DSGEModel.Quantiles)/2);
         for nb=1:NumBounds;
            Lower = DSGEModel.Quantiles(nb).SOmega(pos,:);
            Upper = DSGEModel.Quantiles(length(DSGEModel.Percentiles)-nb+1).SOmega(pos,:);
            PatchColor = ((1+(2*(nb-1)))/length(DSGEModel.Percentiles))*DSGEModel.ConfidenceBandBaseColor;
            ConfBound = DSGEModel.Quantiles(length(DSGEModel.Percentiles)-nb+1).percent-DSGEModel.Quantiles(nb).percent;
            ConfPatch = patch([Omega fliplr(Omega)],[Upper fliplr(Lower)],PatchColor);
            set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',PatchColor);
            hold('on');
            Handles = [Handles ConfPatch];
            LegendStr = strvcat(LegendStr,[num2str(ConfBound,'%0.0f') ' confidence']);
         end;
      end;
      %
      % draw dashed edges for the cyclical area
      %
      plot(LowerFrequency*ones(1,2),[0 1.1],':','Color',[0 0 0],'LineWidth',0.5);
      hold('on');
      plot(UpperFrequency*ones(1,2),[0 1.1],':','Color',[0 0 0],'LineWidth',0.5);
      hold('on');
      %
      % draw a thick line at the upper y-axis limit
      %
      plot([Omega(1) Omega(f)],[1 1],'-','Color',[0 0 0],'LineWidth',2);
      hold('on');
      %
      % plot the coherence
      %
      LineHandle = plot(Omega,SOmega(pos,:),'-','Color',[0.75 0 0],'LineWidth',2);
      hold('on');
      %
      set(gca,'XLim',[Omega(1) Omega(f)],'YLim',[0 1.1]);
      %
      ModelStr = '';
      if isempty(DSGEModel.ModelStr)==0;
         ModelStr = [' for a ' StringTrim(DSGEModel.ModelStr) ' model'];
      end;
      if (strcmp(DSGEModel.EstStr,'Prior Distribution')==0)&(strcmp(DSGEModel.EstStr,'Posterior Distribution')==0);
         SetAxesFonts(CurrINI,'Title',['Coherence of ' FirstVariable ' and ' SecondVariable ' at ' lower(DSGEModel.EstStr) ModelStr],'XLabel','Frequency in radians (\omega)','YLabel','Coherence');
      elseif strcmp(DSGEModel.EstStr,'Prior Distribution')==1;
         SetAxesFonts(CurrINI,'Title',['Coherence of ' FirstVariable ' and ' SecondVariable ' at prior mean' ModelStr],'XLabel','Frequency in radians (\omega)','YLabel','Coherence');
      elseif strcmp(DSGEModel.EstStr,'Posterior Distribution')==1;
         SetAxesFonts(CurrINI,'Title',['Coherence of ' FirstVariable ' and ' SecondVariable ' at posterior mean' ModelStr],'XLabel','Frequency in radians (\omega)','YLabel','Coherence');
      end;
      %
      if isempty(Handles)==0;
         LegHandle = legend([Handles LineHandle],strvcat(LegendStr,[FirstVariable ',' SecondVariable]));
         SetAxesFonts(CurrINI,LegHandle);
      end;
      %
      SetXTickIntegers(gca);
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'checkvars'
      FirstVar = get(controls.firstvariable,'Value');
      SecondVar = get(controls.secondvariable,'Value');
      n = size(DSGEModel.VarNames,1);
      if FirstVar==SecondVar;
         if gcbo==controls.firstvariable;
            if SecondVar<n;
               set(controls.secondvariable,'Value',SecondVar+1);
            else;
               set(controls.secondvariable,'Value',1);
            end;
         else;
            if FirstVar<n;
               set(controls.firstvariable,'Value',FirstVar+1);
            else;
               set(controls.firstvariable,'Value',1);
            end;
         end;
      end;
   case 'lowerperiod'
      lowervalue = get(controls.lowerperiod,'Value');
      uppervalue = get(controls.upperperiod,'Value');
      h = size(get(controls.upperperiod,'String'),1);
      if lowervalue>=uppervalue;
         if lowervalue<h
            set(controls.upperperiod,'Value',lowervalue+1);
         else;
            set(controls.lowerperiod,'Value',h-1);
            set(controls.upperperiod,'Value',h);
         end;
      end;
   case 'upperperiod'
      lowervalue = get(controls.lowerperiod,'Value');
      uppervalue = get(controls.upperperiod,'Value');
      if lowervalue>=uppervalue;
         if uppervalue>1
            set(controls.lowerperiod,'Value',uppervalue-1);
         else;
            set(controls.lowerperiod,'Value',1);
            set(controls.upperperiod,'Value',2);
         end;
      end;
   case 'done'
      set(controls.done,'UserData','done');
      delete(CoherenceDistributionGUI);
      drawnow;
      pause(0.02);
end;
if strcmp(selector,'init')==1;
   set(CoherenceDistributionGUI,'Visible','on');
   drawnow;
   SetYADAIcon(CoherenceDistributionGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of CoherenceDistributionDLG.m
%
