function GraphActualsDataDLG(selector,CurrINI,DSGEModel)
% GraphActualsDataDLG: Displays graphs of the observed data as well as all alternative actuals data.
%
% USAGE:
%
%       GraphActualsDataDLG(selector,CurrINI,DSGEModel)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: May 10, 2023.
%                        This version: June 22, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 11-05-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==0;
   GraphActualsDataGUI = findobj('Type','figure','Tag','GraphActualsDataDLG');
   controls = get(GraphActualsDataGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
switch selector;
   case 'init';
      GraphActualsDataGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','GraphActualsDataDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-130)/2 500 130], ...
         'Tag','GraphActualsDataDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Graph Actuals Data', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','GraphActualsDataDLG done', ...
         'ToolBar','none');

      if MatlabNumber>=7.0;
         set(GraphActualsDataGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 120],'Graphics - Actuals',45,[0.5 1],'on',CurrINI);
      %
      [FirstPeriod,LastPeriod,SampleStrMat] = CreateSubSample(DSGEModel);
      %
      % determine the prediction sample
      %
      [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
      MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
      MaxFH = min(MaxFH,MaxPH);
      %
      % determine the prediction sample
      %
      [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxFH);
      %
      controls.sample = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',PredDateVector, ...
         'Style','text', ...
         'String',['Prediction Sample: ' PredStartYear ':' PredStartPeriod ' - ' PredEndYear ':' PredEndPeriod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      controls.samplebegtext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Start period for variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','StartDataText');
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 55 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Tag','StartDataPopup',...
         'Value',LastPeriod-3);
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 160 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 20 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVarPopup',...
         'Value',1);
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','GraphActualsDataDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','GraphActualsDataDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(GraphActualsDataGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'showgraph'
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.VariableNames(VariableValue,:));
      if VariableValue<size(DSGEModel.VariableNames,1);
         set(controls.variables,'Value',VariableValue+1);
      else;
         set(controls.variables,'Value',1);
      end;
      %
      PredDateVector = get(controls.sample,'UserData');
      %
      % create a vector with dates
      %
      DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
      %
      % determine the sub-sample from the full sample
      %
      [FirstPeriod,LastPeriod,SampleStrMat] = CreateSubSample(DSGEModel);
      %
      StartSample = get(controls.samplebeg,'Value');
      h = length(PredDateVector);
      T = DSGEModel.T;
      if T>=LastPeriod+h;
         y = DSGEModel.Y(VariableValue,FirstPeriod:LastPeriod+h);
      else;
         y = [DSGEModel.Y(VariableValue,FirstPeriod:T) NaN*ones(1,LastPeriod+h-T)];
      end;
      %
      VarNameStr = CurrVariable;
      if isempty(DSGEModel.Actuals)==0;
         m = length(DSGEModel.Actuals);
         for i=1:m;
            yi = DSGEModel.Actuals(i).data(VariableValue,:);
            Ti = length(yi);
            if Ti>=LastPeriod+h;
               yi = yi(FirstPeriod:LastPeriod+h);
            else;
               yi = [yi(FirstPeriod:Ti) NaN*ones(1,LastPeriod+h-Ti)];
            end;
            y = [y;yi];
            VarNameStr = strvcat(VarNameStr,DSGEModel.Actuals(i).title);
         end;
      end;
      DateVector = [DateVector PredDateVector];
      [m1,Th] = size(y);
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                         'Tag',CurrVariable, ...
                         'Name',['Actuals - ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      LineHandles = plot(DateVector,y(1,:),'x-','Color',[0 0 0],'LineWidth',2);
      hold('on');
      %
      if m1>1;
         Colors = CreateDefaultColors(m1-1);
         for i=2:m1;
            LineHandle = plot(DateVector,y(i,:),'-','Color',Colors(i-1,:),'LineWidth',2);
            LineHandles = [LineHandles LineHandle];
         end;
      end;
      %
      set(gca,'XLim',[DateVector(StartSample) DateVector(Th)]);
      SetAxesFonts(CurrINI,'Title',CurrVariable);
      ylim = get(gca,'YLim');
      add_on = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-add_on ylim(2)+add_on];
      set(gca,'YLim',ylim);
      if (ylim(1)<0)&(ylim(2)>0);
         plot([floor(DateVector(1)) ceil(DateVector(Th))],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         set(gca,'Children',flipud(get(gca,'Children')));
      end;
      %
      % add a vertical line from the forecast point
      %
      fp = plot((((3*DateVector(LastPeriod))-DateVector(LastPeriod-1))/2)*[1 1],ylim,':','Color',[0 0 0]);
      %
      if m1>1;
         if MatlabNumber>=7;
            legHandle = legend(LineHandles,VarNameStr,'Location','NorthWest');
         else;
            legHandle = legend(LineHandles,VarNameStr,2);
         end;
      end;
      %
      SetXTickIntegers(gca);
      %
      % display the graph
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(controls.done,'UserData','done');
      delete(GraphActualsDataGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(GraphActualsDataGUI,'Visible','on');
   drawnow;
   SetYADAIcon(GraphActualsDataGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;


%
% end of GraphActualsDataDLG.m
%
