function GraphBivariatePosteriorDensity(selector,thetaPostSample,thetaMode,ModeData,ParameterNames,NumDraws,NumBurnin,CurrChain,UpdatePostSample,UpdateMode,UpdateNames,UpdateParamPartialPhi,DSGEModel,CurrINI,controls)
% GraphBivariatePosteriorDensity: Creates plots of the bivariate posterior densities for all original parameters and updated
%                                 parameters if added.
%
% USAGE:
%
%       GraphBivariatePosteriorDensity(selector,thetaPostSample,thetaMode,ModeData,ParameterNames,NumDraws,NumBurnin,...
%                 CurrChain,UpdatePostSample,UpdateMode,UpdateNames,UpdateParamPartialPhi,DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", "checkparams",
%                          and "done".
%
% OPTIONAL INPUT: thetaPostSample (matrix) of dimension NumIterxNumParam with all the posterior
%                          draws of the original parameters.
%
%                 thetaMode (vector) of dimension NumParam x 1 with the posterior mode estimates
%                          of the model parameters.
%
%                 ModeData (structure) with fields from the posterior or system prior mode estimation.
%
%                 ParameterNames (string matrix) with the names of the estimated parameters.
%
%                 NumDraws (integer) with the total number of available post burnin draws.
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 CurrChain (integer) with the number for the used sampling chain.
%
%                 UpdatePostSample (matrix) of dimension (NumIter-NumBurnin x NumAdditional) with
%                          the post burnin sample draws of the parameters that are functions
%                          of the estimated parameters.
%
%                 UpdateMode (vector) of dimension NumAdditionalx1 with the posterior mode estimates
%                          of the parameters that are functions of the estimated parameters.
%
%                 UpdateNames (string matrix) of dimension (NumAdditional x N1) with the names of the
%                          additional parameters.
%
%                 UpdateParamPartialPhi (matrix) of dimension NumAdditional x NumParam with the numerical
%                          partial derivatives of the updated parameters with respect the the estimated
%                          transformed parameters.
%
%                 CurrINI (structure) with initialization information.
%
%                 controls (structure) whose fields provide handles to all the UI
%                          controls in YADA's main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 10, 2006.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 11-10-2018: Finished the function.
%
% * 12-10-2018: Fixed the x and y-axis bounds for the fast Fourier transform estimator
%               taken from the Kernel density estimation toolbox.
%
% * 15-10-2018: Dealt with the HG2 changes to setting colors of the contours using an undocumented matlab
%               suggestion.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 28-05-2020: Deleted an unnecessary return call.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.thetaPostSample = thetaPostSample;
   DSGEModel.thetaMode = thetaMode;
   DSGEModel.ModeData = ModeData;
   DSGEModel.ParameterNames = ParameterNames;
   DSGEModel.NumDraws = NumDraws;
   DSGEModel.NumBurnin = NumBurnin;
   DSGEModel.CurrChain = CurrChain;
   DSGEModel.UpdatePostSample = UpdatePostSample;
   DSGEModel.UpdateMode = UpdateMode;
   DSGEModel.UpdateNames = UpdateNames;
   DSGEModel.UpdateParamPartialPhi = UpdateParamPartialPhi;
else
   BivariatePosteriorDensityGUI = findobj('Type','figure','Tag','BivariatePosteriorDensityGUI');
   BivariateControls = get(BivariatePosteriorDensityGUI,'UserData');
   CurrINI = get(BivariateControls.parameter1popup,'UserData');
   DSGEModel = get(BivariateControls.parameter1,'UserData');
end;
%
switch selector;
   case 'init';
      if isempty(CurrChain)==0;
         NameStr = 'Bivariate Posterior Densities';
      else;
         NameStr = 'Bivariate Prior Densities';
      end;
      BivariatePosteriorDensityGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','GraphBivariatePosteriorDensity.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-130)/2 500 200], ...
         'Tag','BivariatePosteriorDensityGUI', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',NameStr, ...
         'NumberTitle','off', ...
         'CloseRequestFcn','GraphBivariatePosteriorDensity done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(BivariatePosteriorDensityGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 190],'Graphics - Kernel Density Estimation',45,[0.5 1],'on',CurrINI);
      %
      % Select first parameter
      %
      BivariateControls.parameter1 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 135 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select parameter 1:', ...
         'HorizontalAlignment','left', ...
         'Tag','ParameterOneText');
      %
      BivariateControls.parameter1popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 140 175 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',strvcat(ParameterNames,UpdateNames), ...
         'HorizontalAlignment','center', ...
         'CallBack','GraphBivariatePosteriorDensity checkparams', ...
         'Tag','ParameterOnePopup',...
         'Value',1);
      %
      % Select second parameter
      %
      BivariateControls.parameter2 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 105 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select parameter 2:', ...
         'HorizontalAlignment','left', ...
         'Tag','ParameterTwoText');
      %
      BivariateControls.parameter2popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 110 175 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',strvcat(ParameterNames,UpdateNames), ...
         'HorizontalAlignment','center', ...
         'CallBack','GraphBivariatePosteriorDensity checkparams', ...
         'Tag','ParameterTwoPopup',...
         'Value',2);
      %
      BivariateControls.KDEMethodText = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Kernel density estimation method:', ...
         'HorizontalAlignment','left', ...
         'Tag','MethodText');
      %
      BivariateControls.KDEMethod = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 80 175 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','on', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','Diffusion|Fast Fouries Transform', ...
         'Value',2, ...
         'Tag','MethodPopup');
      %
      BivariateControls.GridWidthText = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select grid width:', ...
         'HorizontalAlignment','left', ...
         'Tag','GridWidthText');
      %
      BivariateControls.GridWidth = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 50 175 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','on', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','mode +/-1 standard deviation|mode +/-2 standard deviations|mode +/-3 standard deviations|mode +/-4 standard deviations|mode +/-5 standard deviations|mode +/-6 standard deviations|mode +/-7 standard deviations|mode +/-8 standard deviations|mode +/-9 standard deviations|mode +/-10 standard deviations', ...
         'Value',DSGEModel.GridWidth, ...
         'Tag','GridWidthPopup');
      %
      BivariateControls.NumGridPointsText = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Number of grid points per axes:', ...
         'HorizontalAlignment','left', ...
         'Tag','NumGridPointsText');
      %
      BivariateControls.NumGridPoints = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 20 175 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','on', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','16|32|64|128|256', ...
         'Value',2, ...
         'Tag','NumberOfGridPointsPopup');
      %
      % Create graph button
      %
      BivariateControls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','GraphBivariatePosteriorDensity showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      BivariateControls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','GraphBivariatePosteriorDensity done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(BivariatePosteriorDensityGUI,'UserData',BivariateControls,...
         'HandleVisibility','callback');
   case 'checkparams';
      Param1Val = get(BivariateControls.parameter1popup,'Value');
      Param2Val = get(BivariateControls.parameter2popup,'Value');
      if Param1Val==Param2Val;
         %
         % check which controls should be adjusted
         %
         NumParams = size(get(BivariateControls.parameter1popup,'String'),1);
         if gco==BivariateControls.parameter1popup;
            %
            % parameter 1 has changed
            %
            if Param2Val<NumParams;
               set(BivariateControls.parameter2popup,'Value',Param2Val+1);
            else;
               set(BivariateControls.parameter2popup,'Value',1);
            end;
         else;
            %
            % parameter 2 has changed
            %
            if Param2Val<NumParams;
               set(BivariateControls.parameter1popup,'Value',Param1Val+1);
            else;
               set(BivariateControls.parameter1popup,'Value',1);
            end;
         end;
      end;
   case 'showgraph';
      set(BivariatePosteriorDensityGUI,'Visible','off');
      drawnow;
      drawnow;
      %
      % initialize
      %
      [NumIter,NumParam] = size(DSGEModel.thetaPostSample);
      NumUpdateParam = size(DSGEModel.UpdatePostSample,2);
      PostSample = [DSGEModel.thetaPostSample DSGEModel.UpdatePostSample];
      theta = [DSGEModel.thetaMode;DSGEModel.UpdateMode];
      %
      Param1Value = get(BivariateControls.parameter1popup,'Value');
      ParameterNames = get(BivariateControls.parameter1popup,'String');
      Param1 = StringTrim(ParameterNames(Param1Value,:));
      %
      Param2Value = get(BivariateControls.parameter2popup,'Value');
      Param2 = StringTrim(ParameterNames(Param2Value,:));
      %
      % check if we have posterior or prior draws
      %
      if isempty(DSGEModel.CurrChain)==0;
         IsPosterior = 1;
         TypeStr = 'Posterior';
      else;
         IsPosterior = 0;
         TypeStr = 'System Prior';
      end;
      %
      KDEMethod = get(BivariateControls.KDEMethod,'value');
      %
      % first we display a wait dialog
      %
      if IsPosterior==1;
         txt = ['Please wait while YADA computes kernel density estimates of the bivariate posterior density for ' Param1 ' and ' Param2 '. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
         WaitHandle = WaitDLG(txt,'information','Bivariate Kernel Density Estimation - Posterior',500,150,CurrINI,0);
      else;
         txt = ['Please wait while YADA computes kernel density estimates of the bivariate prior density for ' Param1 ' and ' Param2 '. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
         WaitHandle = WaitDLG(txt,'information','Bivariate Kernel Density Estimation - Prior',500,150,CurrINI,0);
      end;
      WaitControls = get(WaitHandle,'UserData');
      drawnow;
      pause(0.02);
      %
      % we can now run the bivariate kernel density estimation, either via diffusion algorithm by
      % Botev, Grotowski and Kroese (2010), Annals of Statistics, and based on the function
      % written by Dr. Zdravko Botev; or based on the kernerl density estimation toolbox by
      % Christian C. Beardah 1994.
      %
      GridWidth = get(BivariateControls.GridWidth,'Value');
      %
      % get the upper and lower bounds for the estimated parameters
      %
      ModeData = DSGEModel.ModeData;
      MinMaxGrid = zeros(length(ModeData.theta),2);
      MinMaxGrid(:,1) = ModeData.phiMode-(GridWidth*sqrt(diag(ModeData.InverseHessian)));
      MinMaxGrid(:,2) = ModeData.phiMode+(GridWidth*sqrt(diag(ModeData.InverseHessian)));
      %
      BoundsMatrix = [PhiToTheta(MinMaxGrid(:,1),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound) PhiToTheta(MinMaxGrid(:,2),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound)];
      %
      if NumUpdateParam>0;
         UpdateBoundsMatrix = zeros(NumUpdateParam,2);
         UpdateBoundsMatrix(:,1) = DSGEModel.UpdateMode-(GridWidth*sqrt(diag(DSGEModel.UpdateParamPartialPhi*ModeData.InverseHessian*DSGEModel.UpdateParamPartialPhi')));
         UpdateBoundsMatrix(:,2) = DSGEModel.UpdateMode+(GridWidth*sqrt(diag(DSGEModel.UpdateParamPartialPhi*ModeData.InverseHessian*DSGEModel.UpdateParamPartialPhi')));
         BoundsMatrix = [BoundsMatrix;UpdateBoundsMatrix];
      end;
      %
      LowerLimits = [BoundsMatrix(Param1Value,1) BoundsMatrix(Param2Value,1)];
      UpperLimits = [BoundsMatrix(Param1Value,2) BoundsMatrix(Param2Value,2)];
      %
      % number of grid points
      %
      NumGridPoints = get(BivariateControls.NumGridPoints,'Value')+3;
      %
      if KDEMethod==1;
         [bandwidth,density,X,Y] = YADAkde2d(PostSample(:,[Param1Value Param2Value]),2^NumGridPoints,LowerLimits,UpperLimits);
      elseif KDEMethod==2;
         [X,Y,density] = KernelDensityPlot2D(PostSample(:,Param1Value),PostSample(:,Param2Value),'knorm',2^NumGridPoints);
      end;
      %
      % close the wait dialog
      %
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
      %
      % show the parameter selection dialog
      %
      set(BivariatePosteriorDensityGUI,'Visible','on');
      drawnow;
      pause(0.02);
      %
      % Setting up the surface
      %
      if isempty(density)==0
         FigHandleSurf = figure('Units','pixels', ...
            'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
            'NumberTitle','off', ...
            'Visible','off', ...
            'Tag',['Surf-' Param1 '-' Param2], ...
            'Name',['Surface Plot of Bivariate ' TypeStr ' Densities - ' Param1 ' and ' Param2]);
         %
         SetFigureProperties(FigHandleSurf,CurrINI);
         %
         Surfhandle = surf(X,Y,density,'EdgeColor','none','LineWidth',1,'FaceLighting','gouraud','SpecularStrength',0.5);
         hold('on');
         %
         %
         % ensure the presence of a colormap
         %
         colormap(jet);
         CurrColorMap = colormap;
         %
         % shading
         %
         shading('interp');
         %
         ZLim = get(gca,'ZLim');
         %
         if KDEMethod==2;
            axis('tight');
            BoundsMatrix(Param1Value,:) = get(gca,'XLim');
            BoundsMatrix(Param2Value,:) = get(gca,'YLim');
         end;
         set(gca,'XLim',BoundsMatrix(Param1Value,:),'YLim',BoundsMatrix(Param2Value,:),'ZLim',[0 max(max(density))]);
         %
         % add a light
         %
         if MatlabNumber<=8.3;
            LightHandle = light('Position',[LowerLimits max(get(gca,'ZLim'))/2]);
            hold('on');
         end;
         %
         % add a colorbar
         %
         CurrAxes = gca;
         ColorBarHandle = colorbar;
         set(ColorBarHandle,'Tag','colorbar');
         SetAxesFonts(CurrINI,ColorBarHandle);
         %
         % make sure the the bar is not too wide
         %
         ColorBarPosition = get(ColorBarHandle,'Position');
         AxesPosition = get(CurrAxes,'Position');
         set(ColorBarHandle,'Position',[ColorBarPosition(1)+(2*ColorBarPosition(3)/3) ColorBarPosition(2) ColorBarPosition(3)/3 ColorBarPosition(4)]);
         set(CurrAxes,'Position',AxesPosition);
         hold('off');
         %
         % fix the title, xlabel, ylabel and zlabel
         %
         if IsPosterior==1;
            SetAxesFonts(CurrINI,'Title',['Joint posterior density of ' Param1 ' and ' Param2],'XLabel',Param1,'Ylabel',Param2,'ZLabel',['Density']);
         else;
            SetAxesFonts(CurrINI,'Title',['Joint prior density of ' Param1 ' and ' Param2],'XLabel',Param1,'Ylabel',Param2,'ZLabel',['Density']);
         end;
         %
         if (strcmp(get(gca,'YGrid'),'on')==1)&(strcmp(get(gca,'ZGrid'),'on')==1);
            set(gca,'XGrid','on');
         end;
         %
         % Fix the toolbar
         %
         SetToolbarImages(FigHandleSurf,CurrINI.images);
         set(FigHandleSurf,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
         %
         % plot contours
         %
         ZVec = max(max(density))*[1/32 1/16 1/8 1/4 1/2 3/4];
         %
         CMap = zeros(6,3);
         CMap(1,:) = (1/255)*[150 150 255];
         CMap(2,:) = (1/255)*[25 25 255];
         CMap(3,3) = 200/255;
         CMap(4,3) = 125/255;
         CMap(5,3) = 50/255;
         %
         FigHandleContour = figure('Visible','off', ...
                                   'Position',[((CurrINI.scrsz(3)-650)/2)+32 ((CurrINI.scrsz(4)-500)/2)-32 650 500], ...
                                   'Tag',['Contour-' Param1 '-' Param2], ...
                                   'Colormap',CMap, ...
                                   'Name',['Contour Plot of Bivariate ' TypeStr ' Densities - ' Param1 ' and ' Param2]);
         %
         SetFigureProperties(FigHandleContour,CurrINI);
         %
         % create the filled contour plot
         %
         if MatlabNumber>8.3;
            [c_mat,ContourHandle] = contourf(X,Y,density,ZVec);
         else;
            if MatlabNumber>=7;
               [c_mat,ContourHandle] = contourf('v6',X,Y,density,ZVec);
            else;
               [c_mat,ContourHandle] = contourf(X,Y,density,ZVec);
            end;
         end;
         hold('on');
         %
         % remove edgecolors
         %
         if MatlabNumber>8.3;
            set(ContourHandle,'LineColor','none');
            drawnow;
            %
            % try to change the face color of the filled contours under HG2 graphics
            %
            % HG2 graphics with contour plots are largely undocumented. The following
            % is based on "Undocumented Matlab" at
            %
            %    https://undocumentedmatlab.com/blog/customizing-contour-plots
            %
            ColorChange = 0;
            FacePrims = ContourHandle.FacePrims;
            CMapLev = 255*CMap';
            for i=1:length(FacePrims)
               FacePrims(i).ColorData = uint8([CMapLev(:,i);255]);
               ColorChange = 1;
            end;
         else;
            for i=1:length(ContourHandle);
               if i<=6;
                  set(ContourHandle(i),'EdgeColor','none','FaceColor',CMap(i,:));
               else;
                  set(ContourHandle(i),'EdgeColor','none');
               end;
            end;
         end;
         set(gca,'XLim',BoundsMatrix(Param1Value,:),'YLim',BoundsMatrix(Param2Value,:));
         %
         if IsPosterior==1;
            SetAxesFonts(CurrINI,'Title',['Contours of posterior density of ' Param1 ' and ' Param2],'XLabel',Param1,'Ylabel',Param2);
         else;
            SetAxesFonts(CurrINI,'Title',['Contours of prior density of ' Param1 ' and ' Param2],'XLabel',Param1,'Ylabel',Param2);
         end;
         %
         if strcmp(get(gca,'YGrid'),'on')==1;
            set(gca,'XGrid','on');
         end;
         %
         % Fix the toolbar
         %
         SetToolbarImages(FigHandleContour,CurrINI.images);
         set(FigHandleContour,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
         if MatlabNumber>8.3;
            if ColorChange==0;
               %
               % retry changing the ColorData field of FacePrims
               %
               FacePrims = ContourHandle.FacePrims;
               for i=1:length(FacePrims)
                  FacePrims(i).ColorData = uint8([CMapLev(:,i);255]);
               end;
            end;
         end;
      end;
   case 'done';
      set(BivariateControls.done,'UserData','done');
      delete(BivariatePosteriorDensityGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(BivariatePosteriorDensityGUI,'Visible','on');
   drawnow;
   SetYADAIcon(BivariatePosteriorDensityGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(BivariateControls.done);
   end;
   waitfor(BivariateControls.done,'UserData','done');
end;

%
% end of GraphBivariatePosteriorDensity.m
%
