function ObsVarDecompPathDLG(selector,Y,DecompInitState,DecompSteadyState,DecompMeasureError,DecompShocks,Quantiles,DateVector,ShockNames,ShockType,NameStr,AnnualStr,CurrINI,DSGEModel)
% ObsVarDecompPathDLG: Displays graphs of observed variables decomposition paths.
%
% USAGE:
%
%       ObsVarDecompPathDLG(selector,Y,DecompInitState,DecompSteadyState,DecompMeasureError,DecompShocks,Quantiles,DateVector,ShockNames,ShockType,NameStr,AnnualStr,CurrINI,DSGEModel)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend", "showgraph", and "done".
%
% OPTIONAL INPUT: Y (matrix) of dimension nxT with data on the observed variables.
%
%                 DecompInitState (matrix) of dimension nxT with estimates of the initial state term of the decomposition.
%
%                 DecompSteadyState (matrix) of dimension nxT with estimates of the steady-state term of the decomposition.
%
%                 DecompMeasureError (matrix) of dimension nxT with estimates of the measurement error term of the decomposition.
%
%                 DecompShocks (matrix) of dimension nxTxq with estimates of the shock or shock group terms of the decomposition.
%
%                 Quantiles (structure) with quantile data on the decomposition. Is empty if fixed parameter values were used
%                           for the calculations.
%
%                 DateVector (vector) with numeric dates for the observed variables.
%
%                 ShockNames (string matrix) with names of the shocks or shock groups.
%
%                 ShockType (boolean) that is 1 if shock type is shock group and 0 if individual shocks.
%
%                 NameStr (string vector) taking on the values "Posterior Mode", "Initial Values", "Prior Distribution",
%                           or "Posterior Distribution".
%
%                 AnnualStr (string vector) taking on the values "Original" or "Annualized".
%
%                 CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 22, 2009.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-04-2009: Added the display graph code.
%
% * 24-04-2009: Added the Quantiles input variable and code for displaying confidence bands.
%
% * 06-01-2010: Updated the documentation.
%
% * 25-05-2010: Corrected a text issue on the dialog.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 23-03-2018: Fixed an issue to make legend compatible with matlab 2017a and later.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.NameStr = NameStr;
   DSGEModel.AnnualStr = AnnualStr;
   DSGEModel.YDec = Y;
   DSGEModel.DecompInitState = DecompInitState;
   DSGEModel.DecompSteadyState = DecompSteadyState;
   DSGEModel.DecompMeasureError = DecompMeasureError;
   DSGEModel.DecompShocks = DecompShocks;
   DSGEModel.DateVector = DateVector;
   DSGEModel.ShockNames = ShockNames;
   DSGEModel.Quantiles = Quantiles;
else;
   ObsVarDecompPathGUI = findobj('Type','figure','Tag','ObsVarDecompPathDLG');
   controls = get(ObsVarDecompPathGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
switch selector;
   case 'init';
      ObsVarDecompPathGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ObsVarDecompPathDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-525)/2 (CurrINI.scrsz(4)-135)/2 525 135], ...
         'Tag','ObsVarDecompPathDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Observed Variable Decomposition Paths - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ObsVarDecompPathDLG done', ...
         'ToolBar','none');

      if MatlabNumber>=7.0;
         set(ObsVarDecompPathGUI,'DockControl','off');
      end;
      %
      % Check where the sample of data actually begins
      %
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
      else;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1+(max(DSGEModel.annual)-1));
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 523 125],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Select the sample
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      %
      [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
      %
      DSGEModel = OldDSGEModel;
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 80 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ObsVarDecompPathDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[303 80 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ObsVarDecompPathDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SubSampleStrMat,1));
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 50 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVarPopup',...
         'Value',1);
      %
      % select shock or shock group
      %
      controls.shockstext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 15 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select shock:', ...
         'HorizontalAlignment','left', ...
         'Tag','ShockText');
      %
      controls.shocks = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 20 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ShockNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ShockPopup',...
         'Value',1);
      %
      if ShockType==1;
         set(controls.shockstext,'String','Select shock group:');
      end;
      % 
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ObsVarDecompPathDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ObsVarDecompPathDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(ObsVarDecompPathGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
   case 'showgraph'
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.VariableNames(VariableValue,:));
      ShockValue = get(controls.shocks,'Value');
      CurrShock = StringTrim(DSGEModel.ShockNames(ShockValue,:));
      if ShockValue<size(DSGEModel.ShockNames,1);
         set(controls.shocks,'Value',ShockValue+1);
      else;
         set(controls.shocks,'Value',1);
         if VariableValue<size(DSGEModel.VariableNames,1);
            set(controls.variables,'Value',VariableValue+1);
         else;
            set(controls.variables,'Value',1);
         end;
      end;
      %
      % get the sample values
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      %
      % compute the decomposition term
      %
      YDec = DSGEModel.DecompInitState(VariableValue,:)+DSGEModel.DecompSteadyState(VariableValue,:)+DSGEModel.DecompMeasureError(VariableValue,:)+DSGEModel.DecompShocks(VariableValue,:,ShockValue);
      Y = DSGEModel.YDec(VariableValue,:);
      %
      % check if we can compute the correlation between the terms
      %
      if (isempty(DSGEModel.UnitRootStates)==1)&(DSGEModel.AllowUnitRoot==0);
         YDiff = Y-(sum(Y)/length(Y));
         %
         % make sure that only the shock term is included in the correlation
         %
         YShock = DSGEModel.DecompShocks(VariableValue,:,ShockValue);
         YShockDiff = YShock-(sum(YShock)/length(YShock));
         StdY = sqrt(sum(YDiff.*YDiff)/length(Y));
         StdYShock = sqrt(sum(YShockDiff.*YShockDiff)/length(YShock));
         if (StdYShock>1.0e-06)&(StdY>1.0e-06);
            CorrYYShock = (sum(YDiff.*YShockDiff)/length(Y))/(StdY*StdYShock);
            CorrStr = [' (corr=' num2str(CorrYYShock,'%0.2f') ')'];
         else;
            CorrStr = ' (corr=0)';
         end;
      else;
         CorrStr = '';
      end;
      %
      % open a figure
      %
      FigHandle = figure('Visible','off', ...
                     'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                     'Tag',['OVP-' CurrVariable '-' strrep(CurrShock,' ','-')], ...
                     'Name',['Observed Variable Decomposition Path - ' CurrVariable ' and ' CurrShock]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % check if we have any quantile data
      %
      Handles = [];
      LegendStr = '';
      if isempty(DSGEModel.Quantiles)==0;
         NumBounds = floor(length(DSGEModel.Quantiles)/2);
         for i=1:NumBounds;
            Lower = DSGEModel.Quantiles(i).DecompY(VariableValue,samplebeg:sampleend,ShockValue);
            Upper = DSGEModel.Quantiles(length(DSGEModel.Quantiles)-i+1).DecompY(VariableValue,samplebeg:sampleend,ShockValue);
            PatchColor = ((1+(2*(i-1)))/length(DSGEModel.Quantiles))*DSGEModel.ConfidenceBandBaseColor;
            ConfBound = DSGEModel.Quantiles(length(DSGEModel.Quantiles)-i+1).percent-DSGEModel.Quantiles(i).percent;
            ConfPatch = patch([DSGEModel.DateVector(samplebeg:sampleend) fliplr(DSGEModel.DateVector(samplebeg:sampleend))],[Upper fliplr(Lower)],PatchColor);
            set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',PatchColor);
            hold('on');
            Handles = [Handles ConfPatch];
            LegendStr = strvcat(LegendStr,[num2str(ConfBound,'%0.0f') ' confidence']);
         end;
         set(gca,'Layer','Top');
      end;
      %
      % plot the observed variable
      %
      ObsHandle = plot(DSGEModel.DateVector(samplebeg:sampleend),Y(samplebeg:sampleend),'.-','Color',[0.75 0 0],'LineWidth',2);
      hold('on');
      %
      % plot the decomposition
      %
      DecHandle = plot(DSGEModel.DateVector(samplebeg:sampleend),YDec(samplebeg:sampleend),'--','Color',[0 0 0.76],'LineWidth',2);
      hold('on');
      %
      % setup the figure title
      %
      if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (original data) and ' CurrShock ' term at ' lower(DSGEModel.NameStr)]);
      else;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (annual data) and ' CurrShock ' term at ' lower(DSGEModel.NameStr)]);
      end;
      %
      % see if we should add a zero line
      %
      ylim = get(gca,'YLim');
      add_on = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-add_on ylim(2)+add_on];
      set(gca,'YLim',ylim,'XLim',[DSGEModel.DateVector(samplebeg) DSGEModel.DateVector(sampleend)]);
      if (ylim(1)<0)&(ylim(2)>0);
         hold('on');
         ZeroHandle = plot([DSGEModel.DateVector(samplebeg) DSGEModel.DateVector(sampleend)],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         set(gca,'Children',[ObsHandle DecHandle ZeroHandle fliplr(Handles)]);
      end;
      if MatlabNumber>=7;
         legHandle = legend([ObsHandle DecHandle Handles],strvcat(CurrVariable,[CurrShock ' term' CorrStr],LegendStr),'Location','NorthWest');
      else;
         legHandle = legend([ObsHandle DecHandle Handles],strvcat(CurrVariable,[CurrShock ' term' CorrStr],LegendStr),2);
      end;
      SetAxesFonts(CurrINI,legHandle);
      hold('off');
      %
      SetXTickIntegers(gca);
      %
      % display the graph
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(controls.done,'UserData','done');
      delete(ObsVarDecompPathGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(ObsVarDecompPathGUI,'Visible','on');
   drawnow;
   SetYADAIcon(ObsVarDecompPathGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of ObsVarDecompPathDLG.m
%
