function ScatterDecompDistDLG(selector,OVDs,NameStr,AnnualStr,ShockType,CurrINI,DSGEModel)
% ScatterDecompDistDLG: Displays scatter plots of observed variable decompositions.
%
% USAGE:
%
%       ScatterDecompDistDLG(selector,OVDs,NameStr,AnnualStr,ShockType,CurrINI,DSGEModel)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend", "showgraph", and "done".
%
% OPTIONAL INPUT: OVDs (structure) with fields "Y", "DateVector", "ShockNames" with the observed variables, dates and the
%                      names of the shocks or shock groups. Furthermore, the mean estimates of the decomposition terms are
%                      called "DecompInitState", "DecompSteadyState", "DecompMeasureError" and "DecompShocks". Similarly,
%                      the median estimates are called "DecompMedianInitState", "DecompMedianSteadyState",
%                      "DecompMedianMeasureError" and "DecompMedianShocks".
%
%                 NameStr (string vector) taking on the values "Posterior" or "Prior".
%
%                 AnnualStr (string vector) taking on the values "Original" or "Annualized".
%
%                 ShockType (boolean) which takes the value 1 if shock groups are used and 0 otherwise.
%
%                 CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                                   about the model, such as input files, variable and sample
%                                   data. Actual observations are not stored in this file.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 12, 2023.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-12-2023: Finished the function.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.NameStr = NameStr;
   DSGEModel.AnnualStr = AnnualStr;
   DSGEModel.OVDs = OVDs;
   DSGEModel.ShockType = ShockType;
   DSGEModel.SelectedShocks = (1:length(DSGEModel.StateShockPositions));
else;
   GraphGUI = findobj('Type','figure','Tag','ScatterDecompDistDLG');
   controls = get(GraphGUI,'UserData');
   CurrINI = get(controls.variablesone,'UserData');
   DSGEModel = get(controls.variablesonetext,'UserData');
end;
%
switch selector;
   case 'init';
      GraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ScatterDecompDistDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-525)/2 (CurrINI.scrsz(4)-170)/2 525 170], ...
         'Tag','ScatterDecompDistDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Scatter Plot Decompositions - ' NameStr ' Distribution'], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ScatterDecompDistDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(GraphGUI,'DockControl','off');
      end;
      %
      % Check where the sample of data actually begins
      %
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
      else;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1+(max(DSGEModel.annual)-1));
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 523 160],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Select the sample
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      %
      [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
      %
      DSGEModel = OldDSGEModel;
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 110 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 115 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ScatterDecompDistDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[303 115 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ScatterDecompDistDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SubSampleStrMat,1));
      %
      % Select variable
      %
      controls.variablesonetext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable 1 (X axis):', ...
         'HorizontalAlignment','left', ...
         'Tag','ScatterDecompTextOne');
      %
      controls.variablesone = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 80 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'Callback','ScatterDecompDistDLG checkvars', ...
         'HorizontalAlignment','center', ...
         'Tag','ScatterDecompPopupOne',...
         'Value',1);
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 115 90 20], ...
         'String','Select Shocks', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ScatterDecompDistDLG selectshocks', ...
         'Enable','on', ...
         'Tag','Done');
      %
      controls.variablestwotext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select variable 2 (Y axis):', ...
         'HorizontalAlignment','left', ...
         'Tag','ScatterDecompTextTwo');
      %
      controls.variablestwo = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 50 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',DSGEModel.VariableNames, ...
         'Callback','ScatterDecompDistDLG checkvars', ...
         'HorizontalAlignment','center', ...
         'Tag','ScatterDecompPopupTwo',...
         'Value',2);
      %
      controls.saveplotstofile = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Save figures to file', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','SavePlotsToFileCheckBox');
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ScatterDecompDistDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ScatterDecompDistDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(GraphGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
   case 'selectshocks'
      [action,SelectedShocks] = SelectCondVarShockDLG('init','Scatter Plot','Structural Shocks For',DSGEModel.StateShockNames,DSGEModel.SelectedShocks,CurrINI);
      if strcmp(lower(action),'ok')==1;
         if isempty(SelectedShocks)==0;
            if (length(SelectedShocks)<length(DSGEModel.StateShockPositions))&(isequal(SelectedShocks,DSGEModel.SelectedShocks)==0);
               txt = ['Please note that the chosen selection of ' int2str(length(SelectedShocks)) ' shocks will only be used as a sub-selection of shocks when you don''t use shock groups for the scatter plots of the decompositions of observed and state variables.'];
               About(txt,'information',['Scatter Plot Decompositions - ' DSGEModel.NameStr ' Distribution'],300,600,CurrINI);
            end;
            DSGEModel.SelectedShocks = SelectedShocks;
            set(controls.variablesonetext,'UserData',DSGEModel);
         else;
            txt = 'You didn''t select any shocks for the scatter plots. You need to select at least one.';
            About(txt,'information',['Scatter Plot Decompositions - ' DSGEModel.NameStr ' Distribution'],200,600,CurrINI);
         end;
      end;
   case 'checkvars'
      VarOne = get(controls.variablesone,'Value');
      VarTwo = get(controls.variablestwo,'Value');
      if VarOne==VarTwo;
         %
         % check which controls should be adjusted
         %
         NumVars = size(get(controls.variablesone,'String'),1);
         if gco==controls.variablesone;
            %
            % variable 1 has changed
            %
            if VarTwo<NumVars;
               set(controls.variablestwo,'Value',VarTwo+1);
            else;
               set(controls.variablestwo,'Value',1);
            end;
         else;
            %
            % variable 2 has changed
            %
            if VarOne<NumVars;
               set(controls.variablesone,'Value',VarOne+1);
            else;
               set(controls.variablesone,'Value',1);
            end;
         end;
      end;
   case 'showgraph'
      VariableNames = DSGEModel.VariableNames;
      VariableOneValue = get(controls.variablesone,'Value');
      VariableTwoValue = get(controls.variablestwo,'Value');
      CurrVariableOne = StringTrim(DSGEModel.VariableNames(VariableOneValue,:));
      CurrVariableTwo = StringTrim(DSGEModel.VariableNames(VariableTwoValue,:));
      if VariableTwoValue<DSGEModel.n;
         set(controls.variablestwo,'Value',VariableTwoValue+1);
      else;
         if VariableOneValue+1<DSGEModel.n;
            set(controls.variablesone,'Value',VariableOneValue+1);
            set(controls.variablestwo,'Value',VariableOneValue+2);
         else;
            set(controls.variablesone,'Value',1);
            set(controls.variablestwo,'Value',2);
         end;
      end;
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      SampleStrMat = get(controls.samplebeg,'String');
      %
      % first the data
      %
      VarOne = DSGEModel.OVDs.Y(VariableOneValue,:);
      VarTwo = DSGEModel.OVDs.Y(VariableTwoValue,:);
      %
      [n,T,q] = size(DSGEModel.OVDs.DecompShocks);
      %
      UseShockGroups = DSGEModel.ShockType;
      if UseShockGroups==1;
         ShockGroups = DSGEModel.ShockGroups;
      else;
         ShockGroups = (1:length(DSGEModel.StateShockPositions));
      end;
      %
      % initialize
      %
      YmatOneMean = zeros(max(ShockGroups),T);
      YmatOneMedian = YmatOneMean;
      YmatTwoMean = YmatOneMean;
      YmatTwoMedian = YmatOneMean;
      %
      SteadyStateMean = DSGEModel.OVDs.DecompSteadyState;
      SteadyStateMedian = DSGEModel.OVDs.DecompMedianSteadyState;
      %
      % fill Ymat matrices with shock parts
      %
      for i=1:q;
         YmatOneMean(ShockGroups(i),:) = YmatOneMean(ShockGroups(i),:)+DSGEModel.OVDs.DecompShocks(VariableOneValue,:,i);
         YmatOneMedian(ShockGroups(i),:) = YmatOneMedian(ShockGroups(i),:)+DSGEModel.OVDs.DecompMedianShocks(VariableOneValue,:,i);
         YmatTwoMean(ShockGroups(i),:) = YmatTwoMean(ShockGroups(i),:)+DSGEModel.OVDs.DecompShocks(VariableTwoValue,:,i);
         YmatTwoMedian(ShockGroups(i),:) = YmatTwoMedian(ShockGroups(i),:)+DSGEModel.OVDs.DecompMedianShocks(VariableTwoValue,:,i);
      end;
      YmatOneMean = YmatOneMean+(ones(q,1)*SteadyStateMean(VariableOneValue,:));
      YmatOneMedian = YmatOneMedian+(ones(q,1)*SteadyStateMedian(VariableOneValue,:));
      YmatTwoMean = YmatTwoMean+(ones(q,1)*SteadyStateMean(VariableTwoValue,:));
      YmatTwoMedian = YmatTwoMedian+(ones(q,1)*SteadyStateMedian(VariableTwoValue,:));
      UseAliases = 0;
      if (UseShockGroups==1)&(isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)==length(unique(ShockGroups)));
         ShockGroupColors = DSGEModel.ShockGroupColors(unique(ShockGroups),:);
      else;
         if (isempty(DSGEModel.ShockColors)==0)&(q==size(DSGEModel.ShockColors,1));
            ShockGroupColors = DSGEModel.ShockColors;
         else;
            ShockGroupColors = CreateDefaultColors(q);
         end;
         if (UseShockGroups==0)|(isequal(DSGEModel.ShockGroupNames,DSGEModel.StateShockNames)==1);
            UseAliases = 1;
         end;
      end;
      if UseAliases==0;
         ShockGroupNames = '';
         for i=1:size(DSGEModel.ShockGroupNames,1);
            ShockGroupNames = strvcat(ShockGroupNames,StringTrim(strrep(DSGEModel.ShockGroupNames(i,:),'_',' ')));
         end;
      else;
         ShockGroupNames = DSGEModel.ShockAliases;
         for j=1:size(ShockGroupNames,1);
            ShockGroupNames(j,:) = strrep(ShockGroupNames(j,:),'_',' ');
         end;
      end;
      %
      % get the steady-state values
      %
      SS1Mean = mean(SteadyStateMean(VariableOneValue,:));
      SS2Mean = mean(SteadyStateMean(VariableTwoValue,:));
      SS1Median = median(SteadyStateMedian(VariableOneValue,:));
      SS2Median = median(SteadyStateMedian(VariableTwoValue,:));
      %
      % check if we should include all shocks in the plot or not
      %
      if UseShockGroups==0;
         YmatOneMean = YmatOneMean(DSGEModel.SelectedShocks,:);
         YmatTwoMean = YmatTwoMean(DSGEModel.SelectedShocks,:);
         YmatOneMedian = YmatOneMedian(DSGEModel.SelectedShocks,:);
         YmatTwoMedian = YmatTwoMedian(DSGEModel.SelectedShocks,:);
         ShockGroupNames = ShockGroupNames(DSGEModel.SelectedShocks,:);
         ShockGroupColors = ShockGroupColors(DSGEModel.SelectedShocks,:);
      end;
      %
      % determine how many figures to produce and how many rows and coloumns
      %
      NumPlots = size(YmatOneMean,1)+1;
      NumFigs = ceil(NumPlots/16);
      if NumFigs==1;
         [Nrows,Ncols] = VillaniConstructOptimalSubplot(NumPlots);
      else;
         Nrows = 4;
         Ncols = 4;
      end;
      %
      pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
      %
      % plots for the mean
      %
      np = 0;
      for nf=1:NumFigs;
         %
         % create the figure
         %
         if nf>1;
            pos(1) = pos(1)+22;
            pos(2) = pos(2)-22;
            if pos(1)+1000>CurrINI.scrsz(3);
               pos(1) = 0;
            end;
            if pos(2)<=32;
               pos(2) = CurrINI.scrsz(4)-700;
            end;
         end;
         FigHandles{nf} = figure('Visible','off','Position',pos);
         %
         if NumFigs==1;
            set(FigHandles{nf},'Tag',['ScatterPlot-' CurrVariableOne '-' CurrVariableTwo],'Name',['Scatter-Plot of ' CurrVariableOne ' and ' CurrVariableTwo ' Decompositions For Mean Of ' DSGEModel.NameStr ' Distribution - ' SampleStrMat(samplebeg,:) ' to ' SampleStrMat(sampleend,:)]);
         else;
            set(FigHandles{nf},'Tag',['ScatterPlot-' CurrVariableOne '-' CurrVariableTwo '-' int2str(nf)],'Name',['Scatter-Plot of ' CurrVariableOne ' and ' CurrVariableTwo ' Decompositions For Mean Of ' DSGEModel.NameStr ' Distribution - ' SampleStrMat(samplebeg,:) ' to ' SampleStrMat(sampleend,:) ' - Figure ' int2str(nf)]);
         end;
         SetFigureProperties(FigHandles{nf},CurrINI);
         %
         %
         k = 0;
         for rows=1:Nrows;
            for cols=1:Ncols;
               np = np+1;
               if np<=NumPlots;
                  k = k+1;
                  %
                  % ensure that the handle to the current figure is correct (HG2 issue)
                  %
                  set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{nf});
                  subplot(Nrows,Ncols,k);
                  if np==1;
                     if MatlabNumber>8.3;
                        ScatHandle = scatter(VarOne(samplebeg:sampleend),VarTwo(samplebeg:sampleend),25,zeros(1,3),'filled');
                     else;
                        if MatlabNumber>=7;
                           ScatHandle = scatter('v6',VarOne(samplebeg:sampleend),VarTwo(samplebeg:sampleend),4,zeros(1,3),'filled');
                        else;
                           ScatHandle = scatter(VarOne(samplebeg:sampleend),VarTwo(samplebeg:sampleend),4,zeros(1,3),'filled');
                        end;
                     end;
                     hold('on');
                     axis('tight');
                     %
                     % plot the steady state values
                     %
                     XLim = get(gca,'XLim');
                     YLim = get(gca,'YLim');
                     if YLim(2)-YLim(1)<1.0e-6;
                        YLim = YLim+[-0.1 0.1];
                        set(gca,'YLim',YLim);
                     end;
                     plot(SS1Mean*ones(1,2),YLim,'-','LineWidth',1,'Color',[0 0 0]);
                     plot(XLim,SS2Mean*ones(1,2),'-','LineWidth',1,'Color',[0 0 0]);
                     set(gca,'XLim',XLim,'YLim',YLim);
                     %
                     % Show title and labels
                     %
                     SetAxesFonts(CurrINI,'Title',[CurrVariableOne ' and ' CurrVariableTwo ' (' lower(DSGEModel.AnnualStr) ' data)'],'XLabel',CurrVariableOne,'Ylabel',CurrVariableTwo);
                     if strcmp(get(gca,'YGrid'),'on')==1;
                        set(gca,'XGrid','on');
                     end;
                  else;
                     if MatlabNumber>8.3;
                        ScatHandle = scatter(YmatOneMean(np-1,samplebeg:sampleend),YmatTwoMean(np-1,samplebeg:sampleend),25,ShockGroupColors(np-1,:),'filled');
                     else;
                        if MatlabNumber>=7;
                           ScatHandle = scatter('v6',YmatOneMean(np-1,samplebeg:sampleend),YmatTwoMean(np-1,samplebeg:sampleend),4,ShockGroupColors(np-1,:),'filled');
                        else;
                           ScatHandle = scatter(YmatOneMean(np-1,samplebeg:sampleend),YmatTwoMean(np-1,samplebeg:sampleend),4,ShockGroupColors(np-1,:),'filled');
                        end;
                     end;
                     hold('on');
                     axis('tight');
                     %
                     % plot the steady state values
                     %
                     XLim = get(gca,'XLim');
                     YLim = get(gca,'YLim');
                     if YLim(2)-YLim(1)<1.0e-6;
                        YLim = YLim+[-0.1 0.1];
                        set(gca,'YLim',YLim);
                     end;
                     plot(SS1Mean*ones(1,2),YLim,'-','LineWidth',1,'Color',[0 0 0]);
                     plot(XLim,SS2Mean*ones(1,2),'-','LineWidth',1,'Color',[0 0 0]);
                     set(gca,'XLim',XLim,'YLim',YLim);
                     %
                     % Show title and labels
                     %
                     SetAxesFonts(CurrINI,'Title',[StringTrim(ShockGroupNames(np-1,:)) ' shocks'],'XLabel',CurrVariableOne,'Ylabel',CurrVariableTwo);
                     if strcmp(get(gca,'YGrid'),'on')==1;
                        set(gca,'XGrid','on');
                     end;
                  end;
               end;
            end;
         end;
         %
         % check if we should save the figure to disk
         %
         if get(controls.saveplotstofile,'Value')==1;
            GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
            if DirExist(GraphDirectory)==0;
               stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
            end;
            file = FixFilePath([GraphDirectory '\' get(FigHandles{nf},'Tag') '-Decomp-' DSGEModel.AnnualStr '-Mean-' DSGEModel.NameStr 'Dist.fig']);
            saveas(FigHandles{nf},file);
         end;
         %
         SetToolbarImages(FigHandles{nf},CurrINI.images);
         %
         set(FigHandles{nf},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      end;
      %
      % the median
      %
      np = 0;
      for nf=1:NumFigs;
         %
         % create the figure
         %
         pos(1) = pos(1)+22;
         pos(2) = pos(2)-22;
         if pos(1)+1000>CurrINI.scrsz(3);
            pos(1) = 0;
         end;
         if pos(2)<=32;
            pos(2) = CurrINI.scrsz(4)-700;
         end;
         FigHandles{nf} = figure('Visible','off','Position',pos);
         %
         if NumFigs==1;
            set(FigHandles{nf},'Tag',['ScatterPlot-' CurrVariableOne '-' CurrVariableTwo],'Name',['Scatter-Plot of ' CurrVariableOne ' and ' CurrVariableTwo ' Decompositions For Median Of ' DSGEModel.NameStr ' Distribution - ' SampleStrMat(samplebeg,:) ' to ' SampleStrMat(sampleend,:)]);
         else;
            set(FigHandles{nf},'Tag',['ScatterPlot-' CurrVariableOne '-' CurrVariableTwo '-' int2str(nf)],'Name',['Scatter-Plot of ' CurrVariableOne ' and ' CurrVariableTwo ' Decompositions For Median Of ' DSGEModel.NameStr ' Distribution - ' SampleStrMat(samplebeg,:) ' to ' SampleStrMat(sampleend,:) ' - Figure ' int2str(nf)]);
         end;
         SetFigureProperties(FigHandles{nf},CurrINI);
         %
         %
         k = 0;
         for rows=1:Nrows;
            for cols=1:Ncols;
               np = np+1;
               if np<=NumPlots;
                  k = k+1;
                  %
                  % ensure that the handle to the current figure is correct (HG2 issue)
                  %
                  set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{nf});
                  subplot(Nrows,Ncols,k);
                  if np==1;
                     if MatlabNumber>8.3;
                        ScatHandle = scatter(VarOne(samplebeg:sampleend),VarTwo(samplebeg:sampleend),25,zeros(1,3),'filled');
                     else;
                        if MatlabNumber>=7;
                           ScatHandle = scatter('v6',VarOne(samplebeg:sampleend),VarTwo(samplebeg:sampleend),4,zeros(1,3),'filled');
                        else;
                           ScatHandle = scatter(VarOne(samplebeg:sampleend),VarTwo(samplebeg:sampleend),4,zeros(1,3),'filled');
                        end;
                     end;
                     hold('on');
                     axis('tight');
                     %
                     % plot the steady state values
                     %
                     XLim = get(gca,'XLim');
                     YLim = get(gca,'YLim');
                     if YLim(2)-YLim(1)<1.0e-6;
                        YLim = YLim+[-0.1 0.1];
                        set(gca,'YLim',YLim);
                     end;
                     plot(SS1Median*ones(1,2),YLim,'-','LineWidth',1,'Color',[0 0 0]);
                     plot(XLim,SS2Median*ones(1,2),'-','LineWidth',1,'Color',[0 0 0]);
                     set(gca,'XLim',XLim,'YLim',YLim);
                     %
                     % Show title and labels
                     %
                     SetAxesFonts(CurrINI,'Title',[CurrVariableOne ' and ' CurrVariableTwo ' (' lower(DSGEModel.AnnualStr) ' data)'],'XLabel',CurrVariableOne,'Ylabel',CurrVariableTwo);
                     if strcmp(get(gca,'YGrid'),'on')==1;
                        set(gca,'XGrid','on');
                     end;
                  else;
                     if MatlabNumber>8.3;
                        ScatHandle = scatter(YmatOneMedian(np-1,samplebeg:sampleend),YmatTwoMedian(np-1,samplebeg:sampleend),25,ShockGroupColors(np-1,:),'filled');
                     else;
                        if MatlabNumber>=7;
                           ScatHandle = scatter('v6',YmatOneMedian(np-1,samplebeg:sampleend),YmatTwoMedian(np-1,samplebeg:sampleend),4,ShockGroupColors(np-1,:),'filled');
                        else;
                           ScatHandle = scatter(YmatOneMedian(np-1,samplebeg:sampleend),YmatTwoMedian(np-1,samplebeg:sampleend),4,ShockGroupColors(np-1,:),'filled');
                        end;
                     end;
                     hold('on');
                     axis('tight');
                     %
                     % plot the steady state values
                     %
                     XLim = get(gca,'XLim');
                     YLim = get(gca,'YLim');
                     if YLim(2)-YLim(1)<1.0e-6;
                        YLim = YLim+[-0.1 0.1];
                        set(gca,'YLim',YLim);
                     end;
                     plot(SS1Median*ones(1,2),YLim,'-','LineWidth',1,'Color',[0 0 0]);
                     plot(XLim,SS2Median*ones(1,2),'-','LineWidth',1,'Color',[0 0 0]);
                     set(gca,'XLim',XLim,'YLim',YLim);
                     %
                     % Show title and labels
                     %
                     SetAxesFonts(CurrINI,'Title',[StringTrim(ShockGroupNames(np-1,:)) ' shocks'],'XLabel',CurrVariableOne,'Ylabel',CurrVariableTwo);
                     if strcmp(get(gca,'YGrid'),'on')==1;
                        set(gca,'XGrid','on');
                     end;
                  end;
               end;
            end;
         end;
         %
         % check if we should save the figure to disk
         %
         if get(controls.saveplotstofile,'Value')==1;
            GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
            if DirExist(GraphDirectory)==0;
               stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
            end;
            file = FixFilePath([GraphDirectory '\' get(FigHandles{nf},'Tag') '-Decomp-' DSGEModel.AnnualStr '-Median-' DSGEModel.NameStr 'Dist.fig']);
            saveas(FigHandles{nf},file);
         end;
         %
         SetToolbarImages(FigHandles{nf},CurrINI.images);
         %
         set(FigHandles{nf},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      end;
   case 'done'
      set(controls.done,'UserData','done');
      delete(GraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(GraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(GraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;


%
% end of ScatterDecompDistDLG.m
%
