function status = ALPredictiveLikelihoodExist(DSGEModel,SelectedYData,IsOriginal,IsMarginal,IsPosterior,h,NumDraws,NumBurnin,TotalDraws,CurrChain)
% ALPredictiveLikelihoodExist: Checks if the conditional log predictive likelihood under adaptive learning has already been
%                              computed and stored to disk or not. These data are used when computing the predictive likelihood
%                              with Monte Carlo integration.
%
% USAGE:
%
%       status = ALPredictiveLikelihoodExist(DSGEModel,SelectedYData,IsOriginal,IsMarginal,IsPosterior,h,NumDraws,...
%                                          NumBurnin,TotalDraws,CurrChain)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  SelectedYData (integer) with the selected set of actuals for the observed variables.
%
%                  IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                             annualized values should be ptredicted.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive likelihood should be
%                             computed and 0 if the joint predictive likelihood is the object of interest.
%
%                  IsPosterior (boolean) that takes the value 1 if the parameter draws are taken from the posterior
%                             distribution and 0 otherwise.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameters draws used to compute the log posterior.
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) for the selected MCMC chain when posterior draws are used.
%
% REQUIRED OUTPUT: status (boolean) that is 1 if the results exist on disk and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 4, 2021.
%                        This version: March 12, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-05-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 12-03-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
%
SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
if IsOriginal==1;
   TypeStr = 'Original';
else;
   TypeStr = 'Annualized';
end;
if IsMarginal==1;
   MargStr = 'Marginal';
else;
   MargStr = 'Joint';
end;
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'AdaptiveLearningDSGEPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumBurnin) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'AdaptiveLearningDSGEPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.mat']);
end;
if FileExist(file)==1;
   status = 1;
end;

%
% end of ALPredictiveLikelihoodExist.m
%
