function AdaptiveLearningCheckPosteriorMode(DSGEModel,CurrINI,controls)
% AdaptiveLearningCheckPosteriorMode: Opens graphs of the posterior mode. If the graphs already exists on disk,
%                                     these are shown. Otherwise, new ones are created based on the information
%                                     stored to disk about the posterior mode.
%
% USAGE:
%
%       AdaptiveLearningCheckPosteriorMode(DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT: DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 controls (structure), whose fields provide handles to all the UI
%                           controls in YADA's main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 8, 2021.
%                        This version: March 13, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-02-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 13-03-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% display graphs of the region around the posterior mode
%
drawnow;
pause(0.02);
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
if FileExist(FixFilePath([GraphDirectory '\PosteriorModeOptimum-AdaptiveLearning-phi.fig']))==1;
   txt = 'YADA has located old plots of the posterior mode in the file "PosteriorModeOptimum-AdaptiveLearning-phi.fig". Would you like to open this file?';
   answer = Query(txt,'question',300,'Adative Learning - Check Posterior Mode',600,'yes',CurrINI);
elseif FileExist(FixFilePath([GraphDirectory '\PosteriorModeOptimum-AdaptiveLearning-phi-1.fig']))==1;
   txt = 'YADA has located old plots of the posterior mode in the files "PosteriorModeOptimum-AdaptiveLearning-phi-x.fig", where x=1,2,... Would you like to open these files?';
   answer = Query(txt,'question',300,'Adaptive Learning - Check Posterior Mode',600,'yes',CurrINI);
else;
   answer = 'No';
end;
drawnow;
pause(0.02);
%
% check if we should make new figures
%
if strcmp(lower(answer),'no')==1;
   oldtab = get(controls.tab1,'UserData');
   maingui = get(controls.tab1,'Parent');
   set(controls.about,'UserData',0);
   set(maingui,'CurrentObject',controls.tab6);
   YADAGUI('tab');
   if MatlabNumber>=7;
      uicontrol(controls.tab6);
   end;
   set(controls.about,'UserData',1);
   %
   % check if estimation log is empty
   %
   LogText = get(controls.estimation_log,'String');
   if isempty(LogText)==0;
      SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
   end;
   %
   % we need to load data from mat-file, and run the check optimum functions
   %
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
   ErrorStr = '';
   if MatlabNumber>=8.0;
      try;
         ModeData = load(ModeFile);
      catch ME;
         ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
      end;
   else;
      try;
         ModeData = load(ModeFile);
      catch;
         ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
      end;
   end;
   if isempty(ErrorStr)==1;
      %
      % copy files that we need to have on the path
      %
      [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
      [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.InitializeParameterFile)==1;
         [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.SystemPriorFile)==1;
         [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
      end;
      %
      % check if we should also copy the belief system file
      %
      if DSGEModel.BeliefSystem==2;
         [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
      end;
      %
      % check if we should copy model solver file
      %
      if DSGEModel.ModelSolver==4;
         if FileExist(DSGEModel.ModelSolverFile)==1;
            [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
         end;
         if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
            [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
         end;
      end;
      %
      % and the AIMData file
      %
      if MatlabNumber>=8.0;
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch ME;
            ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
         end;
      else;
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch;
            ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
         end;
      end;
      if isempty(ErrorStr)==1;
         %
         % Set up the sub-sample
         %
         [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
         %
         SetLogText(strvcat(' ','Checking the optimum...',' * Transformed parameters'));
         %
         NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
         NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
         k = DSGEModel.k;
         %
         ConstPos = 0;
         if k>0;
            ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
            if ConstPos==0;
               %
               % we will now check if X has a unit vector
               %
               X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
               T = size(X,2);
               for j=1:k;
                  ConstTest = (DSGEModel.X(j,DSGEModel.FirstPeriod:DSGEModel.LastPeriod)==ones(1,T));
                  if sum(ConstTest)==T;
                     ConstPos = j;
                     break;
                  end;
               end;
            end;
            xbar = zeros(k,1);
            xbar(ConstPos) = 1;
         else;
            xbar = [];
         end;
         %
         MinMaxGrid = zeros(length(ModeData.phiMode),2);
         stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
         %
         % create lower and upper bounds from InverseHessian and phiMode
         %
         MinMaxGrid(:,1) = ModeData.phiMode-(DSGEModel.GridWidth*sqrt(diag(ModeData.InverseHessian)));
         MinMaxGrid(:,2) = ModeData.phiMode+(DSGEModel.GridWidth*sqrt(diag(ModeData.InverseHessian)));
         phiFittedStd = VillaniCheckOptimum('logPosteriorAdaptiveLearningPhiDSGE',ModeData.phiMode,[],MinMaxGrid,ModeData.InverseHessian,NumGridPoints,0,[],ModeData.ParameterNames.estimated,'Checking Curvature at Posterior Mode - Transformed Parameters',CurrINI,GraphDirectory,'phi','','',1,1,1, ...
            ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         %
         if (DSGEModel.CheckTransformedOptimum==0)|(IsOdd(DSGEModel.MaximizeAlgorithmValue)==0);
            if isfield(ModeData,'ParameterBounds')==1;
               ParameterBounds = ModeData.ParameterBounds;
            else;
               ParameterBounds = CreateBounds(ModeData.thetaDist,ModeData.LowerBound,ModeData.UniformBounds);
               ParameterBounds = ParameterBounds(:,1:2);
            end;
            SetLogText(' * Original parameters');
            MinMaxGrid(:,1) = PhiToTheta(MinMaxGrid(:,1),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound);
            MinMaxGrid(:,2) = PhiToTheta(MinMaxGrid(:,2),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound);
            ThetaPhiPartial = PartialThetaPartialPhi(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
            thetaFittedStd = VillaniCheckOptimum('logPosteriorAdaptiveLearningThetaDSGE',ModeData.thetaMode,[],MinMaxGrid,ThetaPhiPartial*ModeData.InverseHessian*ThetaPhiPartial,NumGridPoints,0,[],ModeData.ParameterNames.estimated,'Checking Curvature at Posterior Mode - Original Parameters',CurrINI,GraphDirectory,'theta','','',1,1,1, ...
               ModeData.thetaIndex,ParameterBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         else;
            thetaFittedStd = NaN;
         end;
         if isfield(ModeData,'phiFittedStd')==0;
            save(ModeFile,'phiFittedStd','thetaFittedStd','-APPEND');
         else;
            if isfield(ModeData,'thetaFittedStd')==0;
               save(ModeFile,'thetaFittedStd','-APPEND');
            end;
         end;
         SetLogText('All done!');
         return;
      else;
         About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         drawnow;
         pause(0.02);
         return;
      end;
   else;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
      drawnow;
      pause(0.02);
      return;
   end;
end;
%
% try to open the figures
%
if (FileExist(FixFilePath([GraphDirectory '\PosteriorModeOptimum-AdaptiveLearning-phi.fig']))==1)&(strcmp(lower(answer),'yes')==1);
   %
   % Files of the optimum already exists. Open them
   %
   ErrorStr = '';
   if MatlabNumber>=8.0;
      try;
         open(FixFilePath([GraphDirectory '\PosteriorModeOptimum-AdaptiveLearning-phi.fig']));
      catch ME;
         ErrorStr = ['Unable to load the file "PosteriorModeOptimum-AdaptiveLearning-phi.fig". Message caught is: ' YADALastError(ME)];
      end;
   else;
      try;
         open(FixFilePath([GraphDirectory '\PosteriorModeOptimum-AdaptiveLearning-phi.fig']));
      catch;
         ErrorStr = ['Unable to load the file "PosteriorModeOptimum-AdaptiveLearning-phi.fig". Message caught is: ' YADALastError];
      end;
   end;
   if isempty(ErrorStr)==1;
      set(gcf,'Visible','off');
      SetToolbarImages(gcf,CurrINI.images);
      set(gcf,'Visible','on');
      drawnow;
      pause(0.02);
      if FileExist(FixFilePath([GraphDirectory '\CompLikePostOptimum-AdaptiveLearning-phi.fig']))==1;
         open(FixFilePath([GraphDirectory '\CompLikePostOptimum-AdaptiveLearning-phi.fig']));
         set(gcf,'Visible','off');
         SetToolbarImages(gcf,CurrINI.images);
         set(gcf,'Visible','on');
         drawnow;
         pause(0.02);
      end;
      if (DSGEModel.CheckTransformedOptimum==0)|(IsOdd(DSGEModel.MaximizeAlgorithmValue)==0);
         if FileExist(FixFilePath([GraphDirectory '\PosteriorModeOptimum-AdaptiveLearning-theta.fig']))==1;
            open(FixFilePath([GraphDirectory '\PosteriorModeOptimum-AdaptiveLearning-theta.fig']));
            set(gcf,'Visible','off');
            SetToolbarImages(gcf,CurrINI.images);
            set(gcf,'Visible','on');
            drawnow;
            pause(0.02);
         end;
         if FileExist(FixFilePath([GraphDirectory '\CompLikePostOptimum-AdaptiveLearning-theta.fig']))==1;
            open(FixFilePath([GraphDirectory '\CompLikePostOptimum-AdaptiveLearning-theta.fig']));
            set(gcf,'Visible','off');
            SetToolbarImages(gcf,CurrINI.images);
            set(gcf,'Visible','on');
            drawnow;
            pause(0.02);
         end;
      end;
   else;
      About(ErrorStr,'information','YADA - Bad FIG File',500,600,CurrINI);
   end;
elseif (FileExist(FixFilePath([GraphDirectory '\PosteriorModeOptimum-AdaptiveLearning-phi-1.fig']))==1)&(strcmp(lower(answer),'yes')==1);;
   %
   % There are multiple files to open
   %
   done = 0;
   num = 1;
   ErrorStr = '';
   while done==0;
      %
      % we close the loop once a figure doesn't exist
      %
      file = FixFilePath([GraphDirectory '\PosteriorModeOptimum-AdaptiveLearning-phi-' int2str(num) '.fig']);
      if FileExist(file)==1;
         if MatlabNumber>=8.0;
            try;
               open(file);
            catch ME;
               ErrorStr = ['Unable to load the file "PosteriorModeOptimum-AdaptiveLearning-phi-' int2str(num) '.fig". Message caught is: ' YADALastError(ME)];
               done = 1;
            end;
         else;
            try;
               open(file);
            catch;
               ErrorStr = ['Unable to load the file "PosteriorModeOptimum-AdaptiveLearning-phi-' int2str(num) '.fig". Message caught is: ' YADALastError];
               done = 1;
            end;
         end;
         if isempty(ErrorStr)==1;
            set(gcf,'Visible','off');
            SetToolbarImages(gcf,CurrINI.images);
            set(gcf,'Visible','on');
            drawnow;
            pause(0.02);
            %
            NextFile = FixFilePath([GraphDirectory '\CompLikePostOptimum-AdaptiveLearning-phi-' int2str(num) '.fig']);
            if FileExist(NextFile)==1;
               open(NextFile);
               set(gcf,'Visible','off');
               SetToolbarImages(gcf,CurrINI.images);
               set(gcf,'Visible','on');
               drawnow;
               pause(0.02);
            end;
            if (DSGEModel.CheckTransformedOptimum==0)|(IsOdd(DSGEModel.MaximizeAlgorithmValue)==0);
               file = strrep(file,'phi','theta');
               if FileExist(file)==1;
                  open(file);
                  set(gcf,'Visible','off');
                  SetToolbarImages(gcf,CurrINI.images);
                  set(gcf,'Visible','on');
                  drawnow;
                  pause(0.02);
%               else;
%                  done = 1;
               end;
               if done==0;
                  NextFile = strrep(NextFile,'phi','theta');
                  if FileExist(NextFile)==1;
                     open(NextFile);
                     set(gcf,'Visible','off');
                     SetToolbarImages(gcf,CurrINI.images);
                     set(gcf,'Visible','on');
                     drawnow;
                     pause(0.02);
%                  else;
%                     done = 1;
                  end;
               end;
            end;
            num = num+1;
         else;
            About(ErrorStr,'information','YADA - Bad FIG File',500,600,CurrINI);
            done = 1;
         end;
      else;
         done = 1;
      end;
   end;
   if isempty(ErrorStr)==0;
      About(ErrorStr,'information','YADA - Bad FIG File',500,600,CurrINI);
   end;
end;

%
% end of AdaptiveLearningCheckPosteriorMode.m
%
