function [UpdatePostSample,UpdateMode,UpdateNames,UpdateParamPartialPhi] = AdaptiveLearningGetUpdatePosteriorDraws(thetaPostSample,NumBurnin,CurrentChain,NumChains,CheckForData,MarginalStr,DSGEModel,CurrINI)
% AdaptiveLearningGetUpdatePosteriorDraws: Computes the posterior draw values for the parameters that are functions of
%                                          estimated parameters. Applies to the adaptive learning solution of the DSGE model.
%
% USAGE:
%
%       [UpdatePostSample,UpdateNames,UpdateParamPartialPhi] = AdaptiveLearningGetUpdatePosteriorDraws(thetaPostSample,NumBurnin,...
%                                                                                CurrentChain,NumChains,CheckForData,MarginalStr,...
%                                                                                DSGEModel,CurrINI)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension (NumIterxNumParam) with all the non-discarded
%                                  posterior draws of theta (the original parameters)
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  CurrentChain (integer) with the number of the currently used MCMC draws.
%
%                  NumChains (integer) with the total number of MCMC chains that we can choose from.
%
%                  CheckForData (boolean) that takes the value 1 if the function should check if the
%                                  calculations have already been performed and stored on disk, and
%                                  0 if they should be calculated anyway.
%
%                  MarginalStr (string) that takes the values "Marginal" or "Bivariate".
%
%                  DSGEModel (structure) whose fields contain all required information about the model,
%                                  such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: UpdatePostSample (matrix) of dimension (NumIter-NumBurninxNumAdditional) with the post
%                                  burnin sample values of the parameters that are functions of the
%                                  estimated parameters.
%
%                  UpdateMode (vector) of dimension (NumAdditional x 1) with the posterior mode estimates
%                                  of the parameters that are functions of the estimated parameters.
%
%                  UpdateNames (string matrix) of dimension (NumAdditional x N1) with the names of the
%                                  additional parameters.
%
% OPTIONAL OUTPUT: UpdateParamPartialPhi (matrix) of dimension NumAdditional x NumParam with the numerical
%                                  partial derivatives of the update parameter with respect to the
%                                  estimated transformed parameters phi.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 19, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-05-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed eval call to feval.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
UpdatePostSample = [];
UpdateMode = [];
UpdateNames = '';
if nargout==4;
   UpdateParamPartialPhi = [];
end;
%
[NumIter,NumParam] = size(thetaPostSample);
%
% check if we can load the posterior mode data
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      txt = ['YADA can''t provide posterior densities of parameters that are a function of the estimated parameters. There was a problem trying to load the file with posterior mode data ("' GetFilename(ModeFile) '") in the directory "' GetPath(ModeFile) '". YADA has received the following error message: ' YADALastError(ME) '.'];
      About(txt,'information',[MarginalStr ' Posterior Densities - Additional Parameters'],300,500,CurrINI);
      drawnow;
      pause(0.02);
      return;
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      txt = ['YADA can''t provide posterior densities of parameters that are a function of the estimated parameters. There was a problem trying to load the file with posterior mode data ("' GetFilename(ModeFile) '") in the directory "' GetPath(ModeFile) '". YADA has received the following error message: ' YADALastError '.'];
      About(txt,'information',[MarginalStr ' Posterior Densities - Additional Parameters'],300,500,CurrINI);
      drawnow;
      pause(0.02);
      return;
   end;
end;
%
% copy update parameter file to tmp folder
%
[stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
%
% get the names of the parameters to update
%
UpdateNames = ModeData.ParameterNames.additional;
thetaPositions = ModeData.thetaPositions;
ModelParameters = ModeData.ModelParameters;
%
NumAdditional = size(UpdateNames,1);
UpdatePostSample = zeros(NumIter-NumBurnin,NumAdditional);
%
% compute the posterior mode of the updated parameters
%
UpdateMode = zeros(NumAdditional,1);
%
% 1. set the ModelParameters structure to posterior mode values
%
ModelParameters = ThetaToModelParameters(ModelParameters,ModeData.thetaMode,thetaPositions);
%
% 2. update the additional parameters at posterior mode
%
ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
for j=1:NumAdditional;
   UpdateMode(j) = getfield(ModelParameters,StringTrim(UpdateNames(j,:)));
end;
%
% compute numerical partials for UpdateMode versus phi
%
if nargout==4;
   UpdateParamPartialPhi = zeros(NumAdditional,NumParam);
   phiChange = 0.001*abs(ModeData.phiMode);
   IdentMat = eye(NumParam);
   %
   for j=1:NumAdditional;
      for i=1:NumParam;
         phiNew = ModeData.phiMode+(phiChange(i)*IdentMat(:,i));
         thetaNew = PhiToTheta(phiNew,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound);
         MP = ModelParameters;
         MP = ThetaToModelParameters(MP,thetaNew,thetaPositions);
         %
         % 2. update the additional parameters at posterior mode
         %
         MP = feval(GetName(DSGEModel.UpdateParameterFile),MP);
         NewUpdatej = getfield(MP,StringTrim(UpdateNames(j,:)));
         UpdateParamPartialPhi(j,i) = (NewUpdatej-UpdateMode(j))/phiChange(i);
      end;
   end;
end;
%
%
% number of tempering stages
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
% check if previously calculated values are stored on disk
%
if DSGEModel.PosteriorSampler==1;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-AL-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==2;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\slice\Slice-AL-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==3;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-AL-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==4;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-AL-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==5;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-AL-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==6;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-AL-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==7;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-AL-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==8;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-AL-Additional-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumIter,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==9;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-AL-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumIter,'%0.0f') '.mat']);
elseif DSGEModel.PosteriorSampler==10;
   AddFile = FixFilePath([DSGEModel.OutputDirectory '\is\IS-AL-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumIter,'%0.0f') '.mat']);
end;
if (CheckForData==1)&(FileExist(AddFile)==1);
   AddData = load(AddFile);
   %
   % check if we have the same posterior mode and the same parameter names
   %
   if (isfield(AddData,'UpdateNames')==1)&(isfield(AddData,'UpdateMode')==1)&(isfield(AddData,'UpdatePostSample')==1)&(isequal(UpdateMode,AddData.UpdateMode)==1)&(isequal(UpdateNames,AddData.UpdateNames)==1);
      txt = ['YADA has located previously computed values for the additional parameters using ' int2str(NumIter-NumBurnin) ' post burn-in draws of the estimated parameters. Would you like to use these values?'];
      answer = Query(txt,'question',200,[MarginalStr ' Posterior Densities - Additional Parameters'],500,'yes',CurrINI);
      drawnow;
      pause(0.02);
      if strcmp(lower(answer),'yes')==1;
         UpdatePostSample = AddData.UpdatePostSample;
         return;
      end;
   end;
end;
%
if (CheckForData==1)&(NumIter-NumBurnin>=10000);
   sStr = '';
   RStr = 'is';
   if NumAdditional>1;
      sStr = 's';
      RStr = 'are';
   end;
   txt = ['Please wait while YADA prepares the ' int2str(NumIter-NumBurnin) ' post burn-in sample posterior draws of the ' int2str(NumAdditional) ' parameter' sStr ' that ' RStr ' a function of the estimated parameters. Computations started at: ' StringTrim(datestr(now,14)) '.'];
   WaitHandle = WaitDLG(txt,'information',[MarginalStr ' Posterior Densities - Additional Parameters'],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
for i=NumBurnin+1:NumIter;
   %
   % make sure the ModelParameters structure is properly updated
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,thetaPostSample(i,:)',thetaPositions);
   %
   % now run the update parameter function
   %
   ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   %
   % now we update the vector of additional parameters
   %
   for j=1:NumAdditional;
      UpdatePostSample(i-NumBurnin,j) = getfield(ModelParameters,StringTrim(UpdateNames(j,:)));
   end;
   drawnow;
end;
%
% close the wait dialog
%
if (CheckForData==1)&(NumIter-NumBurnin>=10000);
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% save the data to file
%
save(AddFile,'UpdatePostSample','UpdateNames','UpdateMode');

%
% end of AdaptiveLearningGetUpdatePosteriorDraws.m
%
