function [UpdatePriorSample,UpdateMode,UpdateParamPartialPhi] = AdaptiveLearningGetUpdatePriorDraws(thetaDraws,thetaMode,UpdateNames,thetaPositions,ModelParameters,thetaIndex,UniformBounds,LowerBound,CheckForData,DSGEModel,CurrINI)
% AdaptiveLearningGetUpdatePriorDraws: Computes the prior draw values for the parameters that are functions of
%                                      the estimated parameters. Applies to the adaptive learning solution of
%                                      the DSGE model.
%
% USAGE:
%
%       [UpdatePriorSample,UpdateMode,UpdateParamPartialPhi] = AdaptiveLearningGetUpdatePriorDraws(thetaDraws,thetaMode,...
%                                                                 UpdateNames,thetaPositions,ModelParameters,thetaIndex,...
%                                                                 UniformBounds,LowerBound,CheckForData,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  thetaDraws (matrix) of dimension NumParam x NumIter with the prior draws of the
%                             estimated parameters.
%
%                  thetaMode (vector) with the prior mode estimates of the parameters.
%
%                  UpdateNames (string matrix) of dimension (NumAdditional x N1) with the names of the
%                             additional parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta,
%                             3=uniform, and 0=normal, student-t, cauchy, logistic, gumbel. This is used to
%                             determine the type of transformation that may need to be performed for the
%                             theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                             are 0 1 except for the uniform and the beta distribution, where they represent
%                             the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  CheckForData (boolean) that takes the value 1 if the function should check if the
%                             calculations have already been performed and stored on disk, and 0 if
%                             they should be calculated anyway.
%
%                  DSGEModel (structure) whose fields contain all required information about the model,
%                             such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: UpdatePriorSample (matrix) of dimension NumAdditional x NumIter with the values of
%                             the parameters that are functions of the estimated parameters.
%
%                  UpdateMode (vector) of dimension NumAdditional with the system prior mode estimates
%                             for the updated parameters.
%
% OPTIONAL OUTPUT: UpdateParamPartialPhi (matrix) of dimension NumAdditional x NumParam with the numerical
%                             partial derivatives of the update parameter with respect to the estimated
%                             transformed parameters phi.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 19, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-05-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed eval call to feval.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%


%
% initialize
%
UpdatePriorSample = [];
UpdateMode = [];
if nargout==3;
   UpdateParamPartialPhi = [];
end;
%
[NumParam,NumIter] = size(thetaDraws);
%
% copy update parameter file to tmp folder
%
[stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
NumAdditional = size(UpdateNames,1);
UpdatePriorSample = zeros(NumAdditional,NumIter);
%
% 1. set the ModelParameters structure to posterior mode values
%
ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
%
% 2. update the additional parameters at posterior mode
%
ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
%
% compute the value of phi, the transformed parameters, at thetaMean
%
phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
%
% compute the prior mean of the updated parameters
%
UpdateMode = zeros(NumAdditional,1);
for j=1:NumAdditional;
   UpdateMode(j) = getfield(ModelParameters,StringTrim(UpdateNames(j,:)));
end;
%
% compute numerical partials for UpdateMode versus phi
%
if nargout==3;
   UpdateParamPartialPhi = zeros(NumAdditional,NumParam);
   phiChange = 0.001*abs(phiMode);
   IdentMat = eye(NumParam);
   %
   for j=1:NumAdditional;
      for i=1:NumParam;
         phiNew = phiMode+(phiChange(i)*IdentMat(:,i));
         thetaNew = PhiToTheta(phiNew,thetaIndex,UniformBounds,LowerBound);
         MP = ModelParameters;
         MP = ThetaToModelParameters(MP,thetaNew,thetaPositions);
         %
         % 2. update the additional parameters at posterior mode
         %
         MP = feval(GetName(DSGEModel.UpdateParameterFile),MP);
         NewUpdatej = getfield(MP,StringTrim(UpdateNames(j,:)));
         UpdateParamPartialPhi(j,i) = (NewUpdatej-UpdateMode(j))/phiChange(i);
      end;
   end;
end;
%
% check if the update draws have already been calculated.
%
PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' int2str(NumIter) '-' DSGEModel.NameOfModel '.mat']);
PriorDrawsExtraFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
PriorData = load(PriorDrawsFile);
if (CheckForData==1)&(isfield(PriorData,'UpdatePriorSample')==1);
   UpdatePriorSample = PriorData.UpdatePriorSample;
   return;
end;
%
if NumIter>=10000;
   sStr = '';
   RStr = 'is';
   if NumAdditional>1;
      sStr = 's';
      RStr = 'are';
   end;
   txt = ['Please wait while YADA prepares the ' int2str(NumIter) ' prior draws of the ' int2str(NumAdditional) ' parameter' sStr ' that ' RStr ' a function of the estimated parameters. Computations started at: ' StringTrim(datestr(now,14)) '.'];
   WaitHandle = WaitDLG(txt,'information','Bivariate Prior Densities - Additional Parameters',500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% start the iterations
%
for i=1:NumIter;
   %
   % make sure the ModelParameters structure is properly updated
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,thetaDraws(:,i),thetaPositions);
   %
   % now run the update parameter function
   %
   ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   %
   % now we update the vector of additional parameters
   %
   for j=1:NumAdditional;
      UpdatePriorSample(j,i) = getfield(ModelParameters,StringTrim(UpdateNames(j,:)));
   end;
   drawnow;
end;
%
% close the wait dialog
%
if NumIter>=10000;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% save the data to file
%
save(PriorDrawsFile,'UpdatePriorSample','-APPEND');
save(PriorDrawsExtraFile,'UpdatePriorSample','-APPEND');

%
% end of AdaptiveLearningGetUpdatePriorDraws.m
%
