function [XMargPrior,YMargPrior,BoundsMatrix,XSystemPrior,YSystemPrior] = AdaptiveLearningGraphSystemPriorDensity(DSGEModel,CurrINI)
% AdaptiveLearningGraphSystemPriorDensity: Creates plots of the marginal and system prior densities for all original parameters.
%                                          When the output arguments are listed, then the function simply resturns matrices with
%                                          x-axis and y-axis values for the densities. Applies to the adaptive learning solution
%                                          of the DSGE model.
%
% USAGE:
%
%       [XMargPrior,YMargPrior,BoundsMatrix,XSystemPrior,YSystemPrior] = AdaptiveLearningGraphSystemPriorDensity(DSGEModel,CurrINI)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the model,
%                            such as input files, variable and sample data.
%
%                  CurrINI (structure), with initialization information.
%
% OPTIONAL OUTPUT: XMargPrior (matrix) of dimension NumParamxNumPoints. It contains the x-axis data for all
%                            the marginal prior densities.
%
%                  YMargPrior (matrix) of dimension NumParamxNumPoints. It contains the y-axis data for all
%                            the marginal prior densities.
%
%                  BoundsMatrix (matrix) of dimension NumParamx2. It contains information about possible
%                            upper and lower bounds of the parameters.
%
%                  XSystemPrior (matrix) of dimension NumParamxNumSPoints. It contains the x-axis data for all
%                            the prior densities conditional on the system prior data.
%
%                  YSystemPrior (matrix) of dimension NumParamxNumSPoints. It contains the y-axis data for all
%                            the prior densities conditional on the system prior data.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 19, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-05-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargout==5;
   XMargPrior = [];
   YMargPrior = [];
   BoundsMatrix = [];
   XSystemPrior = [];
   YSystemPrior = [];
end;
%
% check if prior draws already exist
%
ErrorStr = '';
PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
if FileExist(PriorDrawsFile)==1;
   %
   % check if the file contains the correct information
   %
   PriorDrawsData = load(PriorDrawsFile);
else;
   ErrorStr = 'The prior draws file doesn''t exist. You must run prior sampling before YADA can graph the prior densities when these include a system prior.';
end;
%
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','Graph Prior Densities',500,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
% get the prior draws of the estimated parameters
%
thetaDraws = PriorDrawsData.thetaDraws;
%
% read the system prior mode file
%
if isempty(CurrINI.Sequence)==1
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
   if FileExist(ModeFile)==1;
      ModeData = load(ModeFile);
   else;
      TxtStr = ['YADA is unable to locate the system prior mode file under adaptive learning and is therefore unable or unwilling to compute the prior densities.'];
      About(ErrorStr,'information','Marginal Prior Densities',500,600,CurrINI);
      return;
   end;
   ParameterNames = ModeData.ParameterNames;
   %
   % check if there are parameters to update
   %
   if (isempty(CurrINI.Sequence)==1)&(FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
      txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Would you also like to plot prior densities for these additional parameters?'];
      answer = Query(txt,'question',200,'Prior Densities - Additional Parameters',500,'no',CurrINI);
      if strcmp(lower(answer),'yes')==1;
         %
         % get the data
         %
         UpdateNames = ParameterNames.additional;
         stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
         if stat==1;
            %
            % get the update prior draws
            %
            if isfield(PriorDrawsData,'UpdateMode')==1;
               UpdatePriorSample = PriorDrawsData.UpdatePriorSample;
               UpdateMode = PriorDrawsData.UpdateMode;
               UpdateParamPartialPhi = PriorDrawsData.UpdateParamPartialPhi;
            else;
               [UpdatePriorSample,UpdateMode,UpdateParamPartialPhi] = AdaptiveLearningGetUpdatePriorDraws(thetaDraws,ModeData.thetaMode,ParameterNames.additional,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,1,DSGEModel,CurrINI);
               %
               % save these outputs to the prior draws file
               %
               save(PriorDrawsFile,'UpdatePriorSample','UpdateMode','UpdateParamPartialPhi','-APPEND');
            end;
            %
            % setup the CurrINI structure to hold parameter data
            %
            CurrINI.theta = ModeData.thetaMode;
            CurrINI.thetaDist = ModeData.thetaDist;
            CurrINI.LowerBound = ModeData.LowerBound;
            CurrINI.PriorDist = ModeData.PriorDist;
            CurrINI.ParameterNames = ModeData.ParameterNames;
            CurrINI.thetaPositions = ModeData.thetaPositions;
            CurrINI.ModelParameters = ModeData.ModelParameters;
            CurrINI.UpdateNames = UpdateNames;
            NumIter = size(PriorDrawsData.thetaDraws,2);
            maingui = findobj('Type','figure','Tag','YADA');
            controls = get(maingui,'UserData');
            [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = AdaptiveLearningGetUpdateSystemPriorDensity(DSGEModel,CurrINI,controls,[],[],[],NumIter);
         else;
            if ispc==1;
               txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\priordraws". The prior for the parameters that are a function of the estimated parameters can therefore not be computed.'];
            else;
               txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/priordraws". The prior for the parameters that are a function of the estimated parameters can therefore not be computed.'];
            end;
            About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
            UpdateNames = '';
            UpdatePriorX = [];
            UpdatePriorY = [];
            UpdatePriorBounds = [];
            UpdateInitialValue = [];
            UpdateMode = [];
            UpdatePriorSample = [];
         end;
      else;
         UpdateNames = '';
         UpdatePriorX = [];
         UpdatePriorY = [];
         UpdatePriorBounds = [];
         UpdateInitialValue = [];
         UpdateMode = [];
         UpdatePriorSample = [];
      end;
   else;
      UpdateNames = '';
      UpdatePriorX = [];
      UpdatePriorY = [];
      UpdatePriorBounds = [];
      UpdateInitialValue = [];
      UpdateMode = [];
      UpdatePriorSample = [];
   end;
else;
   UpdateNames = '';
   UpdatePriorX = [];
   UpdatePriorY = [];
   UpdatePriorBounds = [];
   UpdateInitialValue = [];
   UpdateMode = [];
   UpdatePriorSample = [];
end;
%
% plot the prior draws
%
if nargout==0;
   AdaptiveLearningGraphPosteriorDensity(thetaDraws',0,[],UpdatePriorSample',UpdateMode,UpdateNames,UpdatePriorX,UpdatePriorY,UpdatePriorBounds,DSGEModel,CurrINI);
else;
   [thetaMargMode,UpdateMargMode,XMargPrior,YMargPrior,BoundsMatrix,XSystemPrior,YSystemPrior] = AdaptiveLearningGraphPosteriorDensity(thetaDraws',0,[],UpdatePriorSample',UpdateMode,UpdateNames,UpdatePriorX,UpdatePriorY,UpdatePriorBounds,DSGEModel,CurrINI);
end;

%
% end of AdaptiveLearningGraphSystemPriorDensity.m
%
