function IRStructure = AdaptiveLearningIRFtheta(Ftilde,Btilde,Fss,Bss,H,F,B0,DSGEModel,UseLevels)
% AdaptiveLearningIRFtheta: Computes the impulse responses for observed variables and state variables for a given
%                           value of the parameters with supported solution matrices.
%
% USAGE:
%
%       IRStructure = AdaptiveLearningIRFtheta(Ftilde,Btilde,Fss,Bss,H,F,B0,DSGEModel,UseLevels)
%
% REQUIRED INPUT:  Ftilde (matrix) of dimension r x r x T with the coefficients on lagged state variables in the
%                         state equation of the adaptive learning based solutions.
%
%                  Btilde (matrix) of dimension r x q x T with the coefficients on the structural shocks in the
%                         state equation of the adaptive learning based solutions.
%
%                  Fss (matrix) of dimension r x r with the steady-state coefficients on lagged state variables
%                         in the state equation of the adaptive learning based solution.
%
%                  Bss (matrix) of dimension r x q with the steady-state coefficients on the structural shocks
%                         in the state equation of the adaptive learning based solution.
%
%                  H (matrix) of dimension r x n that maps the state variables into the observables. It can also
%                         be r x n x T when time-varying.
%
%                  F (matrix) of dimension r x r with coefficients on lagged state variables in the state
%                         equation of the RE solution.
%
%                  B0 (matrix) of dimension r x q with coefficients on the structural shocks in the state
%                         equation of the RE solution.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  UseLevels (integer) which indicates if variables in first differences should be
%                         accumulated (1), annualized (2), or left alone (0).
%
% REQUIRED OUTPUT: IRStructure (structure) with fields AL, ALSS and RE where the former has dimension T x 1 with fields
%                         Ksi, matrix of dimension r x q x h+1, and Y, matrix of dimension n x q x h+1; while the
%                         latter two have fields Y and Ksi.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 6, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-05-2020: Finished the function.
%
% * 22-01-2021: Updated the documentation.
%
% * 28-04-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 16-02-2022: Added the input variables Fss and Bss as well as the field ALSS to IRStructure.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
IRStructure = [];
%
% Determine the impulse response horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
h = SampleFreq*DSGEModel.IRHorizon;
%
% compute the IRFs for the RE case first
%
[r,q,T] = size(Btilde);
n = DSGEModel.n;
if length(size(H))==2;
   if UseLevels==0;
      IRStructure.RE = DSGEImpulseResponseFcn(H,F,B0,h);
   elseif UseLevels==1;
      IRStructure.RE = DSGELevImpulseResponseFcn(H,F,B0,diag(1-DSGEModel.levels),h);
   else;
      IRStructure.RE = DSGEImpulseResponseFcn(H,F,B0,h);
      IRFY = zeros(n,h+1,q);
      for i=1:h+1;
         for j=1:q;
            IRFY(:,i,j) = IRStructure.RE.Y(:,j,i);
         end;
      end;
      %
      % we can now annualize
      %
      for j=1:q;
         IRFY(:,:,j) = Annualize([zeros(DSGEModel.n,max(DSGEModel.annual)-1) IRFY(:,:,j)],DSGEModel.annual,DSGEModel.annualscale);
      end;
      %
      % now we save this to the structure IRStructure.RE.Y again
      %
      for i=1:h+1;
         for j=1:q;
            IRStructure.RE.Y(:,j,i) = IRFY(:,i,j);
         end;
      end;
   end;
   %
   % now we turn to adaptive learning
   %
   % first the steady-state IRFs
   %
   if UseLevels==0;
      IRStructure.ALSS = DSGEImpulseResponseFcn(H,Fss,Bss,h);
   elseif UseLevels==1;
      IRStructure.ALSS = DSGELevImpulseResponseFcn(H,Fss,Bss,diag(1-DSGEModel.levels),h);
   else;
      IRStructure.ALSS = DSGEImpulseResponseFcn(H,Fss,Bss,h);
      IRFY = zeros(n,h+1,q);
      for i=1:h+1;
         for j=1:q;
            IRFY(:,i,j) = IRStructure.ALSS.Y(:,j,i);
         end;
      end;
      %
      % we can now annualize
      %
      for j=1:q;
         IRFY(:,:,j) = Annualize([zeros(DSGEModel.n,max(DSGEModel.annual)-1) IRFY(:,:,j)],DSGEModel.annual,DSGEModel.annualscale);
      end;
      %
      % now we save this to the structure IRStructure.RE.Y again
      %
      for i=1:h+1;
         for j=1:q;
            IRStructure.ALSS.Y(:,j,i) = IRFY(:,i,j);
         end;
      end;
   end;
   %
   % and the the time-dependent IRFs
   %
   for t=1:T;
      if UseLevels==0;
         IRStructure.AL(t) = DSGEImpulseResponseFcn(H,Ftilde(:,:,t),Btilde(:,:,t),h);
      elseif UseLevels==1;
         IRStructure.AL(t) = DSGELevImpulseResponseFcn(H,Ftilde(:,:,t),Btilde(:,:,t),diag(1-DSGEModel.levels),h);
      else;
         IRStructure.AL(t) = DSGEImpulseResponseFcn(H,Ftilde(:,:,t),Btilde(:,:,t),h);
         IRFY = zeros(n,h+1,q);
         for i=1:h+1;
            for j=1:q;
               IRFY(:,i,j) = IRStructure.AL(t).Y(:,j,i);
            end;
         end;
         %
         % we can now annualize
         %
         for j=1:q;
            IRFY(:,:,j) = Annualize([zeros(DSGEModel.n,max(DSGEModel.annual)-1) IRFY(:,:,j)],DSGEModel.annual,DSGEModel.annualscale);
         end;
         %
         % now we save this to the structure IRStructure.AL(t).Y again
         %
         for i=1:h+1;
            for j=1:q;
               IRStructure.AL(t).Y(:,j,i) = IRFY(:,i,j);
            end;
         end;
      end;
   end;
else;
   [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
   Ht = H(:,:,LastPeriod);
   if UseLevels==0;
      IRStructure.RE = DSGEImpulseResponseFcn(Ht,F,B0,h);
   elseif UseLevels==1;
      IRStructure.RE = DSGELevImpulseResponseFcn(Ht,F,B0,diag(1-DSGEModel.levels),h);
   else;
      IRStructure.RE = DSGEImpulseResponseFcn(Ht,F,B0,h);
      IRFY = zeros(n,h+1,q);
      for i=1:h+1;
         for j=1:q;
            IRFY(:,i,j) = IRStructure.RE.Y(:,j,i);
         end;
      end;
      %
      % we can now annualize
      %
      for j=1:q;
         IRFY(:,:,j) = Annualize([zeros(DSGEModel.n,max(DSGEModel.annual)-1) IRFY(:,:,j)],DSGEModel.annual,DSGEModel.annualscale);
      end;
      %
      % now we save this to the structure IRStructure.RE.Y again
      %
      for i=1:h+1;
         for j=1:q;
            IRStructure.RE.Y(:,j,i) = IRFY(:,i,j);
         end;
      end;
   end;
   %
   % now we turn to adaptive learning
   %
   % first the steady-state IRFs
   %
   if UseLevels==0;
      IRStructure.ALSS = DSGEImpulseResponseFcn(Ht,Fss,Bss,h);
   elseif UseLevels==1;
      IRStructure.ALSS = DSGELevImpulseResponseFcn(Ht,Fss,Bss,diag(1-DSGEModel.levels),h);
   else;
      IRStructure.ALSS = DSGEImpulseResponseFcn(Ht,Fss,Bss,h);
      IRFY = zeros(n,h+1,q);
      for i=1:h+1;
         for j=1:q;
            IRFY(:,i,j) = IRStructure.ALSS.Y(:,j,i);
         end;
      end;
      %
      % we can now annualize
      %
      for j=1:q;
         IRFY(:,:,j) = Annualize([zeros(DSGEModel.n,max(DSGEModel.annual)-1) IRFY(:,:,j)],DSGEModel.annual,DSGEModel.annualscale);
      end;
      %
      % now we save this to the structure IRStructure.RE.Y again
      %
      for i=1:h+1;
         for j=1:q;
            IRStructure.ALSS.Y(:,j,i) = IRFY(:,i,j);
         end;
      end;
   end;
   %
   % and the the time-dependent IRFs
   %
   for t=1:T;
      if UseLevels==0;
         IRStructure.AL(t) = DSGEImpulseResponseFcn(H(:,:,t),Ftilde(:,:,t),Btilde(:,:,t),h);
      elseif UseLevels==1;
         IRStructure.AL(t) = DSGELevImpulseResponseFcn(H(:,:,t),Ftilde(:,:,t),Btilde(:,:,t),diag(1-DSGEModel.levels),h);
      else;
         IRStructure.AL(t) = DSGEImpulseResponseFcn(H(:,:,t),Ftilde(:,:,t),Btilde(:,:,t),h);
         IRFY = zeros(n,h+1,q);
         for i=1:h+1;
            for j=1:q;
               IRFY(:,i,j) = IRStructure.AL(t).Y(:,j,i);
            end;
         end;
         %
         % we can now annualize
         %
         for j=1:q;
            IRFY(:,:,j) = Annualize([zeros(DSGEModel.n,max(DSGEModel.annual)-1) IRFY(:,:,j)],DSGEModel.annual,DSGEModel.annualscale);
         end;
         %
         % now we save this to the structure IRStructure.AL(t).Y again
         %
         for i=1:h+1;
            for j=1:q;
               IRStructure.AL(t).Y(:,j,i) = IRFY(:,i,j);
            end;
         end;
      end;
   end;
end;

%
% end of AdaptiveLearningIRFtheta.m
%
