function AdaptiveLearningPosteriorModeEstimation(DSGEModel,CurrINI,controls)
% AdaptiveLearningPosteriorModeEstimation: Runs the posterior mode estimation procedures when
%                                          expectations formation is modelled by a Kalman filter
%                                          based adaptive learning mechanism.
%
% USAGE:
%
%       AdaptiveLearningPosteriorModeEstimation(DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT: DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure), with initialization information.
%
%                  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% REQUIRED OUTPUT: None currently.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 30, 2020
%                        This version: January 12, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 31-03-2020: Continued from the first calculation of the log posterior.
%
% * 01-04-2020: Finished the function from the beginning of the optimization.
%
% * 02-04-2020: Took into account that "ComputeSystemPriorMode" takes the input variable "IsAdaptiveLearning".
%
% * 06-04-2020: The function now saves the "BeliefSys" variable to the posterior mode mat-file.
%
% * 15-04-2020: Increased the display window for error message under csminwel, newrat and gmhmaxlik with ReturnCode -1.
%               Similarly for fminunc and ExitFlag being NaN.
%
% * 16-04-2020: Added information about the number of forward looking variables to the log window text.
%
% * 28-04-2020: Made sure that the PriorDist fields "beta_ab", "gamma_ab", "logistic_ab" and "gumbel_ab"
%               are initialized or filled with proper values before checking the prior of the adaptive
%               learning parameters.
%
% * 08-05-2020: Added the input variable mu to "AdaptiveLearningInitialValuesDSGE".
%
% * 18-05-2020: Added information about the Kalman filter and its initialization.
%
% * 02-07-2020: Removed the field "NonCentralGLSMatrix" from DSGEModel and the input variable "mu" for
%               "AdaptiveLearningInitialValuesDSGE".
%
% * 01-10-2020: Fixed a bug with j instead of : for X matrix.
%
% * 06-10-2020: Fixed a bug with a missing IsAdaptiveLearning variables for VillaniCheckOptimum.
%
% * 18-11-2020: Added extra information about initial values on eigenvalue when unstable.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 22-01-2021: Updated the documentation.
%
% * 29-01-2021: The function supports a user defined model solver under adaptive learning.
%
% * 20-07-2021: Fixed the Warning message with unstable adaptive learning solution at the
%               initial values.
%
% * 23-08-2021: Allow for the possibility of importing initial values of the parameters and
%               the inverse Hessian from a posterior mode results file.
%
% * 25-08-2021: Finished the code for properly importing parameter values and inverseHessian
%               from a posterior mode results file.
%
% * 30-08-2021: Importing of initial parameter values and the inverse Hessian is only asked
%               about when the user has selected maximization options 1 to 4, i.e. csminwel
%               or Marco Ratto's approach.
%
% * 07-10-2021: The estimation of the system prior mode before the posterior mode estimation
%               is now optional.
%
% * 10-12-2021: Added log information about the computation of the gradient.
%
% * 03-01-2022: Updated the documentation.
%
% * 11-01-2022: Fixed a system prior issue when updating ModelParameters.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed additional eval calls to feval.
%
% * 31-03-2022: The function now asks it the csminwel results should be used after running gmhmaxlik.
%               Made sure that "DSGEModel.InitializeParameterFile" exists before trying to run it.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-02-2023: Allow import of posterior mode estimates and inverse Hessian from a rational
%               expectations model that has exactly the same parameters as the adaptive learning
%               model except for the estimated learning parameters.
%
% * 09-02-2023: The estimation now provides the option of quitting after convergence when the
%               maximum absolute value of the gradient is greater than or equal to 10.
%
% * 29-03-2023: Changed the default path for importing initialization data in files.
%
% * 14-08-2023: The function supports an additional numerical gradient function.
%
% * 25-10-2023: Updated the documentation.
%
% * 07-11-2023: Made sure that the inverse Hessian with finite differences does not need to be
%               computed twice.
%
% * 08-11-2023: Removed a check for the same number of parameters when importing data from a previous
%               mat-file with adaptive learning estimates.
%
% * 08-01-2024: Updated the documentation.
%
% * 09-01-2024: Took into account that "YADAHessianMatrix" now takes the input variable "NumWorkers".
%
% * 12-01-2024: The IdleTimeout property of a parallel pool is set to Inf when computing the inverse Hessian
%               with finite differences and when asked to do so.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

EstimationStatus = 0;
ScreenMessages = '';
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
% set ShowFigs
%
if isempty(CurrINI.Sequence)==1;
   ShowFigs = 1;
else;
   ShowFigs = 0;
end;
%
% Check the file with prior information
%
SetLogText(strvcat(' ','Initializing posterior mode estimation...',[' * Checking "' GetFilename(DSGEModel.PriorFile) '" with data on prior...']));
%
[status,theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = VerifyPriorData(DSGEModel,CurrINI);
%
% Transform PriorDist.gamma and PriorDist.beta to the relevant parameters that are used by the logGammaPDF and
% the logBetaPDF functions
%
if isempty(PriorDist.beta)==0;
   [a,b] = MomentToParamStdbetaPDF((PriorDist.beta(:,1)-PriorDist.beta(:,3))./(PriorDist.beta(:,4)-PriorDist.beta(:,3)),PriorDist.beta(:,2)./(PriorDist.beta(:,4)-PriorDist.beta(:,3)));
   PriorDist.beta_ab = [a b];
else;
   PriorDist.beta_ab = [];
end;
%
if isempty(PriorDist.gamma)==0;
   [a,b] = MomentToParamGammaPDF(PriorDist.gamma(:,1)-PriorDist.gamma(:,3),PriorDist.gamma(:,2));
   PriorDist.gamma_ab = [a b];
else;
   PriorDist.gamma_ab = [];
end;
if isempty(PriorDist.logistic)==0;
   [a,b] = MomentToParamLogisticPDF(PriorDist.logistic(:,1),PriorDist.logistic(:,2),PriorDist.logistic(:,3));
   PriorDist.logistic_ab = [a b PriorDist.logistic(:,3)];
else;
   PriorDist.logistic_ab = [];
end;
if isempty(PriorDist.gumbel)==0;
   [a,b] = MomentToParamGumbelPDF(PriorDist.gumbel(:,1),PriorDist.gumbel(:,2));
   PriorDist.gumbel_ab = [a b];
else;
   PriorDist.gumbel_ab = [];
end;
%
if status==1;
   %
   % check the prior of the adaptive learning parameters
   %
   SetLogText([' * Checking "' GetFilename(DSGEModel.BeliefPriorFile) '" with data on prior...'])
   [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
   if status==1;
      [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                   thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                   LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
   else;
      SetLogText(strvcat('   STATUS: Bad prior file or inadequate data.',' '));
      return;
   end;
else;
   SetLogText(strvcat('   STATUS: Bad prior file or inadequate data.',' '));
   return;
end;
if isempty(theta)==1;
   SetLogText(strvcat('   STATUS: No parameters to estimate!',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = 'All parameters of your model have status calibrated. Hence, YADA doesn''t have any parameters to estimate.';
      About(txt,'information','Calibrated DSGE Model',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
OriginalModelParameters = ModelParameters;
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   IsSystemPrior = 1;
else;
   IsSystemPrior = 0;
end;
%
% check if we should also copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
   end;
end;
ErrorStr = '';
ErrorFile = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
end;
%
% The structure AIMData contains fields such as neq, nlead and nlag
%
firstUpdatePos = 0;
lastUpdatePos = 0;
firstInitializePos = 0;
lastInitializePos = 0;
if isempty(ErrorStr)==1;
   if (DSGEModel.RunInitializeFirst==1)&(FileExist(DSGEModel.InitializeParameterFile)==1);
      %
      % check if the file with additional parameters that only needs to be updated is OK
      %
      SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.InitializeParameterFile) '...']));
      %
      % we need to run the file DSGEModel.InitializeParameterFile first
      %
      if MatlabNumber>=8.0;
         try;
            firstInitializePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            lastInitializePos = length(fieldnames(ModelParameters));
         catch ME;
            ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError(ME)];
            ErrorFile = DSGEModel.InitializeParameterFile;
         end;
      else;
         try;
            firstInitializePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            lastInitializePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError];
            ErrorFile = DSGEModel.InitializeParameterFile;
         end;
      end;
      if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ErrorStr)==1);
         SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
         if MatlabNumber>=8.0;
            try;
               firstUpdatePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
               lastUpdatePos = length(fieldnames(ModelParameters));
            catch ME;
               ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError(ME)];
               ErrorFile = DSGEModel.UpdateParameterFile;
            end;
         else;
            try;
               firstUpdatePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
               lastUpdatePos = length(fieldnames(ModelParameters));
            catch;
               ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError];
               ErrorFile = DSGEModel.UpdateParameterFile;
            end;
         end;
      end;
   else;
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
         if MatlabNumber>=8.0;
            try;
               firstUpdatePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
               lastUpdatePos = length(fieldnames(ModelParameters));
            catch ME;
               ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError(ME)];
               ErrorFile = DSGEModel.UpdateParameterFile;
            end;
         else;
            try;
               firstUpdatePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
               lastUpdatePos = length(fieldnames(ModelParameters));
            catch;
               ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError];
               ErrorFile = DSGEModel.UpdateParameterFile;
            end;
         end;
      end;
      if (FileExist(DSGEModel.InitializeParameterFile)==1)&(isempty(ErrorStr)==1);
         SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.InitializeParameterFile) '...']));
         if MatlabNumber>=8.0;
            try;
               firstInitializePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
               lastInitializePos = length(fieldnames(ModelParameters));
            catch ME;
               ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError(ME)];
               ErrorFile = DSGEModel.InitializeParameterFile;
            end;
         else;
            try;
               firstInitializePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
               lastInitializePos = length(fieldnames(ModelParameters));
            catch;
               ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError];
               ErrorFile = DSGEModel.InitializeParameterFile;
            end;
         end;
      end;
   end;
end;
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
% we can go on with checking if the initial values permit a unique solution of the DSGE model
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE model has a unique convergent solution','   for initial parameter values under rational expectations...'));
%
% we can now run the DSGE model solver:
%
if DSGEModel.ModelSolver==1;
   SolverStr = 'AiM';
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein';
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Gensys';
elseif DSGEModel.ModelSolver==4;
   SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
end;
%
% To measure the solution time more accurately we call it twice, with the first call only being
% performed for checking if the model has a unique convergent solution at the initial values.
%
if MatlabNumber>=8.0;
   try;
      if DSGEModel.ModelSolver==1;
         [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
      elseif DSGEModel.ModelSolver==2;
         [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      elseif DSGEModel.ModelSolver==3;
         [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      elseif DSGEModel.ModelSolver==4;
         [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
      end;
   catch ME;
      if DSGEModel.ModelSolver==1;
         ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
      elseif DSGEModel.ModelSolver==2;
         ErrorStr = ['There was an error running Klein''s solver. YADA received the following message: ' YADALastError(ME)];
      elseif DSGEModel.ModelSolver==3;
         ErrorStr = ['There was an error running Sims'' gensys solver. YADA received the following message: ' YADALastError(ME)];
      elseif DSGEModel.ModelSolver==4;
         ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError(ME)];
      end;
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'error','AiM Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
      else;
         SetLogText(ErrorStr);
      end;
      return;
   end;
else;
   try;
      if DSGEModel.ModelSolver==1;
         [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
      elseif DSGEModel.ModelSolver==2;
         [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      elseif DSGEModel.ModelSolver==3;
         [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      elseif DSGEModel.ModelSolver==4;
         [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
      end;
   catch;
      if DSGEModel.ModelSolver==1;
         ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
      elseif DSGEModel.ModelSolver==2;
         ErrorStr = ['There was an error running Klein''s solver. YADA received the following message: ' YADALastError];
      elseif DSGEModel.ModelSolver==3;
         ErrorStr = ['There was an error running Sims'' gensys solver. YADA received the following message: ' YADALastError];
      elseif DSGEModel.ModelSolver==4;
         ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError];
      end;
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'error','AiM Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
      else;
         SetLogText(ErrorStr);
      end;
      return;
   end;
end;
if mcode~=1;
   %
   % There doesn't exist a unique convergent solution for the initial parameter values.
   %
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% check for explosive or unsupported unit roots
%
if isempty(DSGEModel.UnitRootStates)==1;
   F1 = F;
else;
   KeepPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
   F1 = F(KeepPos,KeepPos);
end;
%
if ~((max(abs(eig(F1)))<1)&(IsAlmostEqual(max(abs(eig(F1))),1)==0))&~((isempty(DSGEModel.UnitRootStates)==1)&(max(abs(eig(F1)))<=1));
   SetLogText(strvcat('   STATUS: Non-stationary state equation',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The largest eigenvalue of the state equation is ' num2str(max(abs(eig(F1)))) '. The state equation is therefore non-stationary. You need to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check how long it takes to solve the model
%
tic;
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==4;
   [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
SolveTime = toc;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% check the measurement equation file for internal validity
%
SetLogText(strvcat('   STATUS: OK',['   DSGE model solver:      ' SolverStr],['   Maximum absolute error: ' num2str(MaxAbsError)],['   Solution time:          ' num2str(SolveTime) ' seconds'],[' * Checking ' GetName(DSGEModel.MeasurementEquationFile) '...']));
if MatlabNumber>=8.0;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch ME;
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
      ErrorFile = DSGEModel.MeasurementEquationFile;
      SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
      end;
      return;
   end;
else;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch;
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
      ErrorFile = DSGEModel.MeasurementEquationFile;
      SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
      end;
      return;
   end;
end;
%
SetLogText(strvcat('   STATUS: OK',' * Checking the adaptive learning solution...'));
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
m = BeliefSys.m;
G = BeliefSys.G;
k = DSGEModel.k;
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
Stilde = [SMatrix Sperp];
%
% call the function: 
%
[tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if tstatus==0;
   SetLogText(strvcat('   STATUS: Unable to substitute for expected non-forward looking variables',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem.';
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% get estimation sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if k>0
   x = DSGEModel.X(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
else;
   x = [];
end;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
%
% solve the DSGE model under adaptive learning based on the initial values
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% make sure that DSGEModel has the required fields to AdaptiveLearning:
%
DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
DSGEModel.AdaptiveLearning.r = r;
DSGEModel.AdaptiveLearning.f = f;
DSGEModel.AdaptiveLearning.Sperp = Sperp;
DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
DSGEModel.AdaptiveLearning.Kgf = Kgf;
%
% check Ft1: if largest eigenvalue is greater than 1, give a warning
%
condn = 10^(-DSGEModel.AIMTolerance-2);
if max(abs(eig(Ft1)))>1+condn;
   SetLogText(strvcat('   WARNING: Adaptive learning solution is unstable at initial values.',['            Largest eigenvalue: ' num2str(max(abs(eig(Ft1))),'%0.4f')],'            Continuing'));
else;
   SetLogText('   STATUS: OK');
end;
%
% checking the system prior file if it exists
%
if IsSystemPrior==1;
   SetLogText(' * Checking if the system prior can be calculated...');
   if MatlabNumber>=8.0;
      try;
         lnSP = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
      catch ME;
         ErrorStr  = ['Unable to calculate the log of the system prior. Message caught is: ' YADALastError(ME)];
         ErrorFile = DSGEModel.SystemPriorFile;
         SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
            drawnow;
         end;
         return;
      end;
   else;
      try;
         lnSP = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
      catch;
         ErrorStr  = ['Unable to calculate the log of the system prior. Message caught is: ' YADALastError];
         ErrorFile = DSGEModel.SystemPriorFile;
         SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
            drawnow;
         end;
         return;
      end;
   end;
   SetLogText(strvcat('   STATUS: OK',' * Checking if the system prior mode has been calculated...'));
   SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
   if FileExist(SystemPriorModeFile)==1;
      SystemPriorModeData = load(SystemPriorModeFile);
      %
      % we only need the Laplace approximation of the log marginal likelihood of the system prior
      %
      LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
      SetLogText('   STATUS: OK');
   else;
      %
      % we need to first compute the system prior mode
      %
      if isempty(CurrINI.Sequence)==1;
         txt = strvcat('Would you like to estimate the system prior mode before the posterior mode?',' ','NOTE: The posterior mode results makes use of the Laplace approximation of the log marginal likelihood of the system prior. It is used to obtain a consistent estimate of the log marginal likelihood based on the posterior mode estimates.');
         answer = Query(txt,'question',500,'Posterior Mode Estimation',600,'yes',CurrINI);
      else;
         answer = 'yes';
      end;
      if strcmp(lower(answer),'yes')==1;
         [SystemPriorStatus,LogMargLikeSystemLaplace] = ComputeSystemPriorMode(theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,[firstUpdatePos lastUpdatePos firstInitializePos lastInitializePos],1,AIMData,controls,DSGEModel,CurrINI);
         if SystemPriorStatus==0;
            SetLogText('System prior mode estimation has been aborted.');
            return;
         else;
            SetLogText('   STATUS: OK');
         end;
      else;
         SetLogText('   STATUS: User decided to proceed without a system prior mode');
         LogMargLikeSystemLaplace = 0;
      end;
   end;
else;
   LogMargLikeSystemLaplace = 0;
end;
%
% The model has a unique solution for the initial values
%
MaxIterStr = get(controls.optimize.maximumiterations,'String');
MaxIterStr = StringTrim(strrep(MaxIterStr(DSGEModel.OptMaxIterationsValue,:),',',''));
MaxIter = str2num(MaxIterStr);
%
ToleranceStr = get(controls.optimize.tolerance,'String');
ToleranceStr = StringTrim(ToleranceStr(DSGEModel.OptToleranceValue,:));
ToleranceValue = str2num(ToleranceStr);
%
StepLengthStr = get(controls.optimize.steplength,'String');
StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
%
NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
%
% check if the user wants to import initial values for the parameters and the inverse Hessian matrix
%
if (DSGEModel.MaximizeAlgorithmValue<=4)&(isempty(CurrINI.Sequence)==1);
   txt = ['Would you like to import starting values for the parameters and the inverse Hessian from a posterior mode result file?'];
   answer = Query(txt,'question',500,'Posterior Mode Estimation',600,'yes',CurrINI);
else;
   answer = 'No';
end;
ImportInitialValues = 0;
if strcmp(lower(answer),'yes')==1;
   %
   % select the posterior mode file to import prior data from
   %
   if FileExist(DSGEModel.ALDSGEImportIVFile)==1;
      ImportModeFile = DSGEModel.ALDSGEImportIVFile;
   else;
      if MatlabNumber>=6;
         ImportModeFile = FixFilePath([DSGEModel.OutputDirectory '\']);
      else;
         ImportModeFile = FixFilePath([DSGEModel.OutputDirectory '\*.mat']);
      end;
   end;
   if MatlabNumber>=6;
      FileType = {'*.mat', 'Matlab mat Files (*.mat)'};
      [FileName,FilePath] = uigetfile(FileType,'Posterior Mode File - Select File',ImportModeFile);
   else;
      [FileName,FilePath] = uigetfile(ImportModeFile,'Posterior Mode File - Select File');
   end;
   drawnow;
   pause(0.02);
   %
   if (isnumeric(FileName)==0)&(isunix==1);
      if strcmp(FilePath(1,length(FilePath)),'/')==0;
         FilePath = [FilePath '/'];
      end;
   end;
   %
   % check if a file was selected
   %
   if (isnumeric(FileName)==0)&(FileExist([FilePath FileName])==1);
      ImportModeFile = [FilePath FileName];
      %
      % check if we can import data
      %
      if MatlabNumber>=8.0;
         try;
            ImportModeData = load(ImportModeFile);
         catch ME;
            ErrorStr = ['Unable to load the file "' ImportModeFile '". Message caught is: ' YADALastError(ME)];
         end;
      else;
         try;
            ImportModeData = load(ImportModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ImportModeFile '". Message caught is: ' YADALastError];
         end;
      end;
      if isempty(ErrorStr)==0;
         SetLogText(strvcat(' ',['Unable to import data from the file "' GetFilename(ImportModeFile) '"'],'Posterior mode estimation has been aborted.'));
         About(ErrorStr,'error',['Error - ' GetFilename(ImportModeFile)],CurrINI.scrsz(4)-50,600,CurrINI);
         return;
      end;
      if (isfield(ImportModeData,'thetaMode')==0)|(isfield(ImportModeData,'InverseHessian')==0);
         SetLogText(strvcat(' ',['Bad data in the file "' GetFilename(ImportModeFile) '"'],'Posterior mode estimation has been aborted.'));
         txt = ['The variable "thetaMode" or "InverseHessian" is missing from the file "' GetFilename(ImportModeFile) '". Posterior mode estimation has therefore been aborted.'];
         About(txt,'information',['Bad Data - ' GetFilename(ImportModeFile)],CurrINI.scrsz(4)-50,600,CurrINI);
         return;
      end;
      if (isfield(ImportModeData,'ParameterNames')==0)|(isfield(ImportModeData.ParameterNames,'estimated')==0);
         SetLogText(strvcat(' ',['Bad data in the file "' GetFilename(ImportModeFile) '"'],'Posterior mode estimation has been aborted.'));
         txt = ['The variable "ParameterNames" or "ParameterNames.estimated" is missing from the file "' GetFilename(ImportModeFile) '". Posterior mode estimation has therefore been aborted.'];
         About(txt,'information',['Bad Data - ' GetFilename(ImportModeFile)],CurrINI.scrsz(4)-50,600,CurrINI);
         return;
      end;
%      CheckAL = strfind(ImportModeFile,'-AdaptiveLearning');
      NumParam = length(theta);
%      NumImportParam = length(ImportModeData.thetaMode);
      ParamNames = ParameterNames.estimated;
      OriginalParamNames = ParameterNames.estimated;
      Positions = [];
      thetaLearn = [];
      phiLearn = [];
%      if isempty(CheckAL)==0;
%         if NumImportParam~=NumParam;
%            SetLogText(strvcat(' ',['Bad data in the file "' GetFilename(ImportModeFile) '"'],'Posterior mode estimation has been aborted.'));
%            txt = ['The number of parameters to estimate is ' int2str(NumImportParam) ' in the file "' GetFilename(ImportModeFile) '", while there should be ' int2str(NumParam) ' parameters. Posterior mode estimation has therefore been aborted.'];
%            About(txt,'information',['Bad Data - ' GetFilename(ImportModeFile)],CurrINI.scrsz(4)-50,600,CurrINI);
%            return;
%         end;
%      else;
         %
         % now we exclude comparisons of adaptive learning parameters
         %
         Positions = [FindStringRow(ParamNames,'rhoAL') FindStringRow(ParamNames,'sigmarAL') FindStringRow(ParamNames,'sigmeAL')];
         if isempty(Positions)==0;
            KeepParam = setdiff((1:NumParam),Positions);
            ParamNames = ParamNames(KeepParam,:);
            thetaLearn = theta(Positions);
            phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
            phiLearn = phi(Positions);
         end;
         NumParam = size(ParamNames,1);
%      end;
%      disp(isempty(Positions))
      %
      % check if the correct parameter names exist
      %
      CheckNames = 0;
      for ii=1:NumParam;
         OldName = StringTrim(ParamNames(ii,:));
         NewName = StringTrim(ImportModeData.ParameterNames.estimated(ii,:));
         CheckNames = CheckNames+strcmp(OldName,NewName);
      end;
      if CheckNames==NumParam;
         %
         % check if the InverseHessian is positive definite
         %
         if IsPositiveDefinite(ImportModeData.InverseHessian)==0;
            SetLogText(strvcat(' ',['Bad data in the file "' GetFilename(ImportModeFile) '"'],'Posterior mode estimation has been aborted.'));
            txt = ['The inverse Hessian matrix in the file "' GetFilename(ImportModeFile) '" isn''t positive definite. Posterior mode estimation has therefore been aborted.'];
            About(txt,'information',['Bad Data - ' GetFilename(ImportModeFile)],CurrINI.scrsz(4)-50,600,CurrINI);
            return;
         end;
         InitialInvHessMat = ImportModeData.InverseHessian;
         phi = ImportModeData.phiMode;
         theta = ImportModeData.thetaMode;
         if isempty(phiLearn)==0;
            phi = [phi;phiLearn];
            theta = [theta;thetaLearn];
            LearnParam = length(phiLearn);
            if DSGEModel.InitializeHessian==1;
               Const = 10;
            elseif DSGEModel.InitializeHessian==2;
               Const = 1;
            elseif DSGEModel.InitializeHessian==3;
               Const = 10^(-1);
            elseif DSGEModel.InitializeHessian==4;
               Const = 10^(-2);
            elseif DSGEModel.InitializeHessian==5;
               Const = 10^(-3);
            elseif DSGEModel.InitializeHessian==6;
               Const = 10^(-4);
            else;
               Const = 10^(-5);
            end;
            InitialInvHessMat = [InitialInvHessMat zeros(NumParam,LearnParam);zeros(LearnParam,NumParam) Const*eye(LearnParam)];
         end;
         %
         % check if we should fix the InverseHessian to comply with parameter transformations
         %
         if (IsOdd(ImportModeData.DSGEModel.MaximizeAlgorithmValue)==1)&(IsOdd(DSGEModel.MaximizeAlgorithmValue)==0);
            %
            % we need to transform the initial value of the inverse Hessian from the phi-space to the theta-space
            %
            ThetaPhiPartial = PartialThetaPartialPhi(phi,thetaIndex,UniformBounds);
            InitialInvHessMat = ThetaPhiPartial*InitialInvHessMat*ThetaPhiPartial;
         end;
         %
         % check if the inverse Hessian is good enough
         %
         txt = strvcat(['The maximum and minimum values of the diagonal of the inverse Hessian are: (' num2str(max(diag(InitialInvHessMat))) ',' num2str(min(diag(InitialInvHessMat))) '). Are you sure you want to use this inverse Hessian?'],' ','NOTE: If you answer "No" then YADA will initialize this matrix as selected on the GUI, but use the new parameter values.');
         answer = Query(txt,'question',500,'Posterior Mode Estimation',600,'yes',CurrINI);
         %
         if strcmp(lower(answer),'yes')==1;
            SetLogText(' * Importing initial parameter values and inverse Hessian');
            DSGEModel.ALDSGEImportIVFile = ImportModeFile;
            ImportInitialValues = 1;
         else;
            SetLogText(' * Importing initial parameter values');
         end;
      else;
         SetLogText(strvcat(' ',['Bad data in the file "' GetFilename(ImportModeFile) '"'],'Posterior mode estimation has been aborted.'));
         txt = ['The parameter names in the file "' GetFilename(ImportModeFile) '" doesn''t match the names used by your model. Posterior mode estimation has therefore been aborted.'];
         About(txt,'information',['Bad Data - ' GetFilename(ImportModeFile)],CurrINI.scrsz(4)-50,600,CurrINI);
         return;
      end;
      ParameterNames.estimated = OriginalParamNames;
   end;
else;
   DSGEModel.ALDSGEImportIVFile = pwd;
end;
%
% check the log posterior
%
SetLogText(' * Checking if the log posterior can be calculated...');
%
% Collect additional parameters in ParameterNames.additional
%
ParameterNames = ReadAdditionalParameterNames(ParameterNames,ModelParameters,OriginalModelParameters,[firstUpdatePos lastUpdatePos firstInitializePos lastInitializePos]);
%
% check the names of the calibrated and updated parameters
%
ParameterNames = CheckUpdateParameterNames(ParameterNames,ModelParameters,theta,thetaPositions,DSGEModel);
%
% set up vectors with calibrated parameters and additional parameters. These are only used for printing to the output file
%
[thetaCalibrated,thetaAdditionalInitial] = ReadAdditionalParameterValues(ModelParameters,ParameterNames);
%
% Determine the first observation to use when computing log-likelihood with the Kalman filter
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
%
NumParam = length(theta);
%
% get the upper and lower bounds of the parameters
%
ParameterBounds = CreateBounds(thetaDist,LowerBound,UniformBounds);
ParameterBounds = ParameterBounds(:,1:2);
%
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
      T = size(X,2);
      for j=1:k;
         ConstTest = (DSGEModel.X(j,DSGEModel.FirstPeriod:DSGEModel.LastPeriod)==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% check the parameter target type
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
   OptParameters = 'Transformed';
else;
   OptParameters = 'Original';
end;
%
% check if the log posterior can be computed
%
phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
ErrorStr = '';
%
if MatlabNumber>=8.0;
   try;
      tic;
      if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
         logPost = logPosteriorAdaptiveLearningPhiDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      else;
         logPost = logPosteriorAdaptiveLearningThetaDSGE(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         if logPost~=1000000;
            logPost = logPost-logJacobian(phi,thetaIndex,UniformBounds);
         end;
      end;
      LogPostTime = toc;
      LogPosteriorPhiIV = -logPost;
   catch ME;
      ErrorStr = ['Unable to calculate the log posterior. Message caught is: ' YADALastError(ME)];
      SetLogText(strvcat('   STATUS: Unable to calculate log posterior.',' '));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'error','Error - Log Posterior',500,600,CurrINI);
         drawnow;
      end;
      return;
   end;
else;
   try;
      tic;
      if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
         logPost = logPosteriorAdaptiveLearningPhiDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      else;
         logPost = logPosteriorAdaptiveLearningThetaDSGE(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         if logPost~=1000000;
            logPost = logPost-logJacobian(phi,thetaIndex,UniformBounds);
         end;
      end;
      LogPostTime = toc;
      LogPosteriorPhiIV = -logPost;
   catch;
      ErrorStr = ['Unable to calculate the log posterior. Message caught is: ' YADALastError];
      SetLogText(strvcat('   STATUS: Unable to calculate log posterior.',' '));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'error','Error - Log Posterior',500,600,CurrINI);
         drawnow;
      end;
      return;
   end;
end;
%
if ((exist('logPost')==0)|(logPost==1000000));
   %
   % check the log likelihood
   %
   drawnow;
   if (exist('logPost')==1)&(logPost==1000000);
      [logLikeValue,Solution,aimstatus,kalmanstatus] = logLikelihoodAdaptiveLearningDSGE(ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      if kalmanstatus==0;
         AddOnTxt = 'YADA was unable to compute the Kalman filter successfully for the adaptive learning version.';
         AddLogTxt = '           Kalman filter not run successfully. Check your PLM.';
      else;
         AddOnTxt = 'You probably need to change your initial parameter values.';
         AddLogTxt = '';
      end;
   else;
      AddLogTxt = '';
      AddOnTxt = 'You probably need to change your initial parameter values.';
   end;
   %
   SetLogText(strvcat('   STATUS: Unable to compute log posterior at initial values',AddLogTxt,' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['Unable to compute the log posterior at the initial values. ' AddOnTxt];
      About(txt,'information','YADA - DSGE Model Problem',300,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% Compute a numerical gradient
%
SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogPostTime) ' seconds'],' * Checking if the gradient can be calculated...'));
tic;
try;
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      [g,badg] = YADAnumgrad('logPosteriorAdaptiveLearningPhiDSGE',phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   else;
      [g,badg] = YADAnumgrad('logPosteriorAdaptiveLearningThetaDSGE',theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   end;
catch;
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      [g,badg] = YADANumGradient('logPosteriorAdaptiveLearningPhiDSGE',phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   else;
      [g,badg] = YADANumGradient('logPosteriorAdaptiveLearningThetaDSGE',theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   end;
end;
LogGradientTime = toc;
drawnow;
pause(0.1);
if badg~=0;
   SetLogText(strvcat('   STATUS: Bad gradient of log posterior at initial values',' '));
   [badpar,maxpos] = min(abs(g));
   %
   if badpar>0;
      ValueStr = num2str(badpar);
   else;
      ValueStr = 'NaN';
   end;
   %
   ParamStr = ['"' StringTrim(ParameterNames.estimated(maxpos(1),:)) '"'];
   AddParamStr = ' ';
   if length(maxpos)>=2;
      AddParamStr = 's ';
      for i=2:length(maxpos);
         ParamStr = [ParamStr ', "' StringTrim(ParameterNames.estimated(maxpos(i),:)) '"'];
      end;
   end;
   %
   if isempty(CurrINI.Sequence)==1;
      txt = ['Unable to compute a meaningful numerical gradient of the log posterior at the initial values. The largest absolute value or an improper value (NaN or Inf) of the gradient is found for the parameter' AddParamStr ParamStr ', with value of ' ValueStr '. You probably need to change your initial parameter values.'];
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check how the Kalman filter is initialized
%
if DSGEModel.UseDoublingAlgorithm==1;
   InitialAlgorithmText = '   Initialization method:  Analytic';
elseif DSGEModel.UseDoublingAlgorithm==2;
   InitialAlgorithmText = '   Initialization method:  Numeric (doubling algorithm)';
else;
   InitialAlgorithmText = '   Initialization method:  Constant times identity';
end;
%
% check which Kalman algorithm is used
%
if CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0;
   KalmanAlgorithmText = 'Double adaptive learning filter';
else;
   KalmanAlgorithmText = 'Single adaptive learning filter';
end;
%
% Starting the optimization
%
if (DSGEModel.MaximizeAlgorithmValue==1)|(DSGEModel.MaximizeAlgorithmValue==2);
   %
   % We use Sims' csminwel routine with transformed parameters.
   %
   OptAlgorithm = 'csminwel';
   %
   LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
   LogPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
   if IsSystemPrior==1;
      LogSPIV = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
      SystemPriorStr = ['   Log system prior value: ' num2str(LogSPIV,'%0.4f')];
   else;
      LogSPIV = 0;
      SystemPriorStr = '';
   end;
   LogPosteriorThetaIV = LogPosteriorPhiIV-LogJacobianIV;
   LogLikelihoodIV = LogPosteriorThetaIV-LogPriorIV-LogSPIV;
   %
   % compute the initial inverse Hessian unless such a matrix has already
   % been successfully imported
   %
   if ImportInitialValues==0;
      if DSGEModel.InitializeHessian==1;
         InitialInvHessMat = 10*eye(NumParam);
      elseif DSGEModel.InitializeHessian==2;
         InitialInvHessMat = 1*eye(NumParam);
      elseif DSGEModel.InitializeHessian==3;
         InitialInvHessMat = 10^(-1)*eye(NumParam);
      elseif DSGEModel.InitializeHessian==4;
         InitialInvHessMat = 10^(-2)*eye(NumParam);
      elseif DSGEModel.InitializeHessian==5;
         InitialInvHessMat = 10^(-3)*eye(NumParam);
      elseif DSGEModel.InitializeHessian==6;
         InitialInvHessMat = 10^(-4)*eye(NumParam);
      elseif DSGEModel.InitializeHessian==7;
         InitialInvHessMat = 10^(-5)*eye(NumParam);
      elseif DSGEModel.InitializeHessian==8;
         %
         % sample from the marginal prior
         %
         InitialInvHessMat = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));
      end;
   end;
   %
   % Deelete old error file if present
   %
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogGradientTime) ' seconds'],['   Kalman algorithm:       ' KalmanAlgorithmText],InitialAlgorithmText,['   Log posterior value:    ' num2str(LogPosteriorPhiIV,'%0.4f')],['   Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   Log likelihood value:   ' num2str(LogLikelihoodIV,'%0.4f')],' ','Optimization routine:',sprintf('   Sims'' csminwel for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Selected sample:                  ' DSGEModel.SubBeginYear ':' DSGEModel.SubBeginPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Initializing Kalman filter:       ' num2str(DSGEModel.KalmanFirstObservation-1,'%0.0f') ' observations'],[' * Effective sample size:            ' num2str(DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1),'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')],[' * Number of forward variables:      ' int2str(f)]));
   %
   if MatlabNumber>=8.0;
      try;
         tic;
         if DSGEModel.MaximizeAlgorithmValue==1;
            [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningPhiDSGE',phi,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
               thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         else;
            [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningThetaDSGE',theta,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
               thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            %
            % make sure that the outputs are transformed into phi-space
            %
            phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
            phiEstimates = zeros(size(thetaEstimates));
            for i=1:size(thetaEstimates,2);
               phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
            end;
            ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
            InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
         end;
         MaxTime = toc;
         GradientVector = abs(gh);
      catch ME;
         if DSGEModel.ShowProgress==1;
            WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
            if IsHandle(WaitHandle)==1;
               set(WaitHandle,'UserData','done');
               close(WaitHandle);
               drawnow;
               pause(0.02);
            end;
         else;
            WaitHandle = findobj('Type','figure','Tag','WaitDLG');
            WaitControls = get(WaitHandle,'UserData');
            set(WaitControls.text,'UserData','done');
            delete(WaitHandle);
            drawnow;
            pause(0.02);
         end;
         ReturnCode = -1;
         AbortEstimation = '';
         NumIter = [];
         GradientVector = [];
         ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' YADALastError(ME)];
      end;
   else;
      try;
         tic;
         if DSGEModel.MaximizeAlgorithmValue==1;
            [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningPhiDSGE',phi,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
               thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         else;
            [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningThetaDSGE',theta,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
               thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            %
            % make sure that the outputs are transformed into phi-space
            %
            phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
            phiEstimates = zeros(size(thetaEstimates));
            for i=1:size(thetaEstimates,2);
               phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
            end;
            ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
            InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
         end;
         MaxTime = toc;
         GradientVector = abs(gh);
      catch;
         if DSGEModel.ShowProgress==1;
            WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
            if IsHandle(WaitHandle)==1;
               set(WaitHandle,'UserData','done');
               close(WaitHandle);
               drawnow;
               pause(0.02);
            end;
         else;
            WaitHandle = findobj('Type','figure','Tag','WaitDLG');
            WaitControls = get(WaitHandle,'UserData');
            set(WaitControls.text,'UserData','done');
            delete(WaitHandle);
            drawnow;
            pause(0.02);
         end;
         ReturnCode = -1;
         AbortEstimation = '';
         NumIter = [];
         GradientVector = [];
         ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' YADALastError];
      end;
   end;
   %
   % check if the user would like to continue optimization using the new parameter values as input
   %
   if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter)&(max(GradientVector)>ToleranceValue)&(DSGEModel.CsminwelExtraRuns>1);
      MaxExtraRuns = DSGEModel.CsminwelExtraRuns-1;
      LastFH = -fh;
      if isempty(CurrINI.Sequence)==1;
         txt = strvcat(['According to the convergence criterion used by csminwel, i.e., the change in the value of the log posterior (' num2str(FuncDiff) ') is less than or equal to the tolerance value (' num2str(ToleranceValue) '), the optimization has converged. However, the maximum value of the gradient (' num2str(max(GradientVector)) ') is greater than the tolerance value. Would you like to continue running csminwel using the latest values for the parameters along with a reinitialized inverse Hessian as input?'],' ',['NOTE: The maximum number of extra runs of csminwel is set to ' int2str(MaxExtraRuns) '. YADA will also abort if the new value of the log posterior is smaller than the previous value. In the event that YADA aborts from the extra csminwel runs it will use the original csminwel results.']);
         answer = Query(txt,'question',500,'Posterior Mode Estimation - csminwel',600,'no',CurrINI);
      else;
         answer = CurrINI.Sequence.QueryAnswers{1};
      end;
      if strcmp(lower(answer),'yes')==1;
         %
         % select a suitable convergence criterior
         %
         TolStr = get(controls.optimize.tolerance,'String');
         [action,TolVal] = SelectionDlg(TolStr,DSGEModel.OptToleranceValue,'Tolerance value for the between runs change of optimal value given by csminwel:','Select Tolerance Value','Posterior Mode Estimation - csminwel','','',CurrINI);
         if strcmp(lower(action),'ok')==1;
            NewToleranceValue = num2str(StringTrim(TolStr(TolVal,:)));
            %
            % store all original results
            %
            OldResults.fh = fh;
            OldResults.phiMode = phiMode;
            OldResults.GradientVector = GradientVector;
            OldResults.InverseHessian = InverseHessian;
            OldResults.NumIter = NumIter;
            OldResults.fcount = fcount;
            OldResults.FuncDiff = FuncDiff;
            OldResults.OptResults = OptResults;
            OldResults.ScreenMessages = ScreenMessages;
            OldResults.phiEstimates = phiEstimates;
            OldResults.ReturnCode = ReturnCode;
            OldResults.MaxTime = MaxTime;
            NewResults = OldResults;
            NewResults.LastRun = 0;
            %
            NumExtraRuns = 1;
            done = 0;
            TotalIter = 0;
            while done==0;
               %
               % run csminwel again
               %
               SetLogText(strvcat(' ',['Extra csminwel run number ' int2str(NumExtraRuns) ' out of ' int2str(MaxExtraRuns)],[' * Last value of the log posterior:  ' num2str(-fh,'%0.4f')]));
               tic;
               if DSGEModel.MaximizeAlgorithmValue==1;
                  [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningPhiDSGE',phiMode,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                     thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
               else;
                  [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningThetaDSGE',thetaMode,InitialInvHessMat,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                     thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                  %
                  % make sure that the outputs are transformed into phi-space
                  %
                  phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
                  phiEstimates = zeros(size(thetaEstimates));
                  for i=1:size(thetaEstimates,2);
                     phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
                  end;
                  ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
                  InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
               end;
               MaxTime = MaxTime+toc;
               GradientVector = abs(gh);
               SetLogText(strvcat([' * New value of the log posterior:   ' num2str(-fh,'%0.4f')],[' * Number of iterations:             ' int2str(NumIter)],[' * Maximum value of the gradient:    ' num2str(max(GradientVector))],[' * Original value of the gradient:   ' num2str(max(OldResults.GradientVector))],[' * Tolerance value:                  ' num2str(NewToleranceValue)]));
               %
               % collect the results
               %
               TotalIter = TotalIter+NumIter;
               if isequal(InverseHessian,InitialInvHessMat)==0;
                  NewResults.LastRun = NumExtraRuns;
                  NewResults.fh = fh;
                  NewResults.phiMode = phiMode;
                  NewResults.GradientVector = GradientVector;
                  NewResults.InverseHessian = InverseHessian;
                  NewResults.phiEstimates = [NewResults.phiEstimates phiEstimates];
                  NewResults.ReturnCode = ReturnCode;
                  NewResults.NumIter = NewResults.NumIter+NumIter;
                  NewResults.fcount = NewResults.fcount+fcount;
                  NewResults.FuncDiff = FuncDiff;
                  NewResults.OptResults = [NewResults.OptResults;OptResults];
                  NewResults.ScreenMessages = strvcat(NewResults.ScreenMessages,' ',['Addition run number: ' int2str(NumExtraRuns)],ScreenMessages);
               end;
               NewResults.MaxTime = MaxTime;
               PosTestValue = (-fh-LastFH>=0);
               SmallTestValue = (-fh-LastFH<=NewToleranceValue);
               ConvTest = (PosTestValue==1)&(SmallTestValue);
               if (ConvTest==1)&(TotalIter>NumExtraRuns)&(strcmp(lower(AbortEstimation),'cancel')==0);
                  done = 1;
               elseif strcmp(lower(AbortEstimation),'cancel')==1;
                  done = 2;
               elseif PosTestValue==0;
                  done = 3;
               elseif NumExtraRuns==MaxExtraRuns;
                  done = 4;
               else;
                  NumExtraRuns = NumExtraRuns+1;
                  LastFH = -fh;
               end;
            end;
            if done==1;
               if (NewResults.LastRun>0)&(OldResults.fh-NewResults.fh>0);
                  if isempty(CurrINI.Sequence)==1;
                     txt = ['YADA has finished the extra runs. The new results are taken from extra run number ' int2str(NewResults.LastRun) '. The value of the log posterior has changed from ' num2str(-OldResults.fh,'%0.4f') ' to ' num2str(-NewResults.fh,'%0.4f') ', while the maximum value of the gradient was originally ' num2str(max(OldResults.GradientVector)) ' and is now ' num2str(max(NewResults.GradientVector)) '. At the same time, the smallest eigenvalue of the inverse Hessian was originally ' num2str(min(abs(eig(OldResults.InverseHessian)))) ' and is now ' num2str(min(abs(eig(NewResults.InverseHessian)))) '. Would you like to use the new results?'];
                     answer = Query(txt,'question',500,'Posterior Mode Estimation - csminwel',600,'no',CurrINI);
                  else;
                     answer = CurrINI.Sequence.QueryAnswers{2};
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     NumIter = NewResults.NumIter;
                     fcount = NewResults.fcount;
                     OptResults = NewResults.OptResults;
                     ScreenMessages = NewResults.ScreenMessages;
                     phiEstimates = NewResults.phiEstimates;
                     InverseHessian = NewResults.InverseHessian;
                     phiMode = NewResults.phiMode;
                     FuncDiff = NewResults.FuncDiff;
                     fh = NewResults.fh;
                     GradientVector = NewResults.GradientVector;
                     ReturnCode = NewResults.ReturnCode;
                  else;
                     fh = OldResults.fh;
                     phiMode = OldResults.phiMode;
                     GradientVector = OldResults.GradientVector;
                     InverseHessian = OldResults.InverseHessian;
                     NumIter = OldResults.NumIter;
                     fcount = OldResults.fcount;
                     FuncDiff = OldResults.FuncDiff;
                     OptResults = OldResults.OptResults;
                     ScreenMessages = OldResults.ScreenMessages;
                     phiEstimates = OldResults.phiEstimates;
                     ReturnCode = OldResults.ReturnCode;
                     MaxTime = OldResults.MaxTime;
                  end;
               else;
                  fh = OldResults.fh;
                  phiMode = OldResults.phiMode;
                  GradientVector = OldResults.GradientVector;
                  InverseHessian = OldResults.InverseHessian;
                  NumIter = OldResults.NumIter;
                  fcount = OldResults.fcount;
                  FuncDiff = OldResults.FuncDiff;
                  OptResults = OldResults.OptResults;
                  ScreenMessages = OldResults.ScreenMessages;
                  phiEstimates = OldResults.phiEstimates;
                  ReturnCode = OldResults.ReturnCode;
                  MaxTime = OldResults.MaxTime;
                  SetLogText(strvcat(' ','No real improvement: YADA will use the original csminwel results'));
               end;
            else;
               if done==4;
                  SetLogText(strvcat(' ','No real improvement: YADA will use the original csminwel results'));
               elseif done==3;
                  SetLogText(strvcat(' ','Lower log posterior value: YADA will use the original csminwel results'));
               elseif done==2;
                  SetLogText(strvcat(' ','User aborted: YADA will use the original csminwel results'));
               end;
               fh = OldResults.fh;
               phiMode = OldResults.phiMode;
               GradientVector = OldResults.GradientVector;
               InverseHessian = OldResults.InverseHessian;
               NumIter = OldResults.NumIter;
               fcount = OldResults.fcount;
               FuncDiff = OldResults.FuncDiff;
               OptResults = OldResults.OptResults;
               ScreenMessages = OldResults.ScreenMessages;
               phiEstimates = OldResults.phiEstimates;
               ReturnCode = OldResults.ReturnCode;
               MaxTime = OldResults.MaxTime;
            end;
            clear('OldResults','NewResults');
         else;
            SetLogText(strvcat(' ','User skipped extra runs: YADA will use the original csminwel results'));
         end;
      end;
   end;
   %
   % gh is the numerical gradient, fh the function value at minimum
   %
   % ReturnCode: 0 = minimum found
   %             1 = zero gradient
   %        2 or 4 = back and forth on step length never finished
   %             3 = smallest step still improving too slow
   %             5 = largest step still improving too fast
   %             6 = smallest step still improving too slow, reversed gradient
   %             7 = warning: possible inaccuracy in H matrix
   %
   if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
      %
      % we're OK
      %
      LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
      if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
         LogPostDensity = -fh;
      else;
         LogPostDensity = -fh+LogJacobianMode;
      end;
      %
      % Calculate the log posterior at theta
      %
      thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
      %
      LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
      %
      % make sure ModelParameters is updated
      %
      ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
      end;
      if IsSystemPrior==1;
         LogSPMode = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
         SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
      else;
         LogSPMode = 0;
         SystemPriorStr = '';
      end;
      LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
      %
      SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
      %
      EstimationStatus = 1;
      %
   elseif strcmp(lower(AbortEstimation),'cancel')==1;
      %
      % posteriod mode estimation was aborted
      %
      SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
   elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter>MaxIter);
      %
      % maximum number of iterations was reached
      %
      txt = ['The maximum number of iterations was reached without csminwel converging. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Maximum number of iteration reached.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: Maximum number of iterations reached.';
   elseif ReturnCode==1;
      txt = ['Csminwel return code: zero gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: zero gradient.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: zeros gradient.';
   elseif (ReturnCode==2)|(ReturnCode==4);
      txt = ['Csminwel return code: back and forth on step length never finished. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: back and forth on step length never finished.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: back and forth on step length never finished.';
   elseif ReturnCode==3;
      txt = ['Csminwel return code: smallest step still improving too slow. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: smallest step still improving too slow.';
   elseif ReturnCode==5;
      txt = ['Csminwel return code: largest step still improving too fast. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: largest step still improving too fast.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: largest step still improving too fast.';
   elseif ReturnCode==6;
      txt = ['Csminwel return code: smallest step still improving too slow, reversed gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: smallest step still improving too slow, reversed gradient.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: smallest step still improving too slow, reversed gradient.';
   elseif ReturnCode==7;
      txt = ['Csminwel return code: possible inaccuracy in inverse Hessian. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Csminwel return code: possible inaccuracy in inverse Hessian.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Csminwel return code: possible inaccuracy in inverse Hessian.';
   elseif ReturnCode==-1;
      SetLogText(strvcat(' ','Csminwel error: optimization ended abruptly.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',CurrINI.scrsz(4)-50,CurrINI.scrsz(3)/2,CurrINI);
         drawnow;
      end;
      return;
   end;
elseif (DSGEModel.MaximizeAlgorithmValue==3)|(DSGEModel.MaximizeAlgorithmValue==4);
   %
   % use Marco Ratto's newrat
   %
   OptAlgorithm = 'newrat';
   %
   % compute the initial inverse Hessian
   %
   if ImportInitialValues==0;
      if DSGEModel.InitializeHessian==1;
         HessianMat = 10^(-1)*eye(NumParam);
      elseif DSGEModel.InitializeHessian==2;
         HessianMat = eye(NumParam);
      elseif DSGEModel.InitializeHessian==3;
         HessianMat = 10*eye(NumParam);
      elseif DSGEModel.InitializeHessian==4;
         HessianMat = 10^2*eye(NumParam);
      elseif DSGEModel.InitializeHessian==5;
         HessianMat = 10^3*eye(NumParam);
      elseif DSGEModel.InitializeHessian==6;
         HessianMat = 10^4*eye(NumParam);
      elseif DSGEModel.InitializeHessian==7;
         HessianMat = 10^5*eye(NumParam);
      elseif DSGEModel.InitializeHessian==8;
         %
         % sample from the prior
         %
         HessianMat = eye(NumParam)/PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));
      end;
   end;
   %
   LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
   LogPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
   if IsSystemPrior==1;
      LogSPIV = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
      SystemPriorStr = ['   Log system prior value: ' num2str(LogSPIV,'%0.4f')];
   else;
      LogSPIV = 0;
      SystemPriorStr = '';
   end;
   LogPosteriorThetaIV = LogPosteriorPhiIV-LogJacobianIV;
   LogLikelihoodIV = LogPosteriorThetaIV-LogPriorIV-LogSPIV;
   %
   % create prior standard deviations
   %
   PriorStd = sqrt(diag(PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue))));
   %
   % call the function
   %
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogGradientTime) ' seconds'],['   Kalman algorithm:       ' KalmanAlgorithmText],InitialAlgorithmText,['   Log posterior value:    ' num2str(LogPosteriorPhiIV,'%0.4f')],['   Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   Log likelihood value:   ' num2str(LogLikelihoodIV,'%0.4f')],' ','Optimization routine:',sprintf('   Ratto''s newrat for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Selected sample:                  ' DSGEModel.SubBeginYear ':' DSGEModel.SubBeginPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Initializing Kalman filter:       ' num2str(DSGEModel.KalmanFirstObservation-1,'%0.0f') ' observations'],[' * Effective sample size:            ' num2str(DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1),'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')],[' * Number of forward variables:      ' int2str(f)]));
   %
   % run the estimation
   %
   if DSGEModel.MaximizeAlgorithmValue==3;
      ParamLUB = [-Inf*ones(NumParam,1) Inf*ones(NumParam,1)];
   else;
      ParamLUB = ParameterBounds;
   end;
   %
   if MatlabNumber>=8.0;
      try;
         tic;
         if DSGEModel.MaximizeAlgorithmValue==3;
            [phiMode,HessianMat,gh,fh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAnewrat('logPosteriorAdaptiveLearningPhiDSGE',phi,HessianMat,[],[],ToleranceValue,MaxIter,2,ParamLUB,PriorStd,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - newrat',CurrINI, ...
               thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         else;
            [thetaMode,HessianMat,gh,fh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAnewrat('logPosteriorAdaptiveLearningThetaDSGE',theta,HessianMat,[],[],ToleranceValue,MaxIter,2,ParamLUB,PriorStd,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - newrat',CurrINI, ...
               thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            %
            % make sure that the outputs are transformed into phi-space
            %
            phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
            phiEstimates = zeros(size(thetaEstimates));
            for i=1:size(thetaEstimates,2);
               phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
            end;
            ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
            InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
         end;
         %
         MaxTime = toc;
         GradientVector = abs(gh);
         if isinf(fh)==1;
            ReturnCode = -2;
            ErrorStr = 'The value of the objective function is infinite. Optimization aborted.';
         elseif isnan(fh)==1;
            ReturnCode = -3;
            ErrorStr = 'The objective function is a NaN value. Optimization aborted.';
         end;
      catch ME;
         if DSGEModel.ShowProgress==1;
            WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
            if IsHandle(WaitHandle)==1;
               set(WaitHandle,'UserData','done');
               close(WaitHandle);
               drawnow;
               pause(0.02);
            end;
         else;
            WaitHandle = findobj('Type','figure','Tag','WaitDLG');
            WaitControls = get(WaitHandle,'UserData');
            set(WaitControls.text,'UserData','done');
            delete(WaitHandle);
            drawnow;
            pause(0.02);
         end;
         ReturnCode = -1;
         AbortEstimation = '';
         NumIter = [];
         GradientVector = [];
         ErrorStr = ['There was an error running newrat. YADA caught the following error message: ' YADALastError(ME)];
      end;
   else;
      try;
         tic;
         if DSGEModel.MaximizeAlgorithmValue==3;
            [phiMode,HessianMat,gh,fh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAnewrat('logPosteriorAdaptiveLearningPhiDSGE',phi,HessianMat,[],[],ToleranceValue,MaxIter,2,ParamLUB,PriorStd,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - newrat',CurrINI, ...
               thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         else;
            [thetaMode,HessianMat,gh,fh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAnewrat('logPosteriorAdaptiveLearningThetaDSGE',theta,HessianMat,[],[],ToleranceValue,MaxIter,2,ParamLUB,PriorStd,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - newrat',CurrINI, ...
               thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            %
            % make sure that the outputs are transformed into phi-space
            %
            phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
            phiEstimates = zeros(size(thetaEstimates));
            for i=1:size(thetaEstimates,2);
               phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
            end;
            ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
            InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
         end;
         %
         MaxTime = toc;
         GradientVector = abs(gh);
         if isinf(fh)==1;
            ReturnCode = -2;
            ErrorStr = 'The value of the objective function is infinite. Optimization aborted.';
         elseif isnan(fh)==1;
            ReturnCode = -3;
            ErrorStr = 'The objective function is a NaN value. Optimization aborted.';
         end;
      catch;
         if DSGEModel.ShowProgress==1;
            WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
            if IsHandle(WaitHandle)==1;
               set(WaitHandle,'UserData','done');
               close(WaitHandle);
               drawnow;
               pause(0.02);
            end;
         else;
            WaitHandle = findobj('Type','figure','Tag','WaitDLG');
            WaitControls = get(WaitHandle,'UserData');
            set(WaitControls.text,'UserData','done');
            delete(WaitHandle);
            drawnow;
            pause(0.02);
         end;
         ReturnCode = -1;
         AbortEstimation = '';
         NumIter = [];
         GradientVector = [];
         ErrorStr = ['There was an error running newrat. YADA caught the following error message: ' YADALastError];
      end;
   end;
   %
   % gh is the numerical gradient, fh the function value at minimum
   %
   % ReturnCode: 0 = minimum found
   %             1 = zero gradient
   %        2 or 4 = back and forth on step length never finished
   %             3 = smallest step still improving too slow
   %             5 = largest step still improving too fast
   %             6 = smallest step still improving too slow, reversed gradient
   %             7 = warning: possible inaccuracy in H matrix
   %            -2 infinite value of function
   %            -3 NaN value of function
   %
   if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
      %
      % we're OK
      %
      LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
      if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
         LogPostDensity = -fh;
      else;
         LogPostDensity = -fh+LogJacobianMode;
      end;
      %
      % Calculate the log posterior at theta
      %
      thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
      %
      LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
      %
      % make sure ModelParameters is updated
      %
      ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
      end;
      if IsSystemPrior==1;
         LogSPMode = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
         SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
      else;
         LogSPMode = 0;
         SystemPriorStr = '';
      end;
      LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
      %
      SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
      %
      EstimationStatus = 1;
      %
   elseif strcmp(lower(AbortEstimation),'cancel')==1;
      %
      % posteriod mode estimation was aborted
      %
      SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
   elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter>MaxIter);
      %
      % maximum number of iterations was reached
      %
      txt = ['The maximum number of iterations was reached without newrat converging. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Maximum number of iteration reached.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: Maximum number of iterations reached.';
   elseif ReturnCode==1;
      txt = ['Newrat return code: zero gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: zero gradient.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: zeros gradient.';
   elseif (ReturnCode==2)|(ReturnCode==4);
      txt = ['Newrat return code: back and forth on step length never finished. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: back and forth on step length never finished.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: back and forth on step length never finished.';
   elseif ReturnCode==3;
      txt = ['Newrat return code: smallest step still improving too slow. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: smallest step still improving too slow.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: smallest step still improving too slow.';
   elseif ReturnCode==5;
      txt = ['Newrat return code: largest step still improving too fast. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: largest step still improving too fast.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: largest step still improving too fast.';
   elseif ReturnCode==6;
      txt = ['Newrat return code: smallest step still improving too slow, reversed gradient. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: smallest step still improving too slow, reversed gradient.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: smallest step still improving too slow, reversed gradient.';
   elseif ReturnCode==7;
      txt = ['Newrat return code: possible inaccuracy in inverse Hessian. YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
      SetLogText(strvcat(' ','Newrat return code: possible inaccuracy in inverse Hessian.'));
      if isempty(CurrINI.Sequence)==1;
         About(txt,'information','Optimization Problem',200,500,CurrINI);
         drawnow;
      end;
      ReturnCodeText = 'Newrat return code: possible inaccuracy in inverse Hessian.';
   elseif ReturnCode==-1;
      SetLogText(strvcat(' ','Newrat error: optimization ended abruptly.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',CurrINI.scrsz(4)-50,CurrINI.scrsz(3)/2,CurrINI);
         drawnow;
      end;
      return;
   elseif ReturnCode==-2;
      SetLogText(strvcat(' ','Newrat error: infinite value of objective function.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',500,600,CurrINI);
         drawnow;
      end;
      return;
   elseif ReturnCode==-3;
      SetLogText(strvcat(' ','Newrat error: NaN value of objective function.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',500,600,CurrINI);
         drawnow;
      end;
      return;
   end;
elseif (DSGEModel.MaximizeAlgorithmValue==5)|(DSGEModel.MaximizeAlgorithmValue==6);
   %
   % use Dynare's gmhmaxlik
   %
   OptAlgorithm = 'gmhmaxlik';
   %
   % determine the initial scale factor
   %
   ScaleStr = get(controls.posterior.scalefactor,'String');
   c = str2num(StringTrim(ScaleStr(DSGEModel.MHScaleFactor,:)));
   %
   % determine the number of draws
   %
   DrawsStr = get(controls.posterior.draws,'String');
   NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
   BurninStr = get(controls.posterior.burnin,'String');
   NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
   NumDraws = NumPostDraws-NumBurnin;
   %
   % initial covariance matrix
   %
   [InitialInvHessian,ParamMean] = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,DSGEModel.RandomNumberValue,IsOdd(DSGEModel.MaximizeAlgorithmValue));;
   %
   % parameter bounds
   %
   if DSGEModel.MaximizeAlgorithmValue==5;
      ParamLUB = [-Inf*ones(NumParam,1) Inf*ones(NumParam,1)];
   else;
      ParamLUB = ParameterBounds;
   end;
   %
   LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
   LogPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
   if IsSystemPrior==1;
      LogSPIV = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
      SystemPriorStr = ['   Log system prior value: ' num2str(LogSPIV,'%0.4f')];
   else;
      LogSPIV = 0;
      SystemPriorStr = '';
   end;
   LogPosteriorThetaIV = LogPosteriorPhiIV-LogJacobianIV;
   LogLikelihoodIV = LogPosteriorThetaIV-LogPriorIV-LogSPIV;
   %
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   %
   SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogGradientTime) ' seconds'],['   Kalman algorithm:       ' KalmanAlgorithmText],InitialAlgorithmText,['   Log posterior value:    ' num2str(LogPosteriorPhiIV,'%0.4f')],['   Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   Log likelihood value:   ' num2str(LogLikelihoodIV,'%0.4f')],' ','Optimization routine:',sprintf('   Dynare''s gmhmaxlik for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Number of Monte Carlo draws:      ' int2str(NumDraws)],[' * Selected sample:                  ' DSGEModel.SubBeginYear ':' DSGEModel.SubBeginPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Initializing Kalman filter:       ' num2str(DSGEModel.KalmanFirstObservation-1,'%0.0f') ' observations'],[' * Effective sample size:            ' num2str(DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1),'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')],[' * Number of forward variables:      ' int2str(f)]));
   %
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.MaximizeAlgorithmValue==5;
            [phiMode,InverseHessian,cEst,phiMean,fh,gh,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,MaxTime] = YADAgmhmaxlik('logPosteriorAdaptiveLearningPhiDSGE',phi,ParamLUB,NumDraws,c,ParamMean,InitialInvHessian,DSGEModel.RandomNumberValue,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - gmhmaxlik',CurrINI, ...
               thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         else;
            [thetaMode,InverseHessian,cEst,phiMean,fh,gh,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,MaxTime] = YADAgmhmaxlik('logPosteriorAdaptiveLearningThetaDSGE',theta,ParamLUB,NumDraws,c,ParamMean,InitialInvHessian,DSGEModel.RandomNumberValue,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - gmhmaxlik',CurrINI, ...
               thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            %
            % make sure that the outputs are transformed into phi-space
            %
            phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
            phiEstimates = zeros(size(thetaEstimates));
            for i=1:size(thetaEstimates,2);
               phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
            end;
            ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
            ThetaInverseHessian = InverseHessian;
            InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
         end;
         %
         GradientVector = abs(gh);
         ReturnCode = 0;
         NumIter = NumDraws;
      catch ME;
         if DSGEModel.ShowProgress==1;
            WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
            if IsHandle(WaitHandle)==1;
               set(WaitHandle,'UserData','done');
               close(WaitHandle);
               drawnow;
               pause(0.02);
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               if IsHandle(WaitHandle)==1;
                  WaitControls = get(WaitHandle,'UserData');
                  set(WaitControls.text,'UserData','done');
                  delete(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            end;
         else;
            WaitHandle = findobj('Type','figure','Tag','WaitDLG');
            if IsHandle(WaitHandle)==1;
               WaitControls = get(WaitHandle,'UserData');
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
            end;
         end;
         ReturnCode = -1;
         AbortEstimation = '';
         NumIter = [];
         GradientVector = [];
         ErrorStr = ['There was an error running gmhmaxlik. YADA caught the following error message: ' YADALastError(ME)];
      end;
   else;
      try;
         if DSGEModel.MaximizeAlgorithmValue==5;
            [phiMode,InverseHessian,cEst,phiMean,fh,gh,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,MaxTime] = YADAgmhmaxlik('logPosteriorAdaptiveLearningPhiDSGE',phi,ParamLUB,NumDraws,c,ParamMean,InitialInvHessian,DSGEModel.RandomNumberValue,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - gmhmaxlik',CurrINI, ...
               thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         else;
            [thetaMode,InverseHessian,cEst,phiMean,fh,gh,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,MaxTime] = YADAgmhmaxlik('logPosteriorAdaptiveLearningThetaDSGE',theta,ParamLUB,NumDraws,c,ParamMean,InitialInvHessian,DSGEModel.RandomNumberValue,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - gmhmaxlik',CurrINI, ...
               thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            %
            % make sure that the outputs are transformed into phi-space
            %
            phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
            phiEstimates = zeros(size(thetaEstimates));
            for i=1:size(thetaEstimates,2);
               phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
            end;
            ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
            ThetaInverseHessian = InverseHessian;
            InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
         end;
         %
         GradientVector = abs(gh);
         ReturnCode = 0;
         NumIter = NumDraws;
      catch;
         if DSGEModel.ShowProgress==1;
            WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
            if IsHandle(WaitHandle)==1;
               set(WaitHandle,'UserData','done');
               close(WaitHandle);
               drawnow;
               pause(0.02);
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               if IsHandle(WaitHandle)==1;
                  WaitControls = get(WaitHandle,'UserData');
                  set(WaitControls.text,'UserData','done');
                  delete(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            end;
         else;
            WaitHandle = findobj('Type','figure','Tag','WaitDLG');
            if IsHandle(WaitHandle)==1;
               WaitControls = get(WaitHandle,'UserData');
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
            end;
         end;
         ReturnCode = -1;
         AbortEstimation = '';
         NumIter = [];
         GradientVector = [];
         ErrorStr = ['There was an error running gmhmaxlik. YADA caught the following error message: ' YADALastError];
      end;
   end;
   %
   % check what we should do
   %
   if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1);
      %
      % check if the user wants to run csminwel using the gmhmaxlik results as input
      %
      if isempty(CurrINI.Sequence)==1;
         txt = 'Would you like to run csminwel once using the gmhmaxlik results as input?';
         answer = Query(txt,'question',200,'YADA - Csminwel Optimization',500,'no',CurrINI);
      else;
         answer = CurrINI.Sequence.QueryAnswers{5};
      end;
      if strcmp(lower(answer),'yes')==1;
         OldResults.fh = fh;
         OldResults.phiMode = phiMode;
         OldResults.GradientVector = GradientVector;
         OldResults.InverseHessian = InverseHessian;
         OldResults.NumIter = NumIter;
         OldResults.fcount = fcount;
         OldResults.FuncDiff = FuncDiff;
         OldResults.OptResults = OptResults;
         OldResults.ScreenMessages = ScreenMessages;
         OldResults.phiEstimates = phiEstimates;
         OldResults.ReturnCode = ReturnCode;
         OldResults.MaxTime = MaxTime;
         NewResults = OldResults;
         NewResults.LastRun = 0;
         if DSGEModel.MaximizeAlgorithmValue==5;
            if IsPositiveDefinite(InverseHessian)==0;
               OldResults.InverseHessian = abs(diag(diag(OldResults.InverseHessian)));
               SetLogText(strvcat(' ','NOTE: The inverse Hessian is not positive definite. Using only diagonal elements.'));
            end;
         else;
            OldResults.ThetaInverseHessian = ThetaInverseHessian;
            if IsPositiveDefinite(ThetaInverseHessian)==0;
               OldResults.ThetaInverseHessian = abs(diag(diag(OldResults.ThetaInverseHessian)));
               SetLogText(strvcat(' ','NOTE: The inverse Hessian is not positive definite. Using only diagonal elements.'));
            end;
         end;
         if MatlabNumber>=8.0;
            try;
               tic;
               if DSGEModel.MaximizeAlgorithmValue==5;
                  [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningPhiDSGE',phiMode,OldResults.InverseHessian,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                     thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
               else;
                  [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningThetaDSGE',thetaMode,OldResults.ThetaInverseHessian,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                     thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                  %
                  % make sure that the outputs are transformed into phi-space
                  %
                  phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
                  phiEstimates = zeros(size(thetaEstimates));
                  for i=1:size(thetaEstimates,2);
                     phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
                  end;
                  ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
                  InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
               end;
               if isempty(CurrINI.Sequence)==1;
                  txt = ['The value of max(abs(gradient)) is ' num2str(OptResults(size(OptResults,1),4)) ' and the objective function change is ' num2str(FuncDiff) '. Would you like to use the results from csminwel?'];
                  answer = Query(txt,'question',200,'YADA - Csminwel Optimization',500,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     AbortEstimation = 'cancel';
                  end;
               end;
               if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
                  MaxTime = MaxTime+toc;
                  GradientVector = abs(gh);
                  SetLogText(strvcat([' * Old value of the log posterior:   '  num2str(-OldResults.fh,'%0.4f')],' ',' * Running csminwel to climb even higher',[' * New value of the log posterior:   ' num2str(-fh,'%0.4f')],[' * Number of iterations:             ' int2str(NumIter)]));
                  NewResults.LastRun = NumIter;
                  NewResults.fh = fh;
                  NewResults.phiMode = phiMode;
                  NewResults.GradientVector = GradientVector;
                  NewResults.InverseHessian = InverseHessian;
                  NewResults.phiEstimates = [NewResults.phiEstimates phiEstimates];
                  NewResults.ReturnCode = ReturnCode;
                  NewResults.NumIter = NewResults.NumIter+NumIter;
                  NewResults.fcount = NewResults.fcount+fcount;
                  NewResults.FuncDiff = FuncDiff;
                  NewResults.OptResults = [NewResults.OptResults;OptResults];
                  NewResults.ScreenMessages = strvcat(NewResults.ScreenMessages,' ','Addition csminwel runs:',ScreenMessages);
                  %
                  PosTestValue = (-fh+OldResults.fh>=0);
                  if (PosTestValue==1)&(IsPositiveDefinite(InverseHessian)==1);
                     NumIter = NewResults.NumIter;
                     fcount = NewResults.fcount;
                     OptResults = NewResults.OptResults;
                     ScreenMessages = NewResults.ScreenMessages;
                     phiEstimates = NewResults.phiEstimates;
                     SetLogText(' * Using the new results from csminwel');
                  else;
                     fh = OldResults.fh;
                     phiMode = OldResults.phiMode;
                     GradientVector = OldResults.GradientVector;
                     InverseHessian = OldResults.InverseHessian;
                     NumIter = OldResults.NumIter;
                     fcount = OldResults.fcount;
                     FuncDiff = OldResults.FuncDiff;
                     OptResults = OldResults.OptResults;
                     ScreenMessages = OldResults.ScreenMessages;
                     phiEstimates = OldResults.phiEstimates;
                     ReturnCode = OldResults.ReturnCode;
                     MaxTime = OldResults.MaxTime;
                     if PosTestValue==1;
                        SetLogText(strvcat('NOTE: Bad inverse Hessian for csminwel',' * Using the results from gmhmaxlik only'));
                     else;
                        SetLogText(strvcat('NOTE: Log posterior kernel value not improving',' * Using the results from gmhmaxlik only'));
                     end;
                  end;
               else;
                  toc;
                  fh = OldResults.fh;
                  phiMode = OldResults.phiMode;
                  GradientVector = OldResults.GradientVector;
                  InverseHessian = OldResults.InverseHessian;
                  NumIter = OldResults.NumIter;
                  fcount = OldResults.fcount;
                  FuncDiff = OldResults.FuncDiff;
                  OptResults = OldResults.OptResults;
                  ScreenMessages = OldResults.ScreenMessages;
                  phiEstimates = OldResults.phiEstimates;
                  ReturnCode = OldResults.ReturnCode;
                  MaxTime = OldResults.MaxTime;
                  SetLogText(' * Using the results from gmhmaxlik only');
               end;
            catch ME;
               if DSGEModel.ShowProgress==1;
                  WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
                  if IsHandle(WaitHandle)==1;
                     set(WaitHandle,'UserData','done');
                     close(WaitHandle);
                     drawnow;
                     pause(0.02);
                  else;
                     WaitHandle = findobj('Type','figure','Tag','WaitDLG');
                     if IsHandle(WaitHandle)==1;
                        WaitControls = get(WaitHandle,'UserData');
                        set(WaitControls.text,'UserData','done');
                        delete(WaitHandle);
                        drawnow;
                        pause(0.02);
                     end;
                  end;
               else;
                  WaitHandle = findobj('Type','figure','Tag','WaitDLG');
                  if IsHandle(WaitHandle)==1;
                     WaitControls = get(WaitHandle,'UserData');
                     set(WaitControls.text,'UserData','done');
                     delete(WaitHandle);
                     drawnow;
                     pause(0.02);
                  end;
               end;
               ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' YADALastError(ME)];
            end;
         else;
            try;
               tic;
               if DSGEModel.MaximizeAlgorithmValue==5;
                  [fh,phiMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,phiEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningPhiDSGE',phiMode,OldResults.InverseHessian,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                     thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
               else;
                  [fh,thetaMode,gh,InverseHessian,NumIter,fcount,FuncDiff,AbortEstimation,OptResults,ScreenMessages,thetaEstimates,ReturnCode] = YADAcsminwel('logPosteriorAdaptiveLearningThetaDSGE',thetaMode,OldResults.ThetaInverseHessian,[],ToleranceValue,MaxIter,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - csminwel',CurrINI, ...
                     thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                  %
                  % make sure that the outputs are transformed into phi-space
                  %
                  phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
                  phiEstimates = zeros(size(thetaEstimates));
                  for i=1:size(thetaEstimates,2);
                     phiEstimates(:,i) = ThetaToPhi(thetaEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
                  end;
                  ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
                  InverseHessian = (ThetaPhiPartial\InverseHessian)/ThetaPhiPartial;
               end;
               if isempty(CurrINI.Sequence)==1;
                  txt = ['The value of max(abs(gradient)) is ' num2str(OptResults(size(OptResults,1),4)) ' and the objective function change is ' num2str(FuncDiff) '. Would you like to use the results from csminwel?'];
                  answer = Query(txt,'question',200,'YADA - Csminwel Optimization',500,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     AbortEstimation = 'cancel';
                  end;
               end;
               if (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')~=1)&(NumIter<=MaxIter);
                  MaxTime = MaxTime+toc;
                  GradientVector = abs(gh);
                  SetLogText(strvcat([' * Old value of the log posterior:   '  num2str(-OldResults.fh,'%0.4f')],' ',' * Running csminwel to climb even higher',[' * New value of the log posterior:   ' num2str(-fh,'%0.4f')],[' * Number of iterations:             ' int2str(NumIter)]));
                  NewResults.LastRun = NumIter;
                  NewResults.fh = fh;
                  NewResults.phiMode = phiMode;
                  NewResults.GradientVector = GradientVector;
                  NewResults.InverseHessian = InverseHessian;
                  NewResults.phiEstimates = [NewResults.phiEstimates phiEstimates];
                  NewResults.ReturnCode = ReturnCode;
                  NewResults.NumIter = NewResults.NumIter+NumIter;
                  NewResults.fcount = NewResults.fcount+fcount;
                  NewResults.FuncDiff = FuncDiff;
                  NewResults.OptResults = [NewResults.OptResults;OptResults];
                  NewResults.ScreenMessages = strvcat(NewResults.ScreenMessages,' ','Addition csminwel runs:',ScreenMessages);
                  %
                  PosTestValue = (-fh+OldResults.fh>=0);
                  if (PosTestValue==1)&(IsPositiveDefinite(InverseHessian)==1);
                     NumIter = NewResults.NumIter;
                     fcount = NewResults.fcount;
                     OptResults = NewResults.OptResults;
                     ScreenMessages = NewResults.ScreenMessages;
                     phiEstimates = NewResults.phiEstimates;
                     SetLogText(' * Using the new results from csminwel');
                  else;
                     fh = OldResults.fh;
                     phiMode = OldResults.phiMode;
                     GradientVector = OldResults.GradientVector;
                     InverseHessian = OldResults.InverseHessian;
                     NumIter = OldResults.NumIter;
                     fcount = OldResults.fcount;
                     FuncDiff = OldResults.FuncDiff;
                     OptResults = OldResults.OptResults;
                     ScreenMessages = OldResults.ScreenMessages;
                     phiEstimates = OldResults.phiEstimates;
                     ReturnCode = OldResults.ReturnCode;
                     MaxTime = OldResults.MaxTime;
                     if PosTestValue==1;
                        SetLogText(strvcat('NOTE: Bad inverse Hessian for csminwel',' * Using the results from gmhmaxlik only'));
                     else;
                        SetLogText(strvcat('NOTE: Log posterior kernel value not improving',' * Using the results from gmhmaxlik only'));
                     end;
                  end;
               else;
                  toc;
                  fh = OldResults.fh;
                  phiMode = OldResults.phiMode;
                  GradientVector = OldResults.GradientVector;
                  InverseHessian = OldResults.InverseHessian;
                  NumIter = OldResults.NumIter;
                  fcount = OldResults.fcount;
                  FuncDiff = OldResults.FuncDiff;
                  OptResults = OldResults.OptResults;
                  ScreenMessages = OldResults.ScreenMessages;
                  phiEstimates = OldResults.phiEstimates;
                  ReturnCode = OldResults.ReturnCode;
                  MaxTime = OldResults.MaxTime;
                  SetLogText(' * Using the results from gmhmaxlik only');
               end;
            catch;
               if DSGEModel.ShowProgress==1;
                  WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
                  if IsHandle(WaitHandle)==1;
                     set(WaitHandle,'UserData','done');
                     close(WaitHandle);
                     drawnow;
                     pause(0.02);
                  else;
                     WaitHandle = findobj('Type','figure','Tag','WaitDLG');
                     if IsHandle(WaitHandle)==1;
                        WaitControls = get(WaitHandle,'UserData');
                        set(WaitControls.text,'UserData','done');
                        delete(WaitHandle);
                        drawnow;
                        pause(0.02);
                     end;
                  end;
               else;
                  WaitHandle = findobj('Type','figure','Tag','WaitDLG');
                  if IsHandle(WaitHandle)==1;
                     WaitControls = get(WaitHandle,'UserData');
                     set(WaitControls.text,'UserData','done');
                     delete(WaitHandle);
                     drawnow;
                     pause(0.02);
                  end;
               end;
               ErrorStr = ['There was an error running csminwel. YADA caught the following error message: ' YADALastError];
            end;
         end;
      else;
         OldResults = [];
      end;
      %
      % we're OK
      %
      LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
      if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
         LogPostDensity = -fh;
      else;
         LogPostDensity = -fh+LogJacobianMode;
      end;
      %
      % Calculate the log posterior at theta
      %
      thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
      %
      LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
      %
      % make sure ModelParameters is updated
      %
      ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
      end;
      if IsSystemPrior==1;
         LogSPMode = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
         SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
      else;
         LogSPMode = 0;
         SystemPriorStr = '';
      end;
      LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
      %
      SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Objective function change:        ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(size(OptResults,1),4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
      %
      EstimationStatus = 1;
      %
   elseif (ReturnCode==0)&(strcmp(lower(AbortEstimation),'cancel')==1);
      %
      % the user selected to abort posterior mode estimation
      %
      SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
   elseif ReturnCode==-1;
      SetLogText(strvcat(' ','Gmhmaxlik error: optimization ended abruptly.'));
      if isempty(CurrINI.Sequence)==1;
         About(ErrorStr,'information','Optimization Problem',CurrINI.scrsz(4)-50,CurrINI.scrsz(3)/2,CurrINI);
         drawnow;
      end;
      return;
   end;
elseif (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
   %
   % use matlab's fminunc
   %
   OptAlgorithm = 'fminunc';
   %
   LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
   LogPriorIV = logPriorDSGE(PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound),thetaDist,PriorDist,LowerBound);
   if IsSystemPrior==1;
      LogSPIV = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
      SystemPriorStr = ['   Log system prior value: ' num2str(LogSPIV,'%0.4f')];
   else;
      LogSPIV = 0;
      SystemPriorStr = '';
   end;
   LogPosteriorThetaIV = LogPosteriorPhiIV-LogJacobianIV;
   LogLikelihoodIV = LogPosteriorThetaIV-LogPriorIV-LogSPIV;
   %
   % set the options
   %
   Options = optimset('MaxIter',MaxIter,'MaxFunEvals',100*MaxIter,'Diagnostics','off','LargeScale','off','TolFun',ToleranceValue,'TolX',ToleranceValue);
   %
   % call the function
   %
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogGradientTime) ' seconds'],['   Kalman algorithm:       ' KalmanAlgorithmText],InitialAlgorithmText,['   Log posterior value:    ' num2str(LogPosteriorPhiIV,'%0.4f')],['   Log prior value:        ' num2str(LogPriorIV,'%0.4f')],['   Log Jacobian value:     ' num2str(LogJacobianIV,'%0.4f')],SystemPriorStr,['   Log likelihood value:   ' num2str(LogLikelihoodIV,'%0.4f')],' ','Optimization routine:',sprintf('   Matlab''s fminunc for %s parameters...',lower(OptParameters)),[' * Time:                             ' StringTrim(datestr(now,14))],[' * Number of parameters to estimate: ' num2str(NumParam)],[' * Total number of parameters:       ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Tolerance level:                  ' ToleranceStr],[' * Maximum number of iterations:     ' MaxIterStr],[' * Selected sample:                  ' DSGEModel.SubBeginYear ':' DSGEModel.SubBeginPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Initializing Kalman filter:       ' num2str(DSGEModel.KalmanFirstObservation-1,'%0.0f') ' observations'],[' * Effective sample size:            ' num2str(DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1),'%0.0f')],[' * Estimation sample:                ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:     ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:    ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:        ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:           ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:     ' num2str(rank(R),'%0.0f')],[' * Number of forward variables:      ' int2str(f)]));
   %
   if MatlabNumber>=7;
      if MatlabNumber>=8.0;
         try;
            tic;
            if DSGEModel.MaximizeAlgorithmValue==7;
               [phiMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc7('logPosteriorAdaptiveLearningPhiDSGE',phi,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            else;
               [thetaMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc7('logPosteriorAdaptiveLearningThetaDSGE',theta,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(OUTPUT.phiEstimates));
               for i=1:size(OUTPUT.phiEstimates,2);
                  phiEstimates(:,i) = ThetaToPhi(OUTPUT.phiEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
               end;
               OUTPUT.phiEstimates = phiEstimates;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               HessianMatrix = ThetaPhiPartial*HessianMatrix*ThetaPhiPartial;
            end;
            MaxTime = toc;
            GradientVector = abs(Grad);
            AbortEstimation = OUTPUT.abort;
            NumIter = OUTPUT.iterations;
            OptResults = OUTPUT.OptResults;
         catch ME;
            if DSGEModel.ShowProgress==1;
               WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
               if IsHandle(WaitHandle)==1;
                  set(WaitHandle,'UserData','done');
                  close(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               WaitControls = get(WaitHandle,'UserData');
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
            end;
            ExitFlag = NaN;
            AbortEstimation = '';
            NumIter = [];
            GradientVector = [];
            ErrorStr = ['There was an error running fminunc. YADA caught the following error message: ' YADALastError(ME)];
         end;
      else;
         try;
            tic;
            if DSGEModel.MaximizeAlgorithmValue==7;
               [phiMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc7('logPosteriorAdaptiveLearningPhiDSGE',phi,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            else;
               [thetaMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc7('logPosteriorAdaptiveLearningThetaDSGE',theta,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(OUTPUT.phiEstimates));
               for i=1:size(OUTPUT.phiEstimates,2);
                  phiEstimates(:,i) = ThetaToPhi(OUTPUT.phiEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
               end;
               OUTPUT.phiEstimates = phiEstimates;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               HessianMatrix = ThetaPhiPartial*HessianMatrix*ThetaPhiPartial;
            end;
            MaxTime = toc;
            GradientVector = abs(Grad);
            AbortEstimation = OUTPUT.abort;
            NumIter = OUTPUT.iterations;
            OptResults = OUTPUT.OptResults;
         catch;
            if DSGEModel.ShowProgress==1;
               WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
               if IsHandle(WaitHandle)==1;
                  set(WaitHandle,'UserData','done');
                  close(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               WaitControls = get(WaitHandle,'UserData');
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
            end;
            ExitFlag = NaN;
            AbortEstimation = '';
            NumIter = [];
            GradientVector = [];
            ErrorStr = ['There was an error running fminunc. YADA caught the following error message: ' YADALastError];
         end;
      end;
      if (ExitFlag>0)&(strcmp(lower(AbortEstimation),'cancel')~=1);
         %
         % we're OK
         %
         FuncDiff = OUTPUT.convergence;
         LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
         if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
            LogPostDensity = -fh;
            thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
         else;
            LogPostDensity = -fh+LogJacobianMode;
         end;
         InverseHessian = HessianMatrix\eye(NumParam);
         phiEstimates = OUTPUT.phiEstimates;
         %
         % compute the value of the prior and of the likelihood         
         %
         LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
         %
         % make sure ModelParameters is updated
         %
         ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
         if FileExist(DSGEModel.UpdateParameterFile)==1;
            ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
         end;
         if IsSystemPrior==1;
            LogSPMode = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
            SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
         else;
            LogSPMode = 0;
            SystemPriorStr = '';
         end;
         LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
         %
         SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
         %
         EstimationStatus = 1;
         %
      elseif strcmp(lower(AbortEstimation),'cancel')==1;
         %
         % posteriod mode estimation was aborted
         %
         SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
      elseif ExitFlag==0;
         %
         % Reached maximum number of iteration
         %
         txt = ['The maximum number of function evaluations or iterations was reached without fminunc converging (ExitFlag is 0). YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
         SetLogText(strvcat(' ','Maximum number of function evaluations or iteration reached.',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
         if isempty(CurrINI.Sequence)==1;
            About(txt,'information','Optimization Problem',200,500,CurrINI);
            drawnow;
         end;
         ReturnCodeText = 'Fminunc return code: the maximum number of function evaluations or iteration was reached without fminunc coverging (ExitFlag is 0).';
      elseif ExitFlag<0;
         %
         % no convergence
         %
         txt = ['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) '). YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
         SetLogText(strvcat(' ',['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) ').'],[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
         if isempty(CurrINI.Sequence)==1;
            About(txt,'information','Optimization Problem',200,500,CurrINI);
            drawnow;
         end;
         ReturnCodeText = ['Fminunc return code: No convergence (ExitFlag is ' int2str(ExitFlag) ').'];
      elseif isnan(ExitFlag)==1;
         SetLogText(strvcat(' ','Fminunc error: optimization ended abruptly.'));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'information','Optimization Problem',CurrINI.scrsz(4)-50,CurrINI.scrsz(3)/2,CurrINI);
            drawnow;
         end;
         return;
      end;
   else;
      if MatlabNumber>=8.0;
         try;
            tic;
            if DSGEModel.MaximizeAlgorithmValue==7;
               [phiMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc5('logPosteriorAdaptiveLearningPhiDSGE',phi,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            else;
               [thetaMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc5('logPosteriorAdaptiveLearningThetaDSGE',theta,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(OUTPUT.phiEstimates));
               for i=1:size(OUTPUT.phiEstimates,2);
                  phiEstimates(:,i) = ThetaToPhi(OUTPUT.phiEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
               end;
               OUTPUT.phiEstimates = phiEstimates;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               HessianMatrix = ThetaPhiPartial*HessianMatrix*ThetaPhiPartial;
            end;
            MaxTime = toc;
            AbortEstimation = OUTPUT.abort;
            NumIter = OUTPUT.iterations;
            OptResults = OUTPUT.OptResults;
            GradientVector = abs(Grad);
         catch ME;
            if DSGEModel.ShowProgress==1;
               WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
               if IsHandle(WaitHandle)==1;
                  set(WaitHandle,'UserData','done');
                  close(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               WaitControls = get(WaitHandle,'UserData');
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
            end;
            ExitFlag = NaN;
            AbortEstimation = '';
            NumIter = [];
            GradientVector = [];
            ErrorStr = ['There was an error running fminunc. YADA caught the following error message: ' YADALastError(ME)];
         end;
      else;
         try;
            tic;
            if DSGEModel.MaximizeAlgorithmValue==7;
               [phiMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc5('logPosteriorAdaptiveLearningPhiDSGE',phi,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
            else;
               [thetaMode,fh,ExitFlag,OUTPUT,Grad,HessianMatrix] = YADAfminunc5('logPosteriorAdaptiveLearningThetaDSGE',theta,Options,DSGEModel.ShowProgress,DSGEModel.ShowProgressClock,'Posterior Mode Estimation - fminunc',CurrINI, ...
                  thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
               %
               % make sure that the outputs are transformed into phi-space
               %
               phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
               phiEstimates = zeros(size(OUTPUT.phiEstimates));
               for i=1:size(OUTPUT.phiEstimates,2);
                  phiEstimates(:,i) = ThetaToPhi(OUTPUT.phiEstimates(:,i),thetaIndex,UniformBounds,LowerBound);;
               end;
               OUTPUT.phiEstimates = phiEstimates;
               ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
               HessianMatrix = ThetaPhiPartial*HessianMatrix*ThetaPhiPartial;
            end;
            MaxTime = toc;
            AbortEstimation = OUTPUT.abort;
            NumIter = OUTPUT.iterations;
            OptResults = OUTPUT.OptResults;
            GradientVector = abs(Grad);
         catch;
            if DSGEModel.ShowProgress==1;
               WaitHandle = findobj('Type','figure','Tag','TMWProgressDLG');
               if IsHandle(WaitHandle)==1;
                  set(WaitHandle,'UserData','done');
                  close(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
            else;
               WaitHandle = findobj('Type','figure','Tag','WaitDLG');
               WaitControls = get(WaitHandle,'UserData');
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               drawnow;
               pause(0.02);
            end;
            ExitFlag = NaN;
            AbortEstimation = '';
            NumIter = [];
            GradientVector = [];
            ErrorStr = ['There was an error running fminunc. YADA caught the following error message: ' YADALastError];
         end;
      end;
      if (ExitFlag>0)&(strcmp(lower(AbortEstimation),'cancel')~=1);
         %
         % we're OK
         %
         FuncDiff = OUTPUT.convergence;
         LogJacobianMode = logJacobian(phiMode,thetaIndex,UniformBounds);
         if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
            LogPostDensity = -fh;
            thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
         else;
            LogPostDensity = -fh+LogJacobianMode;
         end;
         InverseHessian = HessianMatrix\eye(NumParam);
         phiEstimates = OUTPUT.phiEstimates;
         %
         % compute the value of the prior density and the likelihood         
         %
         LogPriorMode = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
         %
         % make sure ModelParameters is updated
         %
         ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
         if FileExist(DSGEModel.UpdateParameterFile)==1;
            ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
         end;
         if IsSystemPrior==1;
            LogSPMode = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
            SystemPriorStr = [' * Log system prior at mode:         ' num2str(LogSPMode,'%0.4f')];
         else;
            LogSPMode = 0;
            SystemPriorStr = '';
         end;
         LogLikelihoodMode = LogPostDensity-(LogPriorMode+LogJacobianMode+LogSPMode);
         %
         SetLogText(strvcat(' ','Optimization completed!',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)'],[' * Number of iterations:             ' num2str(NumIter)],[' * Convergence value:                ' num2str(FuncDiff)],[' * max(abs(gradient)):               ' num2str(OptResults(NumIter,4))],[' * Log posterior of phi at mode:     ' num2str(LogPostDensity,'%0.4f')],[' * Log posterior of theta at mode:   ' num2str(LogPostDensity-LogJacobianMode,'%0.4f')],[' * Log prior at mode:                ' num2str(LogPriorMode,'%0.4f')],[' * Log Jacobian at mode:             ' num2str(LogJacobianMode,'%0.4f')],SystemPriorStr,[' * Log likelihood at mode:           ' num2str(LogLikelihoodMode,'%0.4f')]));
         %
         EstimationStatus = 1;
         %
      elseif strcmp(lower(AbortEstimation),'cancel')==1;
         %
         % posteriod mode estimation was aborted
         %
         SetLogText(strvcat(' ','Posterior mode estimation was aborted.'));
      elseif ExitFlag==0;
         %
         % Reached maximum number of iteration
         %
         txt = ['The maximum number of function evaluations or iterations was reached without fminunc converging (ExitFlag is 0). YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
         SetLogText(strvcat(' ','Maximum number of function evaluations or iteration reached.',[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
         if isempty(CurrINI.Sequence)==1;
            About(txt,'information','Optimization Problem',200,500,CurrINI);
            drawnow;
         end;
         ReturnCodeText = 'Fminunc return code: the maximum number of function evaluations or iteration was reached without fminunc coverging (ExitFlag is 0).';
      elseif ExitFlag<0;
         %
         % no convergence
         %
         txt = ['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) '). YADA has aborted from posterior mode estimation. Writing optimization information to file: "OptErrorReport-' DSGEModel.NameOfModel '.txt" in "' DSGEModel.OutputDirectory '".'];
         SetLogText(strvcat(' ',['Fminunc didn''t converge (ExitFlag is ' int2str(ExitFlag) ').'],[' * Time:                             ' StringTrim(datestr(now,14))],[' * Optimization time:                ' SecondsToHoursStr(MaxTime) ' (hrs:min:sec)']));
         if isempty(CurrINI.Sequence)==1;
            About(txt,'information','Optimization Problem',200,500,CurrINI);
            drawnow;
         end;
         ReturnCodeText = ['Fminunc return code: No convergence (ExitFlag is ' int2str(ExitFlag) ').'];
      elseif isnan(ExitFlag)==1;
         SetLogText(strvcat(' ','Fminunc error: optimization ended abruptly.'));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'information','Optimization Problem',CurrINI.scrsz(4)-50,CurrINI.scrsz(3)/2,CurrINI);
            drawnow;
         end;
         return;
      end;
   end;
end;
%
% check if estimation was unsuccessful for some reason
%
if (EstimationStatus==0)&(strcmp(lower(AbortEstimation),'cancel')==0);
   %
   % write error information to file
   %
   SetLogText(strvcat('Writing optimization error summary to disk.',['   Directory: ' DSGEModel.OutputDirectory],['   File name: OptErrorReport-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']));
   %
   % transform phiMode to thetaMode taking into account that certain entries may be Inf or Nan's
   %
   NotFinite = 1-isfinite(phiMode);
   if sum(NotFinite)>=1;
      HelpIndex = (1:NumParam)';
      SwapParams = (NotFinite .* HelpIndex);
      SwapParams = SwapParams(SwapParams>0);
      phiModeOld = phiMode;
      phiMode(SwapParams) = 1;
   end;
   thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
   %
   % compute the partial of theta
   %
   phiGradientVector = abs((PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds)\eye(NumParam))*GradientVector);
   %
   % Now we have to reset all those elelemts that were either NaN or Inf
   %
   if sum(NotFinite)>=1;
      thetaMode(SwapParams) = phiModeOld(SwapParams);
      phiGradientVector(SwapParams) = abs(phiModeOld(SwapParams));
   end;
   %
   % check what we can save to mat-file
   %
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      LogPostDensity = -fh;
   else;
      LogPostDensity = -fh+logJacobian(phiMode,thetaIndex,UniformBounds);
   end;
   if (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
      if (isempty(HessianMatrix)==0)&(rank(HessianMatrix)==size(HessianMatrix,2));
         InverseHessian = HessianMatrix\eye(length(phiMode));
      else;
         InverseHessian = [];
      end;
      phiEstimates = OUTPUT.phiEstimates;
   end;
   %
   % see if we should check the optimum
   %
   if (DSGEModel.CheckOptimum==1)&(isempty(InverseHessian)==0)&(IsPositiveDefinite(InverseHessian)==1);
      %
      % Check if GraphDirectory exists
      %
      GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
      MinMaxGrid = zeros(NumParam,2);
      stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
      %
      % create lower and upper bounds from InverseHessian and phiMode
      %
      MinMaxGrid(:,1) = phiMode-(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
      MinMaxGrid(:,2) = phiMode+(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
      try;
         phiFittedStd = VillaniCheckOptimum('logPosteriorAdaptiveLearningPhiDSGE',phiMode,[],MinMaxGrid,InverseHessian,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature Of Log Posterior - Transformed Parameters',CurrINI,GraphDirectory,'phi','','',ShowFigs,1,1, ...
            thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      catch;
         phiFittedStd = NaN;
      end;
   else;
      phiFittedStd = NaN;
   end;
   %
   % Save to disk
   %
   ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
   stat = MakeDir(DSGEModel.OutputDirectory,'mode');
   if stat==1;
      SetLogText(strvcat(' ','Saving optimization data to disk using directory:',['   "' ModeFileDir '"...']));
      ModeFile = FixFilePath([ModeFileDir '\PosteriorModeError-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
      save(ModeFile,'phiMode','thetaMode','phi','theta','phiEstimates','ParameterNames','LogPostDensity','InverseHessian','phiFittedStd');
   end;
   %
   % print an error report
   %
   DSGEModel.BeliefSys = BeliefSys;
   TxtFileName = FixFilePath([DSGEModel.OutputDirectory '\OptErrorReport-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']);
   PrintOptimizationErrorReport(TxtFileName,0,[],DSGEModel,CurrINI,AIMData,ParameterNames,PriorDist,thetaDist,thetaMode,theta,phiGradientVector,GradientVector,rank(R),OptResults,ScreenMessages,ReturnCodeText,IsSystemPrior,1);
   %
   % display the file
   %
   if isempty(CurrINI.Sequence)==1;
      TextGUI(TxtFileName,['Optimization Error Summary - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   end;
   return;
elseif (EstimationStatus==0)&(strcmp(lower(AbortEstimation),'cancel')==1);
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
   return;
end;
%
% estimation went well. prepare output
%
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
if isempty(CurrINI.Sequence)==1;
   if (exist('OptResults')==1)&(exist('NumIter')==1);
      [ORn,ORk] = size(OptResults);
      if (NumIter<=ORn)&(ORk>=4);
         if OptResults(NumIter,4)>=10;
            txt = ['The maximum of the absolute values of the gradient vector is ' num2str(OptResults(NumIter,4)) '. It doesn''t look like you''ve found a maximum of the log posterior kernel. Are you sure you want to continue?'];
            answer = Query(txt,'question',500,'Poster Mode Estimation - Gradient Vector',600,'no',CurrINI);
            if strcmp(lower(answer),'no')==1;
               SetLogText(strvcat(' ','Poor gradient convergence','User selected to abort posterior mode estimation'));
               return;
            end;
         end;
      end;
   end;
end;
%
% Record the first observation used by the Kalman filter when calculating the log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
FirstPeriod = DSGEModel.FirstPeriod;
LastPeriod = DSGEModel.LastPeriod;
%
MaxEigenvalueIV = [];
MaxEigenvalueMode = [];
%
% Compute the partial derivatives of the original parameters wrt the transformed parameters, evaluated at the
% posterior mode
%
ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
%
% Check if we should compute the inverse Hessian with finite difference methods
%
if DSGEModel.FiniteDifferenceHessian==1;
   SetLogText(strvcat(' ','Computing posterior standard deviations from inverse Hessian','using finite differences...'));
   %
   % for the transformed parameters
   %
   if (exist('matlabpool')==2)|(exist('parpool')==2);
      if CurrINI.NumWorkers==0;
         if isempty(CurrINI.Sequence)==1;
            txt = 'Would you like to use parallel computations when computing the inverse Hessian with finite differences?';
            answer = Query(txt,'question',500,'Posterior Mode Estimation - Finite Differences',600,'yes',CurrINI);
         else;
            answer = 'Yes';
         end;
         if strcmp(lower(answer),'yes')==1;
            %
            % try to open a parallel pool
            %
            set(controls.about,'UserData',0);
            maingui = findobj('Type','figure','Tag','YADA');
            FileMenuFunctions('OpenPool',maingui);
            %
            CurrINI = get(controls.filemenu,'UserData');
            set(controls.about,'UserData',1);
         end;
      end;
      if (CurrINI.NumWorkers>1)&(exist('parpool')==2);
         poolobj = gcp('nocreate');
         if isempty(poolobj)==0;
            poolobj.IdleTimeout = Inf;
         end;
      end;
   end;
   VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningPhiDSGE',phiMode,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   phiInverseHessianFinDiff = HessianMat\eye(NumParam);
   phiPostModeStdFinDiff = sqrt(diag(phiInverseHessianFinDiff));
   %
   % for the orginal parameters
   %
   VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningThetaDSGE',thetaMode,StepLength,CurrINI.NumWorkers,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   thetaInverseHessianFinDiff = HessianMat\eye(NumParam);
   thetaPostModeStdFinDiff = sqrt(diag(thetaInverseHessianFinDiff));
else;
   if isempty(CurrINI.Sequence)==1;
      txt = 'Would you like to compute the inverse Hessian using finite differences and use it to estimate the marginal likelihood with the Laplace approximation?';
      answer = Query(txt,'question',500,'Posterior Mode Estimation - Finite Differences',600,'yes',CurrINI);
   else;
      answer = CurrINI.Sequence.QueryAnswers{4};
   end;
   if strcmp(lower(answer),'yes')==1;
      if (exist('matlabpool')==2)|(exist('parpool')==2);
         if CurrINI.NumWorkers==0;
            if isempty(CurrINI.Sequence)==1;
               txt = 'Would you like to use parallel computations when computing the inverse Hessian with finite differences?';
               answer = Query(txt,'question',500,'Posterior Mode Estimation - Finite Differences',600,'yes',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               %
               % try to open a parallel pool
               %
               set(controls.about,'UserData',0);
               maingui = findobj('Type','figure','Tag','YADA');
               FileMenuFunctions('OpenPool',maingui);
               %
               CurrINI = get(controls.filemenu,'UserData');
               set(controls.about,'UserData',1);
            end;
         end;
         if (CurrINI.NumWorkers>1)&(exist('parpool')==2);
            poolobj = gcp('nocreate');
            if isempty(poolobj)==0;
               poolobj.IdleTimeout = Inf;
            end;
         end;
      end;
      SetLogText(strvcat(' ','Computing an inverse Hessian using finite differences...'));
      VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningPhiDSGE',phiMode,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
      HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
      if IsPositiveDefinite(HessianMat)==1;
         phiInverseHessianFinDiff = HessianMat\eye(NumParam);
         SetLogText('* Matrix is positive definite.');
      else;
         phiInverseHessianFinDiff = NaN;
         SetLogText('* Matrix is not positive definite.');
      end;
   else;
      phiInverseHessianFinDiff = NaN;
   end;
   %
   phiPostModeStdFinDiff = NaN;
   thetaInverseHessianFinDiff = NaN;
   thetaPostModeStdFinDiff = NaN;
end;
OldInverseHessian = InverseHessian;
if IsPositiveDefinite(InverseHessian)==0;
   %
   % try to compute an inverse Hessian using finite differences
   %
   SetLogText(strvcat(' ','WARNING: The inverse Hessian at the posterior mode is not','         positive definite. Replacing it with estimator','         from finite differences.'));
   if DSGEModel.FiniteDifferenceHessian==1;
      InverseHessian = phiPostModeStdFinDiff;
   else;
      if (exist('phiInverseHessianFinDiff')==1)&(size(phiInverseHessianFinDiff,1)==NumParam);
         InverseHessian = phiPostModeStdFinDiff;
      else;
         VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningPhiDSGE',phiMode,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
         HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
         InverseHessian = HessianMat\eye(NumParam);
      end;
   end;
end;
if IsPositiveDefinite(InverseHessian)==0;
   SetLogText(strvcat(' ','WARNING: The inverse Hessian at the posterior mode','         is not positive definite.'));
   DgInvHess = diag(InverseHessian);
   if length(DgInvHess(DgInvHess>0))==length(DgInvHess);
      if isempty(CurrINI.Sequence)==1;
         txt = 'The inverse Hessian at the posterior mode is not positive definite. Do you want YADA to attempt to create a positive definite matrix from the inverse Hessian?';
         answer = Query(txt,'question',200,'YADA - Bad Inverse Hessian',500,'no',CurrINI);
      else;
         answer = CurrINI.Sequence.QueryAnswers{3};
      end;
      if strcmp(lower(answer),'yes')==1;
         SetLogText('         Attempting the create positive definite matrix...');
         [InvHess,pdstatus] = CreatePositiveDefinite(InverseHessian);
         if pdstatus==1;
            SetLogText(strvcat('         STATUS: Successful. Replacing old inverse Hessian'));
            InverseHessian = InvHess;
         else;
            SetLogText(strvcat('         STATUS: Failed.'));
            EstimationStatus = 0;
         end;
      end;
   end;
end;
%
% Check for problems
%
if EstimationStatus==0;
   SetLogText(strvcat(' ','Writing optimization error summary to disk.',['   Directory: ' DSGEModel.OutputDirectory],['   File name: OptErrorReport-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']));
   %
   % compute the partial of theta
   %
   phiGradientVector = abs((PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds)\eye(NumParam))*GradientVector);
   if (DSGEModel.MaximizeAlgorithmValue==7)|(DSGEModel.MaximizeAlgorithmValue==8);
      phiEstimates = OUTPUT.phiEstimates;
   end;
   phiFittedStd = NaN;
   %
   % Save to disk
   %
   ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
   stat = MakeDir(DSGEModel.OutputDirectory,'mode');
   if stat==1;
      SetLogText(strvcat(' ','Saving optimization data to disk using directory:',['   "' ModeFileDir '"...']));
      ModeFile = FixFilePath([ModeFileDir '\PosteriorModeError-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
      save(ModeFile,'phiMode','thetaMode','phi','theta','phiEstimates','ParameterNames','LogPostDensity','InverseHessian','phiFittedStd');
   end;
   %
   % print an error report
   %
   DSGEModel.BeliefSys = BeliefSys;
   TxtFileName = FixFilePath([DSGEModel.OutputDirectory '\OptErrorReport-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']);
   ReturnCodeText = 'YADA return code: Inverse Hessian is not positive definite.';
   PrintOptimizationErrorReport(TxtFileName,0,[],DSGEModel,CurrINI,AIMData,ParameterNames,PriorDist,thetaDist,thetaMode,theta,phiGradientVector,GradientVector,rank(R),OptResults,ScreenMessages,ReturnCodeText,IsSystemPrior,1);
   %
   % display the file
   %
   if isempty(CurrINI.Sequence)==1;
      TextGUI(TxtFileName,['Optimization Error Summary - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   end;
   SetLogText(' ');
   return;
end;
%
% Check if we should check the optimum
%
if (DSGEModel.CheckOptimum==1)&(IsPositiveDefinite(InverseHessian)==1);
   %
   % make sure we remove the log marginal likelihood of the system prior from the log
   % marginal likelihood of the data
   %
   if isnan(phiInverseHessianFinDiff)==0;
      LogMargLike = MargLikeLaplace(LogPostDensity,phiInverseHessianFinDiff,1)-LogMargLikeSystemLaplace;
      LapStr = 'finite diff.';
   else;
      LogMargLike = MargLikeLaplace(LogPostDensity,InverseHessian,1)-LogMargLikeSystemLaplace;
      LapStr = 'optimization';
   end;
   SetLogText(strvcat(' ',['Computing the Laplace approximation using ' LapStr ' estimator...'],[' * Log marg. likelihood at phi mode: ' num2str(LogMargLike,'%0.4f')],' ','Checking the optimum...',' * Transformed parameters'));
   %
   % Check if GraphDirectory exists
   %
   GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
   MinMaxGrid = zeros(NumParam,2);
   stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
   %
   % create lower and upper bounds from InverseHessian and phiMode
   %
   MinMaxGrid(:,1) = phiMode-(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
   MinMaxGrid(:,2) = phiMode+(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
   phiFittedStd = VillaniCheckOptimum('logPosteriorAdaptiveLearningPhiDSGE',phiMode,[],MinMaxGrid,InverseHessian,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Posterior Mode - Transformed Parameters',CurrINI,GraphDirectory,'phi','','',ShowFigs,1,1, ...
      thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   if (DSGEModel.CheckTransformedOptimum==0)|(IsOdd(DSGEModel.MaximizeAlgorithmValue)==0);
      SetLogText(' * Original parameters');
      MinMaxGrid(:,1) = PhiToTheta(MinMaxGrid(:,1),thetaIndex,UniformBounds,LowerBound);
      MinMaxGrid(:,2) = PhiToTheta(MinMaxGrid(:,2),thetaIndex,UniformBounds,LowerBound);
      thetaFittedStd = VillaniCheckOptimum('logPosteriorAdaptiveLearningThetaDSGE',thetaMode,[],MinMaxGrid,ThetaPhiPartial*InverseHessian*ThetaPhiPartial,NumGridPoints,0,[],ParameterNames.estimated,'Checking Curvature at Posterior Mode - Original Parameters',CurrINI,GraphDirectory,'theta','','',ShowFigs,1,1, ...
         thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
   else;
      thetaFittedStd = NaN;
   end;
else;
   if IsPositiveDefinite(InverseHessian)==1;
      if isnan(phiInverseHessianFinDiff)==0;
         LogMargLike = MargLikeLaplace(LogPostDensity,phiInverseHessianFinDiff,1)-LogMargLikeSystemLaplace;
         LapStr = 'finite diff.';
      else;
         LogMargLike = MargLikeLaplace(LogPostDensity,InverseHessian,1)-LogMargLikeSystemLaplace;
         LapStr = 'optimization';
      end;
      SetLogText(strvcat(' ',['Computing the Laplace approximation using ' LapStr ' estimator...'],[' * Log marg. likelihood at phi mode: ' num2str(LogMargLike,'%0.4f')]));
   end;
   phiFittedStd = NaN;
   thetaFittedStd = NaN;
end;
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
% check the parameters with most extreme correlations based on the inverse Hessian
%
ExtremeCorrBound = 0.7;
CorrHessian = diag(1./sqrt(diag(InverseHessian)))*InverseHessian*diag(1./sqrt(diag(InverseHessian)));
CorrHessian = tril(CorrHessian,-1);
[RowIndex,ColIndex] = find(abs(CorrHessian)>=ExtremeCorrBound);
if isempty(RowIndex)==0;
   ExtremeCorrParameters = cell(length(RowIndex),3);
   for i=1:length(RowIndex)
      ExtremeCorrParameters{i,1} = ParameterNames.estimated(RowIndex(i),:);
      ExtremeCorrParameters{i,2} = ParameterNames.estimated(ColIndex(i),:);
      ExtremeCorrParameters{i,3} = CorrHessian(RowIndex(i),ColIndex(i));
   end;
else;
   ExtremeCorrParameters = [];
end;
%
% check other possible rank deficiencies with the inverse Hessian at the mode
%
[EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(CorrHessian);
SmallEigenValues = EigenValues(EigenValues<0.01);
if isempty(SmallEigenValues)==0;
   p1 = length(SmallEigenValues);
   p = size(EigenVectorMatrix,2);
   SmallEigenVectors = EigenVectorMatrix(:,p-p1+1:p);
else;
   SmallEigenVectors = [];
end;
%
% calculate the log-likelihood at theta and the log prior
%
ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
end;
[LogLikeDensity,Solution,mcode,status] = logLikelihoodAdaptiveLearningDSGE(ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
LogPriorDensity = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
LogJacobianValue = logJacobian(phiMode,thetaIndex,UniformBounds);
LogPriorDensityIV = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
%
% get the mode values of any additional parameters
%
[thetaCalibratedMode,thetaAdditionalMode] = ReadAdditionalParameterValues(ModelParameters,ParameterNames);
%
% Save to disk
%
ModeFileDir = FixFilePath([DSGEModel.OutputDirectory '\mode']);
stat = MakeDir(DSGEModel.OutputDirectory,'mode');
if stat==1;
   SetLogText(strvcat(' ','Saving posterior mode data to disk using directory:',['   "' ModeFileDir '"...']));
   ModeFile = FixFilePath([ModeFileDir '\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
   save(ModeFile,'phiMode','thetaMode','phi','theta','phiEstimates','OptResults','thetaCalibrated','thetaAdditionalInitial','thetaAdditionalMode','ParameterNames','ExtremeCorrParameters','LogPostDensity','LogLikeDensity',...
        'LogPriorDensity','LogJacobianValue','InverseHessian','OldInverseHessian','ThetaPhiPartial','thetaIndex','ParameterBounds','LowerBound','thetaPositions','thetaDist','PriorDist','ModelParameters','UniformBounds',...
        'DSGEModel','AIMData','phiInverseHessianFinDiff','phiPostModeStdFinDiff','thetaInverseHessianFinDiff','thetaPostModeStdFinDiff','phiFittedStd','thetaFittedStd','KalmanFirstObservation','FirstPeriod','LastPeriod',...
        'SmallEigenValues','SmallEigenVectors','LogSPIV','LogSPMode','LogMargLikeSystemLaplace','Sperp','SMatrix','Gtilde','Kgf','BeliefSys');
end;
%
% write output to file
%
DSGEModel.BeliefSys = BeliefSys;
ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeResults-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']);
SetLogText(strvcat(' ','Writing posterior mode estimation results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: PosteriorModeResults-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']));
PrintPosteriorModeResults(ModeTextFile,DSGEModel,CurrINI,ParameterNames,phiMode,thetaMode,phi,theta,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,thetaDist,PriorDist,ThetaPhiPartial,InverseHessian,phiInverseHessianFinDiff,phiFittedStd,thetaFittedStd,phiPostModeStdFinDiff,thetaPostModeStdFinDiff,LogPostDensity,LogLikeDensity,LogPriorDensity,LogJacobianValue,ExtremeCorrParameters,SmallEigenValues,SmallEigenVectors,OptAlgorithm,OptParameters,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,MaxEigenvalueIV,MaxEigenvalueMode,OptResults,ScreenMessages,LogPosteriorPhiIV,LogPosteriorThetaIV,LogPriorDensityIV,rank(R),[],[],[],[],LogSPIV,LogSPMode,IsSystemPrior,LogMargLikeSystemLaplace,CheckAdaptiveLearningFixedBeliefs(ModelParameters));
%
% write summary information to file
%
ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeSummary-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']);
SetLogText(strvcat(' ','Writing posterior mode estimation summary results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: PosteriorModeSummary-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']));
PrintPosteriorModeSummary(ModeTextFile,DSGEModel,CurrINI,ParameterNames,thetaMode,theta,thetaDist,PriorDist,ThetaPhiPartial,InverseHessian,phiInverseHessianFinDiff,phiFittedStd,LogPostDensity,LogLikeDensity,LogPriorDensity,LogJacobianValue,ExtremeCorrParameters,SmallEigenValues,SmallEigenVectors,OptAlgorithm,OptParameters,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,max(GradientVector),MaxEigenvalueIV,MaxEigenvalueMode,LogPosteriorPhiIV,LogPosteriorThetaIV,LogPriorDensityIV,rank(R),[],[],[],[],LogSPIV,LogSPMode,IsSystemPrior,LogMargLikeSystemLaplace,CheckAdaptiveLearningFixedBeliefs(ModelParameters));
SetLogText('All done!');
%
%
% open one of the results files
%
if isempty(CurrINI.Sequence)==1;
   txt = 'Would you like to view only a summary of the posterior mode estimation output? [No means all printed results]';
   answer = Query(txt,'question',120,'Output Selection',500,'yes',CurrINI);
   drawnow;
   if strcmp(lower(answer),'yes')==1;
      ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeSummary-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']);
   else;
      ModeTextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeResults-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']);
   end;
   TextGUI(ModeTextFile,['Posterior Mode Estimation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Adaptive Learning Results',50,CurrINI,1,1);
else;
   SetLogText(' ');
end;

%
% end of AdaptiveLearningPosteriorModeEstimation.m
%
