function [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,S,Sperp)
% AdaptiveLearningTransformDSGEModel: Provides a transformation of the DSGE model such that only the forward looking
%                                     variables appear in the expectation term.
%
% USAGE:
%
%       [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,S,Sperp)
%
% REQUIRED INPUT:  Hlag (matrix) of dimension r x r with the coefficients of lagged state variables in
%                       the structural form of the DSGE model.
%
%                  H0 (matrix) of dimension r x r with the coefficients on contemporaneous state variables
%                       in the structural form of the model.
%
%                  Hexp (matrix) of dimension r x r with the coefficients on expected furture state
%                       variables in the structural form of the model.
%
%                  S (matrix) of dimension r x f which selects the forward looking variables.
%
%                  Sperp (matrix) of dimension r x (r-f) which select the non-forward looking variables.
%
% REQUIRED OUTPUT: TransStatus (boolean) which takes the value 1 if the transformation was successful
%                       and 0 otherwise.
%
%                  Hstarlag (matrix) of dimension r x r with the coefficients of lagged state variables in
%                       the structural form of the DSGE model.
%
%                  Hstar0 (matrix) of dimension r x r with the coefficients on contemporaneous state variables
%                       in the structural form of the model.
%
%                  Hstar1f (matrix) of dimension r x f with the coefficients on the expected future state
%                       variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 19, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-03-2020: Updated the documentation.
%
% * 08-04-2020: Took the need of the C matrix into account.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
TransStatus = 1;
Hstarlag = Hlag;
Hstar0 = H0;
Hstar1f = Hexp*S;
Stilde = [S Sperp];
[r,f] = size(S);
%
% determine the positions of the forward looking variables from S
%
ForwardVariablePositions = (1:r)*S;
ForwardVariablePositions = ForwardVariablePositions(ForwardVariablePositions>0);
C = GetForwardLookingVariableOrder(Hexp,ForwardVariablePositions);
%
% shift equations to match the forward looking variable positions
%
Hlag = C*Hlag;
H0 = C*H0;
Hexp = C*Hexp;
%
% check if Sperp'*C*Hexp*Sperp is a 0 matrix
%
if max(max((abs(Sperp'*Hexp*Sperp))))>1.0e-5;
   TransStatus = 0;
   return;
end;
%
% now check if we need to make adjustments to Hstar0 and Hstar1f
%
if max(max(abs(Hexp*Sperp)))<=1.0e-08;
   %
   % no need for any further work
   %
   return;
end;
%
% now check if Sperp'*C*Hexp is zero
%
if max(max((abs(Sperp'*Hexp))))>1.0e-5;
   TransStatus = 0;
   return;
end;
%
% we're fine, let make the necessary changes to the contemporaneous
% matrix and the one for the expected future.
%
% check if Sperp'*C*H0*Sperp has full rank
%
if rank(Sperp'*H0*Sperp)<(r-f);
   TransStatus = 0;
   return;
end;
%
InvSpH0Sp = (Sperp'*H0*Sperp)\eye(r-f);
H0Trans = zeros(r,r);
H0Trans(1:f,1:f) = S'*Hexp*Sperp*InvSpH0Sp*Sperp'*Hlag*S;
H0Trans(1:f,f+1:r) = S'*Hexp*Sperp*InvSpH0Sp*Sperp'*Hlag*Sperp;
%
Hbar0 = (Stilde'*H0*Stilde)-H0Trans;
Hstar0 = C'*Stilde*Hbar0*Stilde';
%
Hbar1f = zeros(r,f);
Hbar1f(1:f,:) = (S'*Hexp*S)-(S'*Hexp*Sperp*InvSpH0Sp*Sperp'*H0*S);
Hstar1f = C'*Stilde*Hbar1f;

%
% end of AdaptiveLearningTransformDSGEModel.m
%
