function BeliefCoefficientGraphDLG(selector,CurrINI,DSGEModel,StateVarStructure,BeliefVariables,gjvec,NameStr)
% BeliefCoefficientGraphDLG: Displays graphs of belief coefficients
%
% USAGE:
%
%       BeliefCoefficientGraphDLG(selector,CurrINI,DSGEModel,StateVarStructure,BeliefVariables,gjvec,NameStr)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend",
%                          "showgraph", and "done".
%
% OPTIONAL INPUT: CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                          about the model, such as input files, variable and sample
%                          data. Actual observations are not stored in this file.
%
%                 StateVarStructure (structure) with fields that include betatt with the estimated
%                          belief coefficients.
%
%                 BeliefVariables (cell array) with f string matrices with the names of the variables in
%                          each belief equation.
%
%                 gjvec (vector) of dimension f with the number of explanatory variables in each belief
%                          equation.
%
%                 NameStr (string vector) taking on the values "Initial Values", "DSGE Posterior Mode" or
%                          "Posterior Mode".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 6, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 07-04-2020: Updated the documentation.
%
% * 16-04-2020: Added linewidth to the zero line.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.StateVarStructure = StateVarStructure;
   DSGEModel.NameStr = NameStr;
   DSGEModel.BeliefVariables = BeliefVariables;
   DSGEModel.gjvec = cumsum(gjvec);
else
   BeliefCoefficientGUI = findobj('Type','figure','Tag','BeliefCoefficientGraphDLG');
   BeliefControls = get(BeliefCoefficientGUI,'UserData');
   CurrINI = get(BeliefControls.variablespopup,'UserData');
   DSGEModel = get(BeliefControls.variables,'UserData');
end;
%
switch selector;
   case 'init';
      BeliefCoefficientGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','BeliefCoefficientGraphDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-120)/2 500 120], ...
         'Tag','BeliefCoefficientGraphDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Belief Coefficients - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','BeliefCoefficientGraphDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(BeliefCoefficientGUI,'DockControl','off');
      end;
      %
      % Check where the sample of data actually begins
      %
      [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
      %
      DSGEModel = OldDSGEModel;
      %
      % Outer axis
      %
      AxesBox([2 2 498 110],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Sample selection controls
      %
      BeliefControls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      BeliefControls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 55 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','BeliefCoefficientGraphDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      BeliefControls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[278 55 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','BeliefCoefficientGraphDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SubSampleStrMat,1));
      %
      % Select variable
      %
      BeliefControls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 160 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select forward looking variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariablesText');
      %
      BeliefControls.variablespopup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 20 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.ForwardVariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','VariablePopup',...
         'Value',1);
      %
      % Create graph button
      %
      BeliefControls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','BeliefCoefficientGraphDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      BeliefControls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','BeliefCoefficientGraphDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(BeliefCoefficientGUI,'UserData',BeliefControls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(BeliefControls.samplebeg,'Value');
      sampleend = get(BeliefControls.sampleend,'Value');
      if sampleend<samplebeg;
         set(BeliefControls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(BeliefControls.samplebeg,'Value');
      sampleend = get(BeliefControls.sampleend,'Value');
      if sampleend<samplebeg;
         set(BeliefControls.samplebeg,'Value',sampleend);
      end;
   case 'showgraph'
      VarValue = get(BeliefControls.variablespopup,'Value');
      VarString = get(BeliefControls.variablespopup,'String');
      CurrVariable = StringTrim(VarString(VarValue,:));
      if VarValue<size(VarString,1);
         set(BeliefControls.variablespopup,'Value',VarValue+1);
      else;
         set(BeliefControls.variablespopup,'Value',1);
      end;
      %
      % create a vector with dates
      %
      DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
      DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
      %
      % get the selected sample
      %
      StartSample = get(BeliefControls.samplebeg,'Value');
      EndSample = get(BeliefControls.sampleend,'Value');
      %
      % select the betatt coefficients that apply to the current variable
      %
      if VarValue==1;
         StartValue = 1;
         EndValue = DSGEModel.gjvec(1);
      else;
         StartValue = DSGEModel.gjvec(VarValue-1)+1;
         EndValue = DSGEModel.gjvec(VarValue);
      end;
      betatt = DSGEModel.StateVarStructure.betatt(StartValue:EndValue,:);
      BeliefVariableNames = DSGEModel.BeliefVariables{VarValue,1};
      gj = size(betatt,1);
      %
      LineColors = CreateDefaultColors(gj);
      LineType = strvcat('-','--','-.',':');
      NumLoops = ceil(gj/4);
      LineTypes = '';
      for j=1:NumLoops;
         LineTypes = strvcat(LineTypes,LineType);
      end;
      LineTypes = LineTypes(1:gj,:);
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                         'Tag',CurrVariable, ...
                         'Name',['Graph of Belief Coefficients for ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      LineHandles = [];
      for j=1:gj;
         LH = plot(DateVector(StartSample:EndSample),betatt(j,StartSample:EndSample),StringTrim(LineTypes(j,:)),'LineWidth',2,'Color',LineColors(j,:));
         if j==1;
            hold('on');
         end;
         LineHandles = [LineHandles LH];
      end;
      kids = get(gca,'Children');
      if size(kids,2)>size(kids,1);
         kids = kids';
      end;
      ylim = get(gca,'YLim');
      if (ylim(1)<0)&(ylim(2)>0);
         ZeroHandle = plot([DateVector(StartSample) DateVector(EndSample)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
         set(gca,'Children',[ZeroHandle;kids]);
      end;
      set(gca,'XLim',[DateVector(StartSample) DateVector(EndSample)]);
      %
      SetAxesFonts(CurrINI,'Title',[CurrVariable ' belief coefficients at ' lower(DSGEModel.NameStr)]);
      %
      % add the legend
      %
      legHandle = legend(LineHandles,BeliefVariableNames);
      SetAxesFonts(CurrINI,legHandle);
      %
      SetXTickIntegers(gca);
      %
      % Fix the toolbar
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      %
      % display figures
      %
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(BeliefControls.done,'UserData','done');
      delete(BeliefCoefficientGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(BeliefCoefficientGUI,'Visible','on');
   drawnow;
   SetYADAIcon(BeliefCoefficientGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(BeliefControls.done);
   end;
   waitfor(BeliefControls.done,'UserData','done');
end;

%
% end of BeliefCoefficientGraphDLG.m
%
