function [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel)
% GetAdaptiveLearningInputVariables: Computes the main unique adaptive learning related variables
%                                    needed as input for calculating the log likelihood.
%
% USAGE:
%
%       [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about
%                            the model, such as input files, variable and sample data.
%
% REQUIRED OUTPUT: SMatrix (matrix) of dimension r x f which selects the forward looking variables among the
%                            state variables.
%
%                  Sperp (matrix) of dimension r x (r-f) which select the non-forward looking variables.
%
%                  m (integer) with the maximum lag for the state variables in the belief equations.
%
%                  G (cell array) of dimension f for selecting the explanatory variables among deterministic and lagged
%                            state variables in each belief equation for the forward looking variables.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                            of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                            A is a g x f matrix.
%
%                  xbar (vector) of dimension k with unit element for a constant and zeros elsewhere.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 29, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-06-2020: Corrected a bug for lastpos in Gtilde determination.
%
% * 01-10-2020: Fixed a bug with j instead of : for X matrix.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 21-02-2022: Changed eval call to feval.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if isfield(DSGEModel,'FirstPeriod')==0;
   [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
end;
%
if (DSGEModel.BeliefSystem==2)&(FileExist(FixFilePath([pwd '\tmp\' GetFilename(DSGEModel.BeliefSystemFile)]))==0);
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% we need to construct a number of variables that are stored in DSGEModel.AdaptiveLearning
% as the fields r, f, Sperp, SMatrix, Gtilde and Kgf
%
f = length(DSGEModel.ForwardVariablePositions);
r = length(DSGEModel.StateVariablePositions);
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% compute the BeliefSys structure
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
G = BeliefSys.G;
m = BeliefSys.m;
k = DSGEModel.k;
%
% determine Gtilde and Kgf
%
krm = k+(r*m);
gsize = zeros(f,1);
for j=1:f;
   gsize(j) = size(G{j,1},2);
end;
g = sum(gsize);
Ghelp = zeros(g,krm);
Gtilde = zeros(f*g,krm);
lastpos = 0;
for j=1:f;
   Gtildej = Ghelp;
   Gtildej(lastpos+(1:gsize(j)),:) = G{j}';
   Gtilde((g*(j-1))+1:(g*j),:) = Gtildej;
   lastpos = lastpos+gsize(j);
end;
%
% The commutation matrix Kgf
%
Kgf = CommutationMatrix(g,f);
ConstPos = 0;
if k>0
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
      T = size(X,2);
      for j=1:k;
         ConstTest = (DSGEModel.X(j,DSGEModel.FirstPeriod:DSGEModel.LastPeriod)==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;

%
% end of GetAdaptiveLearningInputVariables.m
%
