function [BeliefVariables,gjvec] = GetBeliefCoefficientNames(G,m,ConstPos,BeliefSystem,XVariableNames,StateVariableNames,ForwardVariableNames)
% GetBeliefCoefficientNames: Sets up string matrices with names of the variables used in the perceived law of
%                            motion (belief equations) for the forward looking variables.
%
% USAGE:
%
%       [BeliefVariables,gjvec] = GetBeliefCoefficientNames(G,m,ConstPos,BeliefSystem,XVariableNames,...
%                                                           StateVariableNames,ForwardVariableNames)
%
% REQUIRED INPUT:  G (cell array) with f matrices, one for each forward looking variable of dimension
%                    (k+r*m) x g_j where g_j is the number of explanatory variables in the belief
%                    equation for forward looking variable j.
%
%                  m (integer) with the maximum lag order of the PLM.
%
%                  ConstPos (integer) with the position of the constant term among the deterministic variables
%                    if any, otherwise it is 0.
%
%                  BeliefSystem (integer) is 1 if the default belief system in YADA is used (constant if any)
%                    and two own lags; otherwise the variable is 2.
%
%                  XVariableNames (string matrix) of dimension k x k1 with the names of the deterministic variables.
%
%                  StateVariableNames (string matrix) of dimension r x r1 with the names of the state variables.
%
%                  ForwardVariableNames (string matrix) of dimension f x f1 with the names of the forward
%                    looking variables.
%
% REQUIRED OUTPUT: BeliefVariables (cell array) with f string matrices with the names of the variables in
%                    each belief equation.
%
%                  gjvec (vector) of dimension f with the number of belief coefficients for each belief equation.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 6, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 07-04-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
k = size(XVariableNames,1);
r = size(StateVariableNames,1);
f = size(ForwardVariableNames,1);
BeliefVariables = cell(f,1);
gjvec = zeros(1,f);
%
% check if we have YADA's default belief system
%
if BeliefSystem==1;
   for j=1:f;
      if ConstPos>0;
         BeliefVariablesj = StringTrim(XVariableNames(ConstPos,:));
         gjvec(j) = 3;
      else;
         BeliefVariablesj = '';
         gjvec(j) = 2;
      end;
      BeliefVariablesj = strvcat(BeliefVariablesj,[StringTrim(ForwardVariableNames(j,:)) '(persistence: lag 1 + 2)'],[StringTrim(ForwardVariableNames(j,:)) '(growth rate: -lag 2)']);
      BeliefVariables{j,1} = BeliefVariablesj;
   end;
   return;
end;
%
% we have a user defined belief system
%
AllNames = '';
if k>0;
   AllNames = XVariableNames;
end;
for j=1:m;
   for i=1:r;
      AllNames = strvcat(AllNames,[StringTrim(StateVariableNames(i,:)) '(t-' int2str(j) ')']);
   end;
end;
krm = k+(r*m);
krmvec = (1:krm)';
for j=1:f;
   Gj = G{j,1};
   BeliefVariablesj = '';
   gj = size(Gj,2);
   gjvec(j) = gj;
   for j1=1:gj;
      Gjnz = Gj(:,j1)~=0;
      posvec = Gjnz.*krmvec;
      posvec = posvec(posvec>0);
      GjName = '';
      for i1=1:length(posvec);
         SelGj = Gj(posvec(i1),j1);
         if SelGj==1;
            if isempty(GjName)==1;
               GjName = StringTrim(AllNames(posvec(i1),:));
            else;
               GjName = [GjName '+' StringTrim(AllNames(posvec(i1),:))];
            end;
         elseif SelGj>0;
            if isempty(GjName)==1;
               GjName = [num2str(SelGj) '*' AllNames(posvec(i1),:)];
            else;
               GjName = [GjName '+' num2str(SelGj) '*'  StringTrim(AllNames(posvec(i1),:))];
            end;
         elseif SelGj<0;
            if isempty(GjName)==1;
               GjName = [num2str(SelGj) '*' StringTrim(AllNames(posvec(i1),:))];
            else;
               GjName = [GjName '-' num2str(abs(SelGj)) '*' StringTrim(AllNames(posvec(i1),:))];
            end;
         end;
      end;
      BeliefVariablesj = strvcat(BeliefVariablesj,GjName);
   end;
   BeliefVariables{j,1} = BeliefVariablesj;
end;

%
% end of GetBeliefCoefficientNames.m
%
