function [C,ForwardEquationPositions] = GetForwardLookingVariableOrder(Hexp,ForwardVariablePositions)
% GetForwardLookingVariableOrder: Determines a reorder of the DSGE model equations such that the order of
%                                 the forward looking variables corresponds to the order of the equations.
%                                 This means, for example, that if variable 2 is a forward looking variable
%                                 then variable 2 appears in expectations in equation 2, and so on for all
%                                 forward looking variables.
%
% USAGE:
%
%       C = GetForwardLookingVariableOrder(Hexp,ForwardVariablePositions)
%
% REQUIRED INPUT:  Hexp (matrix) of dimension r x r with coefficients on the expected future
%                       values of the model variables.
%
%                  ForwardVariablePositions (vector) of dimension f with integer positions in
%                       Hexp (columns) of the forward looking variables.
%
% REQUIRED OUTPUT: C (matrix) of dimension r x r which guarantees that the ordering of the r
%                       equations is such that the f forward looking variables (columns) in
%                       Hexp has the matching numbers for the equations (rows).
%
% OPTIONAL OUTPUT: ForwardEquationPositions (vector) of dimension f with integer positions in
%                       Hexp (rows) of the equations where forward looking variables appear
%                       as well as possible zero rows.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 7, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-04-2020: Updated the documentation.
%
% * 09-04-2020: Changed the procedure of determining equation positions. It is
%               slower since it is based on a QR decomposition, but ensures that
%               the number of selected equations is f and that the rank of the
%               rows in ForwardEquationPositions and the columns in
%               ForwardVariablePositions of Hexp is equal to the rank of Hexp
%               based on ForwardVariablePositions.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

r = size(Hexp,1);
Ir = eye(r);
f = length(ForwardVariablePositions);
%ForwardEquationPositions = [];
%for j=1:r;
%   if sum(abs(Hexp(j,:)))>1.0e-7;
%      ForwardEquationPositions = [ForwardEquationPositions j];
%   end;
%end;
[EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(Hexp*Hexp');
ForwardEqPositions = sort(ParameterOrdering(1:f,2))';
%
C = Ir;
C(ForwardEqPositions,:) = Ir(ForwardVariablePositions,:);
C(ForwardVariablePositions,:) = Ir(ForwardEqPositions,:);
if nargout==2;
   ForwardEquationPositions = ForwardEqPositions;
end;

%
% end of GetForwardLookingVariableOrder.m
%
