function LawsOfMotionGraphDLG(selector,CurrINI,DSGEModel,ForwardVariableValues,ALMProjections,PLMProjections,DateVector,NameStr)
% LawsOfMotionGraphDLG: Displays update estimates of the forward looking variables along with the actual and perceived
%                       laws of motion for these variables.
%
% USAGE:
%
%       LawsOfMotionGraphDLG(selector,CurrINI,DSGEModel,ForwardVariableValues,ALMProjections,PLMProjections,DateVector,NameStr)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 ForwardVariableValues (matrix) of dimension f x T with the Kalman filter
%                           update estimates of the forward looking variables based on the
%                           adaptive learning solution.
%
%                 ALMProjections (matrix) of dimension f x T with the Kalman filter forecast
%                           estimates of the forward looking variables based on the adaptive
%                           learning solution. Represents tha actual law of motion.
%
%                 PLMProjections (matrix) of dimension f x T with the perceived law of motion.
%
%                 DateVector (vector) of dimension 1 x T with the dates for the estimates.
%
%                 NameStr (string vector) taking on the values "Initial Values", "DSGE Posterior Mode"
%                           and "Posterior Mode".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 14, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-05-2020: Finished the function.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.ForwardVariableValues = ForwardVariableValues;
   DSGEModel.ALMProjections = ALMProjections;
   DSGEModel.PLMProjections = PLMProjections;
   DSGEModel.DateVector = DateVector;
   DSGEModel.NameStr = NameStr;
else
   LawsOfMotionGraphGUI = findobj('Type','figure','Tag','LawsOfMotionGraphDLG');
   LOMcontrols = get(LawsOfMotionGraphGUI,'UserData');
   CurrINI = get(LOMcontrols.variable,'UserData');
   DSGEModel = get(LOMcontrols.variabletext,'UserData');
end;
%
switch selector;
   case 'init';
      LawsOfMotionGraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','IRGraphDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-90)/2 500 110], ...
         'Tag','LawsOfMotionGraphDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Laws Of Motion - Adaptive Learning - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','LawsOfMotionGraphDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(LawsOfMotionGraphGUI,'DockControl','off');
      end;
      %
      % Check where the sample of data actually begins
      %
      [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
      %
      DSGEModel = OldDSGEModel;
      %
      % Outer axis
      %
      AxesBox([2 2 498 100],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Sample selection controls
      %
      LOMcontrols.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      LOMcontrols.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 55 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','LawsOfMotionGraphDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      LOMcontrols.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[278 55 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','LawsOfMotionGraphDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SubSampleStrMat,1));
      %
      % select variable
      %
      LOMcontrols.variabletext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 160 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select forward looking variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariablesText');
      %
      LOMcontrols.variable = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 20 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.ForwardVariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','VariablePopup',...
         'Value',1);
      %
      % Create graph button
      %
      LOMcontrols.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','LawsOfMotionGraphDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      LOMcontrols.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','LawsOfMotionGraphDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(LawsOfMotionGraphGUI,'UserData',LOMcontrols,...
         'HandleVisibility','callback');
   case 'samplebegin'
      
   case 'sampleend'
      
   case 'showgraph'
      VariableValue = get(LOMcontrols.variable,'Value');
      VariableNames = get(LOMcontrols.variable,'String');
      CurrVariable = StringTrim(VariableNames(VariableValue,:));
      if VariableValue<size(VariableNames,1);
         set(LOMcontrols.variable,'Value',VariableValue+1);
      else;
         set(LOMcontrols.variable,'Value',1);
      end;
      %
      DateVector = DSGEModel.DateVector;
      %
      % get the selected sample
      %
      StartSample = get(LOMcontrols.samplebeg,'Value');
      EndSample = get(LOMcontrols.sampleend,'Value');
      %
      % select the data to display
      %
      zftt = DSGEModel.ForwardVariableValues(VariableValue,StartSample:EndSample);
      almt = DSGEModel.ALMProjections(VariableValue,StartSample:EndSample);
      plmt = DSGEModel.PLMProjections(VariableValue,StartSample:EndSample);
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                         'Tag',CurrVariable, ...
                         'Name',['Laws Of Motion For ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % Kalman update estimate of the forward looking variable
      %
      zfttHandle = plot(DateVector(StartSample:EndSample),zftt,'-.','LineWidth',2,'Color',[0.5 0.5 0.5]);
      hold('on');
      %
      % Kalman forecast estimate of the variable as actual law of motion
      %
      almtHandle = plot(DateVector(StartSample:EndSample),almt,'--','LineWidth',2,'Color',[0 0 0.75]);
      %
      % perceived law of motion estimate of the variable
      %
      plmtHandle = plot(DateVector(StartSample:EndSample),plmt,':','LineWidth',2,'Color',[1 0 0]);
      %
      kids = get(gca,'Children');
      if size(kids,2)>size(kids,1);
         kids = kids';
      end;
      ylim = get(gca,'YLim');
      if (ylim(1)<0)&(ylim(2)>0);
         ZeroHandle = plot([DateVector(StartSample) DateVector(EndSample)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
         set(gca,'Children',[ZeroHandle;kids]);
      end;
      set(gca,'XLim',[DateVector(StartSample) DateVector(EndSample)]);
      %
      SetAxesFonts(CurrINI,'Title',[CurrVariable ' laws of motion at ' lower(DSGEModel.NameStr)]);
      %
      % add the legend
      %
      if MatlabNumber>=7;
         legHandle = legend([zfttHandle almtHandle plmtHandle],strvcat('Update estimate, z^f(t|t)','Actual law of motion, z^f(t|t-1)','Perceived law of motion'),'Location','SouthWest');
      else;
         legHandle = legend([zfttHandle almtHandle plmtHandle],strvcat('Update estimate, z^f(t|t)','Actual law of motion, z^f(t|t-1)','Perceived law of motion'),0);
      end;
      SetAxesFonts(CurrINI,legHandle);
      %
      SetXTickIntegers(gca);
      %
      % Fix the toolbar
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      %
      % display figures
      %
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(LOMcontrols.done,'UserData','done');
      delete(LawsOfMotionGraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(LawsOfMotionGraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(LawsOfMotionGraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(LOMcontrols.done);
   end;
   waitfor(LOMcontrols.done,'UserData','done');
end;

%
% end of LawsOfMotionGraphDLG.m
%
