function PlotMargLikeModifiedHarmonicAdaptiveLearning(PostSample,LogPost,NumBurnin,CurrChain,DSGEModel,CurrINI)
% PlotMargLikeModifiedHarmonicAdaptiveLearning: Calculates sequentially the modified harmonic mean estimator
%                                               of the log marginal likelihood under adaptive learning.
%
% USAGE:
%
%       PlotMargLikeModifiedHarmonicAdaptiveLearning(PostSample,NumBurnin,CurrChain,DSGEModel,CurrINI)
%
% REQUIRED INPUT: PostSample (matrix) of dimension NumIterxNumParam with the posterior sample draws
%                            of the phi parameters.
%
%                 LogPost (vector) of dimension (NumIterx1) with all the log posterior values for the
%                            non-discarded posterior draws. These posterior values are calculated
%                            using the "logPosteriorPhiDSGE" function.
%
%                 NumBurnin (integer), with the number of burn-in draws.
%
%                 CurrChain (integer) with the current MCMC chain number.
%
%                 DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 28, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-05-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% locate the controls structure
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
%
% Get the base values number of draws, etc.
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% Coverage probabilities for the Modified Harmonic Mean estimator of
% the marginal likelihood
%
CovStartStr = get(controls.dsge.coveragestart,'String');
DSGEModel.CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
CovIncStr = get(controls.dsge.coverageincrement,'String');
DSGEModel.CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
CovEndStr = get(controls.dsge.coverageend,'String');
DSGEModel.CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
%
% sequential marginal likelihood start value and step length
%
SeqStartStr = get(controls.dsge.sequentialstart,'String');
DSGEModel.SequentialStartIterationValue = str2num(strrep(StringTrim(SeqStartStr(DSGEModel.SequentialStartIteration,:)),',',''));
SeqStepStr = get(controls.dsge.sequentialstep,'String');
DSGEModel.SequentialStepLengthValue = str2num(strrep(StringTrim(SeqStepStr(DSGEModel.SequentialStepLength,:)),',',''));
%
% Open up a wait dialog.
%
txt = ['Please wait while YADA computes Geweke''s modified harmonic mean estimator of the log marginal likelihood under adaptive learning. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
WaitHandle = WaitDLG(txt,'information','Sequential Marginal Likelihood',600,300,CurrINI,1);
WaitControls = get(WaitHandle,'UserData');
%
% estimate the marginal likelihood
%
[LogMargsMH,CovProb,NWStdErr] = MargLikeModifiedHarmonic(PostSample(NumBurnin+1:NumPostDraws,:),LogPost(NumBurnin+1:NumPostDraws,:),1,[],DSGEModel,WaitControls.cancel);
%
% shut down the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
if isempty(LogMargsMH)==0;
   %
   % ask if we write to file
   %
   txt = 'Would you like to save the sequential estimation data to file?';
   answer = Query(txt,'question',200,'Save - Geweke''s Modified Harmonic Mean',500,'yes',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      %
      Xvalues = LogMargsMH(:,1)+NumBurnin;
      LogMargLikeMH = LogMargsMH(:,2:size(LogMargsMH,2));
      %
      file = FixFilePath([DSGEModel.OutputDirectory '\SeqGewekeModifiedHarmonic-AdaptiveLearning-' DSGEModel.NameOfModel '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.mat']);
      save(file,'CovProb','Xvalues','LogMargLikeMH');
      %
      mfile = FixFilePath([DSGEModel.OutputDirectory '\SeqGMHAL' strrep(DSGEModel.NameOfModel,'-','') int2str(NumPostDraws) int2str(CurrChain) '.m']);
      fid = fopen(mfile,'wt');
      fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
      fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
      fprintf(fid,'%%\n%% Number of posterior draws used for sequential estimation (N x 1) vector\n%%\n');
      fprintf(fid,'Xvalues = DataStructure.Xvalues;\n');
      fprintf(fid,'%%\n%% coverage probabilities used (1 x p vector)\n%%\n');
      fprintf(fid,'CovProb = DataStructure.CovProb;\n');
      fprintf(fid,'%%\n%% N x p matrix with log marginal likelihood values using\n%% Geweke''s modified harmonic mean estimator\n%%\n');
      fprintf(fid,'LogMargLikeMH = DataStructure.LogMargLikeMH;\n');
      fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
      %
      % end the script file
      %
      fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
      fclose(fid);
      %
      txt = ['The data on sequential estimation of the log marginal likelihood under adaptive learning has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: Xvalues (Nx1 vector with the number of posteror draws used for the sequential estimation), CovProb (1xp vector with coverage probabilities), and LogMargLikeMH (Nxp matrix with estimates of the log marginal likelihood with Geweke''s modified harmonic mean). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
      About(txt,'information','Geweke''s Modified Harmonic Mean',200,500,CurrINI);
   end;
   %
   % prepare the graph
   %
   GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
   %
   FigHandle = figure('Visible','off', ...
                      'Position',[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500], ...
                      'Tag','MargLikeMHM', ...
                      'Name','Sequential Marginal Likelihood - Geweke''s Harmonic Mean');
   %
   %
   SetFigureProperties(FigHandle,CurrINI);
   %
   Xvalues = LogMargsMH(:,1)+NumBurnin;
   Yvalues = LogMargsMH(:,2:size(LogMargsMH,2));
   CurrMap = colormap;
   NumColor = 1;
   NumStyle = 1;
   NumMarker = 1;
   ColorOrder = [(1:8:57) (2:8:58) (3:8:59) (4:8:60) (5:8:61) (6:8:62) (7:8:63) (8:8:64)];
   LegendStr = '';
   for i=1:size(Yvalues,2);
      if NumColor<=length(CurrMap);
         CurrColor = CurrMap(ColorOrder(i),:);
      else;
         CurrColor = CurrMap(ColorOrder(1),:);
         NumColor = 1;
      end;
      if NumStyle==1;
         CurrStyle = '-';
      elseif NumStyle==2;
         CurrStyle = '--';
      elseif NumStyle==3;
         CurrStyle = ':';
      elseif NumStyle==4;
         NumStyle = 0;
         CurrStyle = '-.';
      end;
      plot(Xvalues,Yvalues(:,i),'LineStyle',CurrStyle,'Color',CurrColor,'LineWidth',1);
      hold('on');
      NumColor = NumColor+1;
      NumStyle = NumStyle+1;
      LegendStr = strvcat(LegendStr,[num2str(CovProb(i)*100,'%0.0f') ' percent coverage prob']);
   end;
   %
   % plot the average
   %
   plot(Xvalues,(1/size(Yvalues,2))*sum(Yvalues,2),'LineStyle','-','Color',[0 0 0],'LineWidth',1);
   LegendStr = strvcat(LegendStr,'Overall mean');
   hold('off');
   if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
      SetAxesFonts(CurrINI,'Title','Geweke''s Harmonic Mean Estimates','XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
   elseif DSGEModel.PosteriorSampler==2;
      SetAxesFonts(CurrINI,'Title','Geweke''s Harmonic Mean Estimates','XLabel',['Slice sampler draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
   end;
   axis('tight');
   %
   % check the YLim
   %
   ylim = get(gca,'YLim');
   add_on = 0.1*(ylim(2)-ylim(1));
   ylim = [ylim(1)-add_on ylim(2)+add_on];
   set(gca,'YLim',ylim);
   %
   if MatlabNumber>=7;
      legHandle = legend(LegendStr,'Location','Best');
   else;
      legHandle = legend(LegendStr,0);
   end;
   SetAxesFonts(CurrINI,legHandle);
   %
   % save the graph
   %
   if DirExist(GraphDirectory);
      if DSGEModel.PosteriorSampler==1;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-RWM-AL-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==2;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Slice-AL-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==3;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-RWM-t-AL-' int2str(DSGEModel.StudenttDegFree) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==4;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Fixed-AL-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==5;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Fixed-AL-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==6;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Random-AL-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==7;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ModifiedHarmonic-Random-AL-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      end;
      saveas(FigHandle,file);
   end;
   %
   % display the graph
   %
   SetToolbarImages(FigHandle,CurrINI.images);
   set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of PlotMargLikeModifiedHarmonicAdaptiveLearning.m
%
