function [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(Mtilde,Ftilde,Btilde,BAtilde,M)
% ZLBProjectionMatrices: Sets up the projection matrices under adaptive learning for use when
%                        dealing with the ZLB.
%
% USAGE:
%
%       [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(Mtilde,Ftilde,Btilde,BAtilde,M)
%
% REQUIRED INPUT:  Mtilde (matrix) of dimension p x k with the coefficients on the deterministic
%                         variables in the adaptive learning solution.
%
%                  Ftilde (matrix) of dimension p x p with the coefficients on the lagged state/model
%                         variables.
%
%                  Btilde (matrix) of dimension p x q with the coefficients on the structural shocks.
%
%                  BAtilde (matrix) of dimension p x (T_ZLB+1) with the coefficients on the anticipated
%                         shocks.
%
%                  M (matrix) of dimension (T_ZLB+1) x (T_ZLB+1) with the transitions for the anticipated
%                         shocks.
%
% REQUIRED OUTPUT: Mt (matrix) of dimension (q+p+T_ZLB+1) x k with coefficients on the deterministic
%                         variables in the transition equations for the predetermined variables
%                         (structural shocks, lagged state/model variables and anticipated shocks).
%
%                  Gt (matrix) of dimension (q+p+T_ZLB+1) x (q+p+T_ZLB+1) with coefficients on the lagged
%                         predetermined variables in the transition equations for these variables.
%
%                  C1 (matrix) of dimension (q+p+T_ZLB+1) x (q+T_ZLB+1) with coefficients on the vector
%                         of structural shocks and anticipated shock innovations.
%
%                  Mtilde (matrix) of dimension p x k with the coefficients on the deterministic
%                         variables in the projection equation linking the predetermined variables
%                         to the projected variables.
%
%                  Pt (matrix) of dimension p x (q+p+T_ZLB+1) with coefficients on the predetermined
%                         variables in the projection equation.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 29, 2024.
%                        This version: March 1, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 01-03-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Mt = [];
k = size(Mtilde,2);
[p,q] = size(Btilde);
T1 = size(BAtilde,2);
if k>0;
   Mt = zeros(q+p+T1,k);
   Mt(q+1:q+p,:) = Mtilde;
end;
Gt = zeros(q+p+T1,q+p+T1);
Gt(q+1:q+p,:) = [Btilde Ftilde BAtilde];
C1 = zeros(q+p+T1,q+T1);
C1(1:q,1:q) = eye(q);
C1(q+p+1:q+p+T1,q+1:q+T1) = eye(T1);
Pt = [Btilde Ftilde BAtilde];

%
% end of ZLBProjectionMatrices.m
%
