function [logLikeValue,lnL,PredVars,mcode,status] = logMargCondLikeAdaptiveLearningDSGEAnnual(ModelParameters,r,f,SMatrix,Sperp,BeliefSys,Gtilde,Kgf,x,xbar,YData,X,h,DSGEModel,AIMData,OrderQZ,NumWorkers)
% logMargCondLikeAdaptiveLearningDSGEAnnual: Computes the marginal conditional likelihood for h-step-ahead forecasts of annualized
%                                            observed variables in a DSGE model under adaptive learning.
%
% USAGE:
%
%       [logLikeValue,lnL,PredVars,mcode,status] = logMargCondLikeAdaptiveLearningDSGEAnnual(ModelParameters,r,f,...
%                                                        SMatrix,Sperp,BeliefSys,Gtilde,Kgf,x,xbar,YDataAnnual,X,...
%                                                        h,DSGEModel,AIMData,OrderQZ,NumWorkers)
%
% REQUIRED INPUT:  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  r (integer) with the number of state variables.
%
%                  f (integer) with the number of forward looking variables.
%
%                  SMatrix (matrix) of dimension r x f that selects the forward looking variables among the state
%                                  variables.
%
%                  Sperp (matrix) of dimension r x (r-f) which select the non-forward looking variables.
%
%                  BeliefSys (structure) with fields "m" an integer denoting the maximum number of lags of the state
%                                  variables in the belief equations; "Name" a string with the  name of the belief system;
%                                  and G a cell array with f matrices, one for each forward looking variable of dimension
%                                  (k+r*m) x g_j where g_j is the number of explanatory variables in the belief equation
%                                  for forward looking variable j.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                                  of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                                  A is a g x f matrix.
%
%                  x (matrix) of dimension k x T with the sample values of the deterministic variables.
%
%                  xbar (vector) of dimension k with unit element for a constant and zeros elsewhere.
%
%                  YData (matrix) of dimension nxh with the realizations for the annualized observed variables
%                                  over the prediction sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  h (integer) with the prediction horizon.
%
%                  DSGEModel (structure) whose fields contain all required information
%                                  about the model, such as input files, variable and sample
%                                  data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                                  The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
%                  NumWorkers (integer) with the number of workers in a parallel pool.
%
% REQUIRED OUTPUT: logLikeValue (vector) of dimension h with the value of the log conditional likelihood for
%                                  marginal h-step-ahead forecasts.
%
%                  lnL (scalar) with the value of the log-likelhood over the historical sample.
%
%                  PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                                  variables for each prediction period.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not.
%                                  mcode is 1 when unique, 0 if not computed, and various other values when
%                                  there are problems with the solution. Specifically:
%
%                                     mcode = 2:      Roots not correctly computed by real_schur
%                                     mcode = 3:      Too many big roots
%                                     mcode = 35:     Too many big roots, and q(:,right) is singular
%                                     mcode = 4:      Too few big roots
%                                     mcode = 45:     Too few big roots, and q(:,right) is singular
%                                     mcode = 5:      q(:,right) is singular
%                                     mcode = 61:     Too many exact shiftrights
%                                     mcode = 62:     Too many numeric shiftrights
%                                     mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                     cannot be calculated.
%                                     mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                     numbers.
%                                     mcode = -1:     No stable solution.
%                                     mcode = -2:     Too many large generalized eigenvalues.
%                                     mcode = -3:     Too few large generalized eigenvalues.
%                                     else            Return code not properly specified
%
%                  status (integer) Indicates if the log-likelihood was calculated (status=0), or not (status=1).
%                                  In addition, status can be -1 if ths forecast covariance matrix is not positive
%                                  definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 13, 2024.
%                        This version: March 14, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-03-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
logLikeValue = zeros(h,1);
lnL = 0;
PredVars = [];
mcode = 1;
status = 0;
%
% See if we can solve the model
%
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==4;
   [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
if mcode~=1;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix,ModelEigenvalues] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
   if NumWorkers<=1;
      save(FixFilePath([pwd '\tmp\AiMErrorData.mat']),'mcode','ModelEigenvalues');
   end;
   return;
else;
   if (NumWorkers<=1)&(FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1);
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
end;
%
% transform into state space
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% Evaluate the measurement equation file
%
[A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,BeliefSys.m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
%
% call the transformation fucntion
%
[TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if TransStatus==0;
   status = 1;
   return;
end;
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,BeliefSys.m,DSGEModel.k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,BeliefSys.m,DSGEModel.k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% run the Kalman filter
%
if MatlabNumber>=8.0;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(BeliefSys.m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),BeliefSys.m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(BeliefSys.m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
   catch ME;
      status = 1;
      return;
   end;
else;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(BeliefSys.m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),BeliefSys.m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(BeliefSys.m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
   catch;
      status = 1;
      return;
   end;
end;
%
if bstatus==0;
   status = 1;
   return;
end;
%
T = size(Ksitt1,2);
ksimarg = Ksitt(:,T);
Pmarg = Ptt(:,:,T);
s = DSGEModel.Frequency;
r = length(ksimarg);
n = DSGEModel.n;
%
% compute the height of the predictive densities
%
if DSGEModel.k>0;
   Ax = A'*X;
else;
   Ax = zeros(DSGEModel.n,h);
   X = zeros(size(Mtilde,1),h)
end;
Ptti = Pmarg;
Ksiti = zeros(r,r);
Yinit = DSGEModel.Yinit;
DiffPos = DSGEModel.annual'==s;
LevPos = DSGEModel.annual'==1;
%
YData = YData(DSGEModel.PredictionVariables,:);
log2pi = log(2*pi);
%
% initial solution matrices
%
MT1 = Mtilde(:,:,T);
FT1 = Ftilde(:,:,T);
BT1 = Btilde(:,:,T);
%
% check if we have fixed or updated beliefs
%
Q = BT1*BT1';
F_i = eye(r);
Fi = eye(r);
Q_i = Q;
if length(size(H))==2;
   for i=1:h;
      %
      % levels variables forecast and covariance
      %
      ksimarg = (MT1*X(:,i))+(FT1*ksimarg);
      Pmarg = (FT1*Pmarg*FT1')+Q;
      Yhatmarg = Ax(:,i)+(H'*ksimarg);
      SigmaYmarg = (H'*Pmarg*H)+R;
      %
      % point forecasts for sums of first differences
      %
      Yagg = [Yinit Yhatmarg];
      Yhatsum = Annualize(Yagg,DSGEModel.annual,ones(n,1));
      Yinit = Yagg(:,2:s);
      %
      % annualized first differenced variables forecast and covariance
      %
      if i<=s;
         Cti = (FT1*Ptti*FT1')+Q_i;
         Sigmati = (H'*Cti*H)+(i*R);
         %
         % covariances between levels and sums of first differenced variables
         %
         Ksiti = Pmarg+(FT1*Ksiti);
         %
         % updating matrices
         %
         if i<s;
            Fi = FT1*Fi;
            F_i = F_i+Fi;
            Q_i = Q_i+(F_i*Q*F_i');
            Ptti = F_i*Ptti*F_i';
         end;
      else;
         Cti = (FT1*Cti*FT1')+Q_i;
         Sigmati = (H'*Cti*H)+(s*R);
         %
         Ksiti = (FT1*Ksiti*FT1')+(F_i*Q);
      end;
      Upsilonti = (H'*Ksiti*H)+R;
      %
      % place levels variances along the diagonal
      %
      Sigmati(LevPos,LevPos) = SigmaYmarg(LevPos,LevPos);
      %
      % place levels and sums of first diff covariances in the appropriate
      % off-dialognal elements
      %
      Sigmati(LevPos,DiffPos) = Upsilonti(LevPos,DiffPos);
      Sigmati(DiffPos,LevPos) = Upsilonti(LevPos,DiffPos)';
      %
      % select the subset
      %
      Yhatsum = Yhatsum(DSGEModel.PredictionVariables);
      Sigmati = Sigmati(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      %
      % check for missing obs
      %
      pos = ~isnan(YData(:,i));
      nstar = sum(pos);
      if nstar>0;
         zerror = YData(pos,i)-Yhatsum(pos);
         Sigmati = Sigmati(pos,pos);
         logLikeValue(i) = -((nstar*log2pi)+LogDet(Sigmati)+((zerror'/Sigmati)*zerror))/2;
         PredVars{i} = DSGEModel.PredictionVariables(pos);
      else;
         PredVars{i} = [];
      end;
      %
      % check if we should update the beliefs
      %
      if (DSGEModel.FixedBeliefsForProj>1)&(IsALUpdate==1);
         if i==1;
            MT1 = LastSolution.MTT;
            FT1 = LastSolution.FTT;
            BT1 = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         elseif i<h;
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MT1,FT1,BT1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,BeliefSys.m,DSGEModel.k,betaTiT);
            else;
               [MT1,FT1,BT1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,BeliefSys.m,DSGEModel.k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
         Q = BT1*BT1';
      end;
   end;
else;
   for i=1:h;
      Ht = H(:,:,DSGEModel.LastPeriod+i);
      %
      % levels variables forecast and covariance
      %
      ksimarg = (MT1*X(:,i))+(FT1*ksimarg);
      Pmarg = (FT1*Pmarg*FT1')+Q;
      Yhatmarg = Ax(:,i)+(Ht'*ksimarg);
      SigmaYmarg = (Ht'*Pmarg*Ht)+R;
      %
      % point forecasts for sums of first differences
      %
      Yagg = [Yinit Yhatmarg];
      Yhatsum = Annualize(Yagg,DSGEModel.annual,ones(n,1));
      Yinit = Yagg(:,2:s);
      %
      % annualized first differenced variables forecast and covariance
      %
      if i<=s;
         Cti = (FT1*Ptti*FT1')+Q_i;
         Sigmati = (Ht'*Cti*Ht)+(i*R);
         %
         % covariances between levels and sums of first differenced variables
         %
         Ksiti = Pmarg+(FT1*Ksiti);
         %
         % updating matrices
         %
         if i<s;
            Fi = FT1*Fi;
            F_i = F_i+Fi;
            Q_i = Q_i+(F_i*Q*F_i');
            Ptti = F_i*Ptti*F_i';
         end;
      else;
         Cti = (FT1*Cti*FT1')+Q_i;
         Sigmati = (Ht'*Cti*Ht)+(s*R);
         %
         Ksiti = (FT1*Ksiti*FT1')+(F_i*Q);
      end;
      Upsilonti = (Ht'*Ksiti*Ht)+R;
      %
      % place levels variances along the diagonal
      %
      Sigmati(LevPos,LevPos) = SigmaYmarg(LevPos,LevPos);
      %
      % place levels and sums of first diff covariances in the appropriate
      % off-dialognal elements
      %
      Sigmati(LevPos,DiffPos) = Upsilonti(LevPos,DiffPos);
      Sigmati(DiffPos,LevPos) = Upsilonti(LevPos,DiffPos)';
      %
      % select the subset
      %
      Yhatsum = Yhatsum(DSGEModel.PredictionVariables);
      Sigmati = Sigmati(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      %
      % check for missing obs
      %
      pos = ~isnan(YData(:,i));
      nstar = sum(pos);
      if nstar>0;
         zerror = YData(pos,i)-Yhatsum(pos);
         Sigmati = Sigmati(pos,pos);
         logLikeValue(i) = -((nstar*log2pi)+LogDet(Sigmati)+((zerror'/Sigmati)*zerror))/2;
         PredVars{i} = DSGEModel.PredictionVariables(pos);
      else;
         PredVars{i} = [];
      end;
      %
      % check if we should update the beliefs
      %
      if (DSGEModel.FixedBeliefsForProj>1)&(IsALUpdate==1);
         if i==1;
            MT1 = LastSolution.MTT;
            FT1 = LastSolution.FTT;
            BT1 = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         elseif i<h;
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MT1,FT1,BT1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,BeliefSys.m,DSGEModel.k,betaTiT);
            else;
               [MT1,FT1,BT1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,BeliefSys.m,DSGEModel.k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
         Q = BT1*BT1';
      end;
   end;
end;

%
% end of logMargCondLikeAdaptiveLearningDSGEAnnual.m
%
