function [logPost,logLike,logLikeT] = logPosteriorAdaptiveLearningThetaDSGE(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,S,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,OrderQZ,NumWorkers)
% logPosteriorAdaptiveLearningThetaDSGE: computes the log posterior kernel of the DSGE model subject to adaptive learning for the
%                                        original parameters theta.
%
% USAGE:
%
%       [logPost,logLike,logLikeT] = logPosteriorAdaptiveLearningThetaDSGE(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,...
%                                                     thetaDist,PriorDist,ModelParameters,S,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,...
%                                                     AIMData,OrderQZ,NumWorkers)
%
% REQUIRED INPUT:  theta (vector) with all parameters to be estimated.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  ParameterBounds (matrix) of dimension NumParam x 2 with the lower and upper bounds of the parameters.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  S (matrix) of dimension r x f which selects the forward looking variables among the
%                             state variables.
%
%                  Sperp (matrix) of dimension r x (r-f) which select the non-forward looking variables.
%
%                  m (integer) with the maximum lag for the state variables in the belief equations.
%
%                  G (cell array) of dimension f for selecting the explanatory variables among deterministic and lagged
%                             state variables in each belief equation for the forward looking variables.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                             of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                             A is a g x f matrix.
%
%                  xbar (vector) of dimension k with unit element for a constant and zeros elsewhere.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
%                  NumWorkers (integer) with the number of workers in a parallel pool.
%
% REQUIRED OUTPUT: logPost (scalar) the value of the log posterior at phi when we take the log of the Jacobian in the
%                             transformation theta = g(phi) into account, i.e., the log likelihood plus the log prior
%                             of the transformed parameters.
%
% OPTIONAL OUTPUT: logLike (scalar) with the value of the log likelihood.
%
%                  logLikeT (vector) with the time t values of the log-likelihood function.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 31, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 01-04-2020: Updated the documentation.
%
% * 15-04-2020: Added the max(abs(theta))>10000 criterion as an additional upper bound
%               for the parameters.
%
% * 30-04-2020: Added try-catch code around call to log-likelihood function.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 22-02-2022: Changed the last eval call to feval.
%
% * 02-01-2023: Updated the documentation.
%
% * 01-09-2023: The mcode variable is set to zero when log likelihood call fails.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

logPost = [];
if nargout==2;
   logLike = NaN;
end;
%
% check if the bounds are satisfied by theta
%
if (any(theta<ParameterBounds(:,1))==1)|(any(theta>ParameterBounds(:,2))==1)|(max(abs(theta))>10000);
   logPost = 1000000;
   return;
end;
%
% update ModelParameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameters that need to be updated
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
end;
%
% Calculate the log-likelihood
%
try;
   [logLikeValue,Solution,mcode,status] = logLikelihoodAdaptiveLearningDSGE(ModelParameters,S,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,OrderQZ,NumWorkers);
catch;
   logPost = 1000000;
   mcode = 0;
   return;
end;
%
% add things up and take into account that the function will be minimized
%
if (mcode==1)&(status==1)&(isinf(logLikeValue)==0);
   %
   % calculate the value of the log prior density at theta
   %
   logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
   if (isnan(logPrior)==0)&(isinf(logPrior)==0);
      %
      % check if we should compute the system prior
      %
      if FileExist(DSGEModel.SystemPriorFile)==0;
         %
         % compute the function value
         %
         logPost = -(logPrior+logLikeValue);
         if nargout==2;
            logLike = logLikeValue;
         end;
      else;
         DSGEModel.Solution = Solution;
         logSP = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,OrderQZ,1);
         if isnan(logSP)==0;
            %
            % compute the function value
            %
            logPost = -(logPrior+logLikeValue+logSP);
            if nargout==2;
               logLike = logLikeValue;
            end;
         else;
            logPost = 1000000;
         end;
      end;
   else;
      logPost = 1000000;
   end;
else;
   logPost = 1000000;
end;

%
% end of logPosteriorAdaptiveLearningThetaDSGE.m
%
