function status = ZLBPredictionsExist(OutputDirectory,ModelStr,CondStr,ShockMethod,NameOfModel,ZLBSampleT,RtildePosition,REqPosition,ZLBRuleType,h,NumDraws,NumPredPaths,AnnualizeData,TransData,CurrChain,IsPosterior,IsCentered,YDataStr)
% ZLBPredictionsExist: Tests is all the results for the predictive distributions subject to the zero lower
%                      bound exist on disk.
%
% USAGE:
%
%       status = ZLBPredictionsExist(OutputDirectory,ModelStr,CondStr,ShockMethod,NameOfModel,ZLBSampleT,...
%                                    RtildePosition,REqPosition,ZLBRuleType,h,NumDraws,NumPredPaths,...
%                                    AnnualizeData,TransData,CurrChain,IsPosterior,IsCentered,YDataStr)
%
% REQUIRED INPUT:  OutputDirectory (string vector) with the full path to the base output directory.
%
%                  ModelStr (string vector) that is either "DSGE" or "AdaptiveLearningDSGE-x".
%
%                  CondStr (string vector) that is either "Cond" or "".
%
%                  ShockMethod (string vector) that is either "Direct-", "WaggonerZha-", "Subset-" or "".
%
%                  NameOfModel (string vector) with the name of the model.
%
%                  ZLBSampleT (integer)  with the length of the sample over which the zero lower
%                             bound may be binding.
%
%                  RtildePosition (integer) with the position of the monetary policy interest rate.
%
%                  REqPosition (integer) with the position of the monetary policy rule.
%
%                  ZLBRuleType (integer) which is either 1 for lagged unrestricted policy rate (the
%                             default) or 2 for lagged restricted policy rate.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  AnnualizeData (boolean) that take the value 1 if annualized results should be checked and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                             if draws from the prior are supplied.
%
%                  IsCentered (boolean) that takes the value 1 if the distributions not subject to the lower
%                             bound have been centered on the observed data and 0 otherwise.
%
%                  YDataStr (string) reflecting which data for the observed variables has been used. Is an
%                             empty string when IsCentered is 0.
%
% REQUIRED OUTPUT: status (boolean) that is 1 if the results exist on disk and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: September 1, 2022.
%                        This version: January 16, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-09-2022: Updated the documentation.
%
% * 08-09-2022: Added the input variable "YDataStr".
%
% * 06-01-2023: Updated the documentation.
%
% * 23-11-2023: Added the input variable "ZLBRuleType".
%
% * 16-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

status = 0;
if IsPosterior==1;
   for ParamIndex=1:NumDraws;
      if AnnualizeData==0;
         if TransData==0;
            file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         else;
            file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         end;
      else;
         file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      end;
      if FileExist(file)==0;
         return;
      end;
   end;
else;
   for ParamIndex=1:NumDraws;
      if AnnualizeData==0;
         if TransData==0;
            file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         else;
            file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         end;
      else;
         file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      end;
      if FileExist(file)==0;
         return;
      end;
   end;
end;
status = 1;

%
% end of ZLBPredictionsExist.m
%
