function [FilterMatrix,ConjFilterMatrix] = AnnualFilter(annual,annualscale,omega)
% AnnualFilter: Computes an annualization filter at frequency omega.
%
% USAGE:
%
%       [FilterMatrix,ConjFilterMatrix] = AnnualFilter(annual,annualscale,omega)
%
% REQUIRED INPUT:  annual (nx1) vector with entries 1 (annual), 4 (quarterly), or 12 (monthly).
%
%                  annualscale (nx1) vector with coefficients that the annualized rows will be
%                              multiplied with.
%
%                  omega (real) with the current frequency (between 0 and pi).
%
% REQUIRED OUTPUT: FilterMatrix (nxn) diagonal matrix with the filter values at frequency omega.
%
%                  ConjFilterMatrix (nxn) diagonal matrix with the conjugate filter values at frequency omega.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 6, 2009.
%                        This version: January 17, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-10-2009: Simplified the code for the mnthly filter.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 30-04-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 17-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
n = length(annual);
FilterMatrix = eye(n);
ConjFilterMatrix = eye(n);
if max(annual)==1;
   return;
end;
%
% check if we have quarterly or month data
%
if max(annual)==4;
   QFilt = (annual==4);
   QPos = QFilt.*(1:n);
   QPos = QPos(QPos>0);
   QLength = length(QPos);
   zNeg = exp(-i*omega);
   zPos = exp(i*omega);
   FilterMatrix(QPos,QPos) = (1+zNeg+(zNeg^2)+(zNeg^3))*eye(QLength);
   ConjFilterMatrix(QPos,QPos) = (1+zPos+(zPos^2)+(zPos^3))*eye(QLength);
elseif max(annual)==12;
   zNeg = exp(-i*omega);
   zPos = exp(i*omega);
   MFilt = (annual==12);
   MPos = MFilt.*(1:n);
   MPos = MPos(MPos>0);
   MLength = length(MPos);
   FilterMatrix(MPos,MPos) = (1+zNeg+(zNeg^2)+(zNeg^3)+(zNeg^4)+(zNeg^5)+(zNeg^6)+(zNeg^7)+(zNeg^8)+(zNeg^9)+(zNeg^10)+(zNeg^11))*eye(MLength);
   ConjFilterMatrix(MPos,MPos) = (1+zPos+(zPos^2)+(zPos^3)+(zPos^4)+(zPos^5)+(zPos^6)+(zPos^7)+(zPos^8)+(zPos^9)+(zPos^10)+(zPos^11))*eye(MLength);
end;
FilterMatrix = diag(annualscale)*FilterMatrix;
ConjFilterMatrix = diag(annualscale)*ConjFilterMatrix;

%
% end of AnnualFilter.m
%
