function AnnualYPaths = AnnualizePaths(YPaths,DSGEModel,FirstPeriod,LastPeriod,KsitT,A,H)
% AnnualizePaths: Computes paths for annualized data.
%
% USAGE:
%
%       AnnualYPaths = AnnualizePaths(YPaths,DSGEModel,FirstPeriod,LastPeriod,KsitT,A,H)
%
% REQUIRED INPUT:  YPaths (matrix) of dimension nxhxNumPaths with the paths for the data in original form.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  FirstPeriod (integer) with the first period in the sample.
%
%                  LastPeriod (integer) with the last period in the sample.
%
%                  KsitT (matrix) of dimension rxT with the smooth estimates of the state variables.
%
%                  A (matrix) of dimension kxn with the steady-state parameters.
%
%                  H (matrix) of dimension rxn (xT) with the coefficients on the state variables
%                    in the measurement equations.
%
% REQUIRED OUTPUT: AnnualYPaths (matrix) of dimension nxhxNumPaths with the paths for the data in annualized form.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: September 4, 2014.
%                        This version: January 17, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-09-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 30-04-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 17-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[n,h,NumPaths] = size(YPaths);
AnnualYPaths = zeros(n,h,NumPaths);
%
% determine historical values for the observed variabled required for annualization
%
if DSGEModel.YNaNs==0;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
   else;
      Yinit = [];
   end;
else;
   YSmooth = DSGEModel.Y(:,FirstPeriod:LastPeriod);
   nanpos = isnan(YSmooth);
   if max(max(nanpos))==1;
      if DSGEModel.k>0;
         if length(size(H))==2;
            YHistPred = (A'*DSGEModel.X(:,FirstPeriod:LastPeriod))+(H'*KsitT);
         else;
            YHistPred = A'*DSGEModel.X(:,FirstPeriod:LastPeriod);
            for t=1:size(KsitT,2);
               YHistPred(:,t) = YHistPred(:,t)+(H(:,:,FirstPeriod+t-1)'*KsitT(:,t));
            end;
         end;
      else;
         if length(size(H))==2;
            YHistPred = H'*KsitT;
         else;
            YHistPred = zeros(DSGEModel.n,size(KsitT,2));
            for t=1:size(KsitT,2);
               YHistPred(:,t) = H(:,:,FirstPeriod+t-1)'*KsitT(:,t);
            end;
         end;
      end;
      YSmooth(nanpos) = YHistPred(nanpos);
   end;
   if FirstPeriod>1;
      YSmooth = [DSGEModel.Y(:,1:FirstPeriod-1) YSmooth];
   end;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
   else;
      Yinit = [];
   end;
end;
%
% now compute the annualization
%
for it=1:NumPaths;
   AnnualYPaths(:,:,it) = Annualize([Yinit YPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
end;

%
% end of AnnualizePaths.m
%
