function [PSq,CError] = PSDCholeski(P)
% PSDCholeski: Computes a lower triangular Choleski decomposition for any positive semidefinite matrices.
%              If the input matrix has reduced rank the function uses a combination of an eigenvalue
%              decomposition (to obtain a square-root matrix) and then a QR decomposition (to obtain
%              a lower tringular form).
%
% USAGE:
%
%       [PSq,CError] = PSDCholeski(P)
%
% REQUIRED INPUT:  P (nxn) positive semidefinite matrix.
%
% REQUIRED OUTPUT: PSq (nxn) lower triangular matrix such that PSq*PSq' = P
%
% OPTIONAL OUTPUT: CError (positive scalar) with the computation error.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 19, 2010.
%                        This version: January 17, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-08-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 20-01-2011: Made sure that the diagonal elements of PSq are always nonnegative.
%
% * 04-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 30-04-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 17-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

rp = rank(P);
if rp<size(P,1);
   if rp>0;
      [Vals,Eigs] = eig(P);
      PH = real(Vals)*diag(sqrt(diag(abs(Eigs))));
      [G,PT] = qr(PH');
      PSq = PT';
      dPSq = diag(PSq);
      PSq = PSq*diag((dPSq>=0)-(dPSq<0));
   else;
      PSq = zeros(size(P));
   end;
else;
   PSq = chol(P)';
end;
if nargout==2;
   CError = max(max(abs(P-(PSq*PSq'))));
end;

%
% end of PSDCholeski.m
%
