function y = TriGamma(x)
% TriGamma: Computes d^2 log(gamma(x))/dx^2, the trigamma function.
%
% USAGE:
%
%       y = TriGamma(x)
%
% REQUIRED INPUT:  x (vector) with positive entries.
%
% REQUIRED OUTPUT: y (vector) with values of the first derivative of the log of the gamma
%                    function.
%
% REFERENCE: The algorithm is based on Schneider, B.E. (1978), "Trigamma Function",
%            (Algorithm AS 121), Applied Statistics, 27(1), 97-99.
%
% NOTE: The algorithm also uses modifications suggested by Tom Minka in the function "trigamma.m"
%       from the Lightspeed Toolbox. <http://research.microsoft.som/en-us/um/people/minka/>
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 3, 2009.
%                        This version: January 17, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 07-04-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 30-04-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 17-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize output
%
y = [];
if any(x<=0)==1;
   return;
end;
%
% we're in business. upper and lower bound for the values
% where we run the loop trigamma(1+x) = trigamma(x) - (1/x^2)
%
A = 1e-4;
B = 8;
%
% Terms for bounds of small x - c is also equal to trigamma(1)
%
c = pi^2/6;
c1 = -2.404113806319188570799476;
%
% constants in asymptotic expansion formula
%
b2 =  1/6;
b4 = -1/30;
b6 =  1/42;
b8 = -1/30;
b10 = 5/66;
%
% initialize the output
%
y = zeros(size(x));
%
% use approximation (5) in Schneider where the O(1) is bounded from
% above by pi^2/6
%
Ind = find(x<=A);
if isempty(Ind)==0;
   y(Ind) = (1./(x(Ind).*x(Ind)))+c+(c1*x(Ind));
end;
%
% for A<x<B we iterate on (1)
%
RunLoop = 1;
while RunLoop==1;
   %
   % find value between S and C. For these we use (5) in Bernardo
   % and then update x by 1 until x+n>=C
   %
   Ind = find((x>A)&(x<B));
   if isempty(Ind)==1;
      RunLoop = 0;
   end;
   if RunLoop==1
      y(Ind) = y(Ind)+(1./(x(Ind).*x(Ind)));
      x(Ind) = x(Ind)+1;
   end;
end;
%
% use asymptotic expansion when x>=B
%
Ind = find(x>=B);
if isempty(Ind)==0
   r = 1./(x(Ind).*x(Ind));
   y(Ind) = y(Ind)+(r/2)+((1+(r.*(b2+(r.*(b4+(r.*(b6+(r.*(b8+(r.*b10))))))))))./x(Ind));
end;

%
% end of TriGamma.m
%
