function [PartialDerivMatrix,Status] = YADANumericalPartialsMatrix(fcn,x,varargin)
% YADANumericalGradientMatrix: Computes a matrix with numerical partial derivatives when the function being
%                              differentiated returns a vector and the argument for the differentiation
%                              also is a vector.
%
% USAGE:
%
%       [PartialDerivMatrix,Status] = YADANumericalPartialsMatrix(fcn,x,varargin)
%
% REQUIRED INPUT:  fcn (string vector) with the name of the function being differentiated.
%                      The function returns a vector of dimension n.
%
%                  x (vector) if dimention m with the values at which the partials of fcn
%                      should be evaluated.
%
%                  varargin (variable length input argument list) which all the variables
%                      that should be sent to fcn.
%
% REQUIRED OUTPUT: PartialDerivMatrix (matrix) of dimension nxm with the numerical partials of fcn.
%
%                  Status (boolean) that takes the value 1 if the compuations were carried
%                      out successfully and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 2, 2010.
%                        This version: January 17, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 03-02-2010: Updated the documentation.
%
% * 01-12-2010: The "scale" parameter is now set to unity for all x-entries.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 30-04-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 17-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
PartialDerivMatrix = [];
Status = 0;
%
% evaluate fcn at x
%
delta = 1e-6;
m = length(x);
tvec = delta*eye(m);
%
f0 = feval(fcn,x,varargin{:});
if isempty(f0);
   return;
end;
PartialDerivMatrix = zeros(length(f0),m);
BadPartial = 0;
for i=1:m;
   %
   % now we make changes to x and evaluate fcn.
   %
%   if abs(x(i))>0;
%      scale = abs(x(i));
%   else;
      scale = 1;
%   end;
   if size(x,1)>size(x,2)
      tvecv=tvec(i,:);
   else
      tvecv=tvec(:,i);
   end
   g0 = (feval(fcn,x+scale*tvecv',varargin{:})-f0)./(scale*delta);
   if max(abs(g0))<1e15;
      PartialDerivMatrix(:,i) = vec(g0);
   else;
      BadPartial = 1;
      break;
   end;
end;
if BadPartial==0;
   Status = 1;
end;

%
% end of YADANumericalPartialsMatrix.m
%
