function controls = ViewMenuControls(maingui,controls,AiMEnable,DataEnable)
% ViewMenuControls: sets up the View menu.
%
% USAGE:
%
%       controls = ViewMenuControls(maingui,controls,AiMEnable,)
%
% REQUIRED INPUT:  maingui (handle) to YADA's main dialog.
%
%                  controls (structure) with handles to the controls on the YADA main dialog.
%
%                  AiMEnable (string) that takes on the value 'on' or 'off' if AiM should
%                            be enabled or not.
%
%                  DataEnable (string) that takes on the value 'on' or 'off' is the data on
%                            observed variables is available or not.
%
% REQUIRED OUTPUT: controls (structure) with handles to the controls on the YADA main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 3, 2010.
%                        This version: January 18, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-03-2010: Updated the documentation.
%
% * 07-03-2010: Changed the cases of the callback indicator from all lower case
%               to mixed upper and lower case.
%
% * 21-04-2010: Added the submenu item "Structural form" to State-space form.
%
% * 09-07-2010: Added the properties controls for contrallability and observability.
%
% * 01-12-2010: Removed the view control for "Save Annualized Data". Added view control
%               for "Laplace Posterior Densities".
%
% * 05-01-2011: Additional controls for displaying modesty analysis are now loaded.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 21-06-2011: Added the parameter values controls.
%
% * 02-01-2012: Updated the documentation.
%
% * 02-03-2012: Added controls for displaying prior and posterior mean and standard deviation
%               of the state-space parameters.
%
% * 06-03-2012: Added controls for computing the Laplace approximation of the marginal likelihood
%               for initial values and posterior mode values.
%
% * 30-11-2012: Added control for computing the Sims, Waggoner and Zha truncated elliptical version
%               of the modified harmonic mean.
%
% * 03-01-2013: Updated the documentation.
%
% * 18-07-2013: The function now supports controls for modesty statistics based on conditioning
%               assumptions on the state variables.
%
% * 28-01-2014: Updated the documentation.
%
% * 20-02-2014: Added controls for plotting the log posterior of the raw posterior draws.
%
% * 24-02-2014: Updated the documentation.
%
% * 25-02-2014: Updated the documnetation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 28-08-2017: Added the Weights case to the raw posterior draws.
%
% * 11-09-2017: Updated the documentation.
%
% * 15-09-2017: Added control for SMC adaption graphs.
%
% * 13-12-2017: Corrected a typo.
%
% * 08-01-2018: Updated the documentation.
%
% * 10-10-2018: Added controls for marginal and bivariate posterior densities.
%
% * 24-10-2018: Added controls for marginal and bivariate prior densities.
%
% * 31-10-2018: Added control for prior correlations (used only with system prior).
%
% * 05-11-2018: Added control for posterior correlations.
%
% * 23-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 18-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% View menu
%
controls.viewmenu = uimenu('Parent',maingui, ...
   'Label','&View', ...
   'Tag','ViewMenu');
%
controls.viewgraphicsmenu = uimenu('Parent',controls.viewmenu, ...
   'Label','Open &Graphics...', ...
   'Callback','ViewMenuFunctions ViewGraphics', ...
   'Enable','on', ...
   'Tag','ViewGraphicsMenu');
%
% Objects on the View menu
%
controls.datainfo = uimenu('Parent',controls.viewmenu, ...
   'Label','&Data Construction Information...', ...
   'Callback','ViewMenuFunctions DataInfo', ...
   'Separator','on', ...
   'Enable',DataEnable, ...
   'Tag','DataInfoMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                             View AiM Model file                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.aimmodelinfo = uimenu('Parent',controls.viewmenu, ...
   'Label','&AiM Model File...', ...
   'Callback','ViewMenuFunctions AiMModelInfo', ...
   'Enable',AiMEnable, ...
   'Tag','AiMModelInfoMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           View prior information                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.priorinfo = uimenu('Parent',controls.viewmenu, ...
   'Label','Prior Distrib&ution Information...', ...
   'Callback','ViewMenuFunctions PriorInfo', ...
   'Enable','off', ...
   'Tag','PriorInfoMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                              State-space form                               %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.statespaceparent = uimenu('Parent',controls.viewmenu, ...
   'Label','State-Space &Form', ...
   'Separator','on', ...
   'Tag','StateSpaceParentMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                              Parameter values                               %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.parametervaluesparent = uimenu('Parent',controls.statespaceparent, ...
   'Label','P&arameter Values', ...
   'Enable','on', ...
   'Tag','ParameterValuesParentMenu');
%
% at initial values
%
controls.parametervalues.initialvalues = uimenu('Parent',controls.parametervaluesparent, ...
   'Label','&Initial Values...', ...
   'Callback','ViewMenuFunctions ParameterValuesInitialValues', ...
   'Enable','off', ...
   'Tag','ParameterValuesInitialValuesMenu');
%
% at posterior mode values
%
controls.parametervalues.postmode = uimenu('Parent',controls.parametervaluesparent, ...
   'Label','&Posterior Mode...', ...
   'Callback','ViewMenuFunctions ParameterValuesPostMode', ...
   'Enable','off', ...
   'Tag','ParameterValuesPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                Controllability and observability properties                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.propertiesparent = uimenu('Parent',controls.statespaceparent, ...
   'Label','Pr&operties', ...
   'Enable','on', ...
   'Tag','PropertiesParentMenu');
%
% initial values
%
controls.properties.initialvalues = uimenu('Parent',controls.propertiesparent, ...
   'Label','&Initial Values...', ...
   'Callback','ViewMenuFunctions PropertiesInitialValues', ...
   'Enable','off', ...
   'Tag','PropertiesInitialValuesMenu');
%
% posterior mode values
%
controls.properties.postmode = uimenu('Parent',controls.propertiesparent, ...
   'Label','&Posterior Mode...', ...
   'Callback','ViewMenuFunctions PropertiesPostMode', ...
   'Enable','off', ...
   'Tag','PropertiesPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                      Structural form of the DSGE model                      %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.structuralformparent = uimenu('Parent',controls.statespaceparent, ...
   'Label','Structural &Form', ...
   'Enable','on', ...
   'Tag','StructuralFormSaveParentMenu');
%
% at intial values
%
controls.structuralforminitialvalues = uimenu('Parent',controls.structuralformparent, ...
   'Label','&Initial Values...', ...
   'Callback','ViewMenuFunctions StructuralFormInitialValues', ...
   'Enable','off', ...
   'Tag','StructuralFormInitialValuesMenu');
%
% at posterior mode values
%
controls.structuralformpostmode = uimenu('Parent',controls.structuralformparent, ...
   'Label','&Posterior Mode...', ...
   'Callback','ViewMenuFunctions StructuralFormPostMode', ...
   'Enable','off', ...
   'Tag','StructuralFormPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                        Save state-space form to disk                        %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.statespacesaveparent = uimenu('Parent',controls.statespaceparent, ...
   'Label','&Save State-Space Matrices', ...
   'Enable','on', ...
   'Tag','StateSpaceSaveParentMenu');
%
% at initial values
%
controls.statespacesaveinitialvalues = uimenu('Parent',controls.statespacesaveparent, ...
   'Label','&Initial Values...', ...
   'Callback','ViewMenuFunctions StateSpaceSaveInitialValues', ...
   'Enable','off', ...
   'Tag','StateSpaceSaveInitialValuesMenu');
%
% at posterior mode
%
controls.statespacesavemode = uimenu('Parent',controls.statespacesaveparent, ...
   'Label','&Posterior Mode...', ...
   'Callback','ViewMenuFunctions StateSpaceSavePostMode', ...
   'Enable','off', ...
   'Tag','StateSpaceSaveModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                            View state-space form                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% at initial values
%
controls.statespaceinitialvalues = uimenu('Parent',controls.statespaceparent, ...
   'Label','&Initial Values...', ...
   'Callback','ViewMenuFunctions StateSpaceInitialValues', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','StateSpaceInitialValuesMenu');
%
% prior distribution
%
controls.statespacepriordist = uimenu('Parent',controls.statespaceparent, ...
   'Label','P&rior Distribution...', ...
   'Callback','ViewMenuFunctions StateSpacePriorDist', ...
   'Enable','off', ...
   'Tag','StateSpacePriorDistMenu');
%
% at posterior mode
%
controls.statespacemode = uimenu('Parent',controls.statespaceparent, ...
   'Label','&Posterior Mode...', ...
   'Callback','ViewMenuFunctions StateSpacePostMode', ...
   'Enable','off', ...
   'Separator','on', ...
   'Tag','StateSpaceModeMenu');
%
% posterior distribution
%
controls.statespaceposteriordist = uimenu('Parent',controls.statespaceparent, ...
   'Label','Posterior &Distribution...', ...
   'Callback','ViewMenuFunctions StateSpacePosteriorDist', ...
   'Enable','off', ...
   'Tag','StateSpacePosteriorDistMenu');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                plot-a-prior                                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.plotaprior = uimenu('Parent',controls.viewmenu, ...
   'Label','Grap&h-a-Prior...', ...
   'Callback','ViewMenuFunctions PlotAPrior', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','PlotAPriorMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                  estimate the parameter covariance matrix                   %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.estimateparamcovmat = uimenu('Parent',controls.viewmenu, ...
   'Label','Parameter Covariance Matri&x...', ...
   'Callback','ViewMenuFunctions EstimateParamCovMat', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','EstimateParameterCovarianceMatrixMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           posterior mode results                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.modeinfo = uimenu('Parent',controls.viewmenu, ...
   'Label','&Posterior Mode Results...', ...
   'Callback','ViewMenuFunctions ModeInfo', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','ModeInfoMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                       posterior mode results summary                        %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.modesummary = uimenu('Parent',controls.viewmenu, ...
   'Label','Posterior &Mode Summary...', ...
   'Callback','ViewMenuFunctions ModeSummaryInfo', ...
   'Enable','off', ...
   'Tag','ModeSummaryInfoMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         optimization error summary                          %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.opterrorsummary = uimenu('Parent',controls.viewmenu, ...
   'Label','Optimi&zation Error Summary...', ...
   'Callback','ViewMenuFunctions OptErrorSummaryInfo', ...
   'Enable','off', ...
   'Tag','OptErrorSummaryInfoMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         posterior sampling summary                          %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsamplesummary.parent = uimenu('Parent',controls.viewmenu, ...
   'Label','Posterior &Sampling Summary', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','PostSampleSummaryMenu');
%
%
for i=1:10;
   ModHandle = uimenu('Parent',controls.postsamplesummary.parent, ...
      'Label',['Chain &' num2str(i,'%0.0f') '...'], ...
      'Callback','ViewMenuFunctions PostSampleSummary', ...
      'Enable','off', ...
      'UserData','', ...
      'Tag',['PostSampleSummaryChain' num2str(i,'%0.0f') 'Menu']);
   if i==10;
      set(ModHandle,'Label','Chain 1&0...');
   end;
   controls.postsamplesummary = setfield(controls.postsamplesummary,['chain' num2str(i,'%0.0f')],ModHandle);
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         posterior plots - raw draws                         %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.rawplots = uimenu('Parent',controls.viewmenu, ...
   'Label','Ra&w Posterior Draws', ...
   'Enable','on', ...
   'Tag','PostSampleRawPlotsMenu');
%
controls.postsample.rawplots_original = uimenu('Parent',controls.postsample.rawplots, ...
   'Label','&Original Parameters...', ...
   'Callback','ViewMenuFunctions RawPlots', ...
   'Enable','off', ...
   'Tag','PostSampleOriginalRawPlotsMenu');
%
controls.postsample.rawplots_transformed = uimenu('Parent',controls.postsample.rawplots, ...
   'Label','&Transformed Parameters...', ...
   'Callback','ViewMenuFunctions RawPlots', ...
   'Enable','off', ...
   'Tag','PostSampleTransformedRawPlotsMenu');
%
controls.postsample.rawplots_logpost = uimenu('Parent',controls.postsample.rawplots, ...
   'Label','&Log Posterior Kernel...', ...
   'Callback','ViewMenuFunctions RawPlots', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','LogPosteriorRawPlotsMenu');
%
controls.postsample.rawplots_weights = uimenu('Parent',controls.postsample.rawplots, ...
   'Label','&Particle Weights...', ...
   'Callback','ViewMenuFunctions RawPlots', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','WeightsRawPlotsMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                        scatter plot posterior draws                         %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.scatterplots = uimenu('Parent',controls.viewmenu, ...
   'Label','Scatt&er-Plot Posterior Draws', ...
   'Enable','on', ...
   'Tag','PostSampleScatterPlotsMenu');
%
controls.postsample.scatterplots_original = uimenu('Parent',controls.postsample.scatterplots, ...
   'Label','&Original Parameters...', ...
   'Callback','ViewMenuFunctions ScatterPlots', ...
   'Enable','off', ...
   'Tag','PostSampleOriginalScatterPlotsMenu');
%
controls.postsample.scatterplots_transformed = uimenu('Parent',controls.postsample.scatterplots, ...
   'Label','&Transformed Parameters...', ...
   'Callback','ViewMenuFunctions ScatterPlots', ...
   'Enable','off', ...
   'Tag','PostSampleTransformedScatterPlotsMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                       marginal likelihood estimation                        %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.marglike = uimenu('Parent',controls.viewmenu, ...
   'Label','Se&quential Marginal Likelihood', ...
   'Enable','on', ...
   'Tag','MargLikeMenu');
%
% Laplace approximation
%
controls.postsample.laplaceparent = uimenu('Parent',controls.postsample.marglike, ...
   'Label','&Laplace Approximation', ...
   'Enable','on', ...
   'Tag','LaplaceMargLikeMenu');
%
% initial values
%
controls.postsample.laplaceinitval = uimenu('Parent',controls.postsample.laplaceparent, ...
   'Label','&Initial Values...', ...
   'Callback','ViewMenuFunctions LaplaceInitialValues', ...
   'Enable','off', ...
   'Tag','LaplaceMargLikeInitialValuesMenu');
%
% posterior mode
%
controls.postsample.laplacepostmode = uimenu('Parent',controls.postsample.laplaceparent, ...
   'Label','&Posterior Mode...', ...
   'Callback','ViewMenuFunctions LaplacePosteriorMode', ...
   'Enable','off', ...
   'Tag','LaplaceMargLikePosteriorModeMenu');
%
% Chib and Jeliazkov
%
controls.postsample.chibjeliazkov = uimenu('Parent',controls.postsample.marglike, ...
   'Label','&Chib and Jeliazkov...', ...
   'Callback','ViewMenuFunctions ChibJeliazkov', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','PostSampleCJMenu');
%
% Geweke's modified harmonic mean
%
controls.postsample.mhm = uimenu('Parent',controls.postsample.marglike, ...
   'Label','&Modified Harmonic Mean...', ...
   'Callback','ViewMenuFunctions ModifiedHarmonicMean', ...
   'Enable','off', ...
   'Tag','PostSampleMHMMenu');
%
% Sims, Waggoner and Zha
%
controls.postsample.swz = uimenu('Parent',controls.postsample.marglike, ...
   'Label','&Sims, Waggoner and Zha...', ...
   'Callback','ViewMenuFunctions TruncatedElliptical', ...
   'Enable','off', ...
   'Tag','PostSampleSWZMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                 convergence                                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.convergence = uimenu('Parent',controls.viewmenu, ...
   'Label','Con&vergence', ...
   'Enable','on', ...
   'Tag','ConvergenceMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                 CUSUM plots                                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.cusumplots.parent = uimenu('Parent',controls.postsample.convergence, ...
   'Label','&CUSUM Plots', ...
   'Enable','on', ...
   'Tag','CusumPlotsMenu');
%
controls.postsample.cusumplots.logposterior = uimenu('Parent',controls.postsample.cusumplots.parent, ...
   'Label','&Log Posterior Kernel...', ...
   'Callback','ViewMenuFunctions CUSUMLogPosterior', ...
   'Enable','off', ...
   'Tag','CusumPlotsLogPosteriorMenu');
%
controls.postsample.cusumplots.original = uimenu('Parent',controls.postsample.cusumplots.parent, ...
   'Label','&Original Parameters...', ...
   'Callback','ViewMenuFunctions CUSUMParameters', ...
   'Enable','off', ...
   'Tag','CusumPlotsOriginalParametersMenu');
%
controls.postsample.cusumplots.transformed = uimenu('Parent',controls.postsample.cusumplots.parent, ...
   'Label','&Transformed Parameters...', ...
   'Callback','ViewMenuFunctions CUSUMParameters', ...
   'Enable','off', ...
   'Tag','CusumPlotsTransformedParametersMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                        Separated partial means test                         %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.spmt.parent = uimenu('Parent',controls.postsample.convergence, ...
   'Label','&Separated Partial Means Test', ...
   'Enable','on', ...
   'Tag','SeparatedPartialMeansTestMenu');
%
controls.postsample.spmt.logposterior = uimenu('Parent',controls.postsample.spmt.parent, ...
   'Label','&Log Posterior Kernel...', ...
   'Callback','ViewMenuFunctions SPMTLogPosterior', ...
   'Enable','off', ...
   'Tag','SPMTLogPosteriorMenu');
%
controls.postsample.spmt.original = uimenu('Parent',controls.postsample.spmt.parent, ...
   'Label','&Original Parameters...', ...
   'Callback','ViewMenuFunctions SPMTParameters', ...
   'Enable','off', ...
   'Tag','CusumPlotsOriginalParametersMenu');
%
controls.postsample.spmt.transformed = uimenu('Parent',controls.postsample.spmt.parent, ...
   'Label','&Transformed Parameters...', ...
   'Callback','ViewMenuFunctions SPMTParameters', ...
   'Enable','off', ...
   'Tag','CusumPlotsTransformedParametersMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                               Acceptance rate                               %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.acceptancerate = uimenu('Parent',controls.postsample.convergence, ...
   'Label','Acceptance &Ratio...', ...
   'Callback','ViewMenuFunctions ConvergeAcceptance', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','AcceptanceRatioConvergeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                SMC adaption                                 %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.smcadaption = uimenu('Parent',controls.postsample.convergence, ...
   'Label','SMC A&daption...', ...
   'Callback','ViewMenuFunctions SMCAdaption', ...
   'Enable','off', ...
   'Tag','SMCAdaptionConvergeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                   MANOVA                                    %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.manova = uimenu('Parent',controls.postsample.convergence, ...
   'Label','Multivariate &ANOVA...', ...
   'Callback','ViewMenuFunctions ConvergeMANOVA', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','MANOVAConvergeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                          Recursive posterior mean                           %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.mean = uimenu('Parent',controls.postsample.convergence, ...
   'Label','&Posterior Mean...', ...
   'Callback','ViewMenuFunctions ConvergeMean', ...
   'Enable','off', ...
   'Tag','PostSampleConvergeMeanMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         Recursive posterior median                          %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.postsample.median = uimenu('Parent',controls.postsample.convergence, ...
   'Label','Posterior &Median...', ...
   'Callback','ViewMenuFunctions ConvergeMedian', ...
   'Enable','off', ...
   'Tag','PostSampleConvergeMedianMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                             modesty statistics                              %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.modesty.parent = uimenu('Parent',controls.viewmenu, ...
   'Label','Modest&y Statistics', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','ModestyMenu');
%
% modesty controls for conditioning assumptions on the state variables.
%
controls.modesty.state.parent = uimenu('Parent',controls.modesty.parent, ...
   'Label','&State Variable Assumptions', ...
   'Tag','ModestyStateParentMenu');
%
% initial values
%
controls.modesty.state.initialvalues = uimenu('Parent',controls.modesty.state.parent, ...
   'Label','&Initial Values...', ...
   'Callback','ViewMenuFunctions ViewModestyStat', ...
   'Enable','off', ...
   'UserData','', ...
   'Tag','ModestyStateInitialValuesMenu');
%
% prior distribution
%
controls.modesty.state.prior = uimenu('Parent',controls.modesty.state.parent, ...
   'Label','P&rior Distribution...', ...
   'Callback','ViewMenuFunctions ViewModestyStat', ...
   'Enable','off', ...
   'UserData','', ...
   'Tag','ModestyStatePriorDistributionMenu');
%
% posterior mode
%
controls.modesty.state.postmode = uimenu('Parent',controls.modesty.state.parent, ...
   'Label','&Posterior Mode...', ...
   'Callback','ViewMenuFunctions ViewModestyStat', ...
   'Enable','off', ...
   'UserData','', ...
   'Tag','ModestyStatePosteriorModeMenu');
%
for i=1:10;
   ModHandle = uimenu('Parent',controls.modesty.state.parent, ...
      'Label',['Chain &' num2str(i,'%0.0f') '...'], ...
      'Callback','ViewMenuFunctions ViewModestyStat', ...
      'Enable','off', ...
      'UserData','', ...
      'Tag',['ModestyChain' num2str(i,'%0.0f') 'Menu']);
   if i==1;
      set(ModHandle,'Separator','on');
   end;
   if i==10;
      set(ModHandle,'Label','Chain 1&0...');
   end;
   controls.modesty.state = setfield(controls.modesty.state,['chain' num2str(i,'%0.0f')],ModHandle);
end;
%
% initial values
%
controls.modesty.initialvalues = uimenu('Parent',controls.modesty.parent, ...
      'Label','&Initial Values...', ...
      'Callback','ViewMenuFunctions ViewModestyStat', ...
      'Separator','on', ...
      'Enable','off', ...
      'UserData','', ...
      'Tag','ModestyInitialValuesMenu');
%
% prior distribution
%
controls.modesty.prior = uimenu('Parent',controls.modesty.parent, ...
      'Label','P&rior Distribution...', ...
      'Callback','ViewMenuFunctions ViewModestyStat', ...
      'Enable','off', ...
      'UserData','', ...
      'Tag','ModestyPriorDistributionMenu');
%
% posterior mode
%
controls.modesty.postmode = uimenu('Parent',controls.modesty.parent, ...
      'Label','&Posterior Mode...', ...
      'Callback','ViewMenuFunctions ViewModestyStat', ...
      'Enable','off', ...
      'UserData','', ...
      'Tag','ModestyPosteriorModeMenu');
%
for i=1:10;
   ModHandle = uimenu('Parent',controls.modesty.parent, ...
      'Label',['Chain &' num2str(i,'%0.0f') '...'], ...
      'Callback','ViewMenuFunctions ViewModestyStat', ...
      'Enable','off', ...
      'UserData','', ...
      'Tag',['ModestyChain' num2str(i,'%0.0f') 'Menu']);
   if i==1;
      set(ModHandle,'Separator','on');
   end;
   if i==10;
      set(ModHandle,'Label','Chain 1&0...');
   end;
   controls.modesty = setfield(controls.modesty,['chain' num2str(i,'%0.0f')],ModHandle);
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                     view iterative parameter estimates                      %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.iteratedparameters = uimenu('Parent',controls.viewmenu, ...
   'Label','&Iterated Parameter Estimates...', ...
   'Callback','ViewMenuFunctions IteratedParameters', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','IteratedParametersMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                check optimum                                %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.checkpostmode = uimenu('Parent',controls.viewmenu, ...
   'Label','Chec&k Posterior Mode...', ...
   'Callback','ViewMenuFunctions CheckPostMode', ...
   'Enable','off', ...
   'Tag','CheckPosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         3D plots of posterior mode                          %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.threedeepostmode.parent = uimenu('Parent',controls.viewmenu, ...
   'Label','Posterior Mode Surfa&ce', ...
   'Enable','on', ...
   'Tag','PosteriorModeSurfaceMenu');
%
% original parameters
%
controls.threedeepostmode.original = uimenu('Parent',controls.threedeepostmode.parent, ...
   'Label','&Original Parameters...', ...
   'Callback','ViewMenuFunctions ThreeDeePostMode', ...
   'Enable','off', ...
   'Tag','OriginalPosteriorModeSurfaceMenu');
%
% transformed parameteres
%
controls.threedeepostmode.transformed = uimenu('Parent',controls.threedeepostmode.parent, ...
   'Label','&Transformed Parameters...', ...
   'Callback','ViewMenuFunctions ThreeDeePostMode', ...
   'Enable','off', ...
   'Tag','TransformedPosteriorModeSurfaceMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                               Graph the prior                               %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.viewpriorparent = uimenu('Parent',controls.viewmenu, ...
   'Label','P&rior Densities', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','ViewPriorParentMenu');
%
% marginal prior densities
%
controls.viewprior = uimenu('Parent',controls.viewpriorparent, ...
   'Label','&Marginal Prior Densities...', ...
   'Callback','ViewMenuFunctions ViewPrior', ...
   'Enable','off', ...
   'Tag','ViewPriorMenu');
%
% bivariate prior densities
%
controls.viewbivariateprior = uimenu('Parent',controls.viewpriorparent, ...
   'Label','&Bivariate Prior Densities...', ...
   'Callback','ViewMenuFunctions ViewBivariatePrior', ...
   'Enable','off', ...
   'Tag','ViewBivariatePriorMenu');
%
% correlations
%
controls.systempriorcorrelations = uimenu('Parent',controls.viewpriorparent, ...
   'Label','&System Prior Correlations...', ...
   'Callback','ViewMenuFunctions SystemPriorCorrelations', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','ViewSystemPriorCorrelationsMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                   Laplace approximation of the posterior                    %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.viewlaplaceposterior = uimenu('Parent',controls.viewmenu, ...
   'Label','&Laplace Posterior Densities...', ...
   'Callback','ViewMenuFunctions ViewLaplacePosterior', ...
   'Enable','off', ...
   'Tag','ViewLaplacePosteriorMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                             Graph the posterior                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.viewposteriorparent = uimenu('Parent',controls.viewmenu, ...
   'Label','Pos&terior Densities', ...
   'Enable','on', ...
   'Tag','ViewPosteriorMenu');
%
controls.viewposterior = uimenu('Parent',controls.viewposteriorparent, ...
   'Label','&Marginal Posterior Densities...', ...
   'Callback','ViewMenuFunctions ViewPosterior', ...
   'Enable','off', ...
   'Tag','ViewMarginalPosteriorMenu');
%
controls.viewbivariateposterior = uimenu('Parent',controls.viewposteriorparent, ...
   'Label','&Bivariate Posterior Densities...', ...
   'Callback','ViewMenuFunctions ViewBivariatePosterior', ...
   'Enable','off', ...
   'Tag','ViewBivariatePosteriorMenu');
%
controls.posteriorcorrelations = uimenu('Parent',controls.viewposteriorparent, ...
   'Label','&Posterior Correlations...', ...
   'Callback','ViewMenuFunctions PosteriorCorrelations', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','ViewPosteriorCorrelationsMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                 show prior value and value of log jacobian                  %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.viewpriorvalue.parent = uimenu('Parent',controls.viewmenu, ...
   'Label','Prior and Log &Jacobian Value', ...
   'Enable','on', ...
   'Tag','ViewPriorValueMenu');
% 
controls.viewpriorvalue.initialvalues = uimenu('Parent',controls.viewpriorvalue.parent, ...
   'Label','&Initial Values...', ...
   'Callback','ViewMenuFunctions ViewPriorValueInitialValues', ...
   'Enable','off', ...
   'Tag','ViewPriorValueInitialValuesMenu');
%
controls.viewpriorvalue.postmode = uimenu('Parent',controls.viewpriorvalue.parent, ...
   'Label','&Posterior Mode...', ...
   'Callback','ViewMenuFunctions ViewPriorValuePostMode', ...
   'Enable','off', ...
   'Tag','ViewPriorValuePosteriorModeMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                 Graph data                                  %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.viewdata = uimenu('Parent',controls.viewmenu, ...
   'Label','&Observed Variables...', ...
   'Callback','ViewMenuFunctions ViewData', ...
   'Separator','on', ...
   'Enable','off', ...
   'Tag','ViewDataMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                            Graph annualized data                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.viewannualdata = uimenu('Parent',controls.viewmenu, ...
   'Label','A&nnualized Observed Variables...', ...
   'Callback','ViewMenuFunctions ViewAnnualData', ...
   'Enable','off', ...
   'Tag','ViewAnnualizedDataMenu');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                           graph transformed data                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
controls.viewtransdata = uimenu('Parent',controls.viewmenu, ...
   'Label','Transformed O&bserved Variables...', ...
   'Callback','ViewMenuFunctions ViewTransData', ...
   'Enable','off', ...
   'Tag','ViewTransDataMenu');

%
% end of ViewMenuControls.m
%
