function [LogMargs,qL,LBound,NWStdErr] = MargLikeSWZModifiedHarmonic(PostSample,LogPost,TotalDraws,PValue,ComputeSequential,ModeData,lambda,DSGEModel,CurrINI,WaitHandle)
% MargLikeSWZModifiedHarmonic: Computes the marginal likelihood from a MCMC sample using the Sims, Waggoner and Zha (2008)
%                              version of the the Modified Harmonic Mean estimator for a DSGE model.
%
% USAGE:
%
%       [LogMargs,qL,LBound,NWStdErr] = MargLikeSWZModifiedHarmonic(PostSample,LogPost,TotalDraws,PValue,ComputeSequential, ...
%                                                                   ModeData,lambda,DSGEModel,CurrINI,WaitHandle)
%
% REQUIRED INPUT:  PostSample (matrix) of dimension (NumIter x NumParam) with posterior draws of parameters.
%
%                  LogPost (vector) of dimension (NumIter x 1) with log posterior values for the parameters.
%
%                  TotalDraws (integer) with the total number of prior or posterior parameter draws.
%
%                  PValue (scalar) with value between 0 and 100.
%
%                  ComputeSequential (boolean), which determines if the marginal likelihood should be calculated
%                              sequentially or just for the full sample
%
%                  ModeData (structure) with the stored data from posterior mode estimation.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for DSGE-VAR models; it is
%                              empty for DSGE models under rational expectations; and it is NaN for DSGE
%                              models under adaptive learning.
%
%                  DSGEModel (structure), whose fields contain all required information
%                              about the model, such as input files, variable and sample
%                              data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% OPTIONAL INPUT:  WaitHandle (handle) to a wait dialog. Allows the user to end the function prematurely.
%
% REQUIRED OUTPUT: LogMargs (matrix) with all calculated marginal likelihoods. The values
%                              for the marginal likelihoods are located from column 2
%                              while the sample size for the sequential estimate is located 
%                              in the first column.
%
%                  qL (scalar) with an estimate of the fraction of iid draws from the elliptical
%                              distribution such that the posterior kernel values at these draws
%                              are greater than LBound.
%
%                  LBound (scalar) with the lower bound such that PValue percent of the posterior draws have
%                              a log posterior kernel values greater then LBound.
%
%                  NWStdErr (scalar) with the numerical standard error for the full sample estimate of the
%                              marginal likelihood. Not computed when ComputeSequential is unity.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 3, 2012.
%                        This version: January 16, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-12-2012: Continued work on the function.
%
% * 05-12-2012: Fixed a few minor glitches.
%
% * 06-12-2012: Added the input variable "PValue".
%
% * 10-12-2012: Added the "lambda" input variable for DSGE-VAR models.
%
% * 02-01-2013: Updated the documentation.
%
% * 04-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 29-05-2017: Added eps to each zero entry of dev to ensure that its logarithm is not infinite.
%
% * 29-08-2017: Took into account that "logPosteriorPhiDSGE" requires the input variable "NumWorkers".
%
% * 30-08-2017: Took into account that "logPosteriorPhiDSGEVAR" requires the input variable "NumWorkers".
%
% * 22-03-2018: Made sure that "SystemPriorFile" is copied to the tmp directory when it exists. The
%               function has also been adapted for sequential estimation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 29-08-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 19-05-2020: Added the code for the adaptive learning case.
%
% * 01-10-2020: Fixed a bug with j instead of : for X matrix.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 05-07-2023: The function now tries to load the AIMData file directly.
%
% * 16-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

LogMargs = [];
qL = [];
LBound = [];
NWStdErr = [];
if (exist('WaitHandle')==0)|(nargin==7);
   WaitHandle = [];
else;
   WaitControls = get(get(WaitHandle,'Parent'),'UserData');
   WaitTxt = get(WaitControls.text,'String');
   k = size(WaitTxt,1);
   kLine = WaitTxt{k};
end;
%
[NumIter,NumParam] = size(PostSample);
%
if ComputeSequential==1;
   Origin = DSGEModel.SequentialStartIterationValue;
   %
   % make sure that Origin is less than NumIter
   %
   if Origin>=NumIter;
      Origin = floor(NumIter/10);
   end;
   StepLength = DSGEModel.SequentialStepLengthValue;
else;
   Origin = NumIter;
   StepLength = 1;
end;
%
% Create the sequential grid over which we will compute the marginal likelihood
%
SeqGrid = (Origin:StepLength:NumIter);
if isempty(SeqGrid)==1;
   SeqGrid = NumIter;
elseif SeqGrid(length(SeqGrid))<NumIter;
   SeqGrid = [SeqGrid NumIter];
end;
%
% try to load the AIMData, other wise use ModeData
%
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      AIMData = ModeData.AIMData;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      AIMData = ModeData.AIMData;
   end;
end;
%
% generate the M iid draws from the truncated elliptical:
%
% step 1: center covariance matrix of parameters around the mode
%
phiMode = ModeData.phiMode;
M = max(NumIter,10000);
phiMean = (1/NumIter)*sum(PostSample,1)';
SigmaMode = ((1/NumIter)*PostSample'*PostSample)+(phiMode*phiMode')-(phiMean*phiMode')-(phiMode*phiMean');
InvSigmaMode = eye(NumParam)/SigmaMode;
SigmaModeSq = chol(SigmaMode)';
%
% step 2: compute the r(s)
%
rVec= zeros(NumIter,1);
if isempty(WaitHandle)==0;
   WaitTxt{k} = [kLine ' Initializing elliptical distribution.'];
   set(WaitControls.text,'String',WaitTxt);
end;
for i=1:NumIter;
   dev = PostSample(i,:)'-phiMode;
   dev(dev==0) = eps;
   rVec(i) = sqrt(dev'*InvSigmaMode*dev);
end;
%
% step 3: compute the parameters of the h-density
%
rDist = sort(rVec);
Q1 = rDist(ceil((1/100)*NumIter));
Q10 = rDist(ceil((1/10)*NumIter));
Q90 = rDist(floor((9/10)*NumIter));
%
c = log(1/9)/log(Q10/Q90);
b = Q90/(0.9^(1/c));
a = Q1;
%
% determine the lower bound from the posterior kernel based on
% 90 percent of the log posterior values being greater than the bound
%
SortLogPost = sort(LogPost);
LBound = SortLogPost(ceil(((100-PValue)/100)*NumIter));
%
% initialize the random number generators
%
if DSGEModel.RandomNumberValue==1;
   rand('state',0);
   randn('state',0);
else;
   %
   % we set the state based on the current time
   %
   rand('state',sum(100*clock));
   randn('state',sum(100*clock));
end;
%
% Construct M iid draws of phi from an elliptical distribution
%
p = rand(1,M);
x = randn(NumParam,M);
FrobNormX = zeros(1,M);
for i=1:M;
   FrobNormX(i) = sqrt(x(:,i)'*x(:,i));
end;
rDraws = (((b^c)-a^(c))*p).^(1/c);
phiDraws = ((ones(NumParam,1)*(rDraws./FrobNormX)).*(SigmaModeSq*x))+(phiMode*ones(1,M));
%
if isempty(lambda)==1;
   %
   % check if we have already stored data to disk
   %
   File = FixFilePath([DSGEModel.OutputDirectory '\rwm\SWZElliptalDSGE-' DSGEModel.NameOfModel '-' int2str(NumIter) '.' int2str(M) '.' int2str(TotalDraws) '.mat']);
   if FileExist(File)==1;
      if isempty(WaitHandle)==0;
         set(get(WaitHandle,'Parent'),'Visible','off');
         drawnow;
         pause(0.02);
      end;
      if isempty(CurrINI.Sequence)==1;
         txt = ['YADA has located previously computed log posterior kernel values for ' int2str(M) ' iid draws from an elliptical distribution that can be used for the Sims, Waggoner and Zha version of the modified harmonic mean estimator. Would you like to use these values?'];
         answer = Query(txt,'question',300,'Marginal Likelihood - SWZ''s Harmonic Mean',500,'yes',CurrINI);
         if isempty(WaitHandle)==0;
            set(get(WaitHandle,'Parent'),'Visible','on');
            drawnow;
            pause(0.02);
         end;
      else;
         answer = CurrINI.Sequence.QueryAnswers{1};
      end;
   else;
      answer = 'No';
   end;
   if strcmp(lower(answer),'no')==1;
      %
      % create the output directory
      %
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      if stat~=1;
         if isempty(CurrINI.Sequence)==1;
            if ispc==1;
               txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\rwm". The computation of the marginal likelihood with the Sims, Waggoner and Zha version of the modified harmonic mean has therefore been aborted.'];
            else;
               txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/rwm". The computation of the marginal likelihood with the Sims, Waggoner and Zha version of the modified harmonic mean has therefore been aborted.'];
            end;
            About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
            drawnow;
         end;
         return;
      end;
      %
      % Set up the sub-sample
      %
      [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
      %
      if DSGEModel.UseOwnInitialState==1;
         if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
            DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
         else;
            DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
         end;
      else;
         DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
      end;
      %
      % copy files to the tmp folder
      %
      [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
      [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.SystemPriorFile)==1;
         [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
      end;
      %
      % check if we should copy model solver file
      %
      if DSGEModel.ModelSolver==4;
         if FileExist(DSGEModel.ModelSolverFile)==1;
            [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
         end;
      end;
      %
      % display a wait dialog if one isn't already displayed
      %
      if isempty(WaitHandle)==1;
         txt = ['Please wait while YADA computes the log posterior for a total number of ' int2str(M) ' iid draws from an elliptical distribution. The calculation was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
         WaitHandle = WaitDLG(txt,'information','Log Posterior - SWZ''s Harmonic Mean',500,200,CurrINI,0);
         WaitControls = get(WaitHandle,'UserData');
         DeleteWaitHandle = 1;
      else;
         DeleteWaitHandle = 0;
      end;
      %
      % compute the posterior kernel values for the iid draws
      %
      LogPostKernel = zeros(1,M);
      for i=1:M;
         LogPostKernel(i) = -logPosteriorPhiDSGE(phiDraws(:,i),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
         drawnow;
      end;
      %
      if DeleteWaitHandle==1;
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
         WaitHandle = [];
      end;
      %
      save(File,'LogPostKernel');
   else;
      Data = load(File);
      LogPostKernel = Data.LogPostKernel;
   end;
elseif isnan(lambda)==1;
   %
   % check if we have already stored data to disk
   %
   File = FixFilePath([DSGEModel.OutputDirectory '\rwm\SWZElliptalDSGE-AdaptiveLearning-' DSGEModel.NameOfModel '-' int2str(NumIter) '.' int2str(M) '.' int2str(TotalDraws) '.mat']);
   if FileExist(File)==1;
      if isempty(WaitHandle)==0;
         set(get(WaitHandle,'Parent'),'Visible','off');
         drawnow;
         pause(0.02);
      end;
      if isempty(CurrINI.Sequence)==1;
         txt = ['YADA has located previously computed log posterior kernel values for ' int2str(M) ' iid draws from an elliptical distribution that can be used for the Sims, Waggoner and Zha version of the modified harmonic mean estimator. Would you like to use these values?'];
         answer = Query(txt,'question',300,'Marginal Likelihood - SWZ''s Harmonic Mean',500,'yes',CurrINI);
         if isempty(WaitHandle)==0;
            set(get(WaitHandle,'Parent'),'Visible','on');
            drawnow;
            pause(0.02);
         end;
      else;
         answer = CurrINI.Sequence.QueryAnswers{1};
      end;
   else;
      answer = 'No';
   end;
   %
   if strcmp(lower(answer),'no')==1;
      %
      % create the output directory
      %
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      if stat~=1;
         if isempty(CurrINI.Sequence)==1;
            if ispc==1;
               txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\rwm". The computation of the marginal likelihood with the Sims, Waggoner and Zha version of the modified harmonic mean has therefore been aborted.'];
            else;
               txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/rwm". The computation of the marginal likelihood with the Sims, Waggoner and Zha version of the modified harmonic mean has therefore been aborted.'];
            end;
            About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
            drawnow;
         end;
         return;
      end;
      %
      % copy files to the tmp folder
      %
      [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
      [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.SystemPriorFile)==1;
         [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
      end;
      %
      % check if we should also copy the belief system file
      %
      if DSGEModel.BeliefSystem==2;
         [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
      end;
      %
      % check if we should copy model solver file
      %
      if DSGEModel.ModelSolver==4;
         if FileExist(DSGEModel.ModelSolverFile)==1;
            [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
         end;
      end;
      %
      % Set up the sub-sample
      %
      [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
      %
      % determine xbar
      %
      ConstPos = 0;
      k = DSGEModel.k;
      if k>0;
         ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
         if ConstPos==0;
            %
            % we will now check if X has a unit vector
            %
            X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
            T = size(X,2);
            for j=1:k;
               ConstTest = (DSGEModel.X(j,DSGEModel.FirstPeriod:DSGEModel.LastPeriod)==ones(1,T));
               if sum(ConstTest)==T;
                  ConstPos = j;
                  break;
               end;
            end;
         end;
         xbar = zeros(k,1);
         xbar(ConstPos) = 1;
      else;
         xbar = [];
      end;
      %
      % display a wait dialog if one isn't already displayed
      %
      if isempty(WaitHandle)==1;
         txt = ['Please wait while YADA computes the log posterior for a total number of ' int2str(M) ' iid draws from an elliptical distribution. The calculation was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
         WaitHandle = WaitDLG(txt,'information','Log Posterior - SWZ''s Harmonic Mean',500,200,CurrINI,0);
         WaitControls = get(WaitHandle,'UserData');
         DeleteWaitHandle = 1;
      else;
         DeleteWaitHandle = 0;
      end;
      %
      % compute the posterior kernel values for the iid draws
      %
      LogPostKernel = zeros(1,M);
      for i=1:M;
         LogPostKernel(i) = -logPosteriorAdaptiveLearningPhiDSGE(phiDraws(:,i),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
         drawnow;
      end;
      %
      if DeleteWaitHandle==1;
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
         WaitHandle = [];
      end;
      %
      save(File,'LogPostKernel');
   else;
      Data = load(File);
      LogPostKernel = Data.LogPostKernel;
   end;
else;
   %
   % check if we have already stored data to disk
   %
   % initialize dimensions
   %
   p = DSGEModel.BVARLags;
   k = DSGEModel.k;
   n = DSGEModel.n;
   npk = (n*p)+k;
   File = FixFilePath([DSGEModel.OutputDirectory '\rwm\SWZElliptalDSGE-VAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(NumIter) '.' int2str(M) '.' int2str(TotalDraws) '.mat']);
   if FileExist(File)==1;
      if isempty(WaitHandle)==0;
         set(get(WaitHandle,'Parent'),'Visible','off');
         drawnow;
         pause(0.02);
      end;
      if isempty(CurrINI.Sequence)==1;
         txt = ['YADA has located previously computed log posterior kernel values for ' int2str(M) ' iid draws from an elliptical distribution that can be used for the Sims, Waggoner and Zha version of the modified harmonic mean estimator. Would you like to use these values?'];
         answer = Query(txt,'question',300,'DSGE-VAR Seq. Marginal Likelihood - SWZ''s Harmonic Mean',500,'yes',CurrINI);
         if isempty(WaitHandle)==0;
            set(get(WaitHandle,'Parent'),'Visible','on');
            drawnow;
            pause(0.02);
         end;
      else;
         answer = CurrINI.Sequence.QueryAnswers{1};
      end;
   else;
      answer = 'No';
   end;
   if strcmp(lower(answer),'no')==1;
      %
      % create the output directory
      %
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      if stat~=1;
         if ispc==1;
            txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\rwm". The computation of the marginal likelihood with the Sims, Waggoner and Zha version of the modified harmonic mean has therefore been aborted.'];
         else;
            txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/rwm". The computation of the marginal likelihood with the Sims, Waggoner and Zha version of the modified harmonic mean has therefore been aborted.'];
         end;
         About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
         return;
      end;
      %
      % create sample dates
      %
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      %
      % Determine the first observation to use when computing log-likelihood
      %
      KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
      %
      % Check if we should adjust this based on p, the lag order
      %
      IVAdjusted = 0;
      if FirstPeriod+KalmanFirstObservation-1<p+1;
         KalmanFirstObservation = p+2-FirstPeriod;
         IVAdjusted = 1;
      end;
      %
      % actual sample that we will use
      %
      DSGEModel.FirstPeriod = FirstPeriod;
      DSGEModel.LastPeriod = LastPeriod;
      DSGEModel.KalmanFirstObservation = KalmanFirstObservation;
      %
      % get the data
      %
      [y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
      T = size(y,2);
      if k>0;
         DetProductMoments = CalcDeterministicProductMoments(x,X);
      else;
         DetProductMoments = [];
      end;
      %
      % compute sample product moments
      %
      GammaHatyy = (1/T)*y*y';
      GammaHatyY = (1/T)*y*Y';
      GammaHatYY = (1/T)*Y*Y';
      %
      if isinf(lambda)==0;
         logGPR = LogGammaProductRatio(n,((1+lambda)*T)-npk,(lambda*T)-npk);
      else;
         logGPR = [];
      end;
      %
      % copy files to the tmp folder
      %
      [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
      [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.SystemPriorFile)==1;
         [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
      end;
      %
      % check if we should copy model solver file
      %
      if DSGEModel.ModelSolver==4;
         if FileExist(DSGEModel.ModelSolverFile)==1;
            [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
         end;
      end;
      %
      % display a wait dialog if one isn't already displayed
      %
      if isempty(WaitHandle)==1;
         txt = ['Please wait while YADA computes the log posterior for a total number of ' int2str(M) ' iid draws from an elliptical distribution and using the DSGE-VAR(' num2str(lambda,'%0.4f') ') model with ' int2str(p) ' lags. The calculation was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
         WaitHandle = WaitDLG(txt,'information','Log Posterior - SWZ''s Harmonic Mean',500,200,CurrINI,0);
         WaitControls = get(WaitHandle,'UserData');
         DeleteWaitHandle = 1;
      else;
         DeleteWaitHandle = 0;
      end;
      %
      % compute the posterior kernel values for the iid draws
      %
      LogPostKernel = zeros(1,M);
      for i=1:M;
         LogPostKernel(i) = -logPosteriorPhiDSGEVAR(phiDraws(:,i),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
         drawnow;
      end;
      %
      if DeleteWaitHandle==1;
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
         WaitHandle = [];
      end;
      %
      save(File,'LogPostKernel');
   else;
      Data = load(File);
      LogPostKernel = Data.LogPostKernel;
   end;
end;
%
% now compute qL using LogPostKernel and LBound
%
qL = sum(LogPostKernel>LBound)/M;
%
% compute the integration constant of the elliptical distribution
%
logE = gammaln(NumParam/2)-((NumParam/2)*log(pi))-log(2)-LogDet(SigmaModeSq);
%
% and now compute the log values of the elliptical distribution at the posterior draws
%
logGphi = logE+log(c/((b^c)-a^c))+(c-NumParam)*log(rVec);
%
% initialize sequential estimation
%
StopEstimation = 0;
TempValue = 0;
LogMargs = [SeqGrid' zeros(length(SeqGrid),1)];
MaxLogPost = max(LogPost);
StartEst = 1;
num = 0;
for N=SeqGrid;
   %
   % update the wait dialog
   %
   if isempty(WaitHandle)==0;
      WaitTxt{k} = [kLine ' Sample size: ' int2str(N) ' (' int2str(NumIter) ')'];
      set(WaitControls.text,'String',WaitTxt);
      if strcmp(get(WaitHandle,'UserData'),'cancel')==1;
         StopEstimation = 1;
      end;
   end;
   if StopEstimation==1;
      LogMargs = [];
      break;
   end;
   %
   num = num+1;
   %
   Fvals = (1/qL)*exp(logGphi(StartEst:N)).*(LogPost(StartEst:N)>LBound);
   LP = LogPost(StartEst:N);
   LP(LP<=LBound) = 0;
   TempValue = TempValue+sum(Fvals./exp(LP-MaxLogPost));
   LogMargs(num,2) = MaxLogPost-log(TempValue/N);
   drawnow;
   StartEst = N+1;
end;
%
if (StopEstimation==0)&(ComputeSequential==0);
   Fvals = (1/qL)*exp(logGphi).*(LogPost>LBound);
   LP = LogPost;
   LP(LP<=LBound) = 0;
   ntheta = Fvals./exp(LP-MaxLogPost);
   nbar = sum(ntheta)/NumIter;
   NWStdErr = sqrt(NeweyWestCovMat(ntheta,floor(NumIter^(1/2.01))))/abs(nbar);
end;

%
% end of MargLikeSWZModifiedHarmonic.m
%
