function PrintDSGEPredictiveLikelihood(ResFile,LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,LogPredLikeBackNow,StdErrBackNow,MinMaxCondLikeBackNow,TestStdBackNow,PredictionSampleStrMat,HistSampleStrMat,YDataStr,IsOriginal,IsMarginal,IsPosterior,IsAdaptiveLearning,NumDraws,NumBurnin,TotalDraws,lambda,CompTime,DSGEModel,CurrINI)
% PrintDSGEPredictiveLikelihood: Writes a summary of the output from successful predictive likelihood calculations to text file.
%
% USAGE:
%
%       PrintDSGEPredictiveLikelihood(ResFile,LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,LogPredLikeBackNow, ...
%                        StdErrBackNow,MinMaxCondLikeBackNow,TestStdBackNow,PredictionSampleStrMat,HistSampleStrMat, ...
%                        YDataStr,IsOriginal,IsMarginal,IsPosterior,IsAdaptiveLearning,NumDraws,NumBurnin,TotalDraws, ...
%                        lambda,CompTime,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the text-file to write output in.
%
%                 LogPredLike (vector) of dimension h with the predictive likelihood for the forecast
%                             periods.
%
%                 StdErr (vector) of dimension h with the numerical standard errors of the log predictive
%                             likelihood.
%
%                 PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                             variables for each prediction period.
%
%                 MinMaxCondLike (matrix) of dimension hx3 with the mean, the minimum, and the maximum of
%                             the log conditional likelihood in the first to third columns.
%
%                 TestStd (boolean) that takes the value 1 if the numerical standard errors are robust to small
%                             values for the conditional likelihood and 0 if not.
%
%                 LogPredLikeBackNow (vector) of dimension 2 with the predictive likelihood for the back- and
%                             nowcast periods.
%
%                 StdErrBackNow (vector) of dimension 2 with the numerical standard errors of the log predictive
%                             likelihood for the back- and nowcast periods.
%
%                 MinMaxCondLikeBackNow (matrix) of dimension 2x3 with the mean, the minimum, and the maximum of
%                             the log conditional likelihood for the back- and nowcast periods in the first to
%                             third columns.
%
%                 TestStdBackNow (boolean) that takes the value 1 if the numerical standard errors of the back- and
%                             nowcast periods are robust to small values for the conditional likelihood and 0 if not.
%
%                 PredictionSampleStrMat (string matrix) with the prediction sample dates.
%
%                 HistSampleStrMat (string matrix) with the sample dates for the back- and nowcasts.
%
%                 YDataStr (string vector) with a description of the actuals data used for the computation.
%
%                 IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                             annualized values should be predicted.
%
%                 IsMarginal (boolean) that takes the value 1 if the marginal predictive likelihood should be
%                             computed and 0 if the joint predictive likelihood is the object of interest.
%
%                 IsPosterior (boolean) that takes the value 1 if the parameter draws are taken from the posterior
%                             distribution and 0 otherwise.
%
%                 IsAdaptiveLearning (boolean) that takes the value 1 if the adaptive learning mechanism is used for the
%                             expectations formation, and 0 otherwise.
%
%                 NumDraws (integer) with the total number of parameters draws used to compute the log posterior.
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                 lambda (scalar) hyperparameter value that is used to measure degree of misspecification
%                             for DSGE-VAR models.
%
%                 CompTime (time) with the computational time.
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure), with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 30, 2012.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-05-2012: Added code for printing the marginal likelihood values over the forecast sample.
%
% * 15-05-2012: Changed "Average" to "Mean" and added "Median" estimate for the modified harmonic mean.
%
% * 18-10-2012: Added the input variable "StdErr" with the numerical standard errors of the log predictive
%               likelihood.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 10-12-2014: Took into account that the standard errors are NaNs when the number of draws used is too small.
%
% * 14-01-2015: Updated the documentation.
%
% * 26-02-2015: The function now takes the two input variables "MinMaxCondLike" and "TestStd".
%
% * 12-06-2015: The function requires the input variable "YDataStr".
%
% * 22-02-2016: Updated the documentation.
%
% * 15-03-2017: Added the input variables "LogPredLikeBackNow", "StdErrBackNow", "MinMaxCondLikeBackNow",
%               "TestStdBackNow", and "HistSampleStrMat".
%
% * 17-03-2017: Finished the code for the back- and nowcast results.
%
% * 18-07-2017: Corrected a bug for the variables being predicted using now and backcasting.
%
% * 20-07-2017: Made sure that printed output of minimum, maximum and range for the back and nowcasts
%               are written as NaN's when the standard errors are NaN's.
%
% * 14-05-2018: Updated the documentation.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 31-08-2020: Added information about data and integration type for the predictions.
%
% * 26-01-2021: Updated the documentation.
%
% * 12-05-2021: Added the input variable "IsAdaptiveLearning".
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(ResFile,'wt');
h = size(LogPredLike,1);
%
% the header information
%
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                               P R E D I C T I V E   L I K E L I H O O D                               *\n');
if IsAdaptiveLearning==1;
   fprintf(fid,'*                                   A D A P T I V E   L E A R N I N G                                   *\n');
end;
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
if IsOriginal==1;
   TypeStr = 'Original';
else;
   TypeStr = 'Annualized';
end;
if IsMarginal==1;
   MargStr = 'Marginal';
else;
   MargStr = 'Joint';
end;
if IsPosterior==1;
   DistStr = 'Posterior';
else;
   DistStr = 'Prior';
end;
%
fprintf(fid,'Parameter draws:                                   %s distribution \n',DistStr);
if IsPosterior==1;
   fprintf(fid,'Number of available posterior draws:               %0.0f\n',TotalDraws);
   fprintf(fid,'Number of used draws:                              %0.0f\n',NumDraws);
   fprintf(fid,'Number of burn-in draws:                           %0.0f\n',NumBurnin);
else;
   fprintf(fid,'Number of available prior draws:                   %0.0f\n',TotalDraws);
end;
if isempty(lambda)==0;
   fprintf(fid,'Number of lags in DSGE-VAR:                        %0.0f \n',DSGEModel.BVARLags);
   fprintf(fid,'Lambda:                                            %0.4f \n',lambda);
end;
if IsAdaptiveLearning==1;
   fprintf(fid,'Number of forward variables:                       %0.0f\n\n',length(DSGEModel.ForwardVariablePositions));
   fprintf(fid,'Name of belief system:                             %s\n',DSGEModel.BeliefSys.Name);
   if DSGEModel.BeliefSystem==1;
      fprintf(fid,'Belief system file:                                YADA''s default (two own lags)\n');
   else;
      fprintf(fid,'Belief system file:                                %s\n',GetFilename(DSGEModel.BeliefSystemFile));
      fprintf(fid,'Maximum lag order:                                 %0.0f\n', DSGEModel.BeliefSys.m);
   end;
end;
if IsOriginal==1;
   fprintf(fid,'Type of predicting variables:                      original data\n');
else;
   fprintf(fid,'Type of predicting variables:                      annualized data\n');
end;
fprintf(fid,'Set of actual data for the observed variables:     %s \n',YDataStr);
fprintf(fid,'Selected data type:                                %s data\n',TypeStr);
fprintf(fid,'Selected integration of predicted variables:       %s predictive likelihood\n',MargStr);
fprintf(fid,'Computation time:                                  %s (hrs:min:sec)\n\n',SecondsToHoursStr(CompTime));
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
%
PrtVal = ['%' num2str(7+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
%
%
% print the predictive likelihood values
%
FirstStr = MultiCharStr('=',105);
DivStr = MultiCharStr('-',105);
PrtStr = ['%s          ' PrtVal '  %s \n'];
%
fprintf(fid,'%s\n',FirstStr);
if size(PredictionSampleStrMat,2)==6;
   fprintf(fid,'period  predictive likelihood  predicted variables \n');
   PrtStr2 = ['standard error: ' PrtVal ' \n\n'];
   PrtStr6 = ['mean:           ' PrtVal ' \n'];
   PrtStr3 = ['minimum:        ' PrtVal ' \n'];
   PrtStr4 = ['maximum:        ' PrtVal ' \n'];
   PrtStr5 = ['range:          ' PrtVal ' \n'];
else;
   fprintf(fid,'period   predictive likelihood  predicted variables \n');
   PrtStr2 = ['standard error:  ' PrtVal ' \n\n'];
   PrtStr6 = ['mean:            ' PrtVal ' \n'];
   PrtStr3 = ['minimum:         ' PrtVal ' \n'];
   PrtStr4 = ['maximum:         ' PrtVal ' \n'];
   PrtStr5 = ['range:           ' PrtVal ' \n'];
end;
for i=1:h;
   fprintf(fid,'%s\n',DivStr);
   if isnan(LogPredLike(i))==0;
      VarStr = vecstr(DSGEModel.VariableNames(PredVars{i},:),', ');
      VarStr = VarStr(1:length(VarStr)-1);
   else;
      VarStr = 'missing obs';
   end;
   fprintf(fid,PrtStr,PredictionSampleStrMat(i,:),LogPredLike(i),VarStr);
   fprintf(fid,PrtStr2,StdErr(i));
   fprintf(fid,PrtStr6,MinMaxCondLike(i,1));
   fprintf(fid,PrtStr3,MinMaxCondLike(i,2));
   fprintf(fid,PrtStr4,MinMaxCondLike(i,3));
   fprintf(fid,PrtStr5,MinMaxCondLike(i,3)-MinMaxCondLike(i,2));
end;
fprintf(fid,'%s\n\n',FirstStr);
if max(isnan(StdErr))==0;
   fprintf(fid,'NOTE: The numerical standard errors have been calculated with the Newey-West estimator.\n      The minimum and the maximum are taken from the values of the conditional\n      log-likelihood, while mean is its average.\n\n');
else;
   fprintf(fid,'NOTE: The numerical standard errors using the Newey-West estimator yields NaNs when the\n      sample is too small. The minimum and the maximum are taken from the values\n      of the conditional log-likelihood, while mean is its average..\n\n');
end;
%
if TestStd==1;
   fprintf(fid,'NOTE: The estimates, based on demeaning the conditional log-likelihood before computing\n      the means and the numerical standard errors, are robust with respect to the\n      original range of the conditional log-likelihood.\n\n');
end;
%
% results for the back- and nowcasts.
%
if (IsMarginal==1)&(IsOriginal==1)&(isempty(LogPredLikeBackNow)==0)&(sum(isnan(LogPredLikeBackNow))<length(LogPredLikeBackNow));
   fprintf(fid,'%s\n',FirstStr);
   fprintf(fid,'Back and Nowcasts \n');
   if size(PredictionSampleStrMat,2)==6;
      fprintf(fid,'period  predictive likelihood  predicted variables \n');
   else;
      fprintf(fid,'period   predictive likelihood  predicted variables \n');
   end;
   for i=1:length(LogPredLikeBackNow);
      fprintf(fid,'%s\n',DivStr);
      if isnan(LogPredLikeBackNow(i))==0;
         %VarStr = vecstr(DSGEModel.VariableNames(PredVars{i},:),', ');
         VarStr = vecstr(DSGEModel.VariableNames(DSGEModel.PredictionVariables,:),', ');
         VarStr = VarStr(1:length(VarStr)-1);
      else;
         VarStr = 'missing obs';
      end;
      fprintf(fid,PrtStr,HistSampleStrMat(i,:),LogPredLikeBackNow(i),VarStr);
      fprintf(fid,PrtStr2,StdErrBackNow(i));
      if isnan(StdErrBackNow(i))==0;
         fprintf(fid,PrtStr6,MinMaxCondLikeBackNow(i,1));
         fprintf(fid,PrtStr3,MinMaxCondLikeBackNow(i,2));
         fprintf(fid,PrtStr4,MinMaxCondLikeBackNow(i,3));
         fprintf(fid,PrtStr5,MinMaxCondLikeBackNow(i,3)-MinMaxCondLikeBackNow(i,2));
      else;
         fprintf(fid,PrtStr6,NaN);
         fprintf(fid,PrtStr3,NaN);
         fprintf(fid,PrtStr4,NaN);
         fprintf(fid,PrtStr5,NaN);
      end;
   end;
   fprintf(fid,'%s\n\n',FirstStr);
   if TestStdBackNow==1;
      fprintf(fid,'NOTE: The estimates, based on demeaning the conditional log-likelihood before computing\n      the means and the numerical standard errors, are robust with respect to the\n      original range of the conditional log-likelihood.\n\n');
   end;
end;
%
% bottom file data
%
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintDSGEPredictiveLikelihood.m
%
