function PrintPosteriorSamplingSummary(PostSampleTextFile,DSGEModel,CurrINI,ParameterNames,PriorDist,thetaDist,ParData,thetaMargMode,UpdateParData,UpdateMargMode,UpdateNames,NumIter,NumBurnin,NumMeasureErrors,ScaleInitial,ScaleProp,PostSampleTime,AcceptRate,LogMargsMH,CovProb,NWStdErrMH,LogMargsSWZ,qL,PValue,LBound,NWStdErrSWZ,LogMargsCJ,StepLength,CurrChain,NumChains,thetaInit,RWvalue,MaxCorr,IsSystemPrior,LogMargLikeSystemLaplace,SPLogMargsMH,SPCovProb,SPNWStdErrMH,IsAdaptiveLearning)
% PrintPosteriorSamplingSummary: Writes summary information about the posterior distributions for the
%                                original parameters to file.
%
% USAGE:
%
%       PrintPosteriorSamplingSummary(PostSampleTextFile,DSGEModel,CurrINI,ParameterNames,PriorDist,thetaDist,...
%                                     ParData,thetaMargMode,UpdateParData,UpdateMargMode,UpdateNames,NumIter,...
%                                     NumBurnin,NumMeasureErrors,ScaleInitial,ScaleProp,PostSampleTime,...
%                                     AcceptRate,LogMargsMH,CovProb,NWStdErrMH,LogMargsSWZ,qL,PValue,LBound,NWStdErrSWZ,...
%                                     LogMargsCJ,StepLength,CurrChain,NumChains,thetaInit,RWvalue,MaxCorr,...
%                                     IsSystemPrior,LogMargLikeSystemLaplace,SPLogMargsMH,SPCovProb,SPNWStdErrMH,
%                                     IsAdaptiveLearning)
%
% REQUIRED INPUT: PostSampleTextFile (string) with the full path and name of the text output file.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this structure.
%
%                 CurrINI (structure) with initialization information.
%
%                 ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                           "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each
%                           field returns a string matrix with the parameter names.
%
%                 PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                           "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                           parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                           and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                           it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                           truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                           values are the lower and the upper bound. For the student-t they are the location (mode),
%                           the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                           and scale parameters are needed. For the Pareto the parameter values are for the mode
%                           and the shape parameter.
%
%                 thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                           truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                           and 11 if pareto.
%
%                 ParData (structure), with fields "SampleMean" containing the sample
%                           mean of the different parameters, "SampleStd" with the sample
%                           standard deviations, and "SampleMedian" with the sample median
%                           parameter values. The field "SampleStdErr" contains numerical
%                           standard errors of the SampleMean estimate. For SMC with likelihood
%                           tempering, this structure also has the fields "LogMargLikeSMC",
%                           "Ntau", "TemperingLambda", "NumBlocks", "alpha", "cstar", "cstarinit",
%                           "pstar", and "ResamplingAlgorithmStr".
%
%                 thetaMargMode (vector) with the marginal posterior mode estimates of the original
%                           parameters.
%
%                 UpdateParData (structure) with the same fields as the ParData structure. The
%                           data is computed from the posterior values of the parameters that
%                           are a function of the estimated parameters.
%
%                 UpdateMargMode (vector) with the marginal posterior mode estimates of the
%                           parameters that are a function of the estimated parameters.
%
%                 UpdateNames (string matrix) with the names of the parameters that are a
%                           function of the estimated parameters.
%
%                 NumIter (integer) with the total number of posterior draws.
%
%                 NumBurnin (integer) with the number of burn-in draws for the posterior sampler.
%
%                 NumMeasureErrors (integer), giving the number of measurement errors in the model.
%
%                 ScaleInitial (non-negative real number), giving the scale factor for the initial phi value (c0)
%
%                 ScaleProp (positive real number), giving the scale factor for the proposal density (c).
%
%                 PostSampleTime (time) with the length it took the posterior sampler to finish from where
%                           it started.
%
%                 AcceptRate (positive real number), giving the acceptance rate for the RWM posterior sampler
%                           and the average number of log posterior evaluations for the slice sampler.
%
%                 LogMargsMH (matrix) with sequential estimates of the log marginal likelihood using Geweke's
%                           modified harmonic mean estimator. First column is number of draws, while the remaining
%                           columns contain the estimated values for the CovProb values.
%
%                 CovProb (vector), with coverage probabilities.
%
%                 NWStdErrMH (vector) with numerical standard errors for the log marginal likelihood estimated using
%                           Geweke's modified harmonic mean for the full sample only.
%
%                 LogMargsSWZ (matrix) with all calculated marginal likelihoods from the Sims-Waggoner-Zha's version of
%                           the modified harmonic mean estimator of the marginal likelihood. The values for the marginal
%                           likelihoods are located from column 2 while the sample sizes for the sequential estimate are
%                           located in the first column.
%
%                 qL (scalar) with an estimate of the fraction of iid draws from the elliptical
%                           distribution such that the posterior kernel values at these draws
%                           are greater than L.
%
%                 PValue (scalar) with value between 0 and 100.
%
%                 LBound (scalar) with the lower bound such that PValue percent of the posterior draws have
%                           a log posterior kernel values greater then LBound.
%
%                 NWStdErrSWZ (scalar) with the numerical standard error for the full sample estimate of the
%                           Sims-Waggoner-Zha estimator of the marginal likelihood.
%
%                 LogMargsCJ (matrix) with 3 columns. The first holds the number of draws used for the computation,
%                           the second the estimated log marginal likelihood based on the Chib & Jeliazkov estimator,
%                           while the third column gives the numerical standard error of the estimate.
%
%                 StepLength (positive scalar) with the step length that is used for the finite difference estimator
%                           of the inverse Hessian at the posterior mode.
%
%                 CurrChain (integer) with the number of the current MCMC chain.
%
%                 NumChains (integer) with the number of parallell MCMC chains.
%
%                 thetaInit (vector) with the initial values for the original parameters.
%
%                 RWvalue (scalar) with the weight on randomization when mutliple chains are used.
%
%                 MaxCorr (scalar) with the maximum absolute correlation scale factor.
%
%                 IsSystemPrior (boolean) that takes the value 1 if a system prior is used and 0 otherwise.
%
%                 LogMargLikeSystemLaplace (scalar) with the value of the log marginal likelihood of the
%                           system prior.
%
%                 SPLogMargsMH (matrix) with the log marginal likelihood of the system prior using Geweke's
%                           modified harmonic mean estimator. First column is number of draws, while the remaining
%                           columns contain the estimated values for the CovProb values.
%
%                 SPCovProb (vector) with coverage probabilities.
%
%                 SPNWStdErrMH (vector) with numerical standard errors for the log marginal likelihood estimated using
%                           Geweke's modified harmonic mean for the full sample only.
%
% OPTIONAL INPUT: IsAdaptiveLearning (boolean) that takes the value 1 if adaptive learning based expectations
%                           have been used and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 7, 2006.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-12-2006: Finished the function.
%
% * 13-12-2006: Added printing of posterior distribution quantiles.
%
% * 10-04-2007: Took the variable "InverseHessianEstimator" into account. The new inputs are "StepLength", "CurrChain",
%               and "NumChains".
%
% * 11-04-2007: Added the inputs "phiInit" and "RWvalue".
%
% * 17-04-2007: Added the "MaxCorr" input.
%
% * 19-04-2007: Took the ParData field "SampleStdAuto" into account.
%
% * 13-11-2007: Updated the documentation.
%
% * 02-01-2008: Removed code using "DSGEModel.InitializeKalman".
%
% * 07-01-2008: Edited the header for the output file.
%
% * 23-05-2008: Updated the documentation.
%
% * 22-07-2008: Took into account that the gamma and inverted gamma distributions now use
%               the lower bound parameter.
%
% * 23-07-2008: Took into account that the beta distribution supports general lower and upper bounds.
%
% * 23-10-2008: Added information about the number of parameters to estimate and the total number of parameters.
%
% * 21-01-2009: Updated the documentation.
%
% * 26-02-2009: Changed the name of the input variable "thetaMode" to "thetaMargMode", reflecting that
%               it holds the values for the mode of the marginal posterior. The input variables "UpdateParData",
%               "UpdateMargMode", and "UpdateNames" were added, as well code for writing to file the information
%               about the parameters that are a function of the estimated parameters.
%
% * 13-03-2009: Added support for the Student-t, the Cauchy, the logistic, and the Pareto prior distributions.
%
% * 31-03-2009: Made sure that the Pareto distribution has 3 parameters.
%
% * 02-04-2009: Took into account that the logistic distribution supports a shape parameter. Added support
%               for the Gumbel prior distribution.
%
% * 20-04-2009: Made sure that the fields in PriorDist have the correct length.
%
% * 27-11-2009: The function now handles different posterior samplers.
%
% * 28-11-2009: Renamed the input variable "AcceptRatio" to "AcceptRate", the text string "acceptance ratio"
%               to "acceptance rate", and made sure that acceptance rate is only written to the file for
%               the RWM sampler, while the text "average log posterior evaluations" is written to the file
%               when the slice sampler has been run.
%
% * 23-12-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 26-02-2010: Added the "PostSampleTextFile" input variable.
%
% * 17-03-2010: Updated the documentation.
%
% * 26-05-2010: Corrected the type "corelation".
%
% * 17-12-2010: The function now prints DSGE model solver information.
%
% * 23-12-2010: The function now prints Kalman algorithm information.
%
% * 05-01-2011: Updated the documentation.
%
% * 13-10-2011: Took into account that "KalmanAlgorithm" now supports the univariate filter and smoother.
%
% * 09-11-2011: Took into accoun that the Kalman filter now supports diffuse initialization.
%
% * 05-01-2012: Updated the documentation.
%
% * 02-05-2012: The average value is now also shown for the modiefied harmonic mean estimator.
%
% * 15-05-2012: Changed "Average" to "Mean" and added "Median" estimate for the modified harmonic mean.
%
% * 16-11-2012: The function takes the use of the Chandrasekhar recursions into account.
%
% * 30-11-2012: Added some preliminary code for the introduction of the Sims, Waggoner and Zha
%               truncated elliptical version of the modified harmonic mean.
%
% * 03-12-2012: Added some additional code to distinguish the SWZ's and Geweke's versions of the modified
%               harmonic mean estimator.
%
% * 11-12-2012: The function now accepts the input variables "NWStdErrMH", as well as the input variables
%               required to print the results from the Sims-Waggoner-Zha version of the modified harmonic
%               mean estimator of the mlog marginal likelihood.
%
% * 12-12-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 25-08-2015: The function now takes into account that the RWM algorithm can be based on
%               a normal proposal density as well as on a Student-t density.
%
% * 27-08-2015: Added output information aboyut the number of degrees of freedom of the
%               Student-t proposal density.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 02-04-2017: The function now supports the fixed blocking RWM posterior sampler with
%               a normal proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with
%               a Student-t proposal density.
%
% * 11-04-2017: Updated the documentation.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with
%               a normal proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with
%               a Student-t proposal density.
%
% * 07-06-2017: Added the input variables "IsSystemPrior,LogMargLikeSystemLaplace,SPLogMargsMH,
%               SPCovProb,SPNWStdErrMH".
%
% * 14-06-2017: Updated the information written by the function.
%
% * 14-08-2017: Added some initial output based on the SMC posterior sampler with likelihood tempering.
%
% * 23-08-2017: Continued adding support for the SMC with likelihood tempering posterior sampler.
%
% * 04-09-2017: Added information about the number of resamples during the selection step.
%
% * 14-05-2018: Updated the documentation.
%
% * 02-08-2018: The function supports user written model solver function.
%
% * 16-10-2018: The function supports the SMC with data tempering posterior sampler.
%
% * 19-10-2018: Made sure that the number of tempering stages is also written to file for SMC with
%               data tempering.
%
% * 04-12-2018: The function partly supports IS based on MitISEM.
%
% * 04-01-2019: Removed an extra parenthesis.
%
% * 08-01-2019: The function now fully supports IS based on MitISEM.
%
% * 10-01-2019: Minor fixes to ensure that output from MitISEM with 1 mixture component reacts correctly
%               to input data.
%
% * 16-01-2019: Made sure that "StepLength" is printed when relevant undet IS based on MitISEM.
%
% * 10-01-2020: Updated the documentation.
%
% * 18-05-2020: Added the optional input variable "IsAdaptiveLearning".
%
% * 20-05-2020: Added information about the number of forward looking variables.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
if nargin==38
   IsAdaptiveLearning = 0;
end;
%
NumParam = length(ParData.SampleMean);
NumAdditional = length(UpdateMargMode);
%
fid = fopen(PostSampleTextFile,'wt');
%
% the header information
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*     P O S T E R I O R   S A M P L I N G   W I T H   Y A D A   U S I N G      *\n');
if DSGEModel.PosteriorSampler==1;
   fprintf(fid,'*   T H E   R A N D O M - W A L K   M E T R O P O L I S   A L G O R I T H M    *\n');
   fprintf(fid,'*         W I T H   A   N O R M A L   P R O P O S A L   D E N S I T Y          *\n');
elseif DSGEModel.PosteriorSampler==2;
   fprintf(fid,'*                      T H E   S L I C E   S A M P L E R                       *\n');
elseif DSGEModel.PosteriorSampler==3;
   fprintf(fid,'*   T H E   R A N D O M - W A L K   M E T R O P O L I S   A L G O R I T H M    *\n');
   fprintf(fid,'*      W I T H   A   S T U D E N T - T   P R O P O S A L   D E N S I T Y       *\n');
elseif DSGEModel.PosteriorSampler==4;
   fprintf(fid,'*       T H E   F I X E D   B L O C K I N G   R W M   A L G O R I T H M        *\n');
   fprintf(fid,'*         W I T H   A   N O R M A L   P R O P O S A L   D E N S I T Y          *\n');
elseif DSGEModel.PosteriorSampler==5;
   fprintf(fid,'*       T H E   F I X E D   B L O C K I N G   R W M   A L G O R I T H M        *\n');
   fprintf(fid,'*      W I T H   A   S T U D E N T - T   P R O P O S A L   D E N S I T Y       *\n');
elseif DSGEModel.PosteriorSampler==6;
   fprintf(fid,'*      T H E   R A N D O M   B L O C K I N G   R W M   A L G O R I T H M       *\n');
   fprintf(fid,'*         W I T H   A   N O R M A L   P R O P O S A L   D E N S I T Y          *\n');
elseif DSGEModel.PosteriorSampler==7;
   fprintf(fid,'*      T H E   R A N D O M   B L O C K I N G   R W M   A L G O R I T H M       *\n');
   fprintf(fid,'*      W I T H   A   S T U D E N T - T   P R O P O S A L   D E N S I T Y       *\n');
elseif DSGEModel.PosteriorSampler==8;
   fprintf(fid,'*           U S I N G   S E Q U E N T I A L   M O N T E   C A R L O            *\n');
   fprintf(fid,'*              W I T H   L I K E L I H O O D   T E M P E R I N G               *\n');
elseif DSGEModel.PosteriorSampler==9;
   fprintf(fid,'*           U S I N G   S E Q U E N T I A L   M O N T E   C A R L O            *\n');
   fprintf(fid,'*                    W I T H   D A T A   T E M P E R I N G                     *\n');
elseif DSGEModel.PosteriorSampler==10;
   fprintf(fid,'*              U S I N G   I M P O R T A N C E   S A M P L I N G               *\n');
   fprintf(fid,'*                       B A S E D   O N   M I T I S E M                        *\n');
end;
if IsAdaptiveLearning==1;
   fprintf(fid,'*                      A D A P T I V E   L E A R N I N G                       *\n');
end;
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% Model info
%
if DSGEModel.ModelSolver==1;
   SolverStr = 'Anderson & Moore (AiM)';
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein (solab)';
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Sims (gensys)';
elseif DSGEModel.ModelSolver==4;
   SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
end;
if IsAdaptiveLearning==0;
   if DSGEModel.KalmanAlgorithm==1;
      AlgorithmText = 'Standard filter';
   elseif DSGEModel.KalmanAlgorithm==2;
      AlgorithmText = 'Square root filter';
   elseif DSGEModel.KalmanAlgorithm==3;
      AlgorithmText = 'Univariate filter';
   elseif DSGEModel.KalmanAlgorithm==4;
      AlgorithmText = 'Chandrasekhar recursions';
   end;
   if DSGEModel.UseDoublingAlgorithm==1;
      InitialAlgorithmText = 'Analytic';
   elseif DSGEModel.UseDoublingAlgorithm==2;
      InitialAlgorithmText = 'Numeric (doubling algorithm)';
   elseif DSGEModel.UseDoublingAlgorithm==3;
      InitialAlgorithmText = 'Constant times identity';
   elseif DSGEModel.UseDoublingAlgorithm==4;
      InitialAlgorithmText = 'Diffuse';
   end;
else;
   AlgorithmText = 'Adaptive learning filter';
   if DSGEModel.UseDoublingAlgorithm==1;
      InitialAlgorithmText = 'Analytic';
   elseif DSGEModel.UseDoublingAlgorithm==2;
      InitialAlgorithmText = 'Numeric (doubling algorithm)';
   else;
      InitialAlgorithmText = 'Constant times identity';
   end;
end;
fprintf(fid,'Name of model:                    %s\n',DSGEModel.NameOfModel);
fprintf(fid,'DSGE model solver:                %s\n',SolverStr);
fprintf(fid,'Kalman algorithm:                 %s\n',AlgorithmText);
fprintf(fid,'Initialization method:            %s\n',InitialAlgorithmText);
fprintf(fid,'Data construction file:           %s\n',GetFilename(DSGEModel.DataConstructionFile));
fprintf(fid,'Measurement equation file:        %s\n',GetFilename(DSGEModel.MeasurementEquationFile));
fprintf(fid,'Prior distribution data file:     %s\n',GetFilename(DSGEModel.PriorFile));
fprintf(fid,'Sheet name:                       %s\n',DSGEModel.PriorFileSheet);
if IsAdaptiveLearning==1;
   fprintf(fid,'Belief system prior data file:    %s\n',GetFilename(DSGEModel.BeliefPriorFile));
   fprintf(fid,'Sheet name:                       %s\n',DSGEModel.BeliefPriorFileSheet);
end;
if IsSystemPrior==1;
   fprintf(fid,'System prior density file:        %s\n',GetFilename(DSGEModel.SystemPriorFile));
end;
if FileExist(DSGEModel.UpdateParameterFile)==1;
   fprintf(fid,'Update parameters file:           %s\n',GetFilename(DSGEModel.UpdateParameterFile));
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   fprintf(fid,'Initialize parameters file:       %s\n',GetFilename(DSGEModel.InitializeParameterFile));
end;
if IsAdaptiveLearning==1;
   fprintf(fid,'Name of belief system:            %s\n',DSGEModel.BeliefSys.Name);
   if DSGEModel.BeliefSystem==1;
      fprintf(fid,'Belief system file:               YADA''s default (two own lags)\n');
   else;
      fprintf(fid,'Belief system file:               %s\n',GetFilename(DSGEModel.BeliefSystemFile));
      fprintf(fid,'Maximum lag order:                %0.0f\n', DSGEModel.BeliefSys.m);
   end;
end;
fprintf(fid,'AiM model file:                   %s\n\n',GetFilename(DSGEModel.AIMFile));
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*              S A M P L E   &   D A T A   I N F O R M A T I O N               *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% Sample information
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
%
fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
fprintf(fid,'Number of observations:           %0.0f\n',DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1);
fprintf(fid,'Number of observations used to\n');
fprintf(fid,'initialize the Kalman filter:     %0.0f\n',DSGEModel.KalmanFirstObservation-1);
fprintf(fid,'Effective number of observations: %0.0f\n',DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1));
fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
fprintf(fid,'Number of observed variables:     %0.0f\n',min(size(DSGEModel.Y)));
fprintf(fid,'Number of exogenous variables:    %0.0f\n',min(size(DSGEModel.X)));
fprintf(fid,'Number of state variables:        %0.0f\n',size(DSGEModel.StateVariableNames,1));
fprintf(fid,'Number of state shocks:           %0.0f\n',size(DSGEModel.StateShockNames,1));
if IsAdaptiveLearning==0;
   fprintf(fid,'Number of measurement errors:     %0.0f\n\n',NumMeasureErrors);
else;
   fprintf(fid,'Number of measurement errors:     %0.0f\n',NumMeasureErrors);
   fprintf(fid,'Number of forward variables:      %0.0f\n\n',length(DSGEModel.ForwardVariablePositions));
end;
%
% observed variable names, state variable names and state shocks
%
VarNames = StrWrap(vecstr(DSGEModel.VariableNames,' '),40);
fprintf(fid,'Names of observed variables:      %s\n',VarNames(1,:));
for i=2:size(VarNames,1);
   if strcmp(StringTrim(VarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',VarNames(i,:));
   end;
end;
if min(size(DSGEModel.X))>0;
   XVarNames = StrWrap(vecstr(DSGEModel.XVariableNames,' '),40);
   fprintf(fid,'Names of exogenous variables:     %s\n',XVarNames(1,:));
   for i=2:size(XVarNames,1);
      if strcmp(StringTrim(XVarNames(i,:)),'')==0;
         fprintf(fid,'                                  %s\n',XVarNames(i,:));
      end;
   end;
end;
StateVarNames = StrWrap(vecstr(DSGEModel.StateVariableNames,' '),40);
fprintf(fid,'Names of state variables:         %s\n',StateVarNames(1,:));
for i=2:size(StateVarNames,1);
   if strcmp(StringTrim(StateVarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateVarNames(i,:));
   end;
end;
StateShockNames = StrWrap(vecstr(DSGEModel.StateShockNames,' '),40);
fprintf(fid,'Names of the state shocks:        %s\n',StateShockNames(1,:));
for i=2:size(StateShockNames,1);
   if strcmp(StringTrim(StateShockNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateShockNames(i,:));
   end;
end;
if IsAdaptiveLearning==1;
   ForwardVarNames = StrWrap(vecstr(DSGEModel.ForwardVariableNames,' '),40);
   fprintf(fid,'Names of forward variables:       %s\n',ForwardVarNames(1,:));
   for i=2:size(ForwardVarNames,1);
      if strcmp(StringTrim(ForwardVarNames(i,:)),'')==0;
         fprintf(fid,'                                  %s\n',ForwardVarNames(i,:));
      end;
   end;
end;
%
DivStr = MultiCharStr('=',80);
%
% posterior sampler information
%
fprintf(fid,'\n********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*            P O S T E R I O R   D R A W S   I N F O R M A T I O N             *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
fprintf(fid,'Number of posterior draws:             %0.0f\n',NumIter);
if DSGEModel.PosteriorSampler<=7;
   fprintf(fid,'Number of burn-in draws:               %0.0f\n',NumBurnin);
   fprintf(fid,'MCMC chain number:                     %0.0f\n',CurrChain);
elseif DSGEModel.PosteriorSampler==10;
   fprintf(fid,'IS sample number:                      %0.0f\n',CurrChain);
   fprintf(fid,'Share of draws with a non-unique or\n');
   fprintf(fid,'         a non-convergent solution:    %0.4f\n',ParData.ShareBadDraws);
else;
   if DSGEModel.PosteriorSampler==8;
      fprintf(fid,'Number of tempering stages:            %0.0f\n',ParData.Ntau);
      fprintf(fid,'Bending parameter for temp. schedule:  %0.0f\n',ParData.TemperingLambda);
   elseif DSGEModel.PosteriorSampler==9;
      fprintf(fid,'Number of tempering stages:            %0.0f\n',ParData.NumResamples+1);
   end;
   fprintf(fid,'SMC sample number:                     %0.0f\n',CurrChain);
end;
if DSGEModel.InverseHessianEstimator==1;
   InvHessianTxt = 'Optimization routine';
elseif DSGEModel.InverseHessianEstimator==2;
   InvHessianTxt = 'Quadratic modification';
elseif DSGEModel.InverseHessianEstimator==3;
   InvHessianTxt = 'Finite difference';
else;
   InvHessianTxt = 'Own matrix';
end;
if DSGEModel.PosteriorSampler<=7;
   fprintf(fid,'Inverse Hessian estimator:             %s\n',InvHessianTxt);
   if DSGEModel.InverseHessianEstimator==3;
      fprintf(fid,'Step length:                           %s\n',StepLength);
   end;
   fprintf(fid,'Max absolute correlation scale factor: %0.4f\n',MaxCorr);
   if NumChains>1;
      fprintf(fid,'Weight on randomization:               %0.4f\n',RWvalue);
   else;
      fprintf(fid,'Scale factor for initial phi density:  %0.4f\n',ScaleInitial);
   end;
end;
if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
   if DSGEModel.PosteriorSampler<=7;
      fprintf(fid,'Scale factor for proposal phi density: %0.4f\n',ScaleProp);
   end;
   if DSGEModel.PosteriorSampler==3;
      fprintf(fid,'Deg. of freedom of Student-t proposal: %0.0f\n',DSGEModel.StudenttDegFree);
   elseif DSGEModel.PosteriorSampler==4;
      fprintf(fid,'Number of fixed parameter blocks:      %0.0f\n',DSGEModel.FixedNumParamBlocks);
   elseif DSGEModel.PosteriorSampler==5;
      fprintf(fid,'Number of fixed parameter blocks:      %0.0f\n',DSGEModel.FixedNumParamBlocks);
      fprintf(fid,'Deg. of freedom of Student-t proposal: %0.0f\n',DSGEModel.StudenttDegFree);
   elseif DSGEModel.PosteriorSampler==6;
      fprintf(fid,'Number of random parameter blocks:     Lower=%0.0f, Upper=%0.0f\n',DSGEModel.BlockSize(1),DSGEModel.BlockSize(2));
      fprintf(fid,'                                       Uniform discrete distribution\n');
   elseif DSGEModel.PosteriorSampler==7;
      fprintf(fid,'Number of random parameter blocks:     Lower=%0.0f, Upper=%0.0f\n',DSGEModel.BlockSize(1),DSGEModel.BlockSize(2));
      fprintf(fid,'                                       Uniform discrete distribution\n');
      fprintf(fid,'Deg. of freedom of Student-t proposal: %0.0f\n',DSGEModel.StudenttDegFree);
   elseif (DSGEModel.PosteriorSampler==8)|(DSGEModel.PosteriorSampler==9);
      %
      % need more information here
      %
      fprintf(fid,'Resampling algorithm:                  %s\n',ParData.ResamplingAlgorithmStr);
      fprintf(fid,'Number of resamples:                   %0.0f\n',ParData.NumResamples);
      fprintf(fid,'Number of fixed parameter blocks:      %0.0f\n',ParData.NumBlocks);
      fprintf(fid,'Number of MH steps:                    %0.0f\n',ParData.SMCNumMHSteps);
      fprintf(fid,'Mixing weight (alpha):                 %0.4f\n',ParData.alpha);
      fprintf(fid,'Initial scale factor for proposal:     %0.4f\n',ParData.cstarinit);
      fprintf(fid,'Adapted scale factor value:            %0.4f\n',ParData.cstar);
   elseif DSGEModel.PosteriorSampler==10;
      %
      % add IS based on MitISEM related information here
      %
      fprintf(fid,'Maximum number of t-densities:         %0.0f\n',ParData.Hstar);
      fprintf(fid,'Initial df value for t-density:        %0.0f\n',ParData.ISInitialDF);
      fprintf(fid,'Inverse Hessian estimator:             %s\n',InvHessianTxt);
      if DSGEModel.InverseHessianEstimator==3;
         fprintf(fid,'Step length:                           %s\n',StepLength);
      end;
      fprintf(fid,'Max absolute correlation scale factor: %0.4f\n',MaxCorr);
      fprintf(fid,'Draws & weight shares for H+1 comp.:   %s\n',ParData.alphaStr);
      fprintf(fid,'Initial weight on H+1 comp.:           %0.4f\n',ParData.piH1);
      fprintf(fid,'Initial df value for H+1 comp.:        %0.0f\n\n',ParData.dfH1);
      fprintf(fid,'Optimal number of mixture components\n');
      fprintf(fid,'              for candidate density:   %0.0f\n',length(ParData.df));
      fprintf(fid,'Best CoV value obtained for share:     %0.6f\n',ParData.alphastar);
      fprintf(fid,'Coefficient of Variation (CoV):        %0.6f\n',ParData.CoV);
      fprintf(fid,'Relative change of CoV:                %0.6f\n',ParData.RelCoV);
      fprintf(fid,'Tolerance level for CoV:               %0.6f\n',ParData.CoVTolerance);
      %
      % check why the MitISEM algorithm converged
      %
      if ParData.UseObjFcnCriterion==1;
         fprintf(fid,'Value of scaled objective function:    %0.6f\n',ParData.logObjFcn);
         fprintf(fid,'Tolerance level of relative change of\n');
         fprintf(fid,'       the scaled objective function:  %0.6f\n',ParData.ToleranceValue);
         fprintf(fid,'Relative change of the scaled\n');
         fprintf(fid,'       objective function:             %0.6f\n',ParData.DlogObjFcn);
         if (ParData.RelCoV>ParData.CoVTolerance)|(isnan(ParData.RelCoV)==1);
            fprintf(fid,'=> Reason for convergence at %0.0f mixture components is:\n',length(ParData.df));
            if (ParData.DlogObjFcn>=0)&(ParData.DlogObjFcn<ParData.ToleranceValue);
               fprintf(fid,'   * Relative objective function change smaller than tolerance\n\n');
            else;
               fprintf(fid,'   * Relative objective function change negative for %0.0f mixture components\n\n',length(ParData.df)+1);
            end;
         end;
      else;
         fprintf(fid,' => Relative CoV change smaller than tolerance\n\n');
      end;
   end;
elseif DSGEModel.PosteriorSampler==2;
   fprintf(fid,'Scale factor for slice sampler:        %0.4f\n',ScaleProp);
end;
fprintf(fid,'Posterior sampling time:               %s (hrs:min:sec)\n',SecondsToHoursStr(PostSampleTime));
if (DSGEModel.PosteriorSampler==1)|((DSGEModel.PosteriorSampler>=3)&(DSGEModel.PosteriorSampler<=9));
   if (DSGEModel.PosteriorSampler==8)|(DSGEModel.PosteriorSampler==9);
      fprintf(fid,'Targeted acceptance rate:              %0.4f\n',ParData.pstar);
   end;
   fprintf(fid,'Acceptance rate:                       %0.4f\n\n',AcceptRate);
elseif DSGEModel.PosteriorSampler==2;
   fprintf(fid,'Average log posterior evaluations:     %0.4f\n\n',AcceptRate);
elseif DSGEModel.PosteriorSampler==10;
   %
   % IS based on MitISEM information about the optimal candidate density
   %
   fprintf(fid,'\n');
   fprintf(fid,'********************************************************************************\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'*    P A R A M E T E R S   O F   T H E   C A N D I D A T E   D E N S I T Y     *\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'********************************************************************************\n\n');
   %
   H = length(ParData.df);
   m = length(thetaMargMode);
   PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
   PrtStr = ['%s%s   ' PrtVal '   ' PrtVal '\n'];
   ns = size(ParameterNames.estimated,2);
   if ns<20;
      AddStr = SpaceStr(20-ns);
   else;
      AddStr = '';
   end;
   % 
   for h=1:H;
      fprintf(fid,'Mixture component number %0.0f\n\n',h);
      fprintf(fid,'Estimated weight on component:        %0.6f\n',ParData.piweights(h));
      fprintf(fid,'Estimated df for component t-density: %0.6f\n\n',ParData.df(h));
      fprintf(fid,'===================================================\n');
      fprintf(fid,'Transformed parameter    location     scale factor\n');
      fprintf(fid,'---------------------------------------------------\n');
      for i=1:m;
         fprintf(fid,PrtStr,ParameterNames.estimated(i,:),AddStr,ParData.mu(i,h),sqrt(ParData.Sigma(i,i,h)));
      end;
      fprintf(fid,'===================================================\n\n');
   end;
   fprintf(fid,'NOTE: The scale factor is the square-root of the diagonal element of the\n      estimated scale matrix. The off-diagonal elements of this positive\n      definite matrix are not reported. The full matrix is stored in the\n      posterior draws file.\n\n');
end;
if DSGEModel.PosteriorSampler<=7;
   if (DSGEModel.MarginalLikelihoodValue~=5)|(((DSGEModel.PosteriorSampler==2)&(DSGEModel.MarginalLikelihoodValue==4))~=1);
      prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
      %
      % write results for log marginal likelihoods
      %
      fprintf(fid,'********************************************************************************\n');
      fprintf(fid,'*                                                                              *\n');
      fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
      fprintf(fid,'*                                                                              *\n');
      fprintf(fid,'********************************************************************************\n\n');
      if IsSystemPrior==1;
         if isempty(SPLogMargsMH)==0;
            %
            % Geweke's modified harmonic mean estimator
            %
            prt_str = 'Geweke''s Harmonic Mean';
            top_str = '                      ';
            std_str = 'Num. standard errors: ';
            for i=1:length(SPCovProb);
               top_str = [top_str '        ' num2str(SPCovProb(i)*100,'%0.0f') ' percent'];
               prt_str = [prt_str '   ' prt_val];
               std_str = [std_str '   ' prt_val];
            end;
            TopStr = MultiCharStr('=',length(top_str));
            MidStr = MultiCharStr('-',length(top_str));
            fprintf(fid,'%s\n',TopStr);
            fprintf(fid,'                                        System Prior Results\n');
            fprintf(fid,'                                        --------------------\n');
            fprintf(fid,'                              Coverage probabilities for the estimator\n');
            fprintf(fid,[top_str '\n']);
            fprintf(fid,'%s\n',MidStr);
            fprintf(fid,[prt_str '\n'],SPLogMargsMH(size(SPLogMargsMH,1),2:size(SPLogMargsMH,2))');
            if isempty(SPNWStdErrMH)==0;
               fprintf(fid,[std_str '\n'],SPNWStdErrMH');
            end;
            fprintf(fid,[MidStr '\n']);
            fprintf(fid,['Mean:                    ' prt_val '\n'],sum(SPLogMargsMH(1,2:size(SPLogMargsMH,2)))/length(SPCovProb));
            fprintf(fid,['Median:                  ' prt_val '\n'],median(SPLogMargsMH(1,2:size(SPLogMargsMH,2))));
            fprintf(fid,'%s\n\n',TopStr);
         end;
         fprintf(fid,'Laplace approximation:         %0.6f\n\n',LogMargLikeSystemLaplace);
         fprintf(fid,'NOTE: The Laplace approximation of the log marginal likelihood of\n      the system prior is computed from the prior mode of phi, the\n      transformed parameters, and the finite difference estimator\n      of the inverse Hessian at the mode.\n\n');
         fprintf(fid,'NOTE: If the harmonic mean estimates differ greatly from the Laplace\n      approximation estimate, you probably need more prior draws.\n\n');
      end;
      if isempty(LogMargsMH)==0;
         %
         % Geweke's modified harmonic mean estimator
         %
         prt_str = 'Geweke''s Harmonic Mean';
         top_str = '                      ';
         std_str = 'Num. standard errors: ';
         for i=1:length(CovProb);
            top_str = [top_str '        ' num2str(CovProb(i)*100,'%0.0f') ' percent'];
            prt_str = [prt_str '   ' prt_val];
            std_str = [std_str '   ' prt_val];
         end;
         TopStr = MultiCharStr('=',length(top_str));
         MidStr = MultiCharStr('-',length(top_str));
         fprintf(fid,'%s\n',TopStr);
         fprintf(fid,'                              Coverage probabilities for the estimator\n');
         fprintf(fid,[top_str '\n']);
         fprintf(fid,'%s\n',MidStr);
         fprintf(fid,[prt_str '\n'],LogMargsMH(size(LogMargsMH,1),2:size(LogMargsMH,2))');
         if isempty(NWStdErrMH)==0;
            fprintf(fid,[std_str '\n'],NWStdErrMH');
         end;
         fprintf(fid,[MidStr '\n']);
         fprintf(fid,['Mean:                    ' prt_val '\n'],sum(LogMargsMH(1,2:size(LogMargsMH,2)))/length(CovProb));
         fprintf(fid,['Median:                  ' prt_val '\n'],median(LogMargsMH(1,2:size(LogMargsMH,2))));
         fprintf(fid,'%s\n\n',TopStr);
         if IsSystemPrior==1;
            fprintf(fid,'NOTE: These marginal likelihood estimates do NOT take the marginal likelihood\n      of the system prior into account. The latter should be subtraced from\n      the former.\n\n');
         end;
      end;
      if isempty(LogMargsSWZ)==0;
         %
         % SWZ's modified harmonic mean estimator
         %
         if isempty(NWStdErrSWZ)==1;
            NWStdErrSWZ = NaN;
         end;
         fprintf(fid,'===============================================================\n');
         fprintf(fid,'                              point estimate        std. error\n');
         fprintf(fid,'---------------------------------------------------------------\n');
         fprintf(fid,['Sims, Waggoner & Zha         ' prt_val '   ' prt_val '\n'],LogMargsSWZ(size(LogMargsSWZ,1),2),NWStdErrSWZ);
         fprintf(fid,'---------------------------------------------------------------\n');
         fprintf(fid,['Lower Bound of log posterior kernel:           ' prt_val ' \n'],LBound);
         fprintf(fid,['Fraction of iid draws from elliptical > lower bound:  %0.6f \n'],qL);
         fprintf(fid,['Fraction of posterior draws > lower bound:            %0.6f \n'],PValue/100);
         fprintf(fid,'===============================================================\n\n');
         fprintf(fid,'NOTE: The "fraction of iid draws from elliptical > lower bound" notation refers to the fraction of\n      the parameter draws from the elliptical distribution which have a log posterior kernel greater\n      than the lower bound. The "fraction of posterior draws > lower bound" notation refers to the\n      log posterior kernel values for the posterior draws.\n\n');
         %
         if IsSystemPrior==1;
            fprintf(fid,'NOTE: These marginal likelihood estimates do NOT take the marginal likelihood\n      of the system prior into account. The latter should be subtraced from\n      the former.\n\n');
         end;
      end;
      if isempty(LogMargsCJ)==0;
         fprintf(fid,'==========================================================\n');
         fprintf(fid,'                          point estimate        std. error\n');
         fprintf(fid,'----------------------------------------------------------\n');
         fprintf(fid,['Chib & Jeliazkov         ' prt_val '   ' prt_val '\n'],LogMargsCJ(size(LogMargsCJ,1),2),LogMargsCJ(size(LogMargsCJ,1),3));
         fprintf(fid,'==========================================================\n\n');
         if IsSystemPrior==1;
            fprintf(fid,'NOTE: These marginal likelihood estimates do NOT take the marginal likelihood\n      of the system prior into account. The latter should be subtraced from\n      the former.\n\n');
         end;
      end;
   end;
else;
   prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
   %
   % write results for log marginal likelihoods
   %
   fprintf(fid,'********************************************************************************\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'********************************************************************************\n\n');
   if IsSystemPrior==1;
      if isempty(SPLogMargsMH)==0;
         %
         % Geweke's modified harmonic mean estimator
         %
         prt_str = 'Geweke''s Harmonic Mean';
         top_str = '                      ';
         std_str = 'Num. standard errors: ';
         for i=1:length(SPCovProb);
            top_str = [top_str '        ' num2str(SPCovProb(i)*100,'%0.0f') ' percent'];
            prt_str = [prt_str '   ' prt_val];
            std_str = [std_str '   ' prt_val];
         end;
         TopStr = MultiCharStr('=',length(top_str));
         MidStr = MultiCharStr('-',length(top_str));
         fprintf(fid,'%s\n',TopStr);
         fprintf(fid,'                                        System Prior Results\n');
         fprintf(fid,'                                        --------------------\n');
         fprintf(fid,'                              Coverage probabilities for the estimator\n');
         fprintf(fid,[top_str '\n']);
         fprintf(fid,'%s\n',MidStr);
         fprintf(fid,[prt_str '\n'],SPLogMargsMH(size(SPLogMargsMH,1),2:size(SPLogMargsMH,2))');
         if isempty(SPNWStdErrMH)==0;
            fprintf(fid,[std_str '\n'],SPNWStdErrMH');
         end;
         fprintf(fid,[MidStr '\n']);
         fprintf(fid,['Mean:                    ' prt_val '\n'],sum(SPLogMargsMH(1,2:size(SPLogMargsMH,2)))/length(SPCovProb));
         fprintf(fid,['Median:                  ' prt_val '\n'],median(SPLogMargsMH(1,2:size(SPLogMargsMH,2))));
         fprintf(fid,'%s\n\n',TopStr);
      end;
      fprintf(fid,'Laplace approximation:         %0.6f\n\n',LogMargLikeSystemLaplace);
      fprintf(fid,'NOTE: The Laplace approximation of the log marginal likelihood of\n      the system prior is computed from the prior mode of phi, the\n      transformed parameters, and the finite difference estimator\n      of the inverse Hessian at the mode.\n\n');
      fprintf(fid,'NOTE: If the harmonic mean estimates differ greatly from the Laplace\n      approximation estimate, you probably need more prior draws.\n\n');
   end;
   %
   if (DSGEModel.PosteriorSampler==8)|(DSGEModel.PosteriorSampler==9);
      fprintf(fid,'    Log marginal likelihood estimate = %0.6f \n\n',ParData.LogMargLikeSMC);
   else;
      %
      % marginal likelihood estimate and numerical standard error from IS based on MitISEM
      %
      fprintf(fid,'    Log marginal likelihood estimate = %0.6f \n',ParData.LogMargLikeIS);
      fprintf(fid,'    Numerical standard error         = %0.6f \n\n',ParData.StErrLogMargLikeIS);
   end;
end;
%
% deal with the parameters
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                             P A R A M E T E R S                              *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
fprintf(fid,'Number of parameters to estimate: %0.0f\n',length(thetaMargMode));
if NumAdditional>0;
   fprintf(fid,'Number of update parameters:      %0.0f\n',NumAdditional);
end;
if size(ParameterNames.calibrated,1)>0;
   fprintf(fid,'Number of calibrated parameters:  %0.0f\n',size(ParameterNames.calibrated,1));
end;
fprintf(fid,'Total number of parameters:       %0.0f\n\n',size(ParameterNames.all,1));
%
ns = size(ParameterNames.estimated,2);
if ns<20;
   add_str = SpaceStr(20-ns);
else;
   add_str = '';
end;
prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
prt_str = ['%s%s   ' prt_val '   ' prt_val '   ' prt_val '   |   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val];
NewDivStr = MultiCharStr('-',129);
fprintf(fid,'=================================================================================================================================\n');
fprintf(fid,'                                                       Original Parameters\n');
fprintf(fid,'           parameter       mean          median          mode       |      st. dev.   auto st. dev.       RNE             IF\n');
fprintf(fid,'                     [std err of mean]\n');
fprintf(fid,'                       initial value\n');
if (isempty(PriorDist.truncnormal)==1)&(isempty(PriorDist.invgamma)==1)&(isempty(PriorDist.gamma)==1)&(isempty(PriorDist.beta)==1)&(isempty(PriorDist.student)==1)&(isempty(PriorDist.pareto)==1)&(isempty(PriorDist.logistic)==1);
   fprintf(fid,'         (prior dist    parameter 1    parameter 2)\n');
else;
   if isempty(PriorDist.beta)==1;
      fprintf(fid,'         (prior dist    parameter 1    parameter 2    parameter 3)\n');
   else;
      fprintf(fid,'         (prior dist    parameter 1    parameter 2    parameter 3\n');
      fprintf(fid,'                                                      parameter 4)\n');
   end;
end;
%
n_gamma = 0;
n_beta = 0;
n_normal = 0;
n_invgamma = 0;
n_truncnormal = 0;
n_uniform = 0;
n_student = 0;
n_cauchy = 0;
n_logistic = 0;
n_gumbel = 0;
n_pareto = 0;
for i=1:NumParam;
   if i>1;
      fprintf(fid,'%s\n',NewDivStr);
   else;
      fprintf(fid,'=================================================================================================================================\n');
   end;
   if thetaDist(i)==1;
      dist_str = 'gamma';
      dist_add = SpaceStr(6);
      n_gamma = n_gamma+1;
      prior_val = PriorDist.gamma(n_gamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==2;
      dist_str = 'beta';
      dist_add = SpaceStr(7);
      n_beta = n_beta+1;
      prior_val = PriorDist.beta(n_beta,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0 1];
      end;
   elseif thetaDist(i)==3;
      dist_str = 'normal';
      dist_add = SpaceStr(5);
      n_normal = n_normal+1;
      prior_val = PriorDist.normal(n_normal,:);
   elseif thetaDist(i)==4;
      dist_str = 'invgamma';
      dist_add = SpaceStr(3);
      n_invgamma = n_invgamma+1;
      prior_val = PriorDist.invgamma(n_invgamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==5;
      dist_str = 'truncnormal';
      dist_add = '';
      n_truncnormal = n_truncnormal+1;
      prior_val = PriorDist.truncnormal(n_truncnormal,:);
   elseif thetaDist(i)==6;
      dist_str = 'uniform';
      dist_add = SpaceStr(4);
      n_uniform = n_uniform+1;
      prior_val = PriorDist.uniform(n_uniform,:);
   elseif thetaDist(i)==7;
      dist_str = 'student';
      dist_add = SpaceStr(4);
      n_student = n_student+1;
      prior_val = PriorDist.student(n_student,:);
   elseif thetaDist(i)==8;
      dist_str = 'cauchy';
      dist_add = SpaceStr(5);
      n_cauchy = n_cauchy+1;
      prior_val = PriorDist.cauchy(n_cauchy,:);
   elseif thetaDist(i)==9;
      dist_str = 'logistic';
      dist_add = SpaceStr(3);
      n_logistic = n_logistic+1;
      prior_val = PriorDist.logistic(n_logistic,:);
   elseif thetaDist(i)==10;
      dist_str = 'gumbel';
      dist_add = SpaceStr(5);
      n_gumbel = n_gumbel+1;
      prior_val = PriorDist.gumbel(n_gumbel,:);
   elseif thetaDist(i)==11;
      dist_str = 'pareto';
      dist_add = SpaceStr(5);
      n_pareto = n_pareto+1;
      prior_val = PriorDist.pareto(n_pareto,:);
   end;
   fprintf(fid,[prt_str '\n'],add_str,ParameterNames.estimated(i,:),ParData.SampleMean(i),ParData.SampleMedian(i),thetaMargMode(i),ParData.SampleStd(i),ParData.SampleStdAuto(i),(ParData.SampleStd(i)/ParData.SampleStdAuto(i))^2,(ParData.SampleStdAuto(i)/ParData.SampleStd(i))^2);
   fprintf(fid,['                      [' prt_val ']\n'],ParData.SampleStdErr(i));
   fprintf(fid,['                       ' prt_val '\n'],thetaInit(i));
   if (strcmp(dist_str,'truncnormal')==0)&(strcmp(dist_str,'invgamma')==0)&(strcmp(dist_str,'gamma')==0)&(strcmp(dist_str,'beta')==0)&(strcmp(dist_str,'student')==0)&(strcmp(dist_str,'pareto')==0)&(strcmp(dist_str,'logistic')==0);
      fprintf(fid,['        %s(%s   ' prt_val '   ' prt_val ')\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2));
   else;
      if strcmp(dist_str,'beta')==0;
         fprintf(fid,['        %s(%s   ' prt_val '   ' prt_val '   ' prt_val ')\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2),prior_val(1,3));
      else;
         fprintf(fid,['        %s(%s   ' prt_val '   ' prt_val '   ' prt_val '\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2),prior_val(1,3));
         fprintf(fid,['                                                     ' prt_val ')\n'],prior_val(1,4));
      end;
   end;
end;
fprintf(fid,'=================================================================================================================================\n\n');
%
% check if we have additional parameters
%
if isempty(UpdateMargMode)==0;
   nns = size(UpdateNames,2);
   if nns<20;
      AddUpdStr = SpaceStr(20-nns);
   else;
      AddUpdStr = '';
   end;
   fprintf(fid,'=================================================================================================================================\n');
   fprintf(fid,'                                                       Update Parameters\n');
   fprintf(fid,'           parameter       mean          median          mode       |      st. dev.   auto st. dev.       RNE             IF\n');
   fprintf(fid,'                     [std err of mean]\n');
   for i=1:NumAdditional;
      if i>1;
         fprintf(fid,'%s\n',NewDivStr);
      else;
         fprintf(fid,'=================================================================================================================================\n');
      end;
      fprintf(fid,[prt_str '\n'],AddUpdStr,UpdateNames(i,:),UpdateParData.SampleMean(i),UpdateParData.SampleMedian(i),UpdateMargMode(i),UpdateParData.SampleStd(i),UpdateParData.SampleStdAuto(i),(UpdateParData.SampleStd(i)/UpdateParData.SampleStdAuto(i))^2,(UpdateParData.SampleStdAuto(i)/UpdateParData.SampleStd(i))^2);
      fprintf(fid,['                      [' prt_val ']\n'],UpdateParData.SampleStdErr(i));
   end;
   fprintf(fid,'=================================================================================================================================\n\n');
end;
%
fprintf(fid,'NOTE: RNE measures the relative numerical efficiency. It is defined as the ratio between the variance\n');
fprintf(fid,'      of the draws under the assumption that they are iid and the variance of the draws when autocorrelation\n');
fprintf(fid,'      is taken into account. This value is equal to the ratio between the previous two columns squared. The\n');
fprintf(fid,'      inverse of the RNE is sometimes called the inefficiency factor (IF) and is equal to 1+2*sum(1,K) rho(k),\n');
fprintf(fid,'      where rho(k) is the autocorrelation at lag k, and K is %0.0f.\n\n',floor((NumIter-NumBurnin)^(1/2.01)));
%
% print parameter quantiles
%
prt_str = ['%s%s   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val];
fprintf(fid,'===========================================================================================================================================================\n');
fprintf(fid,'                                                      Posterior Quantiles for the Original Parameters                                                      \n');
fprintf(fid,'           parameter        2.5%%           5%%            10%%            20%%            50%%            80%%            90%%            95%%            97.5%%\n');
for i=1:NumParam;
   fprintf(fid,'-----------------------------------------------------------------------------------------------------------------------------------------------------------\n');
   fprintf(fid,[prt_str '\n'],add_str,ParameterNames.estimated(i,:),ParData.Quantiles(i,:)');
end;
fprintf(fid,'===========================================================================================================================================================\n\n');
%
if isempty(UpdateMargMode)==0;
   fprintf(fid,'===========================================================================================================================================================\n');
   fprintf(fid,'                                                      Posterior Quantiles for the Update Parameters                                                      \n');
   fprintf(fid,'           parameter        2.5%%           5%%            10%%            20%%            50%%            80%%            90%%            95%%            97.5%%\n');
   for i=1:NumAdditional;
      fprintf(fid,'-----------------------------------------------------------------------------------------------------------------------------------------------------------\n');
      fprintf(fid,[prt_str '\n'],AddUpdStr,UpdateNames(i,:),UpdateParData.Quantiles(i,:)');
   end;
   fprintf(fid,'===========================================================================================================================================================\n\n');
end;
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(PostSampleTextFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(PostSampleTextFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintPosteriorSamplingSummary.m
%
 