function PrintStateVarianceDecompositions(ResFile,DSGEModel,FEVDs,LRVD,UniquenessCheck,PrintGroups,CurrINI,EstStr)
% PrintStateVarianceDecompositions: Writes the forecast error variance decompositions for the state variables
%                                   to a text file.
%
% USAGE:
%
%       PrintStateVarianceDecompositions(ResFile,DSGEModel,FEVDs,LRVD,UniquenessCheck,PrintGroups,...
%                                        Levels,CurrINI,EstStr)
%
% REQUIRED INPUT: ResFile (string vector) with the full path and name of the output file.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 FEVDs (rx(1+q)xh) matrix with the forecast error variance decompositions for the state variables.
%                           For each h, the rx(1+q) matrix is ordered such that the initial state is ordered
%                           first and the q economic shocks in the following columns. The state variables are
%                           determined through the row numbers.
%
%                 LRVD (rxq) matrix with the long run forecast error variance decomposition.
%
%                 UniquenessCheck ((q+1) vector), with largest eigenvalues of (F-K*H'), where K is the asymptote of the
%                           Kalman gain matrix, i.e., K = F*P1*H*inv(H'*P1*H+R). Elements 1 concerns the full forecast error
%                           variance, while the following q relate to the forecast error variances obtained when economic
%                           shock j, j=1,...,q, is allowed for.
%
%                 PrintGroups (boolean) which indicates if shock groups are used (1) or not (0).
%
%                 CurrINI (structure), with initialization information.
%
%                 EstStr (string vector) that takes the value "posterior mode" or "initial".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 19, 2017.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-12-2017: Changed the output such that state variables appear in rows and shocks in columns.
%
% * 14-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'* F O R E C A S T   E R R O R   V A R I A N C E   D E C O M P O S I T I O N S  *\n');
fprintf(fid,'*                        S T A T E   V A R I A B L E S                         *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'NOTE: Using the %s values.\n\n',lower(EstStr));
%
% new code begins here
%
[n,q1,h] = size(FEVDs);
q = q1-1;
vq = size(DSGEModel.StateVariableNames,2);
if PrintGroups==0;
   NamesMatrix = DSGEModel.StateShockNames;
else;
   NamesMatrix = char(zeros(size(DSGEModel.ShockGroupNames)));
   for i=1:q;
      NamesMatrix(i,:) = strrep(DSGEModel.ShockGroupNames(i,:),'_',' ');
   end;
end;
NamesMatrix = strvcat('All state vars',NamesMatrix,'Sum');
vn = size(NamesMatrix,2);
%
% shift all rows of NamesMatrix such that they have a ragged left edge
%
NamesMat = '';
for i=1:q+2;
   NamesMati = StringTrim(NamesMatrix(i,:));
   vni = length(NamesMati);
   if vni<vn;
      AddNameStr = SpaceStr(vn-vni);
   else;
      AddNameStr = '';
   end;
   NamesMat = strvcat(NamesMat,[AddNameStr NamesMati]);
end;
NamesMatrix = NamesMat;
%
if vn<15;
   AddNameStr = SpaceStr(15-vn);
else;
   AddNameStr = '';
end;
if vq<20;
   AddShockStr = SpaceStr(20-vq);
else;
   AddShockStr = '';
end;
NameStr = '                    ';
NameStrLR = '                    ';
MEStr = '';
PrtStr = '%s';
PrtStrLR = '%s';
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
for i=1:q+2;
   NameStr = [NameStr '   ' AddNameStr NamesMatrix(i,:)];
   PrtStr = [PrtStr '      ' PrtVal];
   if i>=2;
      NameStrLR = [NameStrLR '   ' AddNameStr NamesMatrix(i,:)];
      PrtStrLR = [PrtStrLR '      ' PrtVal];
   end;
end;
%
for i=1:n;
   MEStr = strvcat(MEStr,sprintf('%s%s',AddShockStr,DSGEModel.StateVariableNames(i,:)));
end;
FirstStr = MultiCharStr('=',length(NameStr)+1);
DivStr = MultiCharStr('-',length(NameStr)+1);
FirstStrLR = MultiCharStr('=',length(NameStrLR)+1);
DivStrLR = MultiCharStr('-',length(NameStrLR)+1);
%
if PrintGroups==0;
   fprintf(fid,'--------------------------------------------------------------------------------\n');
   fprintf(fid,'              Uniqueness Properties of the Variance Decompositions\n');
   fprintf(fid,'                         For the 1-Step Ahead Forecasts\n');
   fprintf(fid,'--------------------------------------------------------------------------------\n');
   if UniquenessCheck<1-1e-14;
      fprintf(fid,'* Total Forecast Error Variance:%s            Unique asymptote.\n',SpaceStr(vq));
   else;
      fprintf(fid,'* Total Forecast Error Variance:%s            Multiple asymptotes possible.\n',SpaceStr(vq));
   end;
   fprintf(fid,'--------------------------------------------------------------------------------\n\n');
else;
   for i=1:q;
      iGroup = (DSGEModel.ShockGroups==i) .* (1:length(DSGEModel.ShockGroups));
      iGroup = iGroup(iGroup>0);
      ShockNames = StrWrap(vecstr(DSGEModel.StateShockNames(iGroup,:),' '),60);
      GroupName = StringTrim(NamesMatrix(i,:));
      if size(GroupName,2)<25;
         AddSpace = SpaceStr(25-size(GroupName,2));
      else;
         AddSpace = '';
      end;
      fprintf(fid,'%s:%s %s\n',GroupName,AddSpace,ShockNames(1,:));
      for j=2:size(ShockNames,1);
         if strcmp(StringTrim(ShockNames(j,:)),'')==0;
            fprintf(fid,'%s%s\n',SpaceStr(27),ShockNames(j,:));
         end;
      end;
   end;
   fprintf(fid,'\n');
end;
%
% Write to output for horizon h
%
for j=1:size(FEVDs,3);
   fprintf(fid,'%s\n',FirstStr);
   fprintf(fid,'Forecast Horizon: %3.0f\n',j);
   fprintf(fid,'%s\n',NameStr);
   for i=1:n;
      fprintf(fid,'%s\n',DivStr);
      fprintf(fid,[PrtStr '\n'],MEStr(i,:),[abs(FEVDs(i,:,j)) sum(abs(FEVDs(i,:,j)))]');
   end;
   fprintf(fid,'%s\n\n',FirstStr);
end;
%
% and now the long run
%
fprintf(fid,'%s\n',FirstStrLR);
fprintf(fid,'Forecast Horizon: Long Run\n');
fprintf(fid,'%s\n',NameStrLR);
for i=1:n;
   fprintf(fid,'%s\n',DivStrLR);
   fprintf(fid,[PrtStrLR '\n'],MEStr(i,:),[abs(LRVD(i,:)) sum(abs(LRVD(i,:)))]');
end;
fprintf(fid,'%s\n\n',FirstStrLR);
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintStateVarianceDecompositions.m
%
