function PITMatrix = ComputePITsDSGEPaths(YTrueValues,NumDraws,NumPredPaths,MaxPH,CurrChain,IsPosterior,Weights,DSGEModel,CurrINI)
% ComputePITsDSGEPaths: Computes the PITs based on prediction paths from a DSGE model that are loaded from disk for the individual
%                       observed variables.
%
% USAGE:
%
%       PITMatrix = ComputePITsDSGEPaths(YTrueValues,NumDraws,NumPredPaths,MaxPH,CurrChain,IsPosterior,Weights,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  YTrueValues (matrix) of dimension n x h with the true values of the
%                              observed variables over the forecast horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with the total number of prediction paths per parameter draw.
%
%                  MaxPH (integer) with the maximum prediction horizon used for the paths (can be
%                              shorter than the horizon h for which true values are available.
%
%                  CurrChain (integer) with the number of the currently used Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                              if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                              likelihood tempering posterior sampler.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
% REQUIRED OUTPUT: PITMatrix (matrix) of dimension n x h with the estimated probabilities that the predicted variables
%                              are less than the true values.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 26, 2020.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-08-2020: Updated the documentation.
%
% * 28-08-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 15-02-2022: Added the input variable MaxPH.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[n,h] = size(YTrueValues);
PITMatrix = zeros(n,h);
HeadStr = 'Predictive Distributions (PIT)';
if IsPosterior==0;
   HeadStr = [HeadStr ' - Prior'];
end;
%
% set up a wait dialog
%
txt = 'Please wait while YADA computes the probability integral transform (PIT) values for the individual observed variables. Current forecast horizon: 1.';
WaitHandle = WaitDLG(txt,'information',HeadStr,500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% compute the PIT values for each horizon at a time
%
TotalPaths = NumPredPaths*NumDraws;
for j=1:h;
   txt = ['Please wait while YADA computes the probability integral transform (PIT) values for the individual observed variables. Current forecast horizon: ' int2str(j) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % initialize a temporary variable
   %
   try;
      TempVar = zeros(TotalPaths,n);
      for ParamIndex=1:NumDraws;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-Prediction-Original-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-Prediction-Original-Prior-' DSGEModel.NameOfModel '-' int2str(MaxPH) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         end;
         %
         % load the data
         %
         PredData = load(file);
         PredPaths = PredData.PredPaths;
         for p=1:NumPredPaths;
            CurrPath = p+(NumPredPaths*(ParamIndex-1));
            TempVar(CurrPath,:) = PredPaths(:,j,p)';
         end;
         clear('PredData');
         drawnow;
      end;
      %
      % compute the PITs for horizon j based on the paths
      %
      PITMatrix(:,j) = sum(TempVar'<repmat(YTrueValues(:,j),1,TotalPaths),2)/TotalPaths;
   catch;
      for t=1:n;
         TempVar = zeros(TotalPaths,1);
         for ParamIndex=1:NumDraws;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-Prediction-Original-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-Prediction-Original-Prior-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
            end;
            %
            % load the data
            %
            PredData = load(file);
            PredPaths = PredData.PredPaths;
            for p=1:NumPredPaths;
               CurrPath = p+(NumPredPaths*(ParamIndex-1));
               TempVar(CurrPath,1) = PredPaths(t,j,p)';
            end;
            clear('PredData');
            drawnow;
         end;
         %
         % compute the PITs for variable t and horizon j
         %
         PITMatrix(t,j) = sum(TempVar<YTrueValues(t,j))/TotalPaths;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of ComputePITsDSGEPaths.m
%
