function [AsSigmaUpdate,AsSigmaSmooth,status,RiccatiResults,UniquenessCheck] = DSGEAsymptoticShockCovariances(H,R,F,B0,Ttot,T,DAMaxIter,DAConvValue,RicMaxIter,RicConvValue)
% DSGEAsymptoticShockCovariances: Computes the asymptotes of the covariance matrix for the update
%                                 and smooth estimates of the structural shocks.
%
% USAGE:
%
%       [AsSigmaUpdate,AsSigmaSmooth,status,RiccatiResults,UniquenessCheck] = DSGEAsymptoticShockCovariances(H,R,F,B0,Ttot,T,...
%                                                                       DAMaxIter,DAConvValue,RicMaxIter,RicConvValue)
%
% REQUIRED INPUT:  H (matrix) of dimension rxn from the measurement equation linking observables to
%                             state variables.
%
%                  R (matrix) of dimension nxn with the measurement error covariances.
%
%                  F (matrix) of dimension rxr with the first order autocorrelations for the
%                             state variables in the state equation.
%
%                  B0 (matrix) of dimension rxq linking the structural shocks to the state variables
%                             in the state equation.
%
%                  Ttot (integer) with the last time period of the sample used to estimate the parameters.
%
%                  T (integer) with the length of the sample with smooth estimates.
%
%                  DAMaxIter (integer) which determines the maximum number of iterations that
%                             the doubling algorithm for the Lyapunov aquations can use.
%
%                  DAConvValue (integer) which determined the tolerance level for the convergence of the
%                             doubling algorithms for the Lyapunov equations.
%
%                  RicMaxIter (integer) which determined the maximum number of iterations that the
%                             Riccati equation solver can use.
%
%                  RicConvValue (integer) which determines the tolerance level for the convergence of
%                             the Riccati equations solver.
%
% REQUIRED OUTPUT: AsSigmaUpdate (matrix) of dimension nxn with the limit covariance matrix of the
%                             update estimates of the structural shocks.
%
%                  AsSigmaSmooth (matrix) of dimension nxnxT with the limit covariance matrices of the
%                             smoothed estimates of the structural shocks.
%
%                  status (integer) takes the value 0 if the solution method converged and 1 otherwise. It takes
%                             on the value 2 in the special case that infinite or NaN's are found in P1 during the
%                             iterations.
%
%                  RiccatiResults (1x2 vector), with the number of iterations and the TestValue for the Riccati
%                             solver in the columns. The TestValue refers to the degree of convergence.
%
%                  UniquenessCheck (scalar), with largest eigenvalues of (F-K*H'), where K is the asymptote of the
%                             Kalman gain matrix, i.e., K = F*P1*H*inv(H'*P1*H+R) for the full forecast error
%                             variance.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 10, 2018.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 11-01-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
AsSigmaUpdate = [];
AsSigmaSmooth = [];
status = 0;
RiccatiResults = [1 0];
UniquenessCheck = 1;
%
% check if H is time-varying
%
if length(size(H))==3;
   H = H(:,:,Ttot);
end;
%
% replace small numbers with zero
%
Bpos = (B0>1.0e-12);
Bneg = (B0<-1.0e-12);
B0 = (B0 .* Bpos)+(B0 .* Bneg);
%
Q = B0*B0';
P0 = Q - (Q*(H/((H'*Q*H)+R))*H'*Q);
%
if max(abs(eig(F*P0*F')))>1.0e-10;
   [P_init,stat] = DoublingAlgorithmLyapunov(F,Q,DAMaxIter*100,10^(-(DAConvValue+1)));
   [P1,status,NumIter,TestValue] = RiccatiSolver(F,H,Q,R,P_init,RicMaxIter*1000,10^(-(RicConvValue+1)));
   RiccatiResults = [NumIter TestValue];
else;
   P1 = Q;
end;
%
% check if we didn't converge and leave the function in that case
%
if status~=0;
   return;
end;
Sigmay = (H'*P1*H)+R;
K = F*P1*(H/Sigmay);
L = F-(K*H');
AsSigmaUpdate = ((B0'*H)/Sigmay)*H'*B0;
n = size(AsSigmaUpdate,1);
AsSigmaSmooth = zeros(n,n,T);
N0 = (H/Sigmay)*H';
if norm(B0'*L'*L*B0)>1.0e-10;
   AsSigmaSmooth(:,:,T) = B0'*N0*B0;
   N = N0;
   for t=1:T-1;
      N = N0+(L'*N*L);
      AsSigmaSmooth(:,:,T-t) = B0'*N*B0;
   end;
else;
   for t=1:T;
      AsSigmaSmooth(:,:,t) = AsSigmaUpdate;
   end;
end;
%
% check the eigenvalues of L
%
UniquenessCheck = max(abs(eig(L)));

%
% end of DSGEAsymptoticShockCovariances.m
%
