function MargBackNowcast = DSGEBackNowcast(BackYData,BackXData,KsitT,PtT,A,H,PredictionVariables)
% DSGEBackNowcast: Computes the marginalized conditional predictive likelihood for back and nowcasts
%
% USAGE:
%
%       MargBackNowcast = DSGEBackNowcast(BackYData,BackXData,KsitT,PtT,A,H,PredictionVariables)
%
% REQUIRED INPUT:  BackYData (matrix) of dimension nxT with the actual (revised) data for back
%                            and nowcasting.
%                  BackXData (matrix) of dimension kxT with the exogenous variables.
%                  KsitT (matrix) of dimension rxT with the smooth estimates of the state
%                            variables over the historical sample.
%                  PtT (matrix) of dimension rxrxT with the smooth estimates of the state
%                            covariances over the historical sample.
%                  A (matrix) of dimension kxn with the parameters on the exogenous variables
%                            in the measurement equations.
%                  H (matrix) of dimension rxn (possibly xT) with the coefficients on the
%                            state variables in the measurement equations.
%                  PredictionVariables (vector) of dimension n1 with the location among the
%                            n observed variables to the predict.
%
% REQUIRED OUTPUT: MargBackNowcast (vector) of dimension 1xT with the values of the marginalized
%                            conditional predictive likelihood for the back and nowcasts.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 20, 2017.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 17-07-2023: Made sure that all diagonal elements of SigYhat are positive.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
[n,T] = size(BackYData);
MargBackNowcast = NaN*ones(1,T);
nstar = length(PredictionVariables);
Ax = zeros(n,T);
if isempty(BackXData)==0;
   Ax = A'*BackXData;
end;
log2pi = log(2*pi);
if length(size(H)==2);
   for t=1:T;
      Yhat = Ax(:,t) + H'*KsitT(:,t);
      SigYhat = H'*PtT(:,:,t)*H;
      %
      % select the variables of interest
      %
      Yerror = BackYData(PredictionVariables,t)-Yhat(PredictionVariables);
      SigYhat = SigYhat(PredictionVariables,PredictionVariables);
      if (rank(SigYhat)==nstar)&(min(diag(SigYhat))>1.0e-08);
         MargBackNowcast(t) = -((nstar*log2pi)+LogDet(SigYhat)+((Yerror'/SigYhat)*Yerror))/2;
      end;
   end;
else;
   for t=1:T;
      Ht = H(:,:,t);
      Yhat = Ax(:,t) + Ht'*KsitT(:,t);
      SigYhat = Ht'*PtT(:,:,t)*Ht;
      %
      % select the variables of interest
      %
      Yerror = BackYData(PredictionVariables,t)-Yhat(PredictionVariables);
      SigYhat = SigYhat(PredictionVariables,PredictionVariables);
      if (rank(SigYhat)==nstar)&(min(diag(SigYhat))>1.0e-08);
         MargBackNowcast(t) = -((nstar*log2pi)+LogDet(SigYhat)+((Yerror'/SigYhat)*Yerror))/2;
      end;
   end;
end;

%
% end of DSGEBackNowcast.m
%
