function [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = DSGEModestyStatistics(OutputDirectory,ShockMethod,NameOfModel,MaxPH,NumDraws,NumPredPaths,CurrChain,IsPosterior,n,CondTypeStr)
% DSGEModestyStatistics: Computes the modesty statistics for a DSGE model.
%
% USAGE:
%
%       [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = DSGEModestyStatistics(OutputDirectory,...
%                             ShockMethod,NameOfModel,MaxPH,NumDraws,NumPaths,CurrChain,IsPosterior,n,CondTypeStr)
%
% REQUIRED INPUT:  OutputDirectory (string vector) with the full path to the base output directory.
%
%                  ShockMethod (string vector) that takes on the value "Direct-" or "WaggonerZha-".
%
%                  NameOfModel (string vector) with the name of the model.
%
%                  MaxPH (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  CurrChain (integer) with the number of the currently used Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                              if draws from the prior are supplied.
%
%                  n (integer) with the number of observed variables.
%
% OPTIONAL INPUT:  CondTypeStr (string vector) that determines which type of conditioning assumptions have been
%                              used. The default value is an empty string.
%
% REQUIRED OUTPUT: MultiPValue (scalar) with the probability that the multivariate modesty statistic
%                              is smaller than one based on unconditional shocks.
%
%                  MeanMultiModesty (vector) with 2 elements. The first is the mean of the multivariate
%                              modesty statistic based on the conditioning consistent shocks and the second
%                              is the mean of this statistic for randomly drawn shocks.
%
%                  MeanUniModesty (vector) with means of the univariate modesty statistics for all observed
%                              variables.
%
%                  MeanUniModestyLZ (vector) with the means of the univariate modesty statistics suggested
%                              by Leeper-Zha (other shocks are zero over examined prediction horizon) for
%                              all observed variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 12, 2007.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-03-2007: Fixed a silly bug when computing the mean of "UniModestyStatLZ".
%
% * 13-11-2007: Updated the documentation.
%
% * 19-12-2007: Added the input "ShockMethod".
%
% * 23-05-2008: Updated the documentation.
%
% * 22-01-2009: Updated the documentation.
%
% * 26-03-2009: Added the input variable "IsPosterior".
%
% * 22-07-2009: Added the optional input variable "CondTypeStr".
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 13-01-2011: The function now supports unix and mac paths.
%
% * 01-03-2011: Simplified the computation of the mean statistics.
%
% * 06-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin==9;
   CondTypeStr = '';
end;
TotalPaths = NumPredPaths*NumDraws;
MultiModestyStat = zeros(TotalPaths,2);
UniModestyStat = zeros(TotalPaths,n);
UniModestyStatLZ = zeros(NumDraws,n);
LoadIndex = (1:NumPredPaths);
for ParamIndex=1:NumDraws;
   if IsPosterior==1;
      file = FixFilePath([OutputDirectory '\predictions\DSGE-Cond' CondTypeStr 'Prediction-Original-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
   else;
      file = FixFilePath([OutputDirectory '\predictions\DSGE-Cond' CondTypeStr 'Prediction-Original-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
   end;
   %
   % load the data
   %
   PredData = load(file);
   MultiModestyStat(LoadIndex,:) = PredData.MultiModestyStat;
   UniModestyStat(LoadIndex,:) = PredData.UniModestyStat;
   UniModestyStatLZ(ParamIndex,:) = PredData.UniModestyStatLZ;
   clear('PredData');
   drawnow;
   LoadIndex = LoadIndex+NumPredPaths;
end;
%
% check how often we have a greater value of column 1 than column 2 in MultiModestyStat
%
MultiPValue = sum(MultiModestyStat(:,1)<=MultiModestyStat(:,2))/TotalPaths;
MeanMultiModesty = (1/TotalPaths)*sum(MultiModestyStat);
MeanUniModesty = (1/TotalPaths)*sum(UniModestyStat);
MeanUniModestyLZ = (1/NumDraws)*sum(UniModestyStatLZ);

%
% end of DSGEModestyStatistics.m
%
