function [JointPDH,MargPDH,LaplaceMargLike,PredVars,MargPlugin,JointPlugin,MargBackNowcast,status,kalmanstatus] = DSGEPredictiveLikelihoodTheta(theta,thetaPositions,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds,ModelParameters,YData,X,BackYData,IsOriginal,IsPlugin,FirstPeriod,LastPeriod,StepLength,DSGEModel,CurrINI)
% DSGEPredictiveLikelihoodTheta: Computes the joint and marginal predictive likelihood for selected subset of variables
%                                for fixed parameters values using the Laplace approximation of the predictive likelihood.
%
% USAGE:
%
%       [JointPDH,MargPDH,LaplaceMargLike,PredVars,MargPlugin,JointPlugin,MargBackNowcast,status,kalmanstatus] = ...
%                                         DSGEPredictiveLikelihoodTheta(theta,thetaPositions,thetaIndex,thetaDist,PriorDist, ...
%                                                     LowerBound,UniformBounds,ModelParameters,YData,X,BackYData,IsOriginal, ...
%                                                     IsPlugin,FirstPeriod,LastPeriod,StepLength,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                                 0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of
%                                 transformation that may need to be performed for the theta parameters.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                                 truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                                 and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                                 "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices
%                                 with the parameter values for the distributions. In the case of the normal, beta, gamma,
%                                 logistic and gumbel these parameter values are the mean and the standard deviation.
%                                 For the invgamma it is the s and q parameters, where q is a positive integer (degrees
%                                 of freedom). For the left truncated normal there's a third parameter, the lower bound.
%                                 For the uniform the parameter values are the lower and the upper bound. For the student-t
%                                 they are the location (mode), the scale and the degrees of freedom parameter, while for
%                                 the Cauchy only the location and scale parameters are needed. For the Pareto the
%                                 parameter values are for the mode and the shape parameter.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                                 for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                                 sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  BackYData (matrix) of dimension nx2 with the realization for the observed variables in the last
%                                 two historical time periods. Used for back and nowcasts.
%
%                  IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                                 annualized values should be ptredicted.
%
%                  IsPlugin (boolean) that takes the value 1 if the plugin estimate only should be calculated
%                                 and 0 if both the plugin and the Laplace approximation should be dealt with.
%
%                  FirstPeriod (integer) with the first period of the historical sample.
%
%                  LastPeriod (integer) with the last period of the historical sample.
%
%                  StepLength (scalar) with the step length used when computing minus the Hessian with
%                                 finite differences.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
% REQUIRED OUTPUT: JointPDH (vector) of dimension h with the height of the joint predictive likelihood for the
%                                 h forecast periods using Laplace approximations.
%
%                  MargPDH (vector) of dimension h with the height of the marginal predictive likelihood for the
%                                 h forecast periods using Laplace approximations.
%
%                  LaplaceMargLike (scalar) with the value of the Laplace approximation of the log marginal
%                                 likelihood for the historical sample.
%
%                  PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                                 variables for each prediction period.
%
%                  MargPlugin (matrix) of dimension h with the value of the marginal conditional likelihood
%                                 for the h forecast periods (a plugin estimate).
%
%                  JointPlugin (matrix) of dimension h with the value of the joint conditional likelihood
%                                 for the h forecast periods (a plugin estimate).
%
%                  MargBackNowcast (vector) of dimension 1x2 with the marginalized conditional likelihood
%                                 for the backcast and the nowcast of the predicted variables.
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mocde = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%                  kalmanstatus (boolean) indicating the covariance stationarity of the state variables. If all the
%                                 eigenvalues of F are inside the unit circle, then kalmanstatus=0 and the log-likelihood function
%                                 is calculated. If not, then status=1 and initP is set to 0. In addition, kalmanstatus can be
%                                 -1 if ths forecast covariance matrix is not positive definite for some time period.
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 22, 2012.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-03-2012: Finished the part of the function dealing with forecasting the original data.
%
% * 27-03-2012: The function is rewritten to take 2 different Laplace approximations into account.
%
% * 28-03-2012: Continued working on the function.
%
% * 04-04-2012: The function name has changed from "DSGEPredDensityHeightTheta" to "DSGEPredictiveLikelihoodTheta".
%
% * 26-04-2012: Updated the documentation.
%
% * 07-05-2012: Began work on annualized predictive likelihood computations.
%
% * 08-05-2012: Continued work on annualized predictive likelihood computations.
%
% * 10-05-2012: Will try to finish the annualized predictive likelihood code.
%
% * 11-05-2012: Finished the annualized predictive likelihood code.
%
% * 14-05-2012: Added the "MargPlugin" and "JointPlugin" output variables.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 29-08-2012: Fixed a bug with the calculation of the plugin estimates of the log predictive
%               likelihood. Added the "IsPlugin" input variable which controls if the Laplace
%               approximation should be computed in addition to the plugin estimate.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documnetation.
%
% * 20-02-2017: Added the "MargBackNowcast" output variable.
%
% * 09-03-2017: Made sure that "MargBackNowcast" is properly initialized.
%
% * 14-03-2017: Took into account that "logMargPredLikeDSGE" requires the "BackYData" input variable.
%
% * 14-06-2017: The function now supports the system prior.
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
% * 30-08-2017: Took into account that "logMargPredLikeDSGE", "logMargPredLikeDSGEAnnual", "logPredLikeDSGE",
%               and "logPosteriorPhiDSGE" require the input variable "NumWorkers".
%
% * 16-05-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-12-2019: Took into account that the DSGEModel.ModelSolverFile now requires the three additional
%               input variables "DSGEModel.StateVariableNames", "DSGEModel.StateEquationNames" and 
%               "DSGEModel.StateShockNames".
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed additional eval calls to feval.
%
% * 06-01-2023: Updated the documentation.
%
% * 09-01-2024: Took into account that "YADAHessianMatrix" now takes the input variable "NumWorkers".
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors for "RunKalmanSmoother".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
JointPDH = [];
MargPDH = [];
LaplaceMargLike = [];
PredVars = [];
MargPlugin = [];
JointPlugin = [];
MargBackNowcast = [];
if nargout==7;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   status = 0;
   kalmanstatus = 0;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>7;
      status = 0;
      kalmanstatus = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% See if we can solve the model
%
if nargout==7;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch ME;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError(ME)];
         end;
         kalmanstatus = 0;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   else;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError];
         end;
         kalmanstatus = 0;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   end;
end;
if nargout==9;
   status = mcode;
   if status~=1;
      kalmanstatus = 0;
      return;
   end;
else;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      elseif DSGEModel.ModelSolver==4;
         SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% obtain the measurement equation matrices
%
if MatlabNumber>=8.0;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch ME;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
else;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
h = size(YData,2);
n = DSGEModel.n;
k = DSGEModel.k;
%
% Get the sub-sample
%
if length(size(H))==3;
   if size(H,3)<LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations and forecasts (T+h = ' int2str(LastPeriod+h) '). YADA has therefore aborted from computing the predictive likelihood.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% check the system prior
%
if FileExist(DSGEModel.SystemPriorFile)==1;
   if MatlabNumber>=8.0;
      try;
         logSP = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);
      catch ME;
         %
         % display an error message
         %
         ErrorStr = ['Error found in "' GetName(DSGEModel.SystemPriorFile) '.m". Message caught is: ' YADALastError(ME)];
         About(ErrorStr,'error','Error - System Prior Density',500,600,CurrINI);
         return;
      end;
   else;
      try;
         logSP = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);
      catch;
         %
         % display an error message
         %
         ErrorStr = ['Error found in "' GetName(DSGEModel.SystemPriorFile) '.m". Message caught is: ' YADALastError];
         About(ErrorStr,'error','Error - System Prior Density',500,600,CurrINI);
         return;
      end;
   end;
   SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
   if FileExist(SystemPriorModeFile)==1;
      SystemPriorModeData = load(SystemPriorModeFile);
      %
      % we only need the Laplace approximation of the log marginal likelihood of the system prior
      %
      LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
   else;
      txt = 'YADA needs to compute the system prior mode before it can compute the predictive likelihood.';
      About(txt,'information','Predictive Likelihood',500,600,CurrINI);
      return;
   end;
else;
   logSP = 0;
   LogMargLikeSystemLaplace = 0;
end;
%
% We can now run the Kalman filter
%
[logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
%
% we should adjust everything based on DSGEModel.KalmanFirstObservation
%
if nargout==9;
   kalmanstatus = stat;
   if kalmanstatus~=0;
      return;
   end;
end;
T = size(Ksitt1,2);
%
% For back and nowcasting: determine the smoothed estimates of ksi and P at T-1 and T
%
if rank(R)==0;
   %
   % we only backcast and nowcast when the model doesn't have any measurement errors (or they are contained among
   % the state shocks)
   %
   [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,LastPeriod-1,LastPeriod,Yhat(:,T-1:T),Ksitt1(:,T-1:T),Ptt1(:,:,T-1:T),H,F,R,B0,Yerror(:,T-1:T),SigmaSqRoot(:,:,T-1:T),InvMSEY(:,:,T-1:T),KalmanGain(:,:,T-1:T),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   %
   % compute the plugin estimate of the predictive likelihood for the back- and nowcast
   %
   if DSGEModel.k>0;
      BackXData = DSGEModel.X(:,LastPeriod-1:LastPeriod);
   else;
      BackXData = [];
   end;
   %
   MargBackNowcast = DSGEBackNowcast(BackYData,BackXData,KsitT,PtT,A,H,DSGEModel.PredictionVariables);
end;
%
% compute the height of the log prior
%
phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
%
LogPrior = logPriorPhiDSGE(phi,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds);
LogPostDensity = logLikeValue-LogPrior+logSP;
NumParam = length(theta);
%
% and now minus the inverse Hessian at the mode with finite diffs
%
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
VechHessian = YADAHessianMatrix('logPosteriorPhiDSGE',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
%
% and finallly the Laplace approximation of the log marginal likelihood
%
LaplaceMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
logdetHessian = LogDet(HessianMat);
%
% initialize the remaining output
%
MargPDH = zeros(h,1);
JointPDH = MargPDH;
log2pi = log(2*pi);
if IsOriginal==1;
   %
   % compute the marginal and joint predictive posterior
   %
   [MargPDH,lnL,PredVars] = logMargPredLikeDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,h,0,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
   JointPDH = -logPredLikeDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,h,0,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers)-LogMargLikeSystemLaplace;
   MargPDH = -MargPDH-LogMargLikeSystemLaplace;
   %
   MargPlugin = MargPDH;
   JointPlugin = JointPDH;
   if IsPlugin==0;
      %
      % compute Hessian matrices
      %
      VechHessianMargPred = YADAHessianMatrix('logMargPredLikeDSGE',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,h,1,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
      drawnow;
      VechHessianJointPred = YADAHessianMatrix('logPredLikeDSGE',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,h,1,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
      drawnow;
      %
      % compute the Laplace approximations
      %
      for i=1:h;
         %
         % marginal predictive likelihoods
         %
         HessMargth = GeneralizedCholeski(reshape(VechHessianMargPred(i,:),NumParam,NumParam));
         MargPDH(i) = MargPDH(i)+((1/2)*(logdetHessian-LogDet(HessMargth)));
         %
         % joint predictive likelihoods
         %
         HessJointth = GeneralizedCholeski(reshape(VechHessianJointPred(i,:),NumParam,NumParam));
         JointPDH(i) = JointPDH(i)+((1/2)*(logdetHessian-LogDet(HessJointth)));
      end;
   end;
else;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
      DSGEModel.Frequency = 4;
   else;
      Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
      DSGEModel.Frequency = 12;
   end;
   DSGEModel.Yinit = Yinit;
   %
   % make sure that we take NaN's over the historical sample into account appropriately
   %
   if DSGEModel.YNaNs==0;
      DSGEModel.YAnn = DSGEModel.Y;
   else;
      Y = DSGEModel.Y;
      %
      % make sure that Y has NaN's that conform to the pattern implied by annualization
      %
      NaNMatrix = isnan(Y);
      NaNMatrix = Annualize([zeros(DSGEModel.n,DSGEModel.Frequency-1) NaNMatrix],DSGEModel.annual,ones(n,1));
      DSGEModel.YAnn(logical(NaNMatrix)) = NaN;
   end;
   YDataAnnual = Annualize([Yinit YData],DSGEModel.annual,ones(n,1));
   [MargPDH,lnL,PredVars] = logMargPredLikeDSGEAnnual(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YDataAnnual,X,h,0,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
   MargPDH = -MargPDH-LogMargLikeSystemLaplace;
   YData(isnan(YDataAnnual)) = NaN;
   DSGEModel.Y = DSGEModel.YAnn;
   JointPDH = -logPredLikeDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,h,0,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers)-LogMargLikeSystemLaplace;
   %
   MargPlugin = MargPDH;
   JointPlugin = JointPDH;
   if IsPlugin==0;
      %
      % compute Hessian matrices
      %
      VechHessianMargPred = YADAHessianMatrix('logMargPredLikeDSGEAnnual',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YDataAnnual,X,h,1,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
      drawnow;
      VechHessianJointPred = YADAHessianMatrix('logPredLikeDSGE',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,h,1,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
      drawnow;
      %
      % compute the Laplace approximations
      %
      for i=1:h;
         %
         % marginal predictive likelihoods
         %
         HessMargth = GeneralizedCholeski(reshape(VechHessianMargPred(i,:),NumParam,NumParam));
         MargPDH(i) = MargPDH(i)+((1/2)*(logdetHessian-LogDet(HessMargth)));
         %
         % joint predictive likelihoods
         %
         HessJointth = GeneralizedCholeski(reshape(VechHessianJointPred(i,:),NumParam,NumParam));
         JointPDH(i) = JointPDH(i)+((1/2)*(logdetHessian-LogDet(HessJointth)));
      end;
   end;
end;

%
% end of DSGEPredictiveLikelihoodTheta.m
%
