function DoneCalc = DSGESimulationSmoother(theta,thetaPostSample,thetaPositions,ModelParameters,SelectedParameters,EstType,IsPosterior,TotalPaths,TotalDraws,CurrChain,DSGEModel,controls,CurrINI)
% DSGESimulationSmoother: Computes the distribution of the state variables, shocks and measurement errors conditional on
%                              the data for draws from the prior or posterior distribution. Antithetic variables for balancing
%                              the location and scale and for speeding up the simulations are used.
%
% USAGE:
%
%       DoneCalc = DSGESimulationSmoother(theta,thetaPostSample,thetaPositions,ModelParameters,SelectedParameters,...
%                                         EstType,IsPosterior,TotalPaths,TotalDraws,CurrChain,DSGEModel,controls,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with the posterior mode estimates or the initial values of the parameters.
%
%                  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or the
%                                  prior distribution of the original DSGE model parameters.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                                  of the model parameter in position i of theta. The field "distribution" has a string
%                                  vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  SelectedParameters (vector) of dimension NumParam which indicates which parameters can vary from
%                                 one draw to the next (1) and which parameters are fixed at posterior mode (0).
%
%                  EstType (string vector) that describes what type of estimation is the main estimation, i.e.,
%                                 "State Variable", "State Shock" or "Measurement Error".
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                 if draws from the prior are supplied.
%
%                  TotalPaths (integer) with the total number of paths to simulate per parameter value.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  controls (structure) with handles to all controls on the YADA main GUI.
%
%                  CurrINI (structure), with initialization information.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 15, 2010.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-10-2010: Continued work on the function.
%
% * 26-10-2010: Added even more stuff to the function. Tomorrow it should be finished.
%
% * 28-10-2010: Yesterday was spent on other issues. So today is the day when the code
%               is finished.
%
% * 29-10-2010: The code now properly takes "StopSimulation" into account.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ"
%               from the CurrINI structure.
%
% * 03-01-2011: The function now supports the square root filter.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 11-03-2011: Switched to the "RunKalmanFilter" function.
%
% * 15-03-2011: Switched to the "RunKalmanSmoother" function.
%
% * 16-03-2011: The function now supports missing observations, including simulated values of the
%               observed variables.
%
% * 25-10-2011: Took into account that "RunKalmanSmoother" now requires the input variable "DiffuseInit".
%
% * 06-01-2012: Updated the documentation.
%
% * 29-02-2012: Took into account that the number of paths per parameter value can be unity.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
% * 26-02-2024: Changed the way the smooth structural shocks are computed.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors for "RunKalmanSmoother".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
%
% Prepare variables for selecting the parameters that are updated through the
% posterior draws
%
ScenarioParameters = SelectedParameters .* (1:length(theta));
ScenarioParameters = ScenarioParameters(ScenarioParameters>0);
SelParamStr = strrep(num2str(SelectedParameters),' ','');
thetaMode = theta;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'state');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\state". The computation of ' lower(EstType) ' distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/state". The computation of ' lower(EstType) ' distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% set up Y and X
%
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   X = [];
end;
T = size(Y,2);
Tsum = T-DSGEModel.KalmanFirstObservation+1;
%
% solve the model for the posterior mode estimate
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(DSGEModel.LastPeriod) '). YADA has therefore aborted from the computation of the ' lower(EstType) ' distribution.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
if (strcmp(lower(EstType),'measurement error')==1)&(rank(R)==0);
   txt = ['The DSGE model "' DSGEModel.NameOfModel '" subject to the state variable constraints doesn''t have any measurement errors.'];
   About(txt,'information','Measurement Error Distribution',500,600,CurrINI);
   return;
end;
[B0,KeepShocks] = RemoveRedundantColumns(B0);
%
% make sure the solution of the model can be retrieved when the parameter draws lead to problems
%
Astar = A;
Hstar = H;
Rstar = R;
Fstar = F;
B0star = B0;
%
% some important parameters
%
n = DSGEModel.n;
k = DSGEModel.k;
[r,q] = size(B0);
Ksi0 = DSGEModel.InitialStateVector;
if rank(R)>0;
   ErrorIndicator = zeros(n,1);
   for i=1:n;
      if R(i,i)>0;
         ErrorIndicator(i) = 1;
      end;
   end;
else;
   ErrorIndicator = [];
end;
%
% run the Kalman filter
%
if MatlabNumber>=8.0;
   try;
      if length(size(H))==2;
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,X,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      else;
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,X,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      end;
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
else;
   try;
      if length(size(H))==2;
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,X,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      else;
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,X,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      end;
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
end;
%
% we should adjust everything based on DSGEModel.KalmanFirstObservation
%
if nargout>2;
   kalmanstatus = stat;
   if kalmanstatus~=0;
      return;
   end;
end;
%
% Run the state smoother
%
if MatlabNumber>=8.0;
   try;
      if length(size(H))==2;
         [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(Y,1,size(Y,2),Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      else;
         [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(Y,1,size(Y,2),Yhat,Ksitt1,Ptt1,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      end;
   catch ME;
      Ksitt = [];
      KsitT = [];
      NtMat = [];
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
else;
   try;
      if length(size(H))==2;
         [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(Y,1,size(Y,2),Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      else;
         [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(Y,1,size(Y,2),Yhat,Ksitt1,Ptt1,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      end;
   catch;
      Ksitt = [];
      KsitT = [];
      NtMat = [];
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
%
% determine which type of distribution we have for the parameters
%
if IsPosterior==1;
   DistStr = 'Posterior Distribution';
else;
   DistStr = 'Prior Distribution';
end;
%
% check if all shocks can be observed
%
if strcmp(lower(EstType),'state shock')==1;
   CondCMat = eye(q)-(B0'*NtMat(:,:,T)*B0);
   if max(max(abs(CondCMat)))<1.0e-010;
      txt = 'YADA has concluded that all state shocks can be observed from the full sample. There is therefore no reason to run the simulation smoother.';
      About(txt,'error',['State Shock Simulation Smoother - ' DistStr],200,500,CurrINI);
      return;
   end;
end;
%
% set up the date vector
%
DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
DateVector = DateVector(DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
%
% check how critical values from the chi-square should be computed
%
if exist('YADAchi2inv')==2;
   UseYADAVersion = 2;
else;
   if exist('chi2inv')==2;
      UseYADAVersion = 1;
   else;
      UseYADAVersion = 0;
   end;
end;
%
% initialize the random number generator
%
if DSGEModel.RandomNumberValue==1;
   randn('state',0);
   rand('state',0);
else;
   randn('state',sum(100*clock));
   rand('state',sum(100*clock));
end;
%
% begin the loop
%
abort = '';
MeanEstimationTime = 0;
SimSmooth = [];
%
% initialize index for saving
%
NumDraws = size(thetaPostSample,1);
SaveAfterDraws = 1;
NumFiles = ceil(NumDraws/SaveAfterDraws);
%
% determine how many paths to compute
%
NumPathsStr = get(controls.forecast.predpaths,'String');
TotalPaths = max(str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',','')),4);
NumPaths = TotalPaths/4;
%
% check if we should add a progress dialog
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(TotalPaths*NumDraws,'%0.0f') ' ' lower(EstType) ' draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = ['Simulation Smoother - ' DistStr];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the simulation smoother for the ' lower(EstType) 's using ' int2str(NumDraws) ' parameter draws from the ' lower(DistStr) ' with ' int2str(TotalPaths) ' simulated paths per parameter draw. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Simulation Smoother - ' DistStr],500,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% start the iterations
%
StopSimulation = 0;
if length(size(H))==2;
   for it=1:NumDraws;
      %
      % Solve the model for the current parameter vector
      %
      theta(ScenarioParameters) = thetaPostSample(it,ScenarioParameters)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status==1;
         B0 = B0(:,KeepShocks);
      else;
         %
         % use posterior mode estimate as a backup
         %
         A = Astar;
         H = Hstar;
         R = Rstar;
         F = Fstar;
         B0 = B0star;
         status = 1;
      end;
      %
      % compute the smooth estimates based on the parameters of the state-space model
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,X,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(Y,1,size(Y,2),Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      %
      % compute the economic shocks
      %
      etatT = B0'*rTMat;
      if rank(R)>0;
         [Rm,Lm] = SchurDecomposition(R);
         Rmeasure = Rm*sqrt(Lm);
         NumMeasure = size(Lm,1);
         if k>0;
            wtT = Y-(A'*X)-(H'*KsitT);
         else;
            wtT = Y-(H'*KsitT);
         end;
      else;
         Rmeasure = [];
         wSim = 0;
         NumMeasure = 0;
         wtT = [];
      end;
      if DSGEModel.KalmanFirstObservation>1;
         KsitT = KsitT(:,DSGEModel.KalmanFirstObservation:T);
         etatT = etatT(:,DSGEModel.KalmanFirstObservation:T);
         if isempty(wtT)==0;
            wtT = wtT(:,DSGEModel.KalmanFirstObservation:T);
         end;
      end;
      %
      % initial values for the mean and covariance matrix of the distribution of the state variables
      %
      if isempty(DSGEModel.UnitRootStates)==1;
         [P0,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      else;
         StationaryPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
         P0 = DSGEModel.StateCovConstValue*eye(r);
         F1 = F(StationaryPos,StationaryPos);
         Q1 = B0(StationaryPos,:)*B0(StationaryPos,:)';
         [Pstat,stat] = DoublingAlgorithmLyapunov(F1,Q1,DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
         P0(StationaryPos,StationaryPos) = Pstat;
      end;
      [PS,PL] = SchurDecomposition(P0);
      DF = Tsum*(q+NumMeasure);
      if k>0;
         Ax = A'*X;
      else;
         Ax = 0;
      end;
      %
      % start the simulation of data
      %
      for p=1:NumPaths;
         tic;
         NumIter = (4*p)+(TotalPaths*(it-1));
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopSimulation = 1;
               break;
            else;
               ProgressDLG([(NumIter/(NumDraws*TotalPaths)) MeanEstimationTime]);
            end;
         end;
         %
         % draw the economic shocks
         %
         etaSim = randn(q,T);
         %
         % draw the initial state
         %
         StateSim = zeros(r,T+1);
         StateSim(:,1) = Ksi0+(PS*sqrt(PL)*randn(size(PL,2),1));
         %
         % draw the measurement errors
         %
         if isempty(Rmeasure)==0;
            wSim = Rmeasure*randn(NumMeasure,T);
         end;
         %
         % generate the state matrix
         %
         for t=1:T;
            StateSim(:,t+1) = (F*StateSim(:,t))+(B0*etaSim(:,t));
         end;
         drawnow;
         %
         % remove the initial value of the state vector
         %
         StateSim = StateSim(:,2:T+1);
         %
         % generate the data
         %
         YSim = Ax+(H'*StateSim)+wSim;
         if DSGEModel.YNaNs==1;
            %
            % make sure that YSim has NaNs in the same place as Y
            %
            YSim(isnan(Y)) = NaN;
         end;
         %
         % take the training sample into account
         %
         if DSGEModel.KalmanFirstObservation>1;
            etaSim = etaSim(:,DSGEModel.KalmanFirstObservation:T);
            if isempty(Rmeasure)==0;
               wSim = wSim(:,DSGEModel.KalmanFirstObservation:T);
            end;
            StateSim = StateSim(:,DSGEModel.KalmanFirstObservation:T);
         end;
         %
         % compute the smooth estimates for the simulated data
         %
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(YSim,X,1,size(YSim,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
         [Ksitt,Ptt,SimKsitT,PtT,rttMat,SimrTMat,NtMat] = RunKalmanSmoother(YSim,1,size(YSim,2),Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
         %
         % compute shocks and measurement errors
         %
         SimetatT = B0'*SimrTMat;
         if isempty(wtT)==0;
            SimwtT = YSim-Ax-(H'*SimKsitT);
            SimSmooth(p,1).wtT = wtT+wSim-SimwtT(:,DSGEModel.KalmanFirstObservation:T);
            SimSmooth(p,2).wtT = wtT-wSim+SimwtT(:,DSGEModel.KalmanFirstObservation:T);
         end;
         %
         % collect the results
         %
         SimSmooth(p,1).KsitT = KsitT+StateSim-SimKsitT(:,DSGEModel.KalmanFirstObservation:T);
         SimSmooth(p,2).KsitT = KsitT-StateSim+SimKsitT(:,DSGEModel.KalmanFirstObservation:T);
         SimSmooth(p,1).etatT = etatT+etaSim-SimetatT(:,DSGEModel.KalmanFirstObservation:T);
         SimSmooth(p,2).etatT = etatT-etaSim+SimetatT(:,DSGEModel.KalmanFirstObservation:T);
         %
         % antithetics for balancing the scale
         %
         etavec = vec(etaSim);
         wvec = vec(wSim);
         c = (etavec'*etavec)+(wvec'*wvec);
         %
         % compute p and d
         %
         if UseYADAVersion==2;
            d = YADAchi2inv(1-YADAchi2cdf(c,DF),DF);
         elseif UseYADAVersion==1;
            d = chi2inv(1-chi2cdf(c,DF),DF);
         else;
            d = YADACheapGammaInv(1-YADACheapchi2cdf(c,DF),(DF/2))*2;
         end;
         AdjFactor = sqrt(d/c);
         %
         SimSmooth(p,3).KsitT = KsitT+(AdjFactor*(SimSmooth(p,1).KsitT-KsitT));
         SimSmooth(p,4).KsitT = KsitT+(AdjFactor*(SimSmooth(p,2).KsitT-KsitT));
         SimSmooth(p,3).etatT = etatT+(AdjFactor*(SimSmooth(p,1).etatT-etatT));
         SimSmooth(p,4).etatT = etatT+(AdjFactor*(SimSmooth(p,2).etatT-etatT));
         if isempty(wtT)==0;
            SimSmooth(p,3).wtT = wtT+(AdjFactor*(SimSmooth(p,1).wtT-wtT));
            SimSmooth(p,4).wtT = wtT+(AdjFactor*(SimSmooth(p,2).wtT-wtT));
         end;
         %
         MeanEstimationTime = (((NumIter-1)/NumIter)*MeanEstimationTime)+((1/NumIter)*toc);
         drawnow;
      end;
      %
      if StopSimulation==0;
         %
         % save results to disk
         %
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(it) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(it) '-' int2str(NumFiles) '.' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
         end;
         %
         save(file,'SimSmooth','DateVector','R','KeepShocks','ErrorIndicator');
         SimSmooth = [];
      else;
         break;
      end;
      drawnow;
   end;
else;
   for it=1:NumDraws;
      %
      % Solve the model for the current parameter vector
      %
      theta(ScenarioParameters) = thetaPostSample(it,ScenarioParameters)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status==1;
         B0 = B0(:,KeepShocks);
      else;
         %
         % use posterior mode estimate as a backup
         %
         A = Astar;
         H = Hstar;
         R = Rstar;
         F = Fstar;
         B0 = B0star;
         status = 1;
      end;
      %
      % compute the smooth estimates based on the parameters of the state-space model
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,X,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(Y,1,size(Y,2),Yhat,Ksitt1,Ptt1,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      %
      % compute the economic shocks
      %
      etatT = B0'*rTMat;
      if rank(R)>0;
         [Rm,Lm] = SchurDecomposition(R);
         Rmeasure = Rm*sqrt(Lm);
         NumMeasure = size(Lm,1);
         wtT = zeros(n,T);
         if k>0;
            for t=1:T;
               wtT(:,t) = Y(:,t)-(A'*X(:,t))-(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
            end;
         else;
            for t=1:T;
               wtT(:,t) = Y(:,t)-(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
            end;
         end;
      else;
         Rmeasure = [];
         wSim = zeros(n,T);
         NumMeasure = 0;
         wtT = [];
      end;
      if DSGEModel.KalmanFirstObservation>1;
         KsitT = KsitT(:,DSGEModel.KalmanFirstObservation:T);
         etatT = etatT(:,DSGEModel.KalmanFirstObservation:T);
         if isempty(wtT)==0;
            wtT = wtT(:,DSGEModel.KalmanFirstObservation:T);
         end;
      end;
      %
      % initial values for the mean and covariance matrix of the distribution of the state variables
      %
      if isempty(DSGEModel.UnitRootStates)==1;
         [P0,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      else;
         StationaryPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
         P0 = DSGEModel.StateCovConstValue*eye(r);
         F1 = F(StationaryPos,StationaryPos);
         Q1 = B0(StationaryPos,:)*B0(StationaryPos,:)';
         [Pstat,stat] = DoublingAlgorithmLyapunov(F1,Q1,DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
         P0(StationaryPos,StationaryPos) = Pstat;
      end;
      [PS,PL] = SchurDecomposition(P0);
      DF = Tsum*(q+NumMeasure);
      if k>0;
         Ax = A'*X;
      else;
         Ax = zeros(n,T);
      end;
      %
      % start the simulation of data
      %
      for p=1:NumPaths;
         tic;
         NumIter = (4*p)+(TotalPaths*(it-1));
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopSimulation = 1;
               break;
            else;
               ProgressDLG([(NumIter/(NumDraws*TotalPaths)) MeanEstimationTime]);
            end;
         end;
         %
         % draw the economic shocks
         %
         etaSim = randn(q,T);
         %
         % draw the initial state
         %
         StateSim = zeros(r,T+1);
         StateSim(:,1) = Ksi0+(PS*sqrt(PL)*randn(size(PL,2),1));
         %
         % draw the measurement errors
         %
         if isempty(Rmeasure)==0;
            wSim = Rmeasure*randn(NumMeasure,T);
         end;
         %
         % generate the state matrix
         %
         for t=1:T;
            StateSim(:,t+1) = (F*StateSim(:,t))+(B0*etaSim(:,t));
         end;
         drawnow;
         %
         % remove the initial value of the state vector
         %
         StateSim = StateSim(:,2:T+1);
         %
         % generate the data
         %
         YSim = zeros(n,T);
         for t=1:T;
            YSim(:,t) = Ax(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*StateSim(:,t))+wSim(:,t);
         end;
         %
         if DSGEModel.YNaNs==1;
            %
            % make sure that YSim has NaNs in the same place as Y
            %
            YSim(isnan(Y)) = NaN;
         end;
         %
         % take the training sample into account
         %
         if DSGEModel.KalmanFirstObservation>1;
            etaSim = etaSim(:,DSGEModel.KalmanFirstObservation:T);
            if isempty(Rmeasure)==0;
               wSim = wSim(:,DSGEModel.KalmanFirstObservation:T);
            end;
            StateSim = StateSim(:,DSGEModel.KalmanFirstObservation:T);
         end;
         %
         % compute the smooth estimates for the simulated data
         %
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(YSim,X,1,size(YSim,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
         [Ksitt,Ptt,SimKsitT,PtT,rttMat,SimrTMat,NtMat] = RunKalmanSmoother(YSim,1,size(YSim,2),Yhat,Ksitt1,Ptt1,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
         %
         % compute shocks and measurement errors
         %
         SimetatT = B0'*SimrTMat;
         if isempty(wtT)==0;
            SimwtT = zeros(n,T);
            for t=1:T;
               SimwtT(:,t) = YSim(:,t)-Ax(:,t)-(H(:,:,DSGEModel.FirstPeriod+t-1)'*SimKsitT(:,t));
            end;
            SimSmooth(p,1).wtT = wtT+wSim-SimwtT(:,DSGEModel.KalmanFirstObservation:T);
            SimSmooth(p,2).wtT = wtT-wSim+SimwtT(:,DSGEModel.KalmanFirstObservation:T);
         end;
         %
         % collect the results
         %
         SimSmooth(p,1).KsitT = KsitT+StateSim-SimKsitT(:,DSGEModel.KalmanFirstObservation:T);
         SimSmooth(p,2).KsitT = KsitT-StateSim+SimKsitT(:,DSGEModel.KalmanFirstObservation:T);
         SimSmooth(p,1).etatT = etatT+etaSim-SimetatT(:,DSGEModel.KalmanFirstObservation:T);
         SimSmooth(p,2).etatT = etatT-etaSim+SimetatT(:,DSGEModel.KalmanFirstObservation:T);
         %
         % antithetics for balancing the scale
         %
         etavec = vec(etaSim);
         wvec = vec(wSim);
         c = (etavec'*etavec)+(wvec'*wvec);
         %
         % compute p and d
         %
         if UseYADAVersion==2;
            d = YADAchi2inv(1-YADAchi2cdf(c,DF),DF);
         elseif UseYADAVersion==1;
            d = chi2inv(1-chi2cdf(c,DF),DF);
         else;
            d = YADACheapGammaInv(1-YADACheapchi2cdf(c,DF),(DF/2))*2;
         end;
         AdjFactor = sqrt(d/c);
         %
         SimSmooth(p,3).KsitT = KsitT+(AdjFactor*(SimSmooth(p,1).KsitT-KsitT));
         SimSmooth(p,4).KsitT = KsitT+(AdjFactor*(SimSmooth(p,2).KsitT-KsitT));
         SimSmooth(p,3).etatT = etatT+(AdjFactor*(SimSmooth(p,1).etatT-etatT));
         SimSmooth(p,4).etatT = etatT+(AdjFactor*(SimSmooth(p,2).etatT-etatT));
         if isempty(wtT)==0;
            SimSmooth(p,3).wtT = wtT+(AdjFactor*(SimSmooth(p,1).wtT-wtT));
            SimSmooth(p,4).wtT = wtT+(AdjFactor*(SimSmooth(p,2).wtT-wtT));
         end;
         %
         MeanEstimationTime = (((NumIter-1)/NumIter)*MeanEstimationTime)+((1/NumIter)*toc);
         drawnow;
      end;
      %
      if StopSimulation==0;
         %
         % save results to disk
         %
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(it) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-SimSmooth-Prior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(it) '-' int2str(NumFiles) '.' int2str(TotalDraws) '.' int2str(TotalPaths) '.mat']);
         end;
         %
         save(file,'SimSmooth','DateVector','R','KeepShocks','ErrorIndicator');
         SimSmooth = [];
      else;
         break;
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we have computed all output
%
if StopSimulation==0;
   DoneCalc = 1;
end;

%
% end of DSGESimulationSmoother.m
%
