function DoneCalc = DSGEStationaryPredictiveVariance(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,h,TotalDraws,CurrChain,IsPosterior,Weights)
% DSGEStationaryPredictiveVariance: Computes the h-step-ahead stationary predictive variances for prior or posterior draws and
%                                   decomposes them into the shares due to the initial state, the shocls, the measurement
%                                   errors and parameter uncertainty.
%
% USAGE:
%
%       DoneCalc = DSGEStationaryPredictiveVariance(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,h,...
%                                                   TotalDraws,CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                                  distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) with the posterior mode or initial value of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  h (integer) with the prediction horizon.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 22, 2019.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-03-2019: Finished the function.
%
% * 28-03-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
DoneCalc = 0;
[NumDraws NumParam] = size(thetaPostSample);
k = DSGEModel.k;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   else;
      SetLogText('Missing AIM data file. Calculations aborted.');
   end;
   return;
end;
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   else;
      SetLogText('Error when trying to solve the DSGE model. Calculations aborted.');
   end;
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   else;
      SetLogText(strvcat(['The ' SolverStr ' solver provided the return code:'],mcodeStr,'Calculations aborted.'));
   end;
   return;
end;
SelPos = [];
if length(size(H))==3;
   %
   % check if H has the required size
   %
   [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
   if size(H,3)>=LastPeriod;
      %
      % pick a certain period for the H matrix
      %
      YADAHandle = findobj('Type','figure','Tag','YADA');
      controls = get(YADAHandle,'UserData');
      SubSampleMat = get(controls.dsge.initializekalman,'String');
      SubSampleMat = SubSampleMat(DSGEModel.KalmanFirstObservation:size(SubSampleMat,1),:);
      [action,SelectedPosition] = SelectionDlg(SubSampleMat,size(SubSampleMat,1),'Select time period for the measurement matrix H','Measurement Matrix',['Variance Decompositions - ' TypeStr ' Data'],'','',CurrINI);
      %
      SelPos = FirstPeriod+DSGEModel.KalmanFirstObservation-1+(SelectedPosition-1);
      if strcmp(lower(action),'cancel')==1;
         return;
      end;
      H = H(:,:,SelPos);
   else;
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(LastPeriod) '). YADA has therefore aborted from computing the variance decompositions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% store the posterior mode or initial values matrices
%
Astar = A;
Hstar = H;
Rstar = R;
Fstar = F;
B0star = B0;
%
% compute stationary distribution at mode
%
Q = B0*B0';
P0 = Q-(Q*(H/((H'*Q*H)+R))*H'*Q);
%
if max(abs(eig(F*P0*F')))>1.0e-10;
   [P_init,stat] = DoublingAlgorithmLyapunov(F,Q,DAMaxIter*100,10^(-(DAConvValue+1)));
   [P1star,status,NumIter,TestValue] = RiccatiSolver(F,H,Q,R,P_init,RicMaxIter*1000,10^(-(RicConvValue+1)));
   if status==0;
      P0star = P1star-(P1star*(H/((H'*P1star*H)+R))*H'*P1star);
   else;
      P0star = P0;
   end;
else;
   P0star = P0;
end;
%
%
[r,q] = size(B0);
n = DSGEModel.n;
k = DSGEModel.k;
%
% intialize certain moments
%
StateCov = zeros(n,n,h);
ShockCov = StateCov;
MeasureCov = zeros(n,n);
ParameterCov = StateCov;
TotalCov = StateCov;
MeanPredictY = zeros(n,h);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumDraws) ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Draw Number:';
   %
   ProgressStructure.name = ['Stationary Predictive Variance' PriorHeader ' - ' int2str(h) ' Periods Ahead'];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes stationary predictive variances for the original data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Stationary Predictive Variance ' PriorHeader ' - ' int2str(h) ' Periods Ahead'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
ParamIndex = 1;
if length(size(H))==2;
   for ParamIndex=1:NumDraws;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([ParamIndex/NumDraws ParamIndex]);
         end;
      end;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use posterior mode or initial values as a backup
         %
         A = Astar;
         H = Hstar;
         R = Rstar;
         F = Fstar;
         B0 = B0star;
         status = 1;
      end;
      %
      % check if Q is a solution for P0
      %
      status = 0;
      Q = B0*B0';
      P0 = Q-(Q*(H/((H'*Q*H)+R))*H'*Q);
      %
      if max(abs(eig(F*P0*F')))>1.0e-10;
         [P_init,stat] = DoublingAlgorithmLyapunov(F,Q,DAMaxIter*100,10^(-(DAConvValue+1)));
         [P1,status,NumIter,TestValue] = RiccatiSolver(F,H,Q,R,P_init,RicMaxIter*1000,10^(-(RicConvValue+1)));
         RiccatiResults = [NumIter TestValue];
         if status==0;
            P0 = P1-(P1*(H/((H'*P1*H)+R))*H'*P1);
         end;
      end;
      if status~=0;
         P0 = P0star;
      end;
      P0 = diag(abs(diag(P0)))+tril(P0,-1)+tril(P0,-1)';
      %
      % we can now compute the stationary prediction covariance matrices
      %
      MeasureCov = MeasureCov+((1/NumDraws)*R*Weights(ParamIndex));
      PredictY = zeros(n,h);
      Fi = eye(r);
      FjSum = zeros(r,r);
      Q = B0*B0';
      for i=1:h;
         FjSum = FjSum+(Fi*Q*Fi');
         ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*FjSum*H);
         %
         % update Fi
         %
         Fi = F*Fi;
         StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fi*P0*Fi'*H);
         if k>0;
            PredictY(:,i) = (A'*X(:,i));
         end;
         ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
      end;
      MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
      drawnow;
   end;
else;
   for ParamIndex=1:NumDraws;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([ParamIndex/NumDraws ParamIndex]);
         end;
      end;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use posterior mode or initial values as a backup
         %
         A = Astar;
         H = Hstar;
         R = Rstar;
         F = Fstar;
         B0 = B0star;
         status = 1;
      else;
         H = H(:,:,SelPos);
      end;
      %
      % check if Q is a solution for P0
      %
      status = 0;
      Q = B0*B0';
      P0 = Q-(Q*(H/((H'*Q*H)+R))*H'*Q);
      %
      if max(abs(eig(F*P0*F')))>1.0e-10;
         [P_init,stat] = DoublingAlgorithmLyapunov(F,Q,DAMaxIter*100,10^(-(DAConvValue+1)));
         [P1,status,NumIter,TestValue] = RiccatiSolver(F,H,Q,R,P_init,RicMaxIter*1000,10^(-(RicConvValue+1)));
         RiccatiResults = [NumIter TestValue];
         if status==0;
            P0 = P1-(P1*(H/((H'*P1*H)+R))*H'*P1);
         end;
      end;
      if status~=0;
         P0 = P0star;
      end;
      P0 = diag(abs(diag(P0)))+tril(P0,-1)+tril(P0,-1)';
      %
      % we can now compute the stationary prediction covariance matrices
      %
      MeasureCov = MeasureCov+((1/NumDraws)*R*Weights(ParamIndex));
      PredictY = zeros(n,h);
      Fi = eye(r);
      FjSum = zeros(r,r);
      Q = B0*B0';
      for i=1:h;
         FjSum = FjSum+(Fi*Q*Fi');
         ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*FjSum*H);
         %
         % update Fi
         %
         Fi = F*Fi;
         StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fi*P0*Fi'*H);
         if k>0;
            PredictY(:,i) = (A'*X(:,i));
         end;
         ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
      end;
      MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
      drawnow;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
for i=1:h;
   ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
   TotalCov(:,:,i) = StateCov(:,:,i)+ShockCov(:,:,i)+MeasureCov+ParameterCov(:,:,i);
end;
%
% save the output to file
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-StationaryPredictiveVariance-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-StationaryPredictiveVariance-Prior-' DSGEModel.NameOfModel '-' int2str(h) '.' int2str(NumDraws) '.mat']);
end;
save(file,'MeanPredictY','MeasureCov','ShockCov','StateCov','ParameterCov','TotalCov');
DoneCalc = 1;

%
% end of DSGEStationaryPredictiveVariance.m
%
