function [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(StateCov,ShockCov,MeasureCov,ParameterCov)
% SummarizeDSGEPredictions: Summarizes the information about the prediction uncertainty in the DSGE model.
%
% USAGE:
%
%       [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(StateCov,ShockCov,MeasureCov,ParameterCov)
%
% REQUIRED INPUT:  StateCov (n x n x h) matrix with the estimated state uncertainty.
%
%                  ShockCov (n x n x h) matrix with the estimated shock uncertainty.
%
%                  MeasureCov (n x n) matrix the the estimated measurement error uncertainty.
%
%                  ParameterCov (n x n x h) matrix with the estimated parameter uncertainty.
%
% REQUIRED OUTPUT: StateVar (matrix) of dimension nxh with the share of the overvall uncertainty for the n observed
%                           variables and the h predictions that is due to state uncertainty.
%
%                  ShockVar (matrix) of dimension nxh with the share of the overvall uncertainty for the n observed
%                           variables and the h predictions that is due to shock uncertainty.
%
%                  MeasureVar (matrix) of dimension nxh or nxnxh with the share of the overvall uncertainty for the n observed
%                           variables and the h predictions that is due to measurement errors.
%
%                  ParameterVar (matrix) of dimension nxh with the share of the overvall uncertainty for the n observed
%                           variables and the h predictions that is due to parameter uncertainty.
%
%                  TotalVar (matrix) of dimension nxh with the total variance of the n observed variables
%                           for the h predictions.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 23, 2007.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-03-2007: Checks to make sure that all relevant elements are non-negative.
%
% * 13-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 22-01-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 22-07-2010: Added the "TotalVar" output variable.
%
% * 05-01-2011: Updated the documentation.
%
% * 15-02-2011: Took into account that MeasureCov is 3D for conditional forecasts.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
n = size(StateCov,1);
h = size(StateCov,3);
StateVar = zeros(n,h);
ShockVar = zeros(n,h);
MeasureVar = zeros(n,h);
ParameterVar = zeros(n,h);
%
% keep just the variances
%
if length(size(MeasureCov))==2;
   for i=1:h;
      StateVar(:,i) = diag(StateCov(:,:,i));
      ShockVar(:,i) = diag(ShockCov(:,:,i));
      MeasureVar(:,i) = diag(MeasureCov);
      ParameterVar(:,i) = diag(ParameterCov(:,:,i));
   end;
else;
   for i=1:h;
      StateVar(:,i) = diag(StateCov(:,:,i));
      ShockVar(:,i) = diag(ShockCov(:,:,i));
      MeasureVar(:,i) = diag(MeasureCov(:,:,i));
      ParameterVar(:,i) = diag(ParameterCov(:,:,i));
   end;
end;
%
% compute the total
%
TotalVar = StateVar+ShockVar+MeasureVar+ParameterVar;
for i=1:h;
   for j=1:n;
      if TotalVar(j,i)<=1.0e-010;
         TotalVar(j,i) = 1;
         StateVar(j,i) = 0;
         ShockVar(j,i) = 0;
         MeasureVar(j,i) = 0;
         ParameterVar(j,i) = 0;
      end;
   end;
end;
%
% and now the shares
%
StateVar = 100*(StateVar ./ TotalVar);
ShockVar = 100*(ShockVar ./ TotalVar);
MeasureVar = 100*(MeasureVar ./ TotalVar);
ParameterVar = 100*(ParameterVar ./ TotalVar);

%
% end of SummarizeDSGEPredictions.m
%
