function [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ,status,kalmanstatus] = StateCondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,h,NumPaths,AnnualizeData,TransData,NameStr)
% StateCondPredictionPathsThetaWZ: Calculates a certain number of possible paths that the endogenous variables can take out-of-sample
%                                  conditional on the certain out-of-sample data for a given value of the parameters in the DSGE model.
%                                  Uses the Waggoner and Zha (1999) method.
%
% USAGE:
%
%       [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ, ...
%                              status,kalmanstatus] = StateCondPredictionPathsThetaWZ(theta,thetaPositions, ...
%                              ModelParameters,DSGEModel,CurrINI,X,Zeta,Z,U,h,NumPaths,AnnualizeData,TransData,NameStr)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                 the prediction paths.
%
%                  Zeta (matrix) of dimenion qzxg with the out-of-sample state variable data we shall
%                                 condition on.
%
%                  Z (matrix) of dimension mxg with the out-of-sample observed data we shall condition the
%                                 predictions on. May be an empty matrix.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                                 data and the conditioning data. May be an empty matrix.
%
%                  h (integer) with the the prediction horizon.
%
%                  NumPaths (integer)the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  NameStr (string vector) that is sent to the progress dialog name after the string
%                                 "Predictive Distribution".
%
% REQUIRED OUTPUT: PredPaths (n x h x NumPaths) 3D matrix with the NumPaths possible paths for the n
%                                 endogenous variables.
%
%                  PredData (structure) with fields:
%                                 PredMean             - (nxh) matrix with the mean predictions of the observed
%                                                        variables.
%                                 ShockScenarioResults - (nxhxq_used) matrix with shock scenarios for the mean
%                                                        predictions of the observed variables. These are
%                                                        obtained by manipulating the shocks used to
%                                                        compute the mean predictions. q_used<=q since some
%                                                        of the q shocks can be zero always by assumption.
%                                 OrigPredMean         - (nxh) matrix with the non-annualized mean predictions
%                                                        of the observed variables.
%                                 MeanPredPath         - (nxh) matrix with the non-annualized average of the
%                                                        predictions paths for the observed variables.
%                                 etaShocks            - (qxh) matrix with the values of the shocks at the mean
%                                                        predictions.
%                                 KsitT                - (rxT) matrix with the smooth estimates of the states
%                                                        over the estimation sample.
%                                 xiStates             - (rxh) matrix with the values of the states at the mean
%                                                        predictions.
%                                 X                    - (kxh) matrix with the deterministic variables for the
%                                                        prediction horizon.
%                                 etaMean              - (qxh) matrix with the mean values for the economic shocks over
%                                                        the forecast period.
%                                 wMean                - (nxh) matrix with the mean values for the measurement errors over
%                                                        the forecast period.
%                                 A                    - (kxn) matrix with the parameters on the exogenous variables in
%                                                        the measurement equation.
%                                 H                    - (rxn) matrix with parameters on the state variabels in the
%                                                        measurement equation.
%                                 R                    - (nxn) covariance matrix for the measurement errors.
%                                 F                    - (rxr) matrix with state transitions parameters for the state
%                                                        equation.
%                                 B0                   - (rxq) matrix with parameters on the economic shocks in the
%                                                        state equation.
%                                 ShockNames           - (qxqstr) string matrix with the names of the economic shocks.
%                                 Shocks               - (qxhxNumPaths) matrix with the economic shocks for the paths.
%                                 MeasError            - (nxhxNumPaths) matrix with the measurement errors for the paths.
%                                 HistState            - (rxNumPaths) matrix with the time T period draw of the state
%                                                        vector for the paths.
%                                 KernelX    - (3D matrix) of dimension n x 2^8 x h with the x-axis values
%                                              from kernel density estimation of the marginal predictive
%                                              densities.
%                                 KernelY    - (3D matrix) of dimension n x 2^8 x h with the y-axis values
%                                              from kernel density estimation of the marginal predictive
%                                              densities.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                 the PredPaths data satisfies the event, the second hold the number
%                                 of times the PredPaths data is greater than the event, the third
%                                 the number of time the data is above the upper bound, while the
%                                 fourth column holds the number of times the event has been tested.
%                                 The fifth and sixth column measure the average deviation from the
%                                 lower bound when below the lower bound and the square of this, while
%                                 the seventh and eighth columns hold similar data for the upper bound
%                                 of the event when the variables are above the upper bound.
%
%                  YObsEventData (n x 8) matrix where the first column hold the number of times
%                                 the PredPaths (adjusted such that the mean is equal to the realized
%                                 value) satisfies the event, the second holds the number of times
%                                 the adjusted paths data is greater than the event, the third
%                                 the number of time the data is above the upper bound, while the
%                                 fourth column holds the number of tiems the event has been tested.
%                                 The fifth and sixth column measure the average deviation from the
%                                 lower bound when below the lower bound and the square of this, while
%                                 the seventh and eighth columns hold similar data for the upper bound
%                                 of the event when the variables are above the upper bound.
%
%                  MultiModestyStat (matrix) with NumPaths x 2 elements. The first column is the the multivariate
%                                 modesty statistic based on the conditioning consistent shocks and the second
%                                 is the value of this statistic for randomly drawn shocks.
%
%                  UniModestyStat (matrix) with NumPaths x n elements. Each column gives the univariate modesty
%                                 for a particular observed variable.
%
%                  UniModestyStatLZ (vector) with the values of the univariate modesty statistics suggested
%                                 by Leeper-Zha (other shocks are zero over examined prediction horizon) for
%                                 all observed variables.
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%                  kalmanstatus (boolean) indicating the covariance stationarity of the state variables. If all the
%                                 eigenvalues of F are inside the unit circle, then kalmanstatus=0 and the log-likelihood function
%                                 is calculated. If not, then status=1 and initP is set to 0. In addition, kalmanstatus can be
%                                 -1 if ths forecast covariance matrix is not positive definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: July 16, 2009.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-07-2009: Updated the documentation.
%
% * 20-07-2009: Made sure that Astar is empty if there are no exogenous variables.
%
% * 27-08-2009: Added extra input and output to "StateSmoother" and "StateSmootherHt" function.
%
% * 04-12-2009: Made sure that smooth estimation of the states using the conditioning assumption only occurs
%               when the forecast horizon is not longer than the length of the historical sample.
%
% * 05-01-2010: Updated the documentation.
%
% * 18-11-2010: Took into account that "PredEventData" and "YObsEventData" now have 7 columns.
%
% * 19-11-2010: And now 8.
%
% * 17-12-2010: The function now supports the Klein and Sims DSGE model solvers.
%
% * 19-12-2010: Updated the documentation.
%
% * 02-01-2011: The function now supports the square root filter.
%
% * 10-01-2011: Changed the dos copy calls to "CopyFile" calls.
%
% * 25-01-2011: Took into account that "CalcTransformData" takes the input variable "TransMatrix".
%
% * 21-02-2011: Made sure that "SigmaTg" is positive semidefinite and a Choleski factor consistent
%               with this is taken.
%
% * 23-02-2011: Changed the computation of the covariance matrix for the Leeper-Zha statistic.
%
% * 01-03-2011: Changed the calculation of the square-root matrix of "SigmaTg".
%
% * 11-03-2011: Switched to the "RunKalmanFilter" function.
%
% * 15-03-2011: Switched to the "RunKalmanSmoother" function.
%
% * 24-03-2011: Streamlined some of the code.
%
% * 19-05-2011: Added test for calling and call to "CondPredictionSmoother" and "CondPredictionSmootherHt".
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 25-10-2011: Took into account that "RunKalmanSmoother" now requires the input variable "DiffuseInit".
%
% * 09-12-2011: Fixes for dealing with missing observations in historical sample.
%
% * 13-12-2011: Added matrix computations suggested by Ingvar Strid that speeds up the calculations.
%
% * 02-01-2012: Updated the documentation.
%
% * 07-02-2012: Added support for missing observations in the conditioning assumptions and for lag-varying
%               K2 matrices.
%
% * 09-03-2012: Made sure that missing observations don't affect the prediction event calculations.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 29-01-2014: Switched the "inv" function for "\".
%
% * 14-01-2015: Updated the documentation.
%
% * 09-06-2015: The code takes the field into account.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-12-2019: Took into account that the DSGEModel.ModelSolverFile now requires the three additional
%               input variables "DSGEModel.StateVariableNames", "DSGEModel.StateEquationNames" and 
%               "DSGEModel.StateShockNames".
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Chnaged additional eval calss to feval.
%
% * 22-02-2022: Changed the last eval calls to feval.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors for "RunKalmanSmoother".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
PredPaths = [];
PredData = [];
PredEventData = [];
YObsEventData = [];
MultiModestyStat = [];
UniModestyStat = [];
UniModestyStatLZ = [];
%
% set up the ModelParameters structure based on theta and thetaPositions
%
if nargout==7;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>7;
      status = 0;
      kalmanstatus = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% See if we can solve the model
%
if nargout==7;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch ME;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError(ME)];
         end;
         kalmanstatus = 0;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   else;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError];
         end;
         kalmanstatus = 0;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   end;
end;
if nargout>7;
   status = mcode;
   if status~=1;
      kalmanstatus = 0;
      return;
   end;
else;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      elseif DSGEModel.ModelSolver==4;
         SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
[B0,KeepShocks] = RemoveRedundantColumns(B0);
n = DSGEModel.n;
%
% check B0 to make sure that it is not empty
%
if isempty(KeepShocks)==1;
   txt = 'All economic shocks are redundant, i.e., the B0 matrix is a zero matrix. YADA is therefore unable to calculate any conditional forecasts.';
   About(txt,'information','YADA - Shock Selection',120,500,CurrINI);
   return;
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0;
%
% obtain the measurement equation matrices
%
if MatlabNumber>=8.0;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,n,size(F,1),DSGEModel.k);
   catch ME;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
else;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,n,size(F,1),DSGEModel.k);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the conditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% We can now run the Kalman filter
%
[logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
%
% we should adjust everything based on DSGEModel.KalmanFirstObservation
%
if nargout>7;
   kalmanstatus = stat;
   if kalmanstatus~=0;
      return;
   end;
end;
%
% get some parameter values
%
[r,T] = size(Ksitt1);
k = DSGEModel.k;
g = size(Zeta,2);
q = size(B0,2);
K3 = DSGEModel.K3(:,DSGEModel.ZetaConditionalVariables);
nq = size(K3,2);
%
% stack the assumptions, shocks, parameters, and mappings
%
if isempty(Z)==1;
   K1 = [zeros(n,nq);K3];
   K2 = zeros(n+r,nq);
   U = zeros(nq,g);
   K2length = 2;
else;
   K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
   K1 = [K1 zeros(n,nq);zeros(r,size(K1,2)) K3];
   if length(size(DSGEModel.K2))==2;
      K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
      K2 = [K2 zeros(n,nq);zeros(r,size(K2,2)+nq)];
      K2length = 2;
   else;
      K2Tmp = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
      K2length = 3;
      K2 = zeros(n+r,size(K2Tmp,2)+nq);
      for j=1:size(K2Tmp,3);
         K2(1:n,1:size(K2Tmp,2),j) = K2Tmp(:,:,j);
      end;
   end;
   Zeta = [Z;Zeta];
   U = [U;zeros(nq,g)];
end;
SCond = (isequal(U,zeros(size(U)))&isequal(K2,zeros(size(K2))));
%
% setup the alternative measurement equation matrices
%
if k>0;
   Astar = [A zeros(k,r)];
else;
   Astar = [];
end;
if length(size(H))==2;
   Hstar = [H eye(r)];
else;
   Hstar = zeros(r,n+r,size(H,3));
   for i=1:size(H,3);
      Hstar(:,:,i) = [H(:,:,i) eye(r)];
   end;
end;
Rstar = [R zeros(n,r); zeros(r,n+r)];
%
% run the Kalman smoother for g periods
%
[Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
PredData.KsitT = KsitT;
%
% calculate measurement errors for the sample T-g+1 until T
%
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (k>0);
      XPred = X(:,1:g);
   else;
      XPred = [];
   end;
   if length(size(H))==2;
      [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,T),PtT(:,:,T),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Zeta,K1,Astar,Hstar,Rstar,F,B0,DSGEModel.KalmanAlgorithm);
   else;
      [KsiTTz,PTTz] = CondPredictionSmootherHt(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,T),PtT(:,:,T),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Zeta,K1,Astar,Hstar(:,:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+g),Rstar,F,B0,DSGEModel.KalmanAlgorithm);
   end;
elseif (DSGEModel.KsiUseCondData==1)&(size(KsitT,2)>=g);
   PtT = PtT(:,:,size(PtT,3)-g+1:size(PtT,3));
   KsitT = KsitT(:,size(KsitT,2)-g+1:size(KsitT,2));
   if length(size(H))==2;
      if rank(R)>0;
         if k>0;
            wtT = DSGEModel.Y(:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod)-(A'*DSGEModel.X(:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod))-(H'*KsitT);
         else;
            wtT = DSGEModel.Y(:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod)-(H'*KsitT);
         end;
      else;
         wtT = zeros(DSGEModel.n,g);
      end;
   else;
      wtT = zeros(DSGEModel.n,g);
      if rank(R)>0;
         if k>0;
            for i=1:g;
               wtT(:,i) = DSGEModel.Y(:,DSGEModel.LastPeriod-g+i)-(A'*DSGEModel.X(:,DSGEModel.LastPeriod-g+i))-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
            end;
         else;
            for i=1:g;
               wtT(:,i) = DSGEModel.Y(:,DSGEModel.LastPeriod-g+i)-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
            end;
         end;
      end;
   end;
   wtT = [wtT;zeros(r,g)];
   %
   % Smooth estimation of the states at time T using the conditioning assumptions.
   %
   if length(size(H))==2;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Zeta,U,[Xtra X(:,1:g)],K1,K2,Astar,Hstar,Rstar,F,B0);
      else;
         [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Zeta,U,[],K1,K2,Astar,Hstar,Rstar,F,B0);
      end;
   else;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Zeta,U,[Xtra X(:,1:g)],K1,K2,Astar,Hstar(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),Rstar,F,B0);
      else;
         [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Zeta,U,[],K1,K2,Astar,Hstar(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),Rstar,F,B0);
      end;
   end;
else;
   KsiTTz = KsitT(:,size(KsitT,2));
   PTTz = PtT(:,:,size(KsitT,2));
end;
PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
%
% Now we need to determine KsiTT for just the last period
%
KsiTT = KsitT(:,size(KsitT,2));
%
% use the Schur decomposition for the reduced rank matrix PTTz instead of
% the Choleski decomposition
%
[S,L] = SchurDecomposition(PTTz);
%
% set up the output
%
PredPaths = zeros(n,h,NumPaths);
LZPredPaths = zeros(n,g);
%
% check if we have measurement errors and if so compute a Schur decomposition of the R matrix
%
if rank(R)>0;
   [Rm,Lm] = SchurDecomposition(R);
   Rmeasure = Rm*sqrt(Lm);
   NumMeasure = size(Lm,1);
else;
   Rmeasure = [];
   wTh = zeros(n,h);
   NumMeasure = 0;
end;
%
%
% compute the state and shock covariances at the given parameters
% as well as the predictions of the observed variables.
%
if (DSGEModel.RunPredictionEvent==1)&(size(DSGEModel.Y,2)>=DSGEModel.LastPeriod+h);
   YObserved = DSGEModel.Y(:,DSGEModel.LastPeriod+1:DSGEModel.LastPeriod+h);
else;
   YObserved = [];
end;
if AnnualizeData==1;
   if DSGEModel.YNaNs==0;
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
      else;
         Yinit = [];
      end;
   else;
      YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
      nanpos = isnan(YSmooth);
      if max(max(nanpos))==1;
         if DSGEModel.k>0;
            if length(size(H))==2;
               YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*PredData.KsitT);
            else;
               YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
               for t=1:size(PredData.KsitT,2);
                  YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*PredData.KsitT(:,t));
               end;
            end;
         else;
            if length(size(H))==2;
               YHistPred = H'*PredData.KsitT;
            else;
               YHistPred = zeros(DSGEModel.n,size(PredData.KsitT,2));
               for t=1:size(PredData.KsitT,2);
                  YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*PredData.KsitT(:,t);
               end;
            end;
         end;
         YSmooth(nanpos) = YHistPred(nanpos);
      end;
      if DSGEModel.FirstPeriod>1;
         YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod-1) YSmooth];
      end;
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         Yinit = YSmooth(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         Yinit = YSmooth(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
      else;
         Yinit = [];
      end;
   end;
else;
   MultiModestyStat = zeros(NumPaths,2);
   UniModestyStat = zeros(NumPaths,n);
   UniModestyStatLZ = zeros(1,n);
   if TransData==1;
      if DSGEModel.YNaNs==0;
         Yinit = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
      else;
         YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
         nanpos = isnan(YSmooth);
         if max(max(nanpos))==1;
            if DSGEModel.k>0;
               if length(size(H))==2;
                  YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*PredData.KsitT);
               else;
                  YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
                  for t=1:size(KsitT,2);
                     YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*PredData.KsitT(:,t));
                  end;
               end;
            else;
               if length(size(H))==2;
                  YHistPred = H'*PredData.KsitT;
               else;
                  YHistPred = zeros(DSGEModel.n,size(PredData.KsitT,2));
                  for t=1:size(PredData.KsitT,2);
                     YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*PredData.KsitT(:,t);
                  end;
               end;
            end;
            YSmooth(nanpos) = YHistPred(nanpos);
         end;
         if DSGEModel.FirstPeriod>1;
            YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod-1) YSmooth];
         end;
         Yinit = YSmooth(:,1:DSGEModel.LastPeriod);
      end;
   end;
end;
%
% create the matrices for selecting shocks
%
m = size(Zeta,1);
nr = n+r;
%
% stack the model to determine the shock distribution
%
Xbar = zeros(nr*g,1);
G = zeros(nr*g,r);
WTg = zeros(nr*g,1);
K = zeros(nr*g,m*g);
D = zeros(nr*g,q*g);
Fexp = F;
ZTg = zeros(m*g,1);
UT = zeros(m*g,1);
if length(size(Hstar))==2;
   for i=1:g;
      Ind = (1+((g-i)*nr):(g-i+1)*nr);
      Ind1 = (1+((g-i)*q):(g-i+1)*q);
      Ind2 = (1+((g-i)*m):(g-i+1)*m);
      if k>0;
         Xbar(Ind,1) = Astar'*X(:,i);
      end;
      G(Ind,:) = Hstar'*Fexp;
      D(Ind,Ind1) = Hstar'*B0;
      K(Ind,Ind2) = K1;
      Fi = F;
      for j=1:i-1;
         Ind1 = Ind1+q;
         Ind3 = Ind+(j*nr);
         D(Ind,Ind1) = Hstar'*Fi*B0;
         if K2length==2;
            K(Ind3,Ind2) = K2;
         else;
            if j<=size(K2,3);
               K(Ind3,Ind2) = K2(:,:,j);
            end;
         end;
         Fi = F*Fi;
      end;
      ZTg(Ind2,1) = Zeta(:,i);
      UT(Ind2,1) = U(:,g);
      Fexp = F*Fexp;
   end;
else;
   for i=1:g;
      Ind = (1+((g-i)*nr):(g-i+1)*nr);
      Ind1 = (1+((g-i)*q):(g-i+1)*q);
      Ind2 = (1+((g-i)*m):(g-i+1)*m);
      if k>0;
         Xbar(Ind,1) = Astar'*X(:,i);
      end;
      G(Ind,:) = Hstar(:,:,DSGEModel.LastPeriod+i)'*Fexp;
      D(Ind,Ind1) = Hstar(:,:,DSGEModel.LastPeriod+i)'*B0;
      K(Ind,Ind2) = K1;
      Fi = F;
      for j=1:i-1;
         Ind1 = Ind1+q;
         Ind3 = Ind+(j*nr);
         D(Ind,Ind1) = Hstar(:,:,DSGEModel.LastPeriod+i)'*Fi*B0;
         if K2length==2;
            K(Ind3,Ind2) = K2;
         else;
            if j<=size(K2,3);
               K(Ind3,Ind2) = K2(:,:,j);
            end;
         end;
         Fi = F*Fi;
      end;
      ZTg(Ind2,1) = Zeta(:,i);
      UT(Ind2,1) = U(:,g);
      Fexp = F*Fexp;
   end;
end;
%
% create a vector with positions for the conditioning assumptions that should be used
%
PosVec = (1:m*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
%
% adjust for missing restrictions
%
K = K(:,PosVec);
ZTg = ZTg(PosVec,1);
UT = UT(PosVec,1);
%
% speed-up code suggested by Ingvar Strid
%
KpD = K'*D;
KpDDpK = KpD*KpD';
DpKInvKpDDpK = KpD'/KpDDpK;
%
% check the rank of K'D
%
if rank(KpD)<length(PosVec);
   txt = ['Some of the conditioning assumptions are linearly dependent. The rank of K''*D is ' int2str(rank(KpD)) ', while the number of conditioning assumptions is ' int2str(length(PosVec)) '. YADA is therefore unable to perform the conditional forecasts for the Waggoner-Zha approach.'];
   About(txt,'error','Error - Conditioning Assumptions',200,500,CurrINI);
   return;
end;
%
% compute the conditioning relation when state vector and measurement errors are at mean
%
kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
%
% compute the mean of the shocks when state vector and measurement errors are at mean
%
mubarTg = DpKInvKpDDpK*kbarTg;
%
% compute the idempotent covariance matrix for the shocks
%
SigmaTg = eye(q*g)-(DpKInvKpDDpK*KpD);
SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
[SE,LE] = schur(SigmaTg);
SigmaTgStd = SE*sqrt(abs(LE));
%
% add info to the PredData structure
%
PredData.KernelX = [];
PredData.KernelY = [];
PredData.X = X;
PredData.A = A;
if length(size(H))==2;
   PredData.H = H;
else;
   PredData.H = H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod+h);
end;
PredData.R = R;
PredData.B0 = B0;
PredData.F = F;
PredData.etaMean = zeros(q,h);
PredData.wMean = zeros(n,h);
PredData.ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
PredData.Shocks = zeros(q,h,NumPaths);
PredData.MeasError = zeros(DSGEModel.n,h,NumPaths);
PredData.HistState = zeros(r,NumPaths);
%
% compute the mean prediction
%
PredData.PredMean = zeros(n,h);
PredData.MeanPredPath = zeros(n,h);
KsiT = KsiTTz;
PredData.etaShocks = zeros(q,h);
PredData.etaShocks(:,1:g) = fliplr(reshape(mubarTg,q,g));
PredData.xiStates = zeros(r,h);
if length(size(Hstar))==2;
   for i=1:h;
      %
      % update the states
      %
      KsiT = (F*KsiT)+(B0*PredData.etaShocks(:,i));
      PredData.xiStates(:,i) = KsiT;
      %
      % compute the conditional mean predictions
      %
      if k>0;
         PredData.PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
      else;
         PredData.PredMean(:,i) = H'*KsiT;
      end;
   end;
else;
   for i=1:h;
      %
      % update the states
      %
      KsiT = (F*KsiT)+(B0*PredData.etaShocks(:,i));
      PredData.xiStates(:,i) = KsiT;
      %
      % compute the conditional mean predictions
      %
      if k>0;
         PredData.PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT);
      else;
         PredData.PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiT;
      end;
   end;
end;
%
PredData.ShockScenarioResults = zeros(n,h,q);
if length(size(H))==2;
   for j=1:q;
      %
      % set shock j of the shocks manipulated for the theoretical mean of the conditional
      % forecast
      %
      etaShocks = PredData.etaShocks;
      etaShocks(j,:) = zeros(1,h);
      %
      % inistalize the state
      %
      KsiT = KsiTTz;
      for i=1:h;
         KsiT = (F*KsiT)+(B0*etaShocks(:,i));
         if k>0;
            PredData.ShockScenarioResults(:,i,j) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredData.ShockScenarioResults(:,i,j) = H'*KsiT;
         end;
      end;
   end;
else;
   for j=1:q;
      %
      % set shock j of the shocks manipulated for the theoretical mean of the conditional
      % forecast
      %
      etaShocks = PredData.etaShocks;
      etaShocks(j,:) = zeros(1,h);
      %
      % inistalize the state
      %
      KsiT = KsiTTz;
      for i=1:h;
         KsiT = (F*KsiT)+(B0*etaShocks(:,i));
         if k>0;
            PredData.ShockScenarioResults(:,i,j) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT);
         else;
            PredData.ShockScenarioResults(:,i,j) = H(:,:,DSGEModel.LastPeriod+i)'*KsiT;
         end;
      end;
   end;
end;
%
% annualize the mean predictions when needed
%
PredData.OrigPredMean = PredData.PredMean;
if AnnualizeData==1;
   PredData.PredMean = Annualize([Yinit PredData.PredMean],DSGEModel.annual,DSGEModel.annualscale);
   if isempty(YObserved)==0;
      YObserved = Annualize([Yinit YObserved],DSGEModel.annual,DSGEModel.annualscale);;
   end;
elseif TransData==1;
   YTmp = [Yinit PredData.PredMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   PredData.PredMean = YData(Tt-h+1:Tt,:)';
   if isempty(YObserved)==0;
      YObserved = [Yinit YObserved];
      [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YObserved',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
      Tt = size(YData,1);
      YObserved = YData(Tt-h+1:Tt,:)';
   end;
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumPaths,'%0.0f') ' Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = ['Conditional Predictive Distribution' NameStr];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
end;
%
% start the iterations
%
Q = B0*B0';
abort = '';
if length(size(H))==2;
   CovMatShocks = Q*Hstar*K1*((K1'*Hstar'*Q*Hstar*K1)\K1')*Hstar'*Q;
   %
   % additional speed-up code suggested by Ingvar Strid
   %
   PTTg = zeros(r,r);
   Fg = eye(r);
   for i=1:g
      Fg = Fg*F;
      PTTg = (F*PTTg*F')+Q;
   end;
   %
   for it=1:NumPaths;
      %
      % check if we should update the progress dialog
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
            PredPaths = [];
            break;
         else;
            ProgressDLG([it/NumPaths it]);
         end;
      end;
      %
      % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
      %
      KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
      PredData.HistState(:,it) = KsiT;
      KsiCTmp = KsiTT;
      KsiTmp = KsiTT;
      PTTgLZ = zeros(r,r);
      %
      % Draw a sequence of measurement errors
      %
      if isempty(Rmeasure)==0;
         wTh = Rmeasure*randn(NumMeasure,h);
         PredData.MeasError(:,:,it) = wTh;
         PredData.wMean = PredData.wMean+((1/NumPaths)*wTh);
         WTg = vec(fliplr([wTh(:,1:g);zeros(r,g)]));
      end;
      %
      % setut the mean for the shock distribution
      %
      kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
      muTg = DpKInvKpDDpK*kTg;
      kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
      muCTmp = DpKInvKpDDpK*kCTmp;
      %
      % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
      % and from N(0,I) for period g+1 to h.
      %
      ShockDraw = randn(q*g,1);
      etaTg = muTg+(SigmaTgStd*ShockDraw);
      etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
      etaTh = randn(q,h);
      etaCTmp = etaTh;
      etaTmp = etaTh;
      %
      % overlay with restricted shocks
      %
      etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
      etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
      PredData.Shocks(:,:,it) = etaTh;
      %
      % compute the prediction paths
      %
      for i=1:h;
         %
         % update the state
         %
         KsiT = (F*KsiT)+(B0*etaTh(:,i));
         if k>0;
            PredPaths(:,i,it) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
         else;
            PredPaths(:,i,it) = (H'*KsiT)+wTh(:,i);
         end;
         %
         % update modesty related material
         %
         if i<=g;
            KsiCTmp = (F*KsiCTmp)+(B0*etaCTmp(:,i));
            KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
            if it==1;
               PTTgLZ = (F*PTTgLZ*F')+CovMatShocks;
            end;
         end;
      end;
      PredData.MeanPredPath = PredData.MeanPredPath+((1/NumPaths)*PredPaths(:,:,it));
      if AnnualizeData==1;
         PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
      else;
         if TransData==1;
            %
            % transform the data
            %
            YTmp = [Yinit PredPaths(:,:,it)];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
         end;
         %
         % deal with modesty statistic
         %
         if k>0;
            PredictCTmp = (A'*X(:,g))+(H'*KsiCTmp)+wTh(:,g);
            PredictY = (A'*X(:,g))+(H'*Fg*KsiTT);
         else;
            PredictCTmp = (H'*KsiCTmp)+wTh(:,g);
            PredictY = H'*Fg*KsiTT;
         end;
         PredCov = (H'*PTTg*H)+R;
         PredDiff = PredictCTmp-PredictY;
         MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
         UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
         %
         % comparison-path for multivariate statistic
         %
         if k>0;
            PredictTmp = (A'*X(:,g))+(H'*KsiTmp)+wTh(:,g);
         else;
            PredictTmp = (H'*KsiTmp)+wTh(:,g);
         end;
         PredDiff = PredictTmp-PredictY;
         MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
         %
         % Leeper-Zha type statistic
         %
         if it==1;
            PredDiff = PredData.PredMean(:,g)-PredictY;
            CovPredDiff = diag(H'*PTTgLZ*H);
            for i=1:length(CovPredDiff);
               if CovPredDiff(i)==0;
                  PredDiff(i) = 0;
                  CovPredDiff(i) = 1;
               end;
            end;
            UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(CovPredDiff))';
         end;
      end;
      drawnow;
   end;
else;
   %
   % additional speed-up code suggested by Ingvar Strid
   %
   PTTg = zeros(r,r);
   Fg = eye(r);
   for i=1:g
      Fg = Fg*F;
      PTTg = (F*PTTg*F')+Q;
   end;
   %
   for it=1:NumPaths;
      %
      % check if we should update the progress dialog
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
            PredPaths = [];
            break;
         else;
            ProgressDLG([it/NumPaths it]);
         end;
      end;
      %
      % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
      %
      KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
      PredData.HistState(:,it) = KsiT;
      KsiCTmp = KsiTT;
      KsiTmp = KsiTT;
      PTTgLZ = zeros(r,r);
      %
      % Draw a sequence of measurement errors
      %
      if isempty(Rmeasure)==0;
         wTh = Rmeasure*randn(NumMeasure,h);
         PredData.MeasError(:,:,it) = wTh;
         PredData.wMean = PredData.wMean+((1/NumPaths)*wTh);
         WTg = vec(fliplr([wTh(:,1:g);zeros(r,g)]));
      end;
      %
      % setut the mean for the shock distribution
      %
      kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
      muTg = DpKInvKpDDpK*kTg;
      kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
      muCTmp = DpKInvKpDDpK*kCTmp;
      %
      % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
      % and from N(0,I) for period g+1 to h.
      %
      ShockDraw = randn(q*g,1);
      etaTg = muTg+(SigmaTgStd*ShockDraw);
      etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
      etaTh = randn(q,h);
      etaCTmp = etaTh;
      etaTmp = etaTh;
      %
      % overlay with restricted shocks
      %
      etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
      etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
      PredData.Shocks(:,:,it) = etaTh;
      %
      % compute the prediction paths
      %
      for i=1:h;
         %
         % update the state
         %
         KsiT = (F*KsiT)+(B0*etaTh(:,i));
         if k>0;
            PredPaths(:,i,it) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
         else;
            PredPaths(:,i,it) = (H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
         end;
         %
         % update modesty related material
         %
         if i<=g;
            KsiCTmp = (F*KsiCTmp)+(B0*etaCTmp(:,i));
            KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
            if it==1;
               %muTi = PredData.etaShocks(:,i);
               PTTgLZ = (F*PTTgLZ*F')+(Q*Hstar(:,:,DSGEModel.LastPeriod+i)*K1*((K1'*Hstar(:,:,DSGEModel.LastPeriod+i)'*Q*Hstar(:,:,DSGEModel.LastPeriod+i)*K1)\K1')*Hstar(:,:,DSGEModel.LastPeriod+i)'*Q);
            end;
         end;
      end;
      PredData.MeanPredPath = PredData.MeanPredPath+((1/NumPaths)*PredPaths(:,:,it));
      if AnnualizeData==1;
         PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
      else;
         if TransData==1;
            %
            % transform the data
            %
            YTmp = [Yinit PredPaths(:,:,it)];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
         end;
         %
         % deal with modesty statistic
         %
         if k>0;
            PredictCTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
            PredictY = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*Fg*KsiTT);
         else;
            PredictCTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
            PredictY = H(:,:,DSGEModel.LastPeriod+g)'*Fg*KsiTT;
         end;
         PredCov = (H(:,:,DSGEModel.LastPeriod+g)'*PTTg*H(:,:,DSGEModel.LastPeriod+g))+R;
         PredDiff = PredictCTmp-PredictY;
         MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
         UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
         %
         % comparison-path for multivariate statistic
         %
         if k>0;
            PredictTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
         else;
            PredictTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
         end;
         PredDiff = PredictTmp-PredictY;
         MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
         %
         % Leeper-Zha type statistic
         %
         if it==1;
            PredDiff = PredData.PredMean(:,g)-PredictY;
            CovPredDiff = diag(H(:,:,DSGEModel.LastPeriod+g)'*PTTgLZ*H(:,:,DSGEModel.LastPeriod+g));
            for i=1:length(CovPredDiff);
               if CovPredDiff(i)==0;
                  PredDiff(i) = 0;
                  CovPredDiff(i) = 1;
               end;
            end;
            UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(CovPredDiff))';
         end;
      end;
      drawnow;
   end;
end;
%
% close the wait handle
%
if DSGEModel.ShowProgress==1;
   set(WaitHandle,'UserData','done');
   close(WaitHandle);
   drawnow;
   pause(0.02);
end;
if (strcmp(abort,'cancel')==0)&(DSGEModel.RunPredictionEvent==1);
   MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
   if DSGEModel.AdjustPredictionPaths==1;
      %
      % we need to fix the paths such that the mean is correct
      %
      AdjustMean = PredData.PredMean-MeanPrediction;
      for i=1:NumPaths;
         PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
   else;
      PEPredPaths = PredPaths;
   end;
   %
   % compute the prediction events results
   %
   PredEventData = CalculatePredictionEvents(PEPredPaths,DSGEModel.PredictionEvent);
   %
   if isempty(YObserved)==0;
      AdjustMean = YObserved-MeanPrediction;
      %
      % make sure that missing observations don't affect adjustment
      %
      AdjustMean(isnan(AdjustMean)) = 0;
      for i=1:NumPaths;
         PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
      YObsEventData = CalculatePredictionEvents(PEPredPaths,DSGEModel.PredictionEvent);
   end;
end;
%
% check if we should compute kernel density estimates
%
if (strcmp(abort,'cancel')==0)&(NumPaths>=500);
   if DSGEModel.RunPredictionEvent==0;
      if DSGEModel.AdjustPredictionPaths==1;
         MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
         AdjustMean = PredData.PredMean-MeanPrediction;
         for i=1:NumPaths;
            PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
         end;
      else;
         PEPredPaths = PredPaths;
      end;
   end;
   %
   [PredData.KernelX,PredData.KernelY] = PredictionKernelDensity(PEPredPaths);
end;

%
% end of StateCondPredictionPathsThetaWZ.m
%
 