function [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ,status,kalmanstatus] = DSGECondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,h,NumPaths,AnnualizeData,TransData,NameStr)
% DSGECondPredictionPathsThetaWZ: Calculates a certain number of possible paths that the endogenous variables can take out-of-sample
%                                 conditional on the certain out-of-sample data for a given value of the parameters in the DSGE model.
%                                 Uses the Waggoner and Zha (1999) method.
%
% USAGE:
%
%       [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ,status,kalmanstatus] = ...
%                                         DSGECondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,...
%                                                                              X,Z,U,h,NumPaths,AnnualizeData,TransData,NameStr)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                 the prediction paths.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                                 predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                                 data and the conditioning data.
%
%                  h (integer) with the prediction horizon.
%
%                  NumPaths (integer) the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  NameStr (string vector) that is sent to the progress dialog name after the string
%                                 "Predictive Distribution".
%
% REQUIRED OUTPUT: PredPaths (n x h x NumPaths) 3D matrix with the NumPaths possible paths for the n
%                                 endogenous variables.
%
%                  PredData (structure) with fields:
%                                 PredMean             - (nxh) matrix with the mean predictions of the observed
%                                                        variables.
%                                 ShockScenarioResults - (nxhxq_used) matrix with shock scenarios for the mean
%                                                        predictions of the observed variables. These are
%                                                        obtained by manipulating the shocks used to
%                                                        compute the mean predictions. q_used<=q sinze some
%                                                        of the q shocks can be zero always by assumption.
%                                 OrigPredMean         - (nxh) matrix with the non-annualized mean predictions
%                                                        of the observed variables.
%                                 MeanPredPath         - (nxh) matrix with the non-annualized average of the
%                                                        predictions paths for the observed variables.
%                                 etaShocks            - (qxh) matrix with the values of the shocks at the mean
%                                                        predictions.
%                                 KsitT                - (rxT) matrix with the smooth estimates of the states
%                                                        over the estimation sample.
%                                 xiStates             - (rxh) matrix with the values of the states at the mean
%                                                        predictions.
%                                 X                    - (kxh) matrix with the deterministic variables for the
%                                                        prediction horizon.
%                                 etaMean              - (qxh) matrix with the mean values for the economic shocks over
%                                                        the forecast period.
%                                 wMean                - (nxh) matrix with the mean values for the measurement errors over
%                                                        the forecast period.
%                                 KsiMean              - (rx1) vector with the sample mean value of the state variables
%                                                        in the last period of the historical sample.
%                                 KsiT                 - (rx1) vector with the population mean value of the state
%                                                        variables in the last period of the historical sample.
%                                 A                    - (kxn) matrix with the parameters on the exogenous variables in
%                                                        the measurement equation.
%                                 H                    - (rxn) matrix with parameters on the state variabels in the
%                                                        measurement equation.
%                                 R                    - (nxn) covariance matrix for the measurement errors.
%                                 F                    - (rxr) matrix with state transitions parameters for the state
%                                                        equation.
%                                 B0                   - (rxq) matrix with parameters on the economic shocks in the
%                                                        state equation.
%                                 ShockNames           - (qxqstr) string matrix with the names of the economic shocks.
%                                 Shocks               - (qxhxNumPaths) matrix with the economic shocks for the paths.
%                                 MeasError            - (nxhxNumPaths) matrix with the measurement errors for the paths.
%                                 HistState            - (rxNumPaths) matrix with the time T period draw of the state
%                                                        vector for the paths.
%                                 Extra                - with fields "Shocks", "MeasError", and "HistState" containing
%                                                        additional draws.
%                                 KernelX              - (3D matrix) of dimension n x 2^8 x h with the x-axis values
%                                                        from kernel density estimation of the marginal predictive
%                                                        densities.
%                                 KernelY              - (3D matrix) of dimension n x 2^8 x h with the y-axis values
%                                                        from kernel density estimation of the marginal predictive
%                                                        densities.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                 the PredPaths data satisfies the event, the second hold the number
%                                 of times the PredPaths data is greater than the event, the third
%                                 the number of time the data is above the upper bound, while the
%                                 fourth column holds the number of times the event has been tested.
%                                 The fifth and sixth column measure the average deviation from the
%                                 lower bound when below the lower bound and the square of this, while
%                                 the seventh and eighth columns hold similar data for the upper bound
%                                 of the event when the variables are above the upper bound.
%
%                  YObsEventData (n x 8) matrix where the first column hold the number of times
%                                 the PredPaths (adjusted such that the mean is equal to the realized
%                                 value) satisfies the event, the second holds the number of times
%                                 the adjusted paths data is greater than the event, the third
%                                 the number of time the data is above the upper bound, while the
%                                 fourth column holds the number of tiems the event has been tested.
%                                 The fifth and sixth column measure the average deviation from the
%                                 lower bound when below the lower bound and the square of this, while
%                                 the seventh and eighth columns hold similar data for the upper bound
%                                 of the event when the variables are above the upper bound.
%
%                  MultiModestyStat (matrix) with NumPaths x 2 elements. The first column is the the multivariate
%                                 modesty statistic based on the conditioning consistent shocks and the second
%                                 is the value of this statistic for randomly drawn shocks.
%
%                  UniModestyStat (matrix) with NumPaths x n elements. Each column gives the univariate modesty
%                                 for a particular observed variable.
%
%                  UniModestyStatLZ (vector) with the values of the univariate modesty statistics suggested
%                                 by Leeper-Zha (other shocks are zero over examined prediction horizon) for
%                                 all observed variables.
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" return complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%                  kalmanstatus (boolean) indicating the covariance stationarity of the state variables. If all the
%                                 eigenvalues of F are inside the unit circle, then kalmanstatus=0 and the log-likelihood function
%                                 is calculated. If not, then status=1 and initP is set to 0. In addition, kalmanstatus can be
%                                 -1 if ths forecast covariance matrix is not positive definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 17, 2007.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-12-2007: Finished the function.
%
% * 21-12-2007: Added the code for dealing with unit-root state variables when running the Kalman filter.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 15-01-2008: Started work on smooth estimation of state variables using the conditioning assumptions.
%
% * 23-01-2008: Finished work on smooth estimation of the state variables using the conditioning assumptions.
%
% * 24-01-2008: Implemented the smooth estimates of the state variables based on the conditioning assumptions.
%
% * 16-04-2008: The code now checks if the model has a unique convergent solution also at the posterior mode
%               estimates.
%
% * 05-05-2008: Added the "TransData" input variable.
%
% * 16-05-2008: Took the "DSGEModel.AIMTolerance" input for AiMSolver into account.
%
% * 23-05-2008: Updated the documentation.
%
% * 27-05-2008: The mcode value 7 is now allowed for.
%
% * 01-07-2008: Added the fields "xiStates" and "KsitT" to PredData.
%
% * 02-07-2008: Added the variable "DSGEModel.AllowUnitRoot" for the Kalman filters.
%
% * 03-07-2008: Made sure that the measurement equation receives only the active state variable names.
%               Added try-catch code for AiMSolver.
%
% * 04-07-2008: Added code to allow for the case when the measurement matrix H is time-varying.
%
% * 07-07-2008: Updated the documentation.
%
% * 09-07-2008: Made sure that the matrix H for the PredData output has the correct dimension when
%               it is time-varying.
%
% * 15-01-2009: Added the fields "Shocks", "MeasError", and "HistState" to the PredData output variable.
%               The output variable "YObsEventData" was also added along with the code that can compute
%               prediction events based on a predictive distribution whose mean is equal to the observed
%               data.
%
% * 30-01-2009: Made sure that status is only set to mcode when the latter exists.
%
% * 09-02-2009: Changed the input "length(DSGEModel.StateVariablePositions)" to "size(F,1)" when calling the
%               measurement equation function.
%
% * 13-02-2009: The mcode values 7 and 8 are now properly mentioned when encountered.
%
% * 02-03-2009: Added code for kernel density estimation of the marginal predictive distributions.
%
% * 03-03-2009: Made sure that the fields "KernelX" and "KernelY" of PredData are initialized.
%
% * 09-03-2009: Added quotation marks around paths for the dos copy commands.
%
% * 07-04-2009: Changed the maximum height of the "AiM Error" message box.
%
% * 30-04-2009: Updated the documentation.
%
% * 10-07-2009: The smooth estimates for the state variables are now stored in the PredData.KsitT matrix
%               for the full selected sample (i.e., including the training sample).
%
% * 16-07-2009: Added a test that the rank of K'D is equal to the number of conditioning assumptions.
%
% * 27-08-2009: Added extra input to "StateSmoother" and "StateSmootherHt" function.
%
% * 04-12-2009: Made sure that smooth estimation of the states using the conditioning assumption only occurs
%               when the forecast horizon is not longer than the length of the historical sample.
%
% * 05-01-2010: Updated the documentation.
%
% * 18-11-2010: Took into account that "PredEventData" and "YObsEventData" now have 7 columns.
%
% * 19-11-2010: And now 8.
%
% * 17-12-2010: The function now supports the Klein and Sims DSGE model solvers.
%
% * 19-12-2010: Updated the documentation.
%
% * 01-01-2011: The function now supports the square root filter.
%
% * 10-01-2011: Changed the dos copy calls to "CopyFile" calls.
%
% * 25-01-2011: Took into account that "CalcTransformData" takes the input variable "TransMatrix".
%
% * 21-02-2011: Made sure that "SigmaTg" is positive semidefinite and a Choleski factor consistent
%               with this is taken.
%
% * 23-02-2011: Changed the computation of the covariance matrix for the Leeper-Zha statistic.
%
% * 24-02-2011: Updated the documentation.
%
% * 01-03-2011: Changed the calculation of the square-root matrix of "SigmaTg".
%
% * 11-03-2011: Switched to the "RunKalmanFilter" function.
%
% * 15-03-2011: Switched to the "RunKalmanSmoother" function.
%
% * 22-03-2011: The function now tests if K1'*H'*B0 has rank equal to the number of conditioning
%               assumptions per time period.
%
% * 23-03-2011: Corrected a spelling error. Simplified the computation of WTg and etaTh.
%
% * 18-05-2011: Added test for calling and call to "CondPredictionSmoother" and "CondPredictionSmootherHt".
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 25-10-2011: Took into account that "RunKalmanSmoother" now requires the input variable "DiffuseInit".
%
% * 09-12-2011: Fixes for dealing with missing observations in historical sample.
%
% * 13-12-2011: Added matrix computations suggested by Ingvar Strid that speeds up the calculations.
%
% * 02-01-2012: Updated the documentation.
%
% * 13-01-2012: Allow for missing observations in the conditioning assumptions.
%
% * 16-01-2012: Added support for lag-varying K2 matrices.
%
% * 17-01-2012: The default value for lag-varying K2 matrices is zero so that missing lags are
%               simply set to zero.
%
% * 02-02-2012: Fixed a bug when adding 3D K2 matrices into K.
%
% * 09-03-2012: Made sure that missing observations don't affect the prediction event calculations.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 29-01-2014: Switched the "inv" function for "\".
%
% * 19-08-2014: Added the fields "KsiT", "KsiMean", "Extra" to the structure "PredData". Added the "ExtraShocks" input
%               variable.
%
% * 20-08-2014: Added code for computing the additional shocks, state variables, and measurement error draws.
%
% * 14-01-2015: Updated the documentation.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 02-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-12-2019: Took into account that the DSGEModel.ModelSolverFile now requires the three additional
%               input variables "DSGEModel.StateVariableNames", "DSGEModel.StateEquationNames" and 
%               "DSGEModel.StateShockNames".
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed additional eval calls to feval.
%
% * 22-02-2022: Changed the last eval calls to feval.
%
% * 17-11-2022: Added a wait control when the progress dialog should not be displayed but transformed
%               variables are predicted.
%
% * 06-01-2023: Updated the documentation.
%
% * 09-02-2023: The function now fully supports the driver.
%
% * 17-02-2023: Removed "ExtraShocks".
%
% * 17-04-2023: The function now supports parallel computations.
%
% * 27-04-2023: Changed random number initialization to avoid shuffle under parallel computations.
%
% * 25-01-2024: Updated the documentation.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
PredPaths = [];
PredData = [];
PredEventData = [];
YObsEventData = [];
MultiModestyStat = [];
UniModestyStat = [];
UniModestyStatLZ = [];
%
% set up the ModelParameters structure based on theta and thetaPositions
%
if nargout==7;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>7;
      status = 0;
      kalmanstatus = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% See if we can solve the model
%
if nargout==7;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch ME;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError(ME)];
         end;
         kalmanstatus = 0;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   else;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError];
         end;
         kalmanstatus = 0;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   end;
end;
if nargout>7;
   status = mcode;
   if status~=1;
      kalmanstatus = 0;
      return;
   end;
else;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      elseif DSGEModel.ModelSolver==4;
         SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
[B0,KeepShocks] = RemoveRedundantColumns(B0);
n = DSGEModel.n;
%
% check B0 to make sure that it is not empty
%
if isempty(KeepShocks)==1;
   txt = 'All economic shocks are redundant, i.e., the B0 matrix is a zero matrix. YADA is therefore unable to calculate any conditional forecasts.';
   About(txt,'information','YADA - Shock Selection',120,500,CurrINI);
   return;
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0;
%
% obtain the measurement equation matrices
%
if MatlabNumber>=8.0;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,n,size(F,1),DSGEModel.k);
   catch ME;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
else;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,n,size(F,1),DSGEModel.k);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the conditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% We can now run the Kalman filter
%
[logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
%
% we should adjust everything based on DSGEModel.KalmanFirstObservation
%
if nargout>7;
   kalmanstatus = stat;
   if kalmanstatus~=0;
      return;
   end;
end;
%
% get some parameter values
%
[r,T] = size(Ksitt1);
k = DSGEModel.k;
g = size(Z,2);
q = size(B0,2);
%
% create the mapping matrices for conditioning variables
%
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
SCond = (isequal(U,zeros(size(U)))&isequal(K2,zeros(size(K2))));
%
% run the Kalman smoother for g periods
%
[Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
PredData.KsitT = KsitT;
%
% calculate measurement errors for the sample T-g+1 until T
%
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (k>0);
      XPred = X(:,1:g);
   else;
      XPred = [];
   end;
   if length(size(H))==2;
      [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,T),PtT(:,:,T),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H,R,F,B0,DSGEModel.KalmanAlgorithm);
   else;
      [KsiTTz,PTTz] = CondPredictionSmootherHt(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,T),PtT(:,:,T),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H(:,:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+g),R,F,B0,DSGEModel.KalmanAlgorithm);
   end;
elseif (DSGEModel.KsiUseCondData==1)&(size(KsitT,2)>=g);
   PtT = PtT(:,:,size(PtT,3)-g+1:size(PtT,3));
   KsitT = KsitT(:,size(KsitT,2)-g+1:size(KsitT,2));
   if length(size(H))==2;
      if rank(R)>0;
         if k>0;
            wtT = DSGEModel.Y(:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod)-(A'*DSGEModel.X(:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod))-(H'*KsitT);
         else;
            wtT = DSGEModel.Y(:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod)-(H'*KsitT);
         end;
      else;
         wtT = zeros(n,g);
      end;
      %
      % Smooth estimation of the states at time T using the conditioning assumptions.
      %
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,F,B0);
      else;
         [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,F,B0);
      end;
   else;
      wtT = zeros(n,g);
      if rank(R)>0;
         if k>0;
            for t=DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod;
               wtT(:,t-(DSGEModel.LastPeriod-g+1)+1) = DSGEModel.Y(:,t)-(A'*DSGEModel.X(:,t))-(H(:,:,t)'*KsitT(:,t-(DSGEModel.LastPeriod-g+1)+1));
            end;
         else;
            for t=DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod;
               wtT(:,t-(DSGEModel.LastPeriod-g+1)+1) = DSGEModel.Y(:,t)-(H(:,:,t)'*KsitT(:,t-(DSGEModel.LastPeriod-g+1)+1));
            end;
         end;
      end;
      %
      % Smooth estimation of the states at time T using the conditioning assumptions.
      %
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
      else;
         [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
      end;
   end;
else;
   KsiTTz = KsitT(:,size(KsitT,2));
   PTTz = PtT(:,:,size(KsitT,2));
end;
PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
%
% Now we need to determine KsiTT for just the last period
%
KsiTT = KsitT(:,size(KsitT,2));
%
% use the Schur decomposition for the reduced rank matrix PTTz instead of
% the Choleski decomposition
%
[S,L] = SchurDecomposition(PTTz);
%
% set up the output
%
PredPaths = zeros(n,h,NumPaths);
LZPredPaths = zeros(n,g);
%
% check if we have measurement errors and if so compute a Schur decomposition of the R matrix
%
if rank(R)>0;
   [Rm,Lm] = SchurDecomposition(R);
   Rmeasure = Rm*sqrt(Lm);
   NumMeasure = size(Lm,1);
else;
   Rmeasure = [];
   wTh = zeros(n,h);
   NumMeasure = 0;
end;
%
%
% compute the state and shock covariances at the given parameters
% as well as the predictions of the observed variables.
%
if (DSGEModel.RunPredictionEvent==1)&(size(DSGEModel.Y,2)>=DSGEModel.LastPeriod+h);
   YObserved = DSGEModel.Y(:,DSGEModel.LastPeriod+1:DSGEModel.LastPeriod+h);
else;
   YObserved = [];
end;
Yinit = [];
if AnnualizeData==1;
   if DSGEModel.YNaNs==0;
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
      else;
         Yinit = [];
      end;
   else;
      YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
      nanpos = isnan(YSmooth);
      if max(max(nanpos))==1;
         if DSGEModel.k>0;
            if length(size(H))==2;
               YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*PredData.KsitT);
            else;
               YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
               for t=1:size(PredData.KsitT,2);
                  YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*PredData.KsitT(:,t));
               end;
            end;
         else;
            if length(size(H))==2;
               YHistPred = H'*PredData.KsitT;
            else;
               YHistPred = zeros(DSGEModel.n,size(PredData.KsitT,2));
               for t=1:size(PredData.KsitT,2);
                  YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*PredData.KsitT(:,t);
               end;
            end;
         end;
         YSmooth(nanpos) = YHistPred(nanpos);
      end;
      if DSGEModel.FirstPeriod>1;
         YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod-1) YSmooth];
      end;
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         Yinit = YSmooth(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         Yinit = YSmooth(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
      else;
         Yinit = [];
      end;
   end;
else;
   MultiModestyStat = zeros(NumPaths,2);
   UniModestyStat = zeros(NumPaths,n);
   UniModestyStatLZ = zeros(1,n);
   if TransData==1;
      if DSGEModel.YNaNs==0;
         Yinit = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
      else;
         YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
         nanpos = isnan(YSmooth);
         if max(max(nanpos))==1;
            if DSGEModel.k>0;
               if length(size(H))==2;
                  YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*PredData.KsitT);
               else;
                  YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
                  for t=1:size(KsitT,2);
                     YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*PredData.KsitT(:,t));
                  end;
               end;
            else;
               if length(size(H))==2;
                  YHistPred = H'*PredData.KsitT;
               else;
                  YHistPred = zeros(DSGEModel.n,size(PredData.KsitT,2));
                  for t=1:size(PredData.KsitT,2);
                     YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*PredData.KsitT(:,t);
                  end;
               end;
            end;
            YSmooth(nanpos) = YHistPred(nanpos);
         end;
         if DSGEModel.FirstPeriod>1;
            YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod-1) YSmooth];
         end;
         Yinit = YSmooth(:,1:DSGEModel.LastPeriod);
      end;
   end;
end;
%
% create the matrices for selecting shocks
%
%N = eye(q);
%M = N(:,DSGEModel.ConditionalShocks);
m = size(Z,1);
%
% check if shocks can be used
%
if length(size(H))==2;
   if rank(K1'*H'*B0)<m;
      txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
      About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
      PredPaths = [];
      return;
   end;
else;
   BadEtaMat = 0;
   for i=1:g;
      if rank(K1'*H(:,:,DSGEModel.LastPeriod+i)'*B0)<m;
         txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
         About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
         PredPaths = [];
         BadEtaMat = 1;
         break;
      end;
   end;
   if BadEtaMat==1;
      return;
   end;
end;
%
% stack the model to determine the shock distribution
%
Xbar = zeros(n*g,1);
G = zeros(n*g,r);
WTg = zeros(n*g,1);
K = zeros(n*g,m*g);
D = zeros(n*g,q*g);
Fexp = F;
ZTg = zeros(m*g,1);
UT = zeros(m*g,1);
if length(size(H))==2;
   for i=1:g;
      Ind = (1+((g-i)*n):(g-i+1)*n);
      Ind1 = (1+((g-i)*q):(g-i+1)*q);
      Ind2 = (1+((g-i)*m):(g-i+1)*m);
      if k>0;
         Xbar(Ind,1) = A'*X(:,i);
      end;
      G(Ind,:) = H'*Fexp;
      D(Ind,Ind1) = H'*B0;
      K(Ind,Ind2) = K1;
      Fi = F;
      for j=1:i-1;
         Ind1 = Ind1+q;
         Ind3 = Ind+(j*n);
         D(Ind,Ind1) = H'*Fi*B0;
         if K2length==2;
            K(Ind3,Ind2) = K2;
         else;
            if j<=size(K2,3);
               K(Ind3,Ind2) = K2(:,:,j);
            end;
         end;
         Fi = F*Fi;
      end;
      ZTg(Ind2,1) = Z(:,i);
      UT(Ind2,1) = U(:,g);
      Fexp = F*Fexp;
   end;
else;
   for i=1:g;
      Ind = (1+((g-i)*n):(g-i+1)*n);
      Ind1 = (1+((g-i)*q):(g-i+1)*q);
      Ind2 = (1+((g-i)*m):(g-i+1)*m);
      if k>0;
         Xbar(Ind,1) = A'*X(:,i);
      end;
      G(Ind,:) = H(:,:,DSGEModel.LastPeriod+i)'*Fexp;
      D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*B0;
      K(Ind,Ind2) = K1;
      Fi = F;
      for j=1:i-1;
         Ind1 = Ind1+q;
         Ind3 = Ind+(j*n);
         D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*B0;
         if K2length==2;
            K(Ind3,Ind2) = K2;
         else;
            if j<=size(K2,3);
               K(Ind3,Ind2) = K2(:,:,j);
            end;
         end;
         Fi = F*Fi;
      end;
      ZTg(Ind2,1) = Z(:,i);
      UT(Ind2,1) = U(:,g);
      Fexp = F*Fexp;
   end;
end;
%
% create a vector with positions for the conditioning assumptions that should be used
%
PosVec = (1:m*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
%
% adjust for missing restrictions
%
K = K(:,PosVec);
ZTg = ZTg(PosVec,1);
UT = UT(PosVec,1);
%
% speed-up code suggested by Ingvar Strid
%
KpD = K'*D;
KpDDpK = KpD*KpD';
DpKInvKpDDpK = KpD'/KpDDpK;
%
% check the rank of K'D
%
if rank(KpD)<length(PosVec);
   txt = ['Some of the conditioning assumptions are linearly dependent. The rank of K''*D is ' int2str(rank(KpD)) ', while the number of conditioning assumptions is ' int2str(length(PosVec)) '. YADA is therefore unable to perform the conditional forecasts for the Waggoner-Zha approach.'];
   About(txt,'error','Error - Conditioning Assumptions',200,500,CurrINI);
   PredPaths = [];
   return;
end;
%
% compute the conditioning relation when state vector and measurement errors are at mean
%
kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
%
% compute the mean of the shocks when state vector and measurement errors are at mean
%
mubarTg = DpKInvKpDDpK*kbarTg;
%
% compute the idempotent covariance matrix for the shocks
%
SigmaTg = eye(q*g)-(DpKInvKpDDpK*KpD);
SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
[SE,LE] = schur(SigmaTg);
SigmaTgStd = SE*sqrt(abs(LE));
%
% add info to the PredData structure
%
PredData.KernelX = [];
PredData.KernelY = [];
PredData.X = X;
PredData.A = A;
if length(size(H))==2;
   PredData.H = H;
else;
   PredData.H = H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod+h);
end;
PredData.R = R;
PredData.B0 = B0;
PredData.F = F;
PredData.etaMean = zeros(q,h);
PredData.wMean = zeros(n,h);
PredData.KsiT = KsiTTz;
PredData.KsiMean = zeros(r,1);
PredData.ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
PredData.Shocks = zeros(q,h,NumPaths);
PredData.MeasError = zeros(DSGEModel.n,h,NumPaths);
PredData.HistState = zeros(r,NumPaths);
%
% compute the mean prediction
%
PredData.PredMean = zeros(n,h);
PredData.MeanPredPath = zeros(n,h);
KsiT = KsiTTz;
PredData.etaShocks = zeros(q,h);
PredData.xiStates = zeros(r,h);
PredData.etaShocks(:,1:g) = fliplr(reshape(mubarTg,q,g));
if length(size(H))==2;
   for i=1:h;
      %
      % update the states
      %
      KsiT = (F*KsiT)+(B0*PredData.etaShocks(:,i));
      PredData.xiStates(:,i) = KsiT;
      %
      % compute the conditional mean predictions
      %
      if k>0;
         PredData.PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
      else;
         PredData.PredMean(:,i) = H'*KsiT;
      end;
   end;
else;
   for i=1:h;
      %
      % update the states
      %
      KsiT = (F*KsiT)+(B0*PredData.etaShocks(:,i));
      PredData.xiStates(:,i) = KsiT;
      %
      % compute the conditional mean predictions
      %
      if k>0;
         PredData.PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT);
      else;
         PredData.PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiT;
      end;
   end;
end;
%
PredData.ShockScenarioResults = zeros(n,h,q);
if length(size(H))==2;
   for j=1:q;
      %
      % set shock j of the shocks manipulated for the theoretical mean of the conditional
      % forecast
      %
      etaShocks = PredData.etaShocks;
      etaShocks(j,:) = zeros(1,h);
      %
      % inistalize the state
      %
      KsiT = KsiTTz;
      for i=1:h;
         KsiT = (F*KsiT)+(B0*etaShocks(:,i));
         if k>0;
            PredData.ShockScenarioResults(:,i,j) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredData.ShockScenarioResults(:,i,j) = H'*KsiT;
         end;
      end;
   end;
else;
   for j=1:q;
      %
      % set shock j of the shocks manipulated for the theoretical mean of the conditional
      % forecast
      %
      etaShocks = PredData.etaShocks;
      etaShocks(j,:) = zeros(1,h);
      %
      % inistalize the state
      %
      KsiT = KsiTTz;
      for i=1:h;
         KsiT = (F*KsiT)+(B0*etaShocks(:,i));
         if k>0;
            PredData.ShockScenarioResults(:,i,j) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT);
         else;
            PredData.ShockScenarioResults(:,i,j) = H(:,:,DSGEModel.LastPeriod+i)'*KsiT;
         end;
      end;
   end;
end;
%
% annualize the mean predictions when needed
%
Tt = [];
PredData.OrigPredMean = PredData.PredMean;
if AnnualizeData==1;
   PredData.PredMean = Annualize([Yinit PredData.PredMean],DSGEModel.annual,DSGEModel.annualscale);
   if isempty(YObserved)==0;
      YObserved = Annualize([Yinit YObserved],DSGEModel.annual,DSGEModel.annualscale);;
   end;
elseif TransData==1;
   YTmp = [Yinit PredData.PredMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   PredData.PredMean = YData(Tt-h+1:Tt,:)';
   if isempty(YObserved)==0;
      YObserved = [Yinit YObserved];
      [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YObserved',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
      Tt = size(YData,1);
      YObserved = YData(Tt-h+1:Tt,:)';
   end;
end;
%
% check if we should run parallel computations
%
if CurrINI.NumWorkers<2;
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' int2str(NumPaths) ' Prediction Paths'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Path Number:';
      %
      ProgressStructure.name = ['Conditional Predictive Distribution' NameStr];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   elseif (DSGEModel.ShowProgress==0)&(TransData==1)&(DSGEModel.RunDriver==0);
      txt = ['Please wait while YADA computes conditional predictive distributions' lower(NameStr) ' for the transformed data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information','Conditional Predictive Distributions - Transformed Data',600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % start the iterations
   %
   Q = B0*B0';
   abort = '';
   if length(size(H))==2;
      CovMatShocks = Q*H*K1*((K1'*H'*Q*H*K1)\K1')*H'*Q;
      %
      % additional speed-up code suggested by Ingvar Strid
      %
      PTTg = zeros(r,r);
      Fg = eye(r);
      for i=1:g
         Fg = Fg*F;
         PTTg = (F*PTTg*F')+Q;
      end;
      %
      for it=1:NumPaths;
         %
         % check if we should update the progress dialog
         %
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
               PredPaths = [];
               break;
            else;
               ProgressDLG([it/NumPaths it]);
            end;
         end;
         %
         % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
         %
         KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
         PredData.HistState(:,it) = KsiT;
         PredData.KsiMean = PredData.KsiMean+((1/NumPaths)*KsiT);
         KsiCTmp = KsiTT;
         KsiTmp = KsiTT;
         PTTgLZ = zeros(r,r);
         %
         % Draw a sequence of measurement errors
         %
         if isempty(Rmeasure)==0;
            wTh = Rmeasure*randn(NumMeasure,h);
            PredData.MeasError(:,:,it) = wTh;
            PredData.wMean = PredData.wMean+((1/NumPaths)*wTh);
            WTg = vec(fliplr(wTh(:,1:g)));
         end;
         %
         % setut the mean for the shock distribution
         %
         kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
         %
         % use speed-up code
         %
         muTg = DpKInvKpDDpK*kTg;
         kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
         %
         % use speed-up code
         %
         muCTmp = DpKInvKpDDpK*kCTmp;
         %
         % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
         % and from N(0,I) for period g+1 to h.
         %
         ShockDraw = randn(q*g,1);
         etaTg = muTg+(SigmaTgStd*ShockDraw);
         etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
         etaTh = randn(q,h);
         etaCTmp = etaTh;
         etaTmp = etaTh;
         %
         % overlay with restricted shocks
         %
         etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
         etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
         PredData.Shocks(:,:,it) = etaTh;
         PredData.etaMean = PredData.etaMean+((1/NumPaths)*etaTh);
         %
         % compute the prediction paths
         %
         for i=1:h;
            %
            % update the state
            %
            KsiT = (F*KsiT)+(B0*etaTh(:,i));
            if k>0;
               PredPaths(:,i,it) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
            else;
               PredPaths(:,i,it) = (H'*KsiT)+wTh(:,i);
            end;
            %
            % update modesty related material
            %
            if i<=g;
               KsiCTmp = (F*KsiCTmp)+(B0*etaCTmp(:,i));
               KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
               if it==1;
                  PTTgLZ = (F*PTTgLZ*F')+CovMatShocks;
               end;
            end;
         end;
         PredData.MeanPredPath = PredData.MeanPredPath+((1/NumPaths)*PredPaths(:,:,it));
         if AnnualizeData==1;
            PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
         else;
            if TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            end;
            %
            % deal with modesty statistic
            %
            if k>0;
               PredictCTmp = (A'*X(:,g))+(H'*KsiCTmp)+wTh(:,g);
               PredictY = (A'*X(:,g))+(H'*Fg*KsiTT);
            else;
               PredictCTmp = (H'*KsiCTmp)+wTh(:,g);
               PredictY = H'*Fg*KsiTT;
            end;
            PredCov = (H'*PTTg*H)+R;
            PredDiff = PredictCTmp-PredictY;
            MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
            UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
            %
            % comparison-path for multivariate statistic
            %
            if k>0;
               PredictTmp = (A'*X(:,g))+(H'*KsiTmp)+wTh(:,g);
            else;
               PredictTmp = (H'*KsiTmp)+wTh(:,g);
            end;
            PredDiff = PredictTmp-PredictY;
            MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
            %
            % Leeper-Zha type statistic
            %
            if it==1;
               PredDiff = PredData.PredMean(:,g)-PredictY;
               CovPredDiff = diag(H'*PTTgLZ*H);
               for i=1:length(CovPredDiff);
                  if CovPredDiff(i)==0;
                     PredDiff(i) = 0;
                     CovPredDiff(i) = 1;
                  end;
               end;
               UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(CovPredDiff))';
            end;
         end;
         drawnow;
      end;
   else;
      %
      % additional speed-up code suggested by Ingvar Strid
      %
      PTTg = zeros(r,r);
      Fg = eye(r);
      for i=1:g
         Fg = Fg*F;
         PTTg = (F*PTTg*F')+Q;
      end;
      %
      for it=1:NumPaths;
         %
         % check if we should update the progress dialog
         %
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
               PredPaths = [];
               break;
            else;
               ProgressDLG([it/NumPaths it]);
            end;
         end;
         %
         % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
         %
         KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
         PredData.HistState(:,it) = KsiT;
         PredData.KsiMean = PredData.KsiMean+((1/NumPaths)*KsiT);
         KsiCTmp = KsiTT;
         KsiTmp = KsiTT;
         PTTgLZ = zeros(r,r);
         %
         % Draw a sequence of measurement errors
         %
         if isempty(Rmeasure)==0;
            wTh = Rmeasure*randn(NumMeasure,h);
            PredData.MeasError(:,:,it) = wTh;
            PredData.wMean = PredData.wMean+((1/NumPaths)*wTh);
            WTg = vec(fliplr(wTh(:,1:g)));
         end;
         %
         % setut the mean for the shock distribution
         %
         kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
         %
         muTg = DpKInvKpDDpK*kTg;
         kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
         %
         muCTmp = DpKInvKpDDpK*kCTmp;
         %
         % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
         % and from N(0,I) for period g+1 to h.
         %
         ShockDraw = randn(q*g,1);
         etaTg = muTg+(SigmaTgStd*ShockDraw);
         etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
         etaTh = randn(q,h);
         etaCTmp = etaTh;
         etaTmp = etaTh;
         %
         % overlay with restricted shocks
         %
         etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
         etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
         PredData.Shocks(:,:,it) = etaTh;
         PredData.etaMean = PredData.etaMean+((1/NumPaths)*etaTh);
         %
         % compute the prediction paths
         %
         for i=1:h;
            %
            % update the state
            %
            KsiT = (F*KsiT)+(B0*etaTh(:,i));
            if k>0;
               PredPaths(:,i,it) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
            else;
               PredPaths(:,i,it) = (H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
            end;
            %
            % update modesty related material
            %
            if i<=g;
               KsiCTmp = (F*KsiCTmp)+(B0*etaCTmp(:,i));
               KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
               if it==1;
                  PTTgLZ = (F*PTTgLZ*F')+(Q*H(:,:,DSGEModel.LastPeriod+i)*K1*((K1'*H(:,:,DSGEModel.LastPeriod+i)'*Q*H(:,:,DSGEModel.LastPeriod+i)*K1)\K1')*H(:,:,DSGEModel.LastPeriod+i)'*Q);
               end;
            end;
         end;
         PredData.MeanPredPath = PredData.MeanPredPath+((1/NumPaths)*PredPaths(:,:,it));
         if AnnualizeData==1;
            PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
         else;
            if TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            end;
            %
            % deal with modesty statistic
            %
            if k>0;
               PredictCTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
               PredictY = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*Fg*KsiTT);
            else;
               PredictCTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
               PredictY = H(:,:,DSGEModel.LastPeriod+g)'*Fg*KsiTT;
            end;
            PredCov = (H(:,:,DSGEModel.LastPeriod+g)'*PTTg*H(:,:,DSGEModel.LastPeriod+g))+R;
            PredDiff = PredictCTmp-PredictY;
            MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
            UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
            %
            % comparison-path for multivariate statistic
            %
            if k>0;
               PredictTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
            else;
               PredictTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
            end;
            PredDiff = PredictTmp-PredictY;
            MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
            %
            % Leeper-Zha type statistic
            %
            if it==1;
               PredDiff = PredData.PredMean(:,g)-PredictY;
               CovPredDiff = diag(H(:,:,DSGEModel.LastPeriod+g)'*PTTgLZ*H(:,:,DSGEModel.LastPeriod+g));
               for i=1:length(CovPredDiff);
                  if CovPredDiff(i)==0;
                     PredDiff(i) = 0;
                     CovPredDiff(i) = 1;
                  end;
               end;
               UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(CovPredDiff))';
            end;
         end;
         drawnow;
      end;
   end;
   %
   % close the wait handle
   %
   if DSGEModel.ShowProgress==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   elseif (DSGEModel.ShowProgress==0)&(TransData==1)&(DSGEModel.RunDriver==0);
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % we can do parallel computations over the parameter draws
   %
   if DSGEModel.RunDriver==0;
      TypeStr = 'original';
      if AnnualizeData==1;
         TypeStr = 'annualized';
      elseif TransData==1;
         TypeStr = 'transformed';
      end;
      txt = ['Please wait while YADA computes predictive distributions for the ' lower(TypeStr) ' data using parallel computations with ' int2str(CurrINI.NumWorkers) ' workers. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['Predictive Distribution' NameStr],600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % initialize
   %
   HistState = PredData.HistState;
   Shocks = PredData.Shocks;
   MeasError = PredData.MeasError;
   %
   MultiModestyStat = zeros(NumPaths,2);
   UniModestyStat = zeros(NumPaths,n);
   %
   if DSGEModel.RandomNumberValue==1;
      %
      % without eval statements the spmd-code gives an error in older matlab systems
      %
      eval('spmd; rng(0,''combRecursive''); end;');
   else;
      eval('spmd; rng(''default''); end;');
   end;
   %
   % begin the loop
   %
   Q = B0*B0';
   abort = '';
   if length(size(H))==2;
      CovMatShocks = Q*H*K1*((K1'*H'*Q*H*K1)\K1')*H'*Q;
      %
      % additional speed-up code suggested by Ingvar Strid
      %
      PTTg = zeros(r,r);
      Fg = eye(r);
      for i=1:g
         Fg = Fg*F;
         PTTg = (F*PTTg*F')+Q;
      end;
      %
      parfor it=1:NumPaths;
         %
         % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
         %
         KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
         HistState(:,it) = KsiT;
         KsiCTmp = KsiTT;
         KsiTmp = KsiTT;
         PTTgLZ = zeros(r,r);
         %
         % Draw a sequence of measurement errors
         %
         if isempty(Rmeasure)==0;
            wTh = Rmeasure*randn(NumMeasure,h);
            MeasError(:,:,it) = wTh;
            WTg = vec(fliplr(wTh(:,1:g)));
         else;
            wTh = zeros(n,h);
            WTg = 0;
         end;
         %
         % setut the mean for the shock distribution
         %
         kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
         %
         % use speed-up code
         %
         muTg = DpKInvKpDDpK*kTg;
         kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
         %
         % use speed-up code
         %
         muCTmp = DpKInvKpDDpK*kCTmp;
         %
         % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
         % and from N(0,I) for period g+1 to h.
         %
         ShockDraw = randn(q*g,1);
         etaTg = muTg+(SigmaTgStd*ShockDraw);
         etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
         etaTh = randn(q,h);
         etaCTmp = etaTh;
         etaTmp = etaTh;
         %
         % overlay with restricted shocks
         %
         etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
         etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
         Shocks(:,:,it) = etaTh;
         %
         % compute the prediction paths
         %
         PredPath = zeros(n,h);
         for i=1:h;
            %
            % update the state
            %
            KsiT = (F*KsiT)+(B0*etaTh(:,i));
            if k>0;
               PredPath(:,i) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
            else;
               PredPath(:,i) = (H'*KsiT)+wTh(:,i);
            end;
            %
            % update modesty related material
            %
            if i<=g;
               KsiCTmp = (F*KsiCTmp)+(B0*etaCTmp(:,i));
               KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
               if it==1;
                  PTTgLZ = (F*PTTgLZ*F')+CovMatShocks;
               end;
            end;
         end;
         PredPaths(:,:,it) = PredPath;
         MMStat = zeros(1,2);
         %
         if AnnualizeData==1;
            PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
         else;
            if TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            end;
            %
            % deal with modesty statistic
            %
            if k>0;
               PredictCTmp = (A'*X(:,g))+(H'*KsiCTmp)+wTh(:,g);
               PredictY = (A'*X(:,g))+(H'*Fg*KsiTT);
            else;
               PredictCTmp = (H'*KsiCTmp)+wTh(:,g);
               PredictY = H'*Fg*KsiTT;
            end;
            PredCov = (H'*PTTg*H)+R;
            PredDiff = PredictCTmp-PredictY;
            MMStat(1,1) = PredDiff'*(PredCov\PredDiff);
            UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
            %
            % comparison-path for multivariate statistic
            %
            if k>0;
               PredictTmp = (A'*X(:,g))+(H'*KsiTmp)+wTh(:,g);
            else;
               PredictTmp = (H'*KsiTmp)+wTh(:,g);
            end;
            PredDiff = PredictTmp-PredictY;
            MMStat(1,2) = PredDiff'*(PredCov\PredDiff);
            MultiModestyStat(it,:) = MMStat;
            %
            % Leeper-Zha type statistic
            %
            if it==1;
               PredDiff = PredData.PredMean(:,g)-PredictY;
               CovPredDiff = diag(H'*PTTgLZ*H);
               for i=1:length(CovPredDiff);
                  if CovPredDiff(i)==0;
                     PredDiff(i) = 0;
                     CovPredDiff(i) = 1;
                  end;
               end;
               UniModestyStatLZ = (PredDiff ./ sqrt(CovPredDiff))';
            end;
         end;
         drawnow;
      end;
   else;
      %
      % additional speed-up code suggested by Ingvar Strid
      %
      PTTg = zeros(r,r);
      Fg = eye(r);
      for i=1:g
         Fg = Fg*F;
         PTTg = (F*PTTg*F')+Q;
      end;
      %
      parfor it=1:NumPaths;
         %
         % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
         %
         KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
         HistState(:,it) = KsiT;
         KsiCTmp = KsiTT;
         KsiTmp = KsiTT;
         PTTgLZ = zeros(r,r);
         %
         % Draw a sequence of measurement errors
         %
         if isempty(Rmeasure)==0;
            wTh = Rmeasure*randn(NumMeasure,h);
            MeasError(:,:,it) = wTh;
            WTg = vec(fliplr(wTh(:,1:g)));
         else;
            wTh = zeros(n,h);
            WTg = 0;
         end;
         %
         % setut the mean for the shock distribution
         %
         kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
         %
         muTg = DpKInvKpDDpK*kTg;
         kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
         %
         muCTmp = DpKInvKpDDpK*kCTmp;
         %
         % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
         % and from N(0,I) for period g+1 to h.
         %
         ShockDraw = randn(q*g,1);
         etaTg = muTg+(SigmaTgStd*ShockDraw);
         etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
         etaTh = randn(q,h);
         etaCTmp = etaTh;
         etaTmp = etaTh;
         %
         % overlay with restricted shocks
         %
         etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
         etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
         Shocks(:,:,it) = etaTh;
         %
         % compute the prediction paths
         %
         PredPath = zeros(n,h);
         for i=1:h;
            %
            % update the state
            %
            KsiT = (F*KsiT)+(B0*etaTh(:,i));
            if k>0;
               PredPath(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
            else;
               PredPath(:,i) = (H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
            end;
            %
            % update modesty related material
            %
            if i<=g;
               KsiCTmp = (F*KsiCTmp)+(B0*etaCTmp(:,i));
               KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
               if it==1;
                  PTTgLZ = (F*PTTgLZ*F')+(Q*H(:,:,DSGEModel.LastPeriod+i)*K1*((K1'*H(:,:,DSGEModel.LastPeriod+i)'*Q*H(:,:,DSGEModel.LastPeriod+i)*K1)\K1')*H(:,:,DSGEModel.LastPeriod+i)'*Q);
               end;
            end;
         end;
         PredPaths(:,:,it) = PredPath;
         MMStat = zeros(1,2);
         %
         if AnnualizeData==1;
            PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
         else;
            if TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            end;
            %
            % deal with modesty statistic
            %
            if k>0;
               PredictCTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
               PredictY = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*Fg*KsiTT);
            else;
               PredictCTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
               PredictY = H(:,:,DSGEModel.LastPeriod+g)'*Fg*KsiTT;
            end;
            PredCov = (H(:,:,DSGEModel.LastPeriod+g)'*PTTg*H(:,:,DSGEModel.LastPeriod+g))+R;
            PredDiff = PredictCTmp-PredictY;
            MMStat(1,1) = PredDiff'*(PredCov\PredDiff);
            UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
            %
            % comparison-path for multivariate statistic
            %
            if k>0;
               PredictTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
            else;
               PredictTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
            end;
            PredDiff = PredictTmp-PredictY;
            MMStat(1,2) = PredDiff'*(PredCov\PredDiff);
            UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
            %
            % Leeper-Zha type statistic
            %
            if it==1;
               PredDiff = PredData.PredMean(:,g)-PredictY;
               CovPredDiff = diag(H(:,:,DSGEModel.LastPeriod+g)'*PTTgLZ*H(:,:,DSGEModel.LastPeriod+g));
               for i=1:length(CovPredDiff);
                  if CovPredDiff(i)==0;
                     PredDiff(i) = 0;
                     CovPredDiff(i) = 1;
                  end;
               end;
               UniModestyStatLZ = (PredDiff ./ sqrt(CovPredDiff))';
            end;
         end;
         drawnow;
      end;
   end;
   %
   % compute mean values of the initial state, the shocks and the measurement errors
   %
   PredData.KsiMean = (1/NumPaths)*sum(HistState,2);
   PredData.etaMean = (1/NumPaths)*sum(Shocks,3);
   if isempty(Rmeasure)==0;
      PredData.wMean = (1/NumPaths)*sum(MeasError,3);
   end;
   PredData.HistState = HistState;
   PredData.Shocks = Shocks;
   PredData.MeasError = MeasError;
   PredData.MeanPredPath = (1/NumPaths)*sum(PredPaths,3);
   %
   % close the wait dialog
   %
   if DSGEModel.RunDriver==0;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
if (strcmp(abort,'cancel')==0)&(DSGEModel.RunPredictionEvent==1);
   MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
   if DSGEModel.AdjustPredictionPaths==1;
      %
      % we need to fix the paths such that the mean is correct
      %
      AdjustMean = PredData.PredMean-MeanPrediction;
      for i=1:NumPaths;
         PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
   else;
      PEPredPaths = PredPaths;
   end;
   %
   % compute the prediction events results
   %
   PredEventData = CalculatePredictionEvents(PEPredPaths,DSGEModel.PredictionEvent);
   %
   if isempty(YObserved)==0;
      AdjustMean = YObserved-MeanPrediction;
      %
      % make sure that missing observations don't affect adjustment
      %
      AdjustMean(isnan(AdjustMean)) = 0;
      for i=1:NumPaths;
         PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
      YObsEventData = CalculatePredictionEvents(PEPredPaths,DSGEModel.PredictionEvent);
   end;
end;
%
% check if we should compute kernel density estimates
%
if (strcmp(abort,'cancel')==0)&(NumPaths>=500);
   if DSGEModel.RunPredictionEvent==0;
      if DSGEModel.AdjustPredictionPaths==1;
         MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
         AdjustMean = PredData.PredMean-MeanPrediction;
         for i=1:NumPaths;
            PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
         end;
      else;
         PEPredPaths = PredPaths;
      end;
   end;
   %
   [PredData.KernelX,PredData.KernelY] = PredictionKernelDensity(PEPredPaths);
end;

%
% end of DSGECondPredictionPathsThetaWZ.m
%
