function [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,h,NumPaths,AnnualizeData,TransData,CurrChain,IsPosterior,Weights)
% ZLBDSGEPredictionPaths: Calculates a certain number of possible paths that the endogenous variables
%                         can take out-of-sample for a given sample of draws from the posterior of the
%                         parameters in the DSGE model when the monetary policy rate is subject to a
%                         zero lower bound.
%
% USAGE:
%
%       [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions, ...
%                                                      ModelParameters,DSGEModel,CurrINI,X,h,NumPaths,AnnualizeData,...
%                                                      TransData,CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                                  distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) with the posterior mode or initial value of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxMaxPH with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  h (integer) with the prediction horizon.
%
%                  NumPaths (integer) the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                  likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                  the PredPaths data satisfies the event, the second hold the number
%                                  of times the PredPaths data is greater than the event, the third
%                                  the number of time the data is above the upper bound, while the
%                                  fourth column holds the number of times the event has been tested.
%                                  The fifth and sixth column measure the average deviation from the
%                                  lower bound when below the lower bound and the square of this, while
%                                  the seventh and eighth columns hold similar data for the upper bound
%                                  of the event when the variables are above the upper bound.
%
%                  ObsVarMean (n x 1) vector with the steady-state values of the observed variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 29, 2022.
%                        This version: March 5, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 30-08-2022: Continued the function from the KalmanSmoother call for the posterior/prior
%               mode estimates.
%
% * 31-08-2022: Finished work on the function from the population mean forecasts.
%
% * 01-09-2022: Added population mean adjustment.
%
% * 02-09-2022: Added the output variable "ObsVarMean".
%
% * 07-09-2022: removed som duplicate code.
%
% * 08-09-2022: Fixed a bug when setting "KsiTMean" during the estimation of the mean prediction
%               and another bug whenh not updating Ax.
%
% * 09-09-2022: Changed the setup of Rzlb to allow properly for a time-varying lower bound.
%
% * 06-10-2022: Fixed a bug when reading DSGEModel.ZLBData.
%
% * 17-10-2022: Fixed a bug when the zero lower bound includes zero values
%               and the Rzlb matrix has nonzero values after them.
%
% * 27-10-2022: The mean predictions are now computed using the paths instead of a population
%               mean based on zero shocks and anticipated shocks computed under that assumption.
%               This is not the correct approach since the anticipated shocks will typically be
%               zero when the shocks are zero and the ZLB is then usually not binding. A better
%               approach is simply to compute the predictive mean using the simulated paths.
%
% * 28-10-2022: Corrected a parenthesis error.
%
% * 06-01-2023: Updated the documentation.
%
% * 28-02-2023: Made sure that At is only updated when status is unity.
%
% * 17-03-2023: Made sure that a 3D H matrix is fully taken into account.
%
% * 24-04-2023: The function supports parallel computations.
%
% * 25-04-2023: Fixed a small bug when saving prediction event results during parfor loop.
%
% * 26-04-2023: Fixed initialization issues with ObsVarMean and MeanPredictY for the
%               parallel computations.
%
% * 27-04-2023: Changed random number initialization to avoid shuffle under parallel computations.
%
% * 21-11-2023: The function now computes the average shock values when the ZLB is binding.
%
% * 23-11-2023: Took into account that ZLBKleinSolver requires the input variable "ZLBRuleType".
%
% * 26-01-2024: Updated the documentation.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors.
%
% * 05-03-2024: Fixed a bug when computing MeanPredictY using NumParam instead of NumPaths.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[NumDraws,NumParam] = size(thetaPostSample);
k = DSGEModel.k;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
n = DSGEModel.n;
DoneCalc = 0;
PredEventData = [];
ObsVarMean = [];
if (AnnualizeData==1)|(TransData==1);
   if TransData==0;
      TypeStr = 'Annualized';
   else;
      TypeStr = 'Transformed';
   end;
else;
   TypeStr = 'Original';
end;
MeanPredictY = zeros(n,h);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,600,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the unconditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
[S,KeepShocks] = RemoveRedundantColumns(B0);
if isempty(S)==1;
   txt = 'The model doesn''t have any structural shocks. YADA has aborted from the lower bound computations.';
   About(txt,'information',['ZLB Predictive Distributions' PriorHeader],200,600,CurrINI);
   return;
end;
[r,q] = size(B0);
%
% test if the measurement equation for the policy rate maps to the correct state variables only
%
if length(size(H))==2;
   ZLBLink = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   NonZLBLink = H(setdiff((1:r),DSGEModel.RtildePosition),DSGEModel.ZLBPosition);
   if r>1;
      TestValue = isequal(NonZLBLink,zeros(r-1,1));
   else;
      TestValue = isempty(NonZLBLink);
   end;
   txt = '';
   BadValue = 0;
   if ZLBLink==0;
      txt = ['The observed monetary policy interest rate (' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ') doesn''t map to the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. YADA is therefore unable to use the lower bound and the steady-state interest rate and will abort.'];
      BadValue = 1;
   elseif TestValue==0;
      txt = ['The measurement equation for ' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ' maps to other state variables than the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. Are you sure you want to use this mapping?'];
   end;
   if isempty(txt)==0;
      if BadValue==1;
         About(txt,'information',['ZLB Predictive Distributions' PriorHeader],300,600,CurrINI);
         return;
      else;
         answer = Query(txt,'question',300,['ZLB Predictive Distributions' PriorHeader],600,'no',CurrINI);
         if strcmp(lower(answer),'no')==1;
            return;
         end;
      end;
   end;
   Hlength = [];
else;
   Hlength = size(H,3);
   ZLBLink = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2);
   ZLBLTest = min(ZLBLink~=zeros(Hlength-DSGEModel.LastPeriod+1,1));
   NonZLBLink = shiftdim(H(setdiff((1:r),DSGEModel.RtildePosition),DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2);
   if r>1;
      TestValue = isequal(NonZLBLink,zeros(Hlength-DSGEModel.LastPeriod+1,r-1))
   else;
      TestValue = isempty(NonZLBLink);
   end;
   txt = '';
   BadValue = 0;
   if ZLBLink==0;
      txt = ['The observed monetary policy interest rate (' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ') doesn''t map to the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. YADA is therefore unable to use the lower bound and the steady-state interest rate and will abort.'];
      BadValue = 1;
   elseif TestValue==0;
      txt = ['The measurement equation for ' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ' maps to other state variables than the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model for some time periods. Are you sure you want to use this mapping?'];
   end;
   if isempty(txt)==0;
      if BadValue==1;
         About(txt,'information',['ZLB Predictive Distributions' PriorHeader],300,600,CurrIN);
         return;
      else;
         answer = Query(txt,'question',300,['ZLB Predictive Distributions' PriorHeader],600,'no',CurrINI);
         if strcmp(lower(answer),'no')==1;
            return;
         end;
      end;
   end;
end;
%
% check if we can solve the model subject to the ZLB
%
%
% determine the steady-state interest rate path.
%
if DSGEModel.k>0;
   Ax = A'*X;
   XPred = X;
   if size(DSGEModel.X,2)>DSGEModel.LastPeriod;
      X = DSGEModel.X(:,DSGEModel.LastPeriod+1:size(DSGEModel.X,2));
      LastValue = X(:,size(X,2));
   else;
      X = [];
      LastValue = DSGEModel.X(:,DSGEModel.LastPeriod);
   end;
   if isempty(DSGEModel.PredictedX)==0;
      X = [X DSGEModel.PredictedX];
   end;
   Tx = size(X,2);
   Xold = X;
   if Tx>DSGEModel.ZLBSampleT+1;
      X = X(:,1:DSGEModel.ZLBSampleT+1);
   elseif Tx<DSGEModel.ZLBSampleT+1;
      X = [X LastValue*ones(1,DSGEModel.ZLBSampleT+1-Tx)];
   end;
   Rbar = A(:,DSGEModel.ZLBPosition)'*X;
else;
   Ax = zeros(n,h);
   Rbar = zeros(1,DSGEModel.ZLBSampleT+1);
   XPred = [];
end;
%
% setup the zero lower bound path
%
Tzlb = length(DSGEModel.ZLBData);
if Tzlb==1;
   Rzlb = DSGEModel.ZLBData*ones(h,DSGEModel.ZLBSampleT+1);
else;
   if Tzlb>DSGEModel.LastPeriod;
      Rzlb = zeros(h,Tzlb-DSGEModel.LastPeriod);
      for i=1:h;
         Rzlb(i,1:Tzlb-DSGEModel.LastPeriod-i+2) = DSGEModel.ZLBData(DSGEModel.LastPeriod+i-1:Tzlb);
      end;
      TZLB = size(Rzlb,2);
      if TZLB>=DSGEModel.ZLBSampleT+1;
         RzlbLast = Rzlb(1,size(Rzlb,2));
         % make the matrix have the correct dimension
         Rzlb = Rzlb(:,1:DSGEModel.ZLBSampleT+1);
         % replace zeros with the last value of the lower bound
         % provided the locations appear after the last nonzero
         for i=1:h;
            for j=2:DSGEModel.ZLBSampleT+1;
               if (Rzlb(i,j)==0)&(Rzlb(i,j-1)~=0);
                  Rzlb(i,j) = RzlbLast;
               end;
            end;
         end;
         %Rzlb((Rzlb==0)) = RzlbLast;
      else;
         %
         % we should add the last value in each row to fill the matrix
         %
         RzlbLast = Rzlb(1,size(Rzlb,2));
         Rzlb((Rzlb==0)) = RzlbLast;
         Rzlb = [Rzlb RzlbLast*ones(h,DSGEModel.ZLBSampleT+1-TZLB)];
      end;
   else;
      Rzlb = DSGEModel.ZLBData(Tzlb)*ones(h,DSGEModel.ZLBSampleT+1);
   end;
end;
%
% since H may involve estimated parameters we save a copy of Rzlb for later use
%
RzlbMat = Rzlb;
%
% Transform the steady-state interest rate and the lower bound to the same units
% as the unrestricted monetary policy rate
%
if length(size(H))==2;
   Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   Rzlb = (1/Hvalue)*Rzlb;
   Rbar = (1/Hvalue)*Rbar;
else;
   Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
   Th = length(Hvalue);
   if Th>DSGEModel.ZLBSampleT;
      Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
   elseif Th<DSGEModel.ZLBSampleT;
      Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
   end;
   Rzlb = Rzlb./Hvalue;
   Rbar = Rbar./Hvalue;
end;
%
% We can now run the Kalman filter
%
if MatlabNumber>=8.0;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,600,CurrINI);
      return;
   end;
else;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError];
      About(txt,'error','Error - Kalman Filter',200,600,CurrINI);
      return;
   end;
end;
if stat~=0;
   if stat==1;
      txt = 'The state variables are not stationary at the initial values. Issue recorded while running the Kalman filter on the linear DSGE model.';
   else;
      txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab. Issue recorded while running the Kalman filter on the linear DSGE model.';
   end;
   About(txt,'information',['ZLB Predictive Distributions' PriorHeader],200,600,CurrINI);
   return;
end;
%
% check if there are numerical issues with diffuse initialization of the Kalman filter
%
if DSGEModel.UseDoublingAlgorithm==4;
   txt = '';
   if KalmanGain.d==KalmanGain.T;
      txt = 'The diffuse initialization of the Kalman filter didn''t converge within the selected sample. This may be due to non-stationarity or near non-stationarity of some state variable which isn''t observed. YADA therefore recommends that you choose a different initialization method.';
   elseif (KalmanGain.d<KalmanGain.T)&(max(diff(sum(SigmaSqRoot,1)))>0);
      txt = ['Diffuse initialization of the Kalman filter converged within the selected sample in period ' int2str(KalmanGain.d) ' relative to period ' int2str(KalmanGain.T) ', but the rank of the Sigma_Inf matrix has increased over the initialization sample. This indicates a numerical issue with the procedure and it may be due to non-stationarity or near non-stationarity of a state variable which is not observed. YADA therefore recommends that you choose a different initialization method.'];
   end;
   if isempty(txt)==0;
      answer = Query(strvcat(txt,' ','Are you sure you want to continue?'),'question',300,['ZLB Predictive Distributions' PriorHeader],600,'no',CurrINI);
      if strcmp(lower(answer),'no')==1;
         return;
      end;
   end;
end;
%
% run the Kalman smoother for g periods
%
if MatlabNumber>=8.0;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch ME;
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
else;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch;
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
%
% Try to solve the model subject to the anticipated shocks
%
[G,C1,P,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
%
% notice that the row r+1 of P corresponds to the unrestricted interest rate
% and row DSGEModel.RtildePosition to the actual policy interest rate
%
if mcode~=1;
   if mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   txt = ['The Klein solver with anticipated shocks for handling the zero lower bound provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - Klein Solver Error'],200,500,CurrINI);
   return;
end;
%
% setup the parameters that determine the projected restricted policy rate
%
e_pr = zeros(r+1,1);
e_pr(DSGEModel.RtildePosition) = 1;
%
% setup ePG for conmputing the restricted policy rate projections
%
ePG = zeros(DSGEModel.ZLBSampleT+1,size(G,2));
ePG(1,:) = e_pr'*P(1:r+1,:);
for tau=1:DSGEModel.ZLBSampleT;
   ePG(tau+1,:) = ePG(tau,:)*G;
end;
if DSGEModel.ZLBSampleT>1;
   M = [zeros(DSGEModel.ZLBSampleT,1) eye(DSGEModel.ZLBSampleT);zeros(1,DSGEModel.ZLBSampleT+1)];
else;
   M = 0;
end;
%
% compute the mean predictions
%
%ZLBKsiMean = zeros(r,h);
ZLBYMean = zeros(n,h);
T = size(KsitT,2);
etaMean = zeros(q,h);
KsiTMean = KsitT(:,T);
%
At = zeros(DSGEModel.ZLBSampleT+1,1);
Xt = [etaMean(:,1);KsiTMean;KsiTMean(DSGEModel.RtildePosition)];
for t=1:h;
   [At,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
   if status==1;
      Ksit = P(1:r,:)*[Xt;At];
      rhat = P(r+1,:)*[Xt;At];
%      ZLBKsiMean(:,t) = Ksit;
      if length(size(H))==2;
         ZLBYMean(:,t) = Ax(:,t)+(H'*Ksit);
      else;
         ZLBYMean(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit);
      end;
      if t+1<=h;
         At = M*At;
         Xt = [etaMean(:,t+1);Ksit;rhat];
      end;
   else;
      break;
   end;
end;
%
% check if the mean shocks and state failed to find the anticipated shocks.
%
if status==0;
   if AlgorithmInformation(1)==1;
      AddTxt = 'YADA has recorded an invertibility problem when attempting to solve for the anticipated shocks.';
   elseif AlgorithmInformation(2)==1;
      AddTxt = 'YADA has recorded that a solution cannot be found, i.e., Step 3 (ii) holds (see the YADA Manual).';
   elseif AlgorithmInformation(3)==1;
      AddTxt = 'YADA has recorded that the anticipated shocks are nonnegative, that the restricted policy rate satisfies the zero lower bound, but the complementary slackness condition is violated.';
   else;
      AddTxt = ['YADA has recorded that the maximum number of attempts (' int2str(AlgorithmInformation(5)) ') to find the anticipated shocks was reached.'];
   end;
   txt = ['The forward-back shooting algorithm failed to solve for the anticipated shocks for period ' int2str(t) ' using the population means of the structural shocks and the state variables in the last historical period. ' AddTxt ' Are you sure you want to continue?'];
   answer = Query(txt,'question',300,['ZLB Predictive Distributions' PriorHeader],600,'no',CurrINI);
   if strcmp(lower(answer),'no')==1;
      return;
   end;
end;
%
% set up the data matrices
%
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
T = size(Y,2);
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   Xdata = [];
end;
if strcmp(DSGEModel.DataFrequency,'q')==1;
   Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
else;
   Yinit = [];
end;
maxYinit = max(max(isnan(Yinit)));
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
if TransData==1;
   Yinitial = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
   %
   % try to covert: h
   %
   YTmp = [Yinitial DSGEModel.Y(:,DSGEModel.LastPeriod)*ones(1,h)];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   maxYinitial = max(max(isnan(Yinitial)));
else;
   maxYinitial = [];
   Yinitial = [];
   Tt = [];
end;
if k==1;
   ObsVarMean = zeros(n,1);
else;
   ObsVarMean = [];
end;
%
% check if we can use parallel computations or not
%
if CurrINI.NumWorkers<2;
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' int2str(NumPaths*NumDraws) ' Prediction Paths'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Path Number:';
      %
      ProgressStructure.name = ['ZLB Predictive Distributions' PriorHeader ' - ' TypeStr ' Data'];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      txt = ['Please wait while YADA computes predictive distributions subject to the zero (effective) lower bound for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['ZLB Predictive Distributions' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % begin the loop
   %
   abort = '';
   StopEstimation = 0;
   %NumDraws = 1;
   for ParamIndex=1:NumDraws;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use posterior mode or initial values as a backup
         %
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      %
      % We can now run the Kalman filter
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % Now we need to calculate KsiTT for just the last period
      %
      if length(size(H))==2;
         if DSGEModel.YNaNs==0;
            inv_msey = ((H'*Ptt1(:,:,T)*H)+R)\eye(n);
            KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
            PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H*inv_msey*H'*Ptt1(:,:,T));
         else;
            pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
            n1 = sum(pos);
            if n1>0;
               inv_msey = ((H(:,pos)'*Ptt1(:,:,T)*H(:,pos))+R(pos,pos))\eye(n1);
               KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
               PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos)*inv_msey*H(:,pos)'*Ptt1(:,:,T));
            else;
               KsiTT = Ksitt1(:,T);
               PTT = Ptt1(:,:,T);
            end;
         end;
      else;
         if DSGEModel.YNaNs==0;
            inv_msey = ((H(:,:,T)'*Ptt1(:,:,T)*H(:,:,T))+R)\eye(n);
            KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,:,T)*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
            PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,:,T)*inv_msey*H(:,:,T)'*Ptt1(:,:,T));
         else;
            pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
            n1 = sum(pos);
            if n1>0;
               inv_msey = ((H(:,pos,T)'*Ptt1(:,:,T)*H(:,pos,T))+R(pos,pos))\eye(n1);
               KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos,T)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
               PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos,T)*inv_msey*H(:,pos,T)'*Ptt1(:,:,T));
            else;
               KsiTT = Ksitt1(:,T);
               PTT = Ptt1(:,:,T);
            end;
         end;
      end;
      PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
      %
      % Try to solve the model subject to the anticipated shocks
      %
      [G,C1,P,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      if mcode~=1;
         %
         % use the posterior mode estimates of the parameters
         %
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
         if length(size(H))==2;
            if DSGEModel.YNaNs==0;
               inv_msey = ((H'*Ptt1(:,:,T)*H)+R)\eye(n);
               KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
               PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H*inv_msey*H'*Ptt1(:,:,T));
            else;
               pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
               n1 = sum(pos);
               if n1>0;
                  inv_msey = ((H(:,pos)'*Ptt1(:,:,T)*H(:,pos))+R(pos,pos))\eye(n1);
                  KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
                  PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos)*inv_msey*H(:,pos)'*Ptt1(:,:,T));
               else;
                  KsiTT = Ksitt1(:,T);
                  PTT = Ptt1(:,:,T);
               end;
            end;
         else;
            if DSGEModel.YNaNs==0;
               inv_msey = ((H(:,:,T)'*Ptt1(:,:,T)*H(:,:,T))+R)\eye(n);
               KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,:,T)*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
               PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,:,T)*inv_msey*H(:,:,T)'*Ptt1(:,:,T));
            else;
               pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
               n1 = sum(pos);
               if n1>0;
                  inv_msey = ((H(:,pos,T)'*Ptt1(:,:,T)*H(:,pos,T))+R(pos,pos))\eye(n1);
                  KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos,T)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
                  PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos,T)*inv_msey*H(:,pos,T)'*Ptt1(:,:,T));
               else;
                  KsiTT = Ksitt1(:,T);
                  PTT = Ptt1(:,:,T);
               end;
            end;
         end;
         [G,C1,P,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      end;
      %
      % setup ePG for computing the restricted policy rate projections
      %
      ePG = zeros(DSGEModel.ZLBSampleT+1,size(G,2));
      ePG(1,:) = e_pr'*P(1:r+1,:);
      for tau=1:DSGEModel.ZLBSampleT;
         ePG(tau+1,:) = ePG(tau,:)*G;
      end;
      %
      % make sure that we take into account that Rbar depends on the parameters
      %
      if k>0;
         Rbar = A(:,DSGEModel.ZLBPosition)'*X;
      end;
      %
      % Transform the steady-state interest rate and the lower bound to the same units
      % as the unrestricted monetary policy rate
      %
      if length(size(H))==2;
         Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
         Rzlb = (1/Hvalue)*RzlbMat;
         Rbar = (1/Hvalue)*Rbar;
      else;
         Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
         Th = length(Hvalue);
         if Th>DSGEModel.ZLBSampleT;
            Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
         elseif Th<DSGEModel.ZLBSampleT;
            Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
         end;
         Rzlb = RzlbMat./Hvalue;
         Rbar = Rbar./Hvalue;
      end;
      %
      % setup random draws of measurement errors including
      % ensuring that the policy rate does not have measurement errors
      %
      if rank(R)>0;
         R(:,DSGEModel.ZLBPosition) = 0;
         R(DSGEModel.ZLBPosition,:) = 0;
         [Rm,Lm] = SchurDecomposition(R);
         Rmeasure = Rm*sqrt(Lm);
         NumMeasure = size(Lm,1);
      else;
         Rmeasure = [];
         wTh = 0;
         NumMeasure = 0;
      end;
      %
      % make sure the deterministic component is updated
      %
      if k>0;
         Ax = A'*XPred;
      else;
         Ax = zeros(n,h);
      end;
      %
      % compute the population mean of the steady-state
      %
      if k==1;
         ObsVarMean = ObsVarMean+(1/NumDraws)*A';
      end;
      %
      % compute a decomposition of PTT in order to draw random values of
      % the state at T, the end of the historical data
      %
      [S,L] = SchurDecomposition(PTT);
      %
      % we are now ready to compute possible paths at theta
      %
      ShockMatrix = zeros(q,h);
      NumZLBTimes = zeros(1,h);
      PredPaths = zeros(n,h,NumPaths);
      for it=1:NumPaths;
         CurrPath = it+(NumPaths*(ParamIndex-1));
         %
         % check if we should update the progress dialog
         %
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
            end;
         end;
         %
         % Draw a sequence of measurement errors
         %
         if isempty(Rmeasure)==0;
            wTh = Rmeasure*randn(NumMeasure,h);
            wTh(DSGEModel.ZLBPosition,:) = 0;
         end;
         %
         % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
         %
         KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
         PredictY = zeros(n,h);
         At = zeros(DSGEModel.ZLBSampleT+1,1);
         Xt = [zeros(q,1);KsiT;KsiT(DSGEModel.RtildePosition)];
         t = 1;
         while t<=h;
            %
            % add shocks to Xt
            %
            Xt(1:q) = randn(q,1);
            [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
            if status==1;
               At = AtNew;
               if max(At)>0;
                  %
                  % add the shocks when ZLB is binding and count how many times for given horizon
                  %
                  ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                  NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
               end;
               Ksit = P(1:r,:)*[Xt;At];
               rhat = P(r+1,:)*[Xt;At];
               if length(size(H))==2;
                  PredictY(:,t) = Ax(:,t)+(H'*Ksit);
               else;
                  PredictY(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit);
               end;
               if t+1<=h;
                  At = M*At;
                  Xt = [zeros(q,1);Ksit;rhat];
               end;
               t = t+1;
            end;
         end;
         %
         % add measurement error draw (or 0) to the forecast path
         %
         PredictY = PredictY+wTh;
         if AnnualizeData==1;
            PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
         elseif TransData==1;
            YTmp = [Yinitial PredictY];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredictY = YData(Tt-h+1:Tt,:)';
         end;
         %
         % update the mean predictions for all paths and parameter draws
         %
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/(NumDraws*NumPaths))*PredictY);
         %
         % add the projected path to the variable to be saved
         %
         PredPaths(:,:,it) = PredictY;
         drawnow;
      end;
      %
      if StopEstimation==1;
         break;
      end;
   %   if DSGEModel.AdjustPredictionPaths==1;
   %      %
   %      % we need to fix the paths such that the sample mean is equal to the population mean.
   %      %
   %      MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
   %      AdjustMean = PredMean-MeanPrediction;
   %      AdjustMean(DSGEModel.ZLBPosition,:) = 0;
   %      for i=1:NumPaths;
   %         PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
   %      end;
   %   end;
      %
      % save the calculations to disk
      %
      if IsPosterior==1;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Original-0-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Transformed-0-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Annual-0-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      else;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Original-Prior-0-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Transformed-Prior-0-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Annual-Prior-0-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      end;
      save(file,'PredPaths','ShockMatrix','NumZLBTimes');
      if DSGEModel.RunPredictionEvent==1;
         %
         % compute the prediction events results
         %
         PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
      end;
   end;
   %
   % close any wait dialog
   %
   if DSGEModel.ShowProgress==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   else;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % we can do parallel computations over the parameter draws
   %
   txt = ['Please wait while YADA computes predictive distributions subject to the zero (effective) lower bound for the ' lower(TypeStr) ' data using parallel computations with ' int2str(CurrINI.NumWorkers) ' workers. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['ZLB Predictive Distributions ' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   if DSGEModel.RandomNumberValue==1;
      %
      % without eval statements the spmd-code gives an error in older matlab systems
      %
      eval('spmd; rng(0,''combRecursive''); end;');
   else;
      eval('spmd; rng(''default''); end;');
   end;
   %
   % begin the loop
   %
   abort = '';
   StopEstimation = 0;
   MP = ModelParameters;
   parfor ParamIndex=1:NumDraws;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      ModelParameters = MP;
      [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use posterior mode or initial values as a backup
         %
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      %
      % We can now run the Kalman filter
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % Now we need to calculate KsiTT for just the last period
      %
      if length(size(H))==2;
         if DSGEModel.YNaNs==0;
            inv_msey = ((H'*Ptt1(:,:,T)*H)+R)\eye(n);
            KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
            PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H*inv_msey*H'*Ptt1(:,:,T));
         else;
            pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
            n1 = sum(pos);
            if n1>0;
               inv_msey = ((H(:,pos)'*Ptt1(:,:,T)*H(:,pos))+R(pos,pos))\eye(n1);
               KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
               PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos)*inv_msey*H(:,pos)'*Ptt1(:,:,T));
            else;
               KsiTT = Ksitt1(:,T);
               PTT = Ptt1(:,:,T);
            end;
         end;
      else;
         if DSGEModel.YNaNs==0;
            inv_msey = ((H(:,:,T)'*Ptt1(:,:,T)*H(:,:,T))+R)\eye(n);
            KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,:,T)*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
            PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,:,T)*inv_msey*H(:,:,T)'*Ptt1(:,:,T));
         else;
            pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
            n1 = sum(pos);
            if n1>0;
               inv_msey = ((H(:,pos,T)'*Ptt1(:,:,T)*H(:,pos,T))+R(pos,pos))\eye(n1);
               KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos,T)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
               PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos,T)*inv_msey*H(:,pos,T)'*Ptt1(:,:,T));
            else;
               KsiTT = Ksitt1(:,T);
               PTT = Ptt1(:,:,T);
            end;
         end;
      end;
      PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
      %
      % Try to solve the model subject to the anticipated shocks
      %
      [G,C1,P,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      if mcode~=1;
         %
         % use the posterior mode estimates of the parameters
         %
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
         if length(size(H))==2;
            if DSGEModel.YNaNs==0;
               inv_msey = ((H'*Ptt1(:,:,T)*H)+R)\eye(n);
               KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
               PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H*inv_msey*H'*Ptt1(:,:,T));
            else;
               pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
               n1 = sum(pos);
               if n1>0;
                  inv_msey = ((H(:,pos)'*Ptt1(:,:,T)*H(:,pos))+R(pos,pos))\eye(n1);
                  KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
                  PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos)*inv_msey*H(:,pos)'*Ptt1(:,:,T));
               else;
                  KsiTT = Ksitt1(:,T);
                  PTT = Ptt1(:,:,T);
               end;
            end;
         else;
            if DSGEModel.YNaNs==0;
               inv_msey = ((H(:,:,T)'*Ptt1(:,:,T)*H(:,:,T))+R)\eye(n);
               KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,:,T)*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
               PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,:,T)*inv_msey*H(:,:,T)'*Ptt1(:,:,T));
            else;
               pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
               n1 = sum(pos);
               if n1>0;
                  inv_msey = ((H(:,pos,T)'*Ptt1(:,:,T)*H(:,pos,T))+R(pos,pos))\eye(n1);
                  KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos,T)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
                  PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos,T)*inv_msey*H(:,pos,T)'*Ptt1(:,:,T));
               else;
                  KsiTT = Ksitt1(:,T);
                  PTT = Ptt1(:,:,T);
               end;
            end;
         end;
         [G,C1,P,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      end;
      %
      % setup ePG for computing the restricted policy rate projections
      %
      ePG = zeros(DSGEModel.ZLBSampleT+1,size(G,2));
      ePG(1,:) = e_pr'*P(1:r+1,:);
      for tau=1:DSGEModel.ZLBSampleT;
         ePG(tau+1,:) = ePG(tau,:)*G;
      end;
      %
      % make sure that we take into account that Rbar depends on the parameters
      %
      if k>0;
         Rbar = A(:,DSGEModel.ZLBPosition)'*X;
      end;
      %
      % Transform the steady-state interest rate and the lower bound to the same units
      % as the unrestricted monetary policy rate
      %
      if length(size(H))==2;
         Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
         Rzlb = (1/Hvalue)*RzlbMat;
         Rbar = (1/Hvalue)*Rbar;
      else;
         Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
         Th = length(Hvalue);
         if Th>DSGEModel.ZLBSampleT;
            Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
         elseif Th<DSGEModel.ZLBSampleT;
            Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
         end;
         Rzlb = RzlbMat./Hvalue;
         Rbar = Rbar./Hvalue;
      end;
      %
      % setup random draws of measurement errors including
      % ensuring that the policy rate does not have measurement errors
      %
      if rank(R)>0;
         R(:,DSGEModel.ZLBPosition) = 0;
         R(DSGEModel.ZLBPosition,:) = 0;
         [Rm,Lm] = SchurDecomposition(R);
         Rmeasure = Rm*sqrt(Lm);
         NumMeasure = size(Lm,1);
      else;
         Rmeasure = [];
         wTh = 0;
         NumMeasure = 0;
      end;
      %
      % make sure the deterministic component is updated
      %
      if k>0;
         Ax = A'*XPred;
      else;
         Ax = zeros(n,h);
      end;
      %
      % compute the population mean of the steady-state
      %
      if k==1;
         ObsVarMean = (1/NumDraws)*A';
      end;
      %
      % compute a decomposition of PTT in order to draw random values of
      % the state at T, the end of the historical data
      %
      [S,L] = SchurDecomposition(PTT);
      %
      % we are now ready to compute possible paths at theta
      %
      ShockMatrix = zeros(q,h);
      NumZLBTimes = zeros(1,h);
      PredPaths = zeros(n,h,NumPaths);
      MeanPredictY = zeros(n,h);
      for it=1:NumPaths;
         %
         % Draw a sequence of measurement errors
         %
         if isempty(Rmeasure)==0;
            wTh = Rmeasure*randn(NumMeasure,h);
            wTh(DSGEModel.ZLBPosition,:) = 0;
         end;
         %
         % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
         %
         KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
         PredictY = zeros(n,h);
         At = zeros(DSGEModel.ZLBSampleT+1,1);
         Xt = [zeros(q,1);KsiT;KsiT(DSGEModel.RtildePosition)];
         t = 1;
         while t<=h;
            %
            % add shocks to Xt
            %
            Xt(1:q) = randn(q,1);
            [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
            if status==1;
               At = AtNew;
               if max(At)>0;
                  %
                  % add the shocks when ZLB is binding and count how many times for given horizon
                  %
                  ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                  NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
               end;
               Ksit = P(1:r,:)*[Xt;At];
               rhat = P(r+1,:)*[Xt;At];
               if length(size(H))==2;
                  PredictY(:,t) = Ax(:,t)+(H'*Ksit);
               else;
                  PredictY(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit);
               end;
               if t+1<=h;
                  At = M*At;
                  Xt = [zeros(q,1);Ksit;rhat];
               end;
               t = t+1;
            end;
         end;
         %
         % add measurement error draw (or 0) to the forecast path
         %
         PredictY = PredictY+wTh;
         if AnnualizeData==1;
            PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
         elseif TransData==1;
            YTmp = [Yinitial PredictY];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredictY = YData(Tt-h+1:Tt,:)';
         end;
         %
         % update the mean predictions for all paths and parameter draws
         %
         MeanPredictY = MeanPredictY+(Weights(ParamIndex)/(NumDraws*NumPaths))*PredictY;
         %
         % add the projected path to the variable to be saved
         %
         PredPaths(:,:,it) = PredictY;
         drawnow;
      end;
   %   if DSGEModel.AdjustPredictionPaths==1;
   %      %
   %      % we need to fix the paths such that the sample mean is equal to the population mean.
   %      %
   %      MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
   %      AdjustMean = PredMean-MeanPrediction;
   %      AdjustMean(DSGEModel.ZLBPosition,:) = 0;
   %      for i=1:NumPaths;
   %         PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
   %      end;
   %   end;
      %
      % save the calculations to disk
      %
      if IsPosterior==1;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Original-0-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Transformed-0-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Annual-0-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      else;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Original-Prior-0-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Transformed-Prior-0-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Annual-Prior-0-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      end;
      savefileparfor(file,PredPaths,MeanPredictY,ObsVarMean,ShockMatrix,NumZLBTimes);
      if DSGEModel.RunPredictionEvent==1;
         %
         % compute the prediction events results
         %
         PredEventData = CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
         savefileparfor(file,[],[],[],[],[],PredEventData);
      end;
   end;
   %
   % run additional calculations to collect estimates if required to do so PredictionEvent only
   %
   if DSGEModel.RunPredictionEvent==1;
      PredEventData = zeros(n,8);
   end;
   MeanPredictY = zeros(n,h);
   if k==1;
      ObsVarMean = zeros(n,1);
   else;
      ObsVarMean = [];
   end;
   for ParamIndex=1:NumDraws;
      if IsPosterior==1;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Original-0-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Transformed-0-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Annual-0-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      else;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Original-Prior-0-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Transformed-Prior-0-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-ZLB-Prediction-Annual-Prior-0-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      end;
      %
      % load the file
      %
      ParamData = load(file);
      MeanPredictY = MeanPredictY+ParamData.MeanPredictY;
      if isempty(ObsVarMean)==0;
         ObsVarMean = ObsVarMean+ParamData.ObsVarMean;
      end;
      %
      % add up the PredEventData
      %
      if DSGEModel.RunPredictionEvent==1;
         PredEventData = PredEventData+ParamData.PredEventData;
      end;
   end;
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
if StopEstimation==1;
   return;
end;
%
% make sure we also store additional data in the last file
%
DoneCalc = 1;
if isfield(DSGEModel,'PredictionEvent')==1;
   PredictionEvent = DSGEModel.PredictionEvent;
else;
   PredictionEvent = [Inf*ones(n,1) zeros(n,1) ones(n,1)];
end;
save(file,'MeanPredictY','PredEventData','PredictionEvent','ObsVarMean','-APPEND');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% a local function for saving to matfile from a parfor loop
%

function savefileparfor(file,PredPaths,MeanPredictY,ObsVarMean,ShockMatrix,NumZLBTimes,PredEventData)

if nargin==6;
   save(file,'PredPaths','MeanPredictY','ObsVarMean','ShockMatrix','NumZLBTimes');
elseif nargin==7;
   save(file,'PredEventData','-APPEND');
end;

%
% end of ZLBDSGEPredictionPaths.m
%
